# $Header:   //ARCHIVE/Archive_NLS/scripts/BJDiscoveryEnqueue.ksv   1.0   Jul 27 2001 15:58:06   Xiaoyan  $
#
# ca7Enqueue.ksh
#
# Goal: Enqueue the CA-7 Forecast files into the database for processing
#
# Usage: ./ca7Enqueue.ksh (see Usage)


# Get Database Server, User, and Password
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

# Get Install Directory
dirInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

Usage()
{

	cat <<USAGE_DOC

Usage: $0
  This script is used to enqueue temporary forecast files(multi files case) in database for batch discovery.
  The data have not yet been loaded into the database.

#  NOTE: The CA-7 forecast files enqueued with this utility will not be processed until a corresponding 
#   ready flag file has been enqueued.

  Required Parameters:

  -i<file_specification>      Specifies the temporary forecast filename, or ready flag file to be processed.
  -o<file_specification>      Specifies the filename to use when processing the file.
  -f<format type>	      Specifies FORMAT_TYPE of the data source.

  Optional Parameter(s):
  -c<localCodepage>           Specifies local codepage if local codepage <> 1252 (English ANSI).
  -S<db_server>               Specifies server name where database resides.  Default=[$db_server].
  -U<db_user>                 Specifies database user name.  Default=[$db_user].
  -P<db_password>             Specifies password for database user.  Default=[$db_password].
  -E                          Use trusted database connection. Use this instead of -U<db_username> and -P<db_password>
  -r<prefix>                  Specifies prefix to remove from the -o filename
  -v                          Display Results

  Examples:

$0 -f10 -ic:/temp/~042091 -o${dirInstall}Data/CA7/<ssn>.all.txt -c932
$0 -f10 -ic:/temp/~042092 -o${dirInstall}Data/CA7/<ssn>.key0.txt -c932
$0 -f10 -ic:/temp/~042093 -o${dirInstall}Data/CA7/<ssn>.ready.txt -c932

$0 -f10 -ic:/temp/~042094 -o${dirInstall}Data/CA7/TBSM.CA7.<ssn>.key0.txt -rTBSM.CA7. -c932
  The above line would create output to: ${dirInstall}Data/CA7/<ssn>.key0.txt

USAGE_DOC
}

fileEnqueue() 
{

timestamp=`date +"%Y%m%d_%H%M%S"`
isql_input_file="$0"."$timestamp".sql.in
isql_output_file="$0"."$timestamp".sql.out
filenamePrefix=$(print ${filenameOut} | awk '

/\.[Rr][Ee][Aa][Dd][Yy]\.[Tt][Xx][Tt]$/ {

  sub( "\.[Rr][Ee][Aa][Dd][Yy]\.[Tt][Xx][Tt]$", "", $0)
  print $0
}

// {

  next
}')

# determine ISQL command string
SQLCMD="isql -n -w3000"

if [ $bTrusted -eq 1 ]; then
  SQLCMD="${SQLCMD} -S$db_server -E"
else
  SQLCMD="${SQLCMD} -S$db_server -U$db_user -P$db_password"
fi

echo "exec asisp_setBJBulkloadStatus @FormatType = "$format_type", @file_path = \"$filenamePrefix\", @pstep = 0, @pstat = 0, @local_codepage =" $localCodepage > ${isql_input_file}
${SQLCMD} -i${isql_input_file} > ${isql_output_file}

if [ $bVerbose -eq 1 ]; then
  cat ${isql_output_file}
fi

rm -f ${isql_input_file}
rm -f ${isql_output_file}
}

# Default Values

bTrusted=0
bVerbose=0
bPrefixRemoval=0
localCodepage=1252

# Process the command line
while getopts "i:o:f:d:v:C:S:U:P:Er:h?" c; do
  case $c in
  i) filenameIn=$OPTARG;;
  o) filenameOut=$OPTARG;;
  f) format_type=$OPTARG;;
  v) bVerbose=1;;
  C) localCodepage=$OPTARG;;
  S) db_server=$OPTARG;;
  U) db_user=$OPTARG;;
  P) db_password=$OPTARG;;
  E) bTrusted=1;;
  r) bPrefixRemoval=1;sPrefixPattern=$OPTARG;;
  h|?)  Usage; exit 0;;
  *)  Usage >&2; exit 1;;
  esac
done
shift $OPTIND-1


# check for ISQL utility
rc=`which isql.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi


# check for REGDMP utility
rc=`which REGDMP.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find REGDMP.EXE.  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure REGDMP.EXE is in the path." >&2
  exit 1
fi

if [ -z $filenameIn ]; then;
  echo "ERROR: -i option not specified." >&2
  Usage >&2
  exit 1
fi

if [ -z $filenameOut ]; then;
  echo "ERROR: -o option not specified." >&2
  Usage >&2
  exit 1
fi

if [ $bPrefixRemoval -ne 0 -a -z sPrefixPattern ]; then;
  echo "ERROR: Removal of prefix requested, but not specified" >&2
  Usage >&2
  exit 1
fi

if [ $bPrefixRemoval -ne 0 ];then;
  filenameOut=$(echo $filenameOut | awk -v sPrefixPattern=$sPrefixPattern '

// {
  gsub( "\#", "\\\#", sPrefixPattern)
  gsub( "\@", "\\\@", sPrefixPattern)
  gsub( "\.", "\\\.", sPrefixPattern)
  gsub( "\$", "\\\$", sPrefixPattern)
  sub( sPrefixPattern, "", $0)
  print $0
}

')

fi

# filenameOut=$(echo $filenameOut | awk '// { gsub( "#", "", $0); print $0}')
 
# Copy file from temporary location to work area

if [ $bVerbose -eq 1 ]; then
  optionCp="-v "
else
  optionCp=""
fi

cp $optionCp "$filenameIn" "$filenameOut" 

iError=$?

if [ $iError -ne 0 ]; then
  echo "ERROR: copy of ${filennameIn} to ${filenameOut} failed." >&2
  exit iError
fi

# if file created is a <ssn>.ready.txt file; then enqueue to system for processing

bIsReadyFile=$(print ${filenameOut} | awk '/\.[Rr][Ee][Aa][Dd][Yy]\.[Tt][Xx][Tt]$/ { print 1; next} // { print 0}')

if [ $bIsReadyFile -eq 1 ]; then;
  fileEnqueue
fi

exit 0

