# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/HistoryServerSetup.ksv   1.3   Aug 10 2001 09:34:44   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/HistoryServerSetup.ksv  $
# $Revision:   1.3  $
# $Date:   Aug 10 2001 09:34:44  $
# $Author:   lin  $
#!/bin/sh
#set -x
#
# HistoryServerSetup.ksh
#   History server set up script
#
#
usage()
{
  print 'SYNPOSIS'
  print '  History server set up script'
  print ''
  print 'Usage: '$0' [options...]'
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print '  -N <hostname>           The optional <hostname> is where the History server database to be set up.'
  print '                          Without this option, History Server Database will be set up locally.' 
  print '  -R <hist_db_user>       db_user of the History Server.' 
  print '  -W <hist_db_password>   db_password of the History Server.' 

  print ''
}

# Lookup the installation directory from the registry
INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

if [ -z $db_server ] || [ -z $db_user ] || [ -z $db_password ]; then
  print >&2 "TBSM SQL Server is not set up on this system"
  exit 1
fi

# check for ISQL utility
rc=`which osql.exe`; 
rc=$?
if [ $rc -ne 0 ]; then
  echo "ERROR: Could not find OSQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi

# set the default location to LOCAL
location="LOCAL"
localhost="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')";
hostname=$localhost

while getopts "U:S:P:EN:R:W:h?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  N)  hostname=$(print "$OPTARG" | awk '{print toupper($0);}');;
  R)  hist_db_user=$OPTARG;;
  W)  hist_db_password=$OPTARG;;
  h)  usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

db_server=$(echo $db_server | awk '{print toupper($0);}')

if [ "$db_server" != "$localhost" ]; then
  print >&2 "Please run $0 on TBSM SQL Server $db_server"
  exit 1
fi

# if input hostname is not the localhost name, set the location to REMOTE 
if [ "$hostname" != "$localhost" ]; then 
  location="REMOTE"

  # check the network connection to the remote server
  ping -n 1 $hostname  > nul:
  if [ $? -ne 0 ]; then 
    print $hostname 'Unreachable'
    exit 1; 
  fi

  if [ "$hist_db_user" == "" ]; then
    print >&2 "Please -R option to specify History Server DB user name"
    usage
    exit 1
  fi

  if [ "$hist_db_password" == "" ]; then
    print >&2 "Please -W option to specify History Server DB password"
    usage
    exit 1
  fi
fi



if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 "
  SQLCMD="osql -S$db_server -E -n -h-1 -w3000 "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 "
  SQLCMD="osql -S$db_server -U$db_user -P$db_password -n -h-1 -w3000 "
fi

print 'Setting up History Server database on' $location $hostname
#
cd $INSTALLDIR/sql
#
if [ "$location" = "LOCAL" ]; then
  #-------------------------------------------------
  # this is for single(local) server installation
  #-------------------------------------------------
  $SQLCMD -i"$INSTALLDIR/sql/EventHistory.PART.01.sql"

  $SQLCMD -i"$INSTALLDIR/sql/EventHistory.PART.02.Single.DB.sql"

  $SQLCMD -i"$INSTALLDIR/sql/EventHistory.PART.03.sql"

elif [ "$location" = "REMOTE" ]; then
  #-------------------------------------------------
  # this is for dual server installation
  #-------------------------------------------------
  
  # setup database on the REMOTE host
  $SQLCMD -S$hostname -U$hist_db_user -P$hist_db_password -i"$INSTALLDIR/sql/EventHistory.PART.01.sql"

  # setup database on the LOCAL host
  $SQLCMD -E -i"$INSTALLDIR/sql/EventHistory.PART.01.sql"

  # generate & customized the sql script
  # assuming password as ....
  
  awk -v hostname=$hostname -v hist_db_user=$hist_db_user -v hist_db_password=$hist_db_password ' \
    { gsub(/XXXX/, hostname) ; gsub(/ZZZZ/, hist_db_password); gsub(/YYYY/, hist_db_user); print;} ' \
    $INSTALLDIR/sql/EventHistory.PART.02.Two.DB.sql.inp > $INSTALLDIR/sql/EventHistory.PART.02.Two.DB.sql

  # need to run using -E
  $SQLCMD -E -i"$INSTALLDIR/sql/EventHistory.PART.02.Two.DB.sql"

  $SQLCMD -i"$INSTALLDIR/sql/EventHistory.PART.03.sql"

else
  print 'Program Error! Please contact system administrator'
fi

print 'History Server Setup Done!'

