# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/ca7.parse.ksv   1.5   Aug 20 2001 16:47:20   Forthman  $
#
# ca7.parse.ksh
# 
# Goal: Take a series of files:
#   <ssn>.all.txt
#   <ssn>.key<#>.txt
#
# And generate a single bcp file

Usage()	{

	cat <<USAGE_DOC

Usage: $0 <prefix for group of ca7 forecast files>

  This scripts takes the file prefix for a group of ca7 forecast files, 
  and parse all files with that prefix as a unit.

  The result is a bcp'able file that can be loaded into the SQL Server database.

Mandatory Argument
	
  -p<prefix for group of ca7 forecast files>    This prefix is a customer defined prefix to the ca7 
                                                forecast files that they produce.

  To effectively parse the ca7 forecast files there should be a group of the following files:

  <prefix>.all.txt        A file containing the CA7 forecast file for a particular instance.
  <prefix>.key<#>.txt     File(s) containing the CA7 forecast file(s) for those jobs that 
                           should be treated as keyjobs.

  Optional Argument

  -d<directory and prefix>    This is the same as the prefix command, but it is proceeded by the full path
                               where the files are located.

Examples:

$0 -pDEV
$0 -dC:/Tivolimanager/Data/CA7/DEV



USAGE_DOC

}

# Process the command line
while getopts "p:d:h?" c; do
  case $c in
    d) filepathTarget=$OPTARG;
       name_schedule_set=$(print $filepathTarget | awk '// { sub( ".*/", "", $0); print $0}');
       filepathTarget=$(print $filepathTarget | awk '// { match( $0, ".*/"); print substr($0, 1, RLENGTH) }');;
    p) name_schedule_set=$OPTARG;;
    h|?)  Usage; exit 0;;
    *)  Usage >&2; exit 1;;
  esac
done

if [ -z $name_schedule_set ]; then;
  print "ERROR: No schedule set name prefix was provided." >&2
  Usage >&2
  exit 1
fi

if [ $filepathTarget ]; then; 
  cd $filepathTarget
  iError=$?
  if [ iError -ne 0 ]; then;
    print "ERROR: Path provided [${filepathTarget}], Invalid." >&2
    Usage >&2
    exit 1
  fi
fi

if [ $(ls ${name_schedule_set}.all.txt ${name_schedule_set}.key*.txt 2> ca7.parse.error.txt | wc -l) -eq 0 ]; then
  print "ERROR: No files to associate with prefix arguement: $name_schedule_set"
  Usage >&2; exit 1;
fi

# First awk script handles changing \000 to . and embedding FILENAME_CURRENT, NAME_SCHEDULE_SET
# Second awk script parses for data

filename_temp=$0.$$.temp.txt
rm -f $filename_temp
for filename_current in $(ls ${name_schedule_set}.all.txt ${name_schedule_set}.key*.txt 2> ca7.parse.error.txt)
do 
	cat $filename_current | tr '\000' '.' | awk -v FILENAME_CURRENT=$filename_current -v NAME_SCHEDULE_SET=$name_schedule_set '
	BEGIN {
		FS  = "|"
		OFS = "|"
	}
	// {
		print FILENAME_CURRENT, NAME_SCHEDULE_SET, $0
	}' >> $filename_temp
done

awk '

BEGIN	{

	FS  = "|"
	OFS = "|"

	record_init()
	record_header()
	record_write()
	record_init()

}

END	{

	if ( name_job != "" ) {

		record_save()
		record_init()

	}

	record_output()

}

function record_write() {

	string_output = name_schedule_set
	string_output = string_output OFS name_system
	string_output = string_output OFS name_job
	string_output = string_output OFS id_schedule
	string_output = string_output OFS flag_keyjob
	string_output = string_output OFS flag_exec
	string_output = string_output OFS year_start
	string_output = string_output OFS day_start
	string_output = string_output OFS hour_start ":" minute_start
	string_output = string_output OFS hour_duration ":" minute_duration
	string_output = string_output OFS year_stop
	string_output = string_output OFS day_stop
	string_output = string_output OFS hour_stop ":" minute_stop

	print string_output

} 

function record_init()	{

	name_schedule_set = "" 
	name_system       = ""
	name_job          = ""
	id_schedule       = ""
	flag_keyjob       = ""
	flag_exec         = ""
	year_start        = ""
	day_start         = ""
	hour_start        = ""
	minute_start      = ""
	hour_duration     = ""
	minute_duration   = ""
	year_stop         = ""
	day_stop          = ""
	hour_stop         = ""
	minute_stop       = ""

        name_schedule_set = NAME_SCHEDULE_SET

	if (flag_key == 1) {

		flag_keyjob = 1

	} else {
	
		flag_keyjob = 0
	}

}

function record_header() {

	name_schedule_set = "name_schedule_set"
	name_system       = "name_system"
	name_job          = "name_job"
	id_schedule       = "id_schedule"
	flag_keyjob       = "flag_keyjob"
        flag_exec         = "flag_exec"
	year_start        = "year_start"
	day_start         = "day_start"
	hour_start        = "hour_start"
	minute_start      = "minute_start"
	hour_duration     = "hour_duration"
	minute_duration   = "minute_duration"
	year_stop         = "year_stop"
	day_stop          = "day_stop"
	hour_stop         = "hour_stop"
	minute_stop       = "minute_stop"

}

function record_save()	{

	index_record = name_schedule_set 
	index_record = index_record SUBSEP name_system
	index_record = index_record SUBSEP name_job
	index_record = index_record SUBSEP id_schedule
	index_record = index_record SUBSEP flag_exec
	index_record = index_record SUBSEP year_start
	index_record = index_record SUBSEP day_start
	index_record = index_record SUBSEP hour_start
	index_record = index_record SUBSEP minute_start
	index_record = index_record SUBSEP hour_duration
	index_record = index_record SUBSEP minute_duration
	index_record = index_record SUBSEP year_stop
	index_record = index_record SUBSEP day_stop
	index_record = index_record SUBSEP hour_stop
	index_record = index_record SUBSEP minute_stop

	if (index_record in a_record) 	{

		if ( a_record[ index_record ] != 1)	{

			a_record[ index_record ] = flag_keyjob
		}

	} else {

		a_record[ index_record ] = flag_keyjob
	}
}

function record_extract(index_record )	{

	record_init()

	FS = SUBSEP

	$0 = index_record

	$0 = $0

	name_schedule_set = $1
	name_system       = $2
	name_job          = $3
	id_schedule       = $4
	flag_keyjob       = a_record[ index_record ]
	flag_exec         = $5
	year_start        = $6
	day_start         = $7
	hour_start        = $8
	minute_start      = $9
	hour_duration     = $10
	minute_duration   = $11
	year_stop         = $12
	day_stop          = $13
	hour_stop         = $14
	minute_stop       = $15

}

function record_output()	{

	for( item in a_record )	{

		record_extract( item )
		record_write()
	}
}
	
function century_add( year_2, 		year_4 )	{

	if ( year_2 > 89)	{

		year_4 = "19" year_2
	} else {
		year_4 = "20" year_2
	}

	return year_4
}

# Target: Extraction and Removal of FILENAME_CURRENT|NAME_SCHEDULE_SET|
//	{

	FILENAME_CURRENT  = $1
	NAME_SCHEDULE_SET = $2

	pattern_prefix = "^" FILENAME_CURRENT "\\" FS NAME_SCHEDULE_SET "\\" FS

	sub( pattern_prefix, "", $0)

	$0 = $0

}

(FILENAME_CURRENT!=filename_history)	{

	flag_key    = 0
	flag_all    = 0
	flag_ignore = 0

	if ( match( FILENAME_CURRENT, /.*\.[Kk][Ee][Yy][[:digit:]]*\.[Tt][Xx][Tt]/) > 0 ) {
		
		flag_key = 1

	} else  if ( match( FILENAME_CURRENT, /.*\.[Aa][Ll][Ll]\.[Tt][Xx][Tt]/) > 0 ) {

		flag_all = 1

	} else {

	flag_ignore = 1

	}

	if ( name_job != "")	{

		record_save()
	}

	record_init()

	filename_history = FILENAME_CURRENT

}

(flag_ignore==1)	{

	next

}


# Target: name_job
#  JOB=DMD0800C                                           COMPLETED ____________

/^[[:space:]]+JOB=[$#@[:alnum:]]+/	{

	if (name_job != "")	{

		record_save()
		record_init()
	}
	
	match( $0, /^[[:space:]]+JOB=[$#@[:alnum:]]+/ )

	name_job = substr( $0, RSTART, RLENGTH)

	sub( "^[[:space:]]+JOB=", "", name_job)

	next
}

# Target: year_start, day_start, hour_start, minute_start
#         SCHEDULE:   LATEST STRT=00291/0638    SCHD MBR=SJ001375

/LATEST STRT=[[:digit:]]{5}\/[[:digit:]]{4}/	{

	match( $0, /LATEST STRT=[[:digit:]]{5}\/[[:digit:]]{4}/ )

	datetime_start = substr( $0, RSTART, RLENGTH)

	sub( "^LATEST STRT=", "", datetime_start)
	
	year_start   = substr( datetime_start, 1, 2)
	year_start   = century_add( year_start)

	day_start    = substr( datetime_start, 3, 3)
	day_start    = day_start + 0
	
	hour_start   = substr( datetime_start, 7, 2)
	minute_start = substr( datetime_start, 9, 2)

	next
}

# Target: year_stop, day_stop, hour_stop, minute_stop, id_schedule
#                     DUEOUT TIME=00291/0640    SCHED ID=001

/DUEOUT TIME=[[:digit:]]{5}\/[[:digit:]]{4} {4}SCHED ID=[[:digit:]]{3}/    {

	match( $0, /DUEOUT TIME=[[:digit:]]{5}\/[[:digit:]]{4}/ )

	datetime_stop = substr( $0, RSTART, RLENGTH)
	
	sub( "^DUEOUT TIME=", "", datetime_stop)

	year_stop    = substr( datetime_stop, 1, 2)
	year_stop    = century_add( year_stop)

	day_stop     = substr( datetime_stop, 3, 3)
	day_stop     = day_stop + 0

	hour_stop    = substr( datetime_stop, 7, 2)
	minute_stop  = substr( datetime_stop, 9, 2)

	match( $0, /SCHED ID=[[:digit:]]{3}/)

        id_schedule  = substr( $0, RSTART, RLENGTH)
        sub( "SCHED ID=", "", id_schedule)

	next
}

# Target: name_system
#                     SYSTEM=DME                LTERM=

/SYSTEM=[^[:space:]]+/ {

	match( $0, /SYSTEM=[^[:space:]]+/ )
	
	name_system = substr( $0, RSTART, RLENGTH)

	sub( "SYSTEM=", "", name_system)

	next
}

# Target: hour_duration, minute_duration
#                     ELAPTM=0002   TP1 =000    TP2 =000

/ELAPTM=[[:digit:]]{4}/	{

	match( $0, /ELAPTM=[[:digit:]]{4}/ )
	
	datetime_duration = substr( $0, RSTART, RLENGTH)

	sub( "ELAPTM=", "", datetime_duration)

	hour_duration    = substr( datetime_duration, 1, 2)
	minute_duration  = substr( datetime_duration, 3, 2)
	
	next
}

# Target: flag_exec
#          EXECUTION:  RELOAD=NO     EXEC=YES    MAINT=NO

/EXEC=[[:alpha:]]{2,3}/	{

	if ( match( $0, /EXEC=YES/))	{

		flag_exec = 1
	} else {
		flag_exec = 0
	}
}


' $filename_temp > ${name_schedule_set}.ca7.bcp

rm -f $filename_temp
