# $Header:   //ARCHIVE/Archive_NLS/scripts/check_opc_plan.ksv   1.0.1.1   Aug 01 2001 14:01:00   BRENT  $
#
# check_opc_plan.ksh
#
# Goal: for each line in the file that lists the opc daily plan files
#          check the file by ls to see if the date or size has changed, 
#          if it has then call OPCParseDailyPlan.ksh
#

Usage()
{
  print ''
  print ' Usage: ' $0 ' -f<file_input> ...'
  print ''
  print '   Required Param(s):'
  print '   -f<file_input>    OPC Daily Plan Listing File'
  print ''
  print '   Plus all the parameters needed for OPCParseDailyPlan.ksh'
  print '    -l<language>     Optional parameter that specifies an alternate language'
  print '                     to use for keyword parsing. The default is $TM_LANG.'
  print '    -nm              Skip the mapping of the loaded data.'
  print '    -np              Skip the parsing of the data.'
  print '    -nl              No load of the parsed data is performed.'
  print '    -v               Run in verbose mode.'
  print '    -E               Use a trusted database connection'
  print '    -U<user>         User name for the database login'
  print '    -P<password>     Password for the database login'
  print '    -S<server>       Database server'
  print '    -e               Central server: Use a trusted database connection'
  print '    -u<user>         Central server: User name for the database login'
  print '    -p<password>     Central server: Password for the database login'
  print '    -s<server>       Central server: Database server'
  print ''
  print '   OPC Daily Plan Listing File Format'
  print ''
  print '      Each line of the file should be of the format:'
  print ''
  print '      <opc_daily_plan_filename>|<datetime_pattern>'
  print ''
  print '                           or'
  print ''
  print '      <opc_daily_plan_filename>'
  print ''
  print '      Where:'
  print '      <opc_daily_plan_filename> The filename of a OPC daily plan that should be'
  print '                                 checked and loaded on discovery of a new '
  print '                                 version.'
  print '      <datetime_pattern>        The pattern that should override the datetime'
  print '                                 pattern specified by the -l<language> option'
  print '                                 of OPCParseDailyPlan.ksh, this field is'
  print '                                 optional.'
  print ''

}

# all arguments, let others pass thru

while getopts f:hd:i:l:EU:S:P:eu:s:p:n:v c; do
  case $c in 
  f) filename_list=$OPTARG;;
  l) ;;
  i)  ;;
  l)  ;;
  d)  ;;
  E)  ;;
  U)  ;;
  P)  ;;
  S)  ;;
  n)  case $OPTARG in
      l)   ;;
      la)  ;;
      lo)  ;;
      lp)  ;;
      p)  ;;
      m)  ;;
      esac
      ;;
  v)  ;;
  *) Usage; exit 1;;
  esac
done

if [ -z "$filename_list" ]; then
  Usage >&2
  exit 1
fi

awk -v filename_script="$0" -v args_shell="$*" '

BEGIN	{

	# stripout the -f call since the parsing routine does want it

	sub( "-f[^ ]* -", "-", args_shell)
	sub( "-f[^ ]*$", "", args_shell)

	FS  = "|"

	FILENAME_HISTORY = filename_script ".history.txt"
	DIRECTORY_LOG    = "logs"

	filename_history_load()

	if ( directory_check( DIRECTORY_LOG) == 0 ) {
	
		directory_create( DIRECTORY_LOG)
	}
}

function directory_check( filename)	{

	flag_exist = 0

	filename_pattern = pattern_filename_create( filename)

        string_command = "ls -ld " filename_pattern " 2>&1"

        string_command | getline string_listing

 	close( string_command)

	if ( match( string_listing, "^d.*" filename_pattern "$" ) > 0 ) {

		flag_exist = 1
	}

	return flag_exist
}

function directory_create( filename)	{

	string_command = "mkdir " filename

	system( string_command)
   	close(  string_command)
}

function filename_history_load(    count_line, string_line, filename) {

	count_line = 0
	
	while( (getline string_line < FILENAME_HISTORY) > 0) {

		count_line++

		if ( ( count_line % 2) == 1 ) {

			filename = string_line

		} else {

			history[ filename ] = string_line

		}
	}

	close( FILENAME_HISTORY)
}

/^ *#/	{
	# Drop lines that begin with hash marks
	
	next
}

/^ *$/	{
	# Drop blank lines

	next
}

function opc_parse( filename, pattern_date, 		string_command)	{


	filename_log = DIRECTORY_LOG "/" "\"" filename "\"" ".log"

	string_redirect = " 2>&1 > " filename_log

	string_command = "OPCParseDailyPlan.ksh"
	string_command = string_command " " args_shell 

	if ( pattern_date != "" ) {

		string_command = string_command " -d" pattern_date

	}

	string_command = string_command " " "\"" filename "\""

	string_command = string_command " " string_redirect

	# print string_command

	system( string_command)
  	close(  string_command)
	
}

function pattern_filename_create( filename,		max_length, string_pattern, pos, char)	{

	max_length = length( filename)

	string_pattern = ""

	for( pos = 1; pos <= max_length; pos++) {

		char = substr( filename, pos, 1)

		if ( match( char, "[^0-9]") > 0 ) {
			
			string_pattern = string_pattern "\\"
		}

		string_pattern = string_pattern char

	}

	return string_pattern
}

//	{

	# checking directory entry is assumed to be a good enough test.
	# additional test would be sum -r

	filename = $1
	pattern_date = $2

	filename_pattern = pattern_filename_create( filename)

	string_command = "ls -l " filename_pattern " 2>&1"

	string_command | getline string_listing

	close( string_command)

	if ( string_command in history )  {

		# file status has changed

		string_history  = history[ string_command ]

		if ( string_history != string_listing ) {

			if ( match( string_listing, filename_pattern "$" ) > 0 ) {

				opc_parse( filename, pattern_date)
			}

		}
	} else {

		# New file 

		if ( match( string_listing, filename_pattern "$" ) > 0 ) {
	
		        opc_parse( filename, pattern_date)
		}
		
	}

	history[ string_command ] = string_listing
}

END	{

	string_command = "touch " FILENAME_HISTORY

	system( string_command)
	close(  string_command)

        string_command = "rm " FILENAME_HISTORY

	system( string_command)
	close(  string_command)

	for( filename in history ) {

		print filename >> FILENAME_HISTORY 
		print history[ filename ] >> FILENAME_HISTORY
	}

	close( FILENAME_HISTORY)
}
' ${filename_list}
