# $Header:   //ARCHIVE/Archive/scripts/makemvscomponents.ksv   1.18   Sep 22 2000 08:40:26   DAVID  $
#
# MakeMVSComponents.ksh
#
# Goal: Setup a new installation of TMOS390 with Event Handler, Listener, and File Receiver for OS specified.
#
# usage: see MakeMVSComponents.ksh -?

num_params=$#;
component_listing='ALL'
bInstall_EH=0;
bInstall_LS=0;
bInstall_SS=0;
bInstall_FR=0;

# Get Database Server, User, Password from Registry
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

# default values
bTrusted=0
bRemoveComponents=0
bForceDatabaseSettings=0
TMP_LOCALDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\\\\\\\\/g' | sed 's/\\\\\\\\$//g')
TMP_INSTALLDIR='//'$COMPUTERNAME'/'$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's?:?$?g' | sed 's/\/$//g')
UPLOAD_MODENAME='LU62PS';
local_host="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')";
mvs_host="$local_host"

usage   ()
{
  print ''
  print 'Usage: '$0' -B<base_name> -O<os_path> [options...]'
  print ''
  print '  Required Parameters(s) - All Components:'
  print '  -B<base_name>           Base name of created NT Services.  e.g. \047Tivoli BSM MVSEventHandlerSvc-<base_name>\047'
  print '  -O<os_path>             MVS Operating System Path (<os_name> or full/partial:\047<ENT/COMP/MACH/LPAR/OS>\047)'
  print ''
  print '  Additional Required Parameters(s) for Listener Services:'
  print '  -d<download_LU_name>    MVS Listener Service Partner LU (download).'
  print '  -t<local_LU_name>       Local LU name'
  print ''
  print '  Additional Required Parameters(s) for Sender Services:'
  print '  -x<upload_LU_name>      MVS Sender Service Partner LU (upload).'
  print ''
  print '  Additional Required Parameters(s) for Removing All Components:'
  print '  -r                      Remove all MVS components specified by <base_name> and <os_path>'
  print ''
  print '  Optional Parameters(s):'
  print '  -I<install_dir>         UNC Remote Directory Name.  Default=['$TMP_INSTALLDIR']'
  print '  -L<local_dir>           Local Directory Name.  Default=['$TMP_LOCALDIR']'
  print '  -H<mvs_host>            Hostname of MVSEventHandlers.  Default=['$mvs_host']'
  print '  -m<upload_mode>         MVS Sender Service Modename (upload only).  Default=['$UPLOAD_MODENAME']'
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<db_username> and -P<db_password>'
  print '  -f                      Force the inclusion of a Database Settings registry key under'
  print '                          each component instead of using the default database settings'
  print '                          on the MVS Host ('$mvs_host').'
  print '  -c<component_listing>   Comma-separated list of components to install.  Default is "ALL".'
  print '                          See list below for valid component codes.  Note that the [-r] option'
  print '                          is not affected by this option.'
  print ''
  print '  Component Codes:'
  print '  ALL - All 4 components  (EH,LS,SS,FR)'
  print '  EH  - Event Handlers    (Tivoli BSM MVSEventHandlerSvc)'
  print '  LS  - Listener Services (Tivoli BSM MVSListenerSvc)'
  print '  SS  - Sender Services   (Tivoli BSM MVSSenderSvc)'
  print '  FR  - File Receivers    (Tivoli BSM FileReceiverSvc)'
  print ''
  print ''
  print '  Examples:'
  print ''
  print '  1) Create all MVS components for an OS named E003:'
  print '      '$0' -BE003 -OE003 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '  2) Create all MVS components for two OS instances named CPUA (same SMFID) located on different MVS host machines, M1 and M2:'
  print '      '$0' -BM1CPUA -O\047M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print '      '$0' -BM2CPUA -O\047M2/LPARA/CPUA\047 -HASIMVS -L\047C:\\\\TivoliManager\047 -I\047//ASIMVS/C$/TivoliManager\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390' -mLU62PS
  print ''
  print '  3) Create MVS components for an OS named with partially qualified (MACH/LPAR/OS) path \047M1/LPARA/CPUA\047:'
  print '      '$0' -BE003 -O\047M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '  4) Create MVS components for an OS named with fully qualified (ENT/COMP/MACH/LPAR/OS) path \047Tivoli Systems/Eastern/M1/LPARA/CPUA\047:'
  print '      '$0' -BE003 -O\047Tivoli Systems/Eastern/M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '  5) Remove all MVS components for an OS named E003:'
  print '      '$0' -BE003 -OE003 -r'
  print ''
  print '  6) Remove all MVS components for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$0' -BM2CPUA -O\047M2/LPARA/CPUA\047 -r'
  print ''
  print '  7) Install only Event Handler and Listener Services for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$0' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cEH,LS -dUSASI01.NJ1TM390 -tNJUS47A'
  print ''
  print '  8) Install only Sender Service for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$0' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cSS -xNJ1TM390'
  print ''
}

while getopts "H:I:L:U:S:B:O:P:x:d:m:t:c:Erf?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  H)  mvs_host=$(echo $OPTARG | awk '{print toupper($0);}');;
  L)  LOCALDIR=$OPTARG;;
  I)  INSTALLDIR=$OPTARG;;
  B)  BASENAME=$OPTARG;;
  O)  OSPATH=$OPTARG;;
  t)  LOCAL_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  x)  UPLOAD_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  d)  DOWNLOAD_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  m)  UPLOAD_MODENAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  E)  bTrusted=1;;
  r)  bRemoveComponents=1;;
  f)  bForceDatabaseSettings=1;;
  c)  component_listing=$OPTARG;;
	\?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done
shift $OPTIND-1

process_component_listing()
{
  if [ "$component_listing" = "ALL" ]; then;
    component_listing="EH,LS,SS,FR"
  fi
  echo "Validating component code(s): \c"
  for component_type in $(print "${component_listing}" | awk 'BEGIN {RS=","} {print}'); do
  {
    echo "[$component_type] \c";
    case $component_type in
    EH) bInstall_EH=1;;
    LS) bInstall_LS=1;;
    SS) bInstall_SS=1;;
    FR) bInstall_FR=1;;
    *)  print >&2 "\nERROR: invalid component type [$component_type] in component listing [$component_listing]"; usage; exit 1;;
    esac
  }
  done;
  print "";
}


check_nt_res_kit()
{
  filename="${1?"ERROR: check_nt_res_kit() missing required field <filename>"}"
  
  # check for utility
  rc=`which "$filename"`; rc=$?
  if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find "$filename".  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure "$filename" is in the path." >&2
    exit 1
  fi
}

check_for_required_parameters()
{
  # check for required parameters
  if [ -z "$BASENAME" ]; then
    echo "ERROR: Missing required parameter -B<base_name>" >&2
    usage >&2
    exit 2
  fi
  if [ -z "$OSPATH" ]; then
    echo "ERROR: Missing required parameter -O<os_path>" >&2
    usage >&2
    exit 2
  fi
  if [ $bRemoveComponents -eq 0 ]; then;
    if [ $bInstall_LS -eq 1 ]; then
      if [ -z "${DOWNLOAD_LUNAME}" ]; then
        echo "ERROR: Missing required parameter -d<download_LU_name>" >&2
        usage >&2
        exit 2
      fi
      if [ -z "${LOCAL_LUNAME}" ]; then
        echo "ERROR: Missing required parameter -t<local_LU_name>" >&2
        usage >&2
        exit 2
      fi
    fi
    if [ $bInstall_SS -eq 1 -a -z "${UPLOAD_LUNAME}" ]; then
      echo "ERROR: Missing required parameter -x<upload_LU_name>" >&2
      usage >&2
      exit 2
    fi
  else
    if [ $num_params -lt 3 ]; then
      echo "ERROR: Missing required parameter(s)" >&2
      usage >&2
      exit 5
    fi
  fi

  check_nt_res_kit REGFIND.EXE
  check_nt_res_kit REGINI.EXE
  check_nt_res_kit SC.EXE
  check_nt_res_kit REGDMP.EXE
  check_nt_res_kit REG.EXE

  # if local or install directory not specified, look in the registry of host <mvs_host>
  if [ -z "$LOCALDIR" ]; then;
    LOCALDIR=$(REGFIND -m '\\'"$mvs_host" -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\\\\\\\\/g' | sed 's/\\\\\\\\$//g')
  fi
  if [ -z "$INSTALLDIR" ]; then;
    INSTALLDIR='//'"$mvs_host"'/'$(REGFIND -m '\\'"$mvs_host" -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's?:?$?g' | sed 's/\/$//g')
  fi

  # get rid of potential extra backslash ('\')
  LOCALDIR=$(print "$LOCALDIR" | sed 's/\\\\/\\/g')

  # check for ISQL utility
  rc=`which isql.exe`; rc=$?
  if [ $rc -ne 0 ]; then;
    echo "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." >&2
    exit 1
  fi

  if [ -z "$INSTALLDIR" ]; then
    echo "ERROR: Could not access directory [$INSTALLDIR]" >&2
    usage >&2
    exit 3
  fi

  if [ ! -d "$INSTALLDIR" ]; then
    echo "ERROR: Could not access directory [$INSTALLDIR]" >&2
    exit 4
  fi

  if [ -z "$LOCALDIR" ]; then
    echo "ERROR: Could not access directory [$LOCALDIR]" >&2
    usage >&2
    exit 3
  fi

  if [ ! -d "$LOCALDIR" ]; then
    echo "ERROR: Could not access directory [$LOCALDIR]" >&2
    exit 4
  fi
}


validate_os_path()
{
  # Lookup the OS id based on the path
  echo "Using Database Host=[$db_server], MVS Host=[$mvs_host]"
  print "$OSPATH" | awk -F'/' 'BEGIN {
    tables[1]="OS_C";
    tables[2]="LPAR_C";
    tables[3]="MACHINE_C";
    tables[4]="COMPLEX_C";
    tables[5]="ENT_C";

    cids[1]="OS";
    cids[2]="LPAR";
    cids[3]="MACH";
    cids[4]="COMP";
    cids[5]="ENT";
  }
  {
    if (NF > 5) {
      print "ERROR: OS path cannot have more than 5 fields. It must be of the form"
      print "  ENT/COMPLEX/MACHINE/LPAR/OS"
      print "The path does not have to be fully specified. If the OS name is unique"
      print "then only the OS name is required. If OS names are unique by machine"
      print "then the path may be specified as MACHINE/LPAR/OS."
    }
    sql="SET NOCOUNT ON\nDECLARE @rc INT\nSELECT CONVERT(BINARY(3),t1.id)\nFROM OS_V AS t1";
    if ( NF == 1 ) {
      sql=sprintf("%s\n  JOIN link as link0 on link0.dst_cid = '\''OS'\'' AND link0.dst_id = t1.id AND link_type='\''PHYC'\'' AND link0.deleted = 0",
                  sql);
    }
    for (i = 2; i <= NF; i++) {
      idx=i-1;
      table=tables[i];
      sql=sprintf("%s\n  JOIN link AS link%d ON link%d.src_cid='\''%s'\'' AND link%d.dst_cid='\''%s'\'' AND link%d.dst_id=t%d.id AND link%d.deleted=0",
                  sql, idx, idx, cids[i], idx, cids[i-1], idx, idx, idx);
      sql=sprintf("%s\n  JOIN %s AS t%d ON t%d.id = link%d.src_id AND t%d.name='\''%s'\''\n",
                  sql, tables[i], i, i, idx, i, $(NF-(idx)));
    }
    sql = sprintf("%s\nWHERE t1.name = '\''%s'\''", sql, $NF);
    print sql;
    print "SELECT @rc=@@ROWCOUNT"
    print "IF @rc=0"
    print "  RAISERROR(\047ERROR: no matching OS id found\047, 11, 1)"
    print "ELSE IF @rc>1"
    print "  RAISERROR(\047ERROR: duplicate non-deleted OS ids found\047, 11, 1)"
  }
  ' > "$0.IN.SQL"
  if $(cat "$0.IN.SQL" | grep -v RAISERROR | grep -q ERROR:); then
    echo '' >&2
    echo 'Error: processing OS path' >&2
    cat "$0.IN.SQL" | grep -v RAISERROR | grep ERROR: >&2
    echo '' >&2
    echo '*** SQL CODE ********************************' >&2
    cat "$0.IN.SQL" >&2
    echo '*** SQL CODE ********************************' >&2
    exit 1
  fi

  # obtain the OS id from the database
  if [ $bTrusted -eq 1 ]; then
    SQL_CMD_LINE="isql -S$db_server -E -n -w3000 -dObject";
  else
    SQL_CMD_LINE="isql -S$db_server -U$db_user -P$db_password -n -w3000 -dObject";
  fi

  OSID=$($SQL_CMD_LINE -i"$0.IN.SQL" |
    awk '
    /^[ ]*0x/ {
      gsub(/[ ]*/,"");
      sub(/^0x/,"");
      os_id=sprintf("%s0006\n", toupper($0));
    }
    /ERROR/ {
      err_str=$0;
    }
    END {
      if (err_str=="")
        print os_id
      else
        print err_str
    }
  ')
  rm -f "$0.IN.SQL"

  # check for errors in isolating OSID
  if $(print "$OSID" | grep -iq ERROR); then
    if [ $bRemoveComponents -eq 1 ]; then;
      OSID="0000000006"
    else
      print "$OSID" for [$OSPATH] >&2
      exit 5
    fi
  fi

  # check for missing OSID
  if [ -z "$OSID" ]; then
    echo "ERROR: Could not find id from OS $OSNAME" >&2
    exit 6
  fi
  echo 'Found OS id='$OSID' for ['$OSPATH']'
  os_id="0x"${OSID%0006}
}

delete_service()
{
  svc_name="${1?"ERROR: delete_service() missing required parameter <svc_name>"}"

  echo 'Deleting NT service ['$svc_name']...\c'
  rc=$(sc \\\\$mvs_host delete $svc_name 2>&1)
  if $(print "$rc" | grep -iq  -e "OpenSCManager failed" -e "failed"); then;
    msg_no=$(print "$rc" | awk '{print $NF}')
    if [ $msg_no -eq 1060 ]; then;  #1060=The specified service does not exist as an installed service.
      echo 'OK (already deleted)'
    elif [ $msg_no -eq 1072 ]; then; #1072 - The specified service has been marked for deletion.
      echo 'OK (already marked for deletion; delete will occur on next re-boot)'
    else
      print "FAILURE: $rc"
      NET HELPMSG $msg_no
    fi
  else
    echo 'OK'
  fi
}

delete_registry_entry()
{
  svc_type="${1?"ERROR: delete_registry_entry() - missing required parameter <svc_type> "}"
  svc_base="${2?"ERROR: delete_registry_entry() - missing required parameter <svc_base> "}"

  print 'Deleting registry entry for ['$svc_type'-'$svc_base']...\c'
  rc=$(echo 'y' | REG DEL 'HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components\'${svc_type}'\Instances\'"${svc_base}" '\\'"$mvs_host" 2>&1)
  if $(print "$rc" | grep -q "Permanentely delete"); then;
    print "OK"
  elif $(print "$rc" | grep -q "The system was unable to find the specified registry key"); then;
    print "OK (already deleted)"
  else
    print "ERROR: $rc"
  fi
}

delete_file()
{
  filepath="${1?"ERROR: delete_file() - missing required parameter <filepath>}"
  
  echo 'Deleting file ['"${filepath}"']...\c'
  if [ -f "${filepath}" ]; then; 
    rc=$(rm -f "${filepath}" 2>&1);
    if $(print "$rc" | grep -q "Access is denied"); then;
      which DeleteFileDelayed.exe 2>&1 > nul:
      if [ $? -eq 0 ]; then;
        DeleteFileDelayed -dr "${filepath}"
        print 'OK (file is in use; delete will occur on next re-boot)'
      else
        print 'WARNING (file is in use and cannot be deleted)'
      fi
    elif [ -z "$rc" ]; then
      print "OK"
    else
      print "ERROR: $rc"
    fi
  else
    echo 'OK (already deleted)'
  fi
}

remove_mvs_components()
{
  echo 'Removing MVS Components ['$BASENAME']: ['$OSPATH'('$OSID')]'

  echo 'Deleting each of the affected NT services...'
  delete_service "ASIMVSEventHandlerSvc-$BASENAME"
  delete_service "ASIFileReceiverSvc-$BASENAME"
  delete_service "ASIMVSSenderSvc-$BASENAME"

  echo 'Deleting the service bootstrap executable for each of the affected services...'
  delete_file "$INSTALLDIR/bin/services/ASIEH_$BASENAME.exe"
  delete_file "$INSTALLDIR/bin/services/ASIFR_$BASENAME.exe"
  delete_file "$INSTALLDIR/bin/services/ASISS_$BASENAME.exe"

  # Removing registry entries for affected services
  delete_registry_entry ASIMVSSenderSvc "${BASENAME}"
  delete_registry_entry ASIMVSEventHandlerSvc "${BASENAME}"
  delete_registry_entry ASIMVSListenerSvc "${BASENAME}"
  delete_registry_entry ASIFileReceiverSvc "${BASENAME}"

  print 'Deleting row from database table MVSSenderSvcMap for os_id='$OSID', svc_host=['$mvs_host'], svc_name=['$BASENAME']...\c'
  isql_cmd='SET NOCOUNT ON
            DELETE FROM MVSSenderSvcMap 
              WHERE os_id='$os_id' OR (svc_host="'$mvs_host'" AND svc_name="'$BASENAME'")
            IF @@ROWCOUNT=0
              RAISERROR("ERROR: Row not found", 0, 1)
            ELSE
              RAISERROR("OK", 0, 1)'
  if [ $bTrusted -eq 1 ]; then
    rc=$($SQL_CMD_LINE -Q"$isql_cmd" 2>&1)
  else
    rc=$($SQL_CMD_LINE -Q"$isql_cmd" 2>&1)
  fi
  if $(print "$rc" | grep -q "OK"); then
    print "OK"
  elif $(print "$rc" | grep -q "ERROR: Row not found"); then
    print "OK (already deleted)"
  else
    print "ERROR: $rc"
  fi

  print "Done."
}

populate_MVSSenderSvcMap_table()
{
  if [ $bInstall_SS -ne 1 ]; then;
    return
  fi

  # populate the MVSSenderSvcMap table in the database 
  # to avoid problems with duplicate OS names
  echo 'Populating database table MVSSenderSvcMap for os_id='$os_id', svc_host=['$mvs_host'], svc_name=['$BASENAME']'
  isql_cmd='SET NOCOUNT ON DELETE FROM MVSSenderSvcMap WHERE os_id='$os_id' OR (svc_host="'$mvs_host'" AND svc_name="'$BASENAME'")
              INSERT MVSSenderSvcMap(os_id, svc_host, svc_name)
                SELECT '$os_id', "'$mvs_host'", "'$BASENAME'"'
  if [ $bTrusted -eq 1 ]; then
    $SQL_CMD_LINE -Q"$isql_cmd"
  else
    $SQL_CMD_LINE -Q"$isql_cmd"
  fi
}

create_registry_settings()
{
  rm -f "$0.reg" 2> nul:

  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Creating registry settings for the Event Handler...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSEventHandlerSvc
            Instances
                $BASENAME
                    Hosts
                        $mvs_host
                            Settings
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
                            Status
                                DesiredState = STOPPED
                    Settings
                        CreateBusinessObjects = REG_DWORD 0x00000000
                        OSID = $OSID
                        DB
                            Application = $BASENAME
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = MVSE_${BASENAME}_
                        Model
                            Params
                                APPLICATION = ASIMVSEventHandlerSvc-$BASENAME
    " >> "$0.reg"
  fi

  if [ $bInstall_LS -eq 1 ]; then;
    echo 'Creating registry settings for the Listener...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSListenerSvc
            Instances
                $BASENAME
                    PartnerLU = ${DOWNLOAD_LUNAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log        
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = MVSL_${BASENAME}_
    " >> "$0.reg"
  fi


  if [ $bInstall_SS -eq 1 ]; then;
    echo 'Creating registry settings for the Sender Service...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSSenderSvc
            Instances
                ${BASENAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\"${BASENAME}"_Upload.que
                            Status
                                DesiredState = RUNNING
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|${BASENAME}|%i|%F|%L|
                            Prefix = MVSS_${BASENAME}_
                        Partner
                            Luname = ${UPLOAD_LUNAME}
                            Modename = ${UPLOAD_MODENAME}
    " >> "$0.reg"
  fi


  if [ $bInstall_FR -eq 1 ]; then;
    echo 'Creating registry settings for the File Receiver...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIFileReceiverSvc
            Instances
                $BASENAME
                    Hosts
                        $mvs_host
                            Settings
                                InputFile = $LOCALDIR\Logs\\$BASENAME.dat
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = FR_${BASENAME}_
    " >> "$0.reg"
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
}


create_db_registry_settings()
{
  rm -f "$0.reg" 2> nul:

  # Event Handler
  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Forcing Event Handler component-level database settings into the registry in response to [-f] parameter...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSEventHandlerSvc
            Instances
                ${BASENAME}
                    Settings
                        DB
                            Server = ${db_server}
                            User = ${db_user}
                            Password = ${db_password}
    " >> "$0.reg"
  fi

  # Sender Service
  if [ $bInstall_SS -eq 1 ]; then;
    echo 'Forcing Sender Service component-level database settings into the registry in response to [-f] parameter...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSSenderSvc
            Instances
                ${BASENAME}
                    Settings
                        DB
                            Server = ${db_server}
                            User = ${db_user}
                            Password = ${db_password}
    " >> "$0.reg"
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
}

create_service()
{
  svc_name="${1?"ERROR: create_service() missing required param <svc_name>"}"
  display_name="${2?"ERROR: create_service() missing required param <display_name>"}"
  bin_path="${3?"ERROR: create_service() missing required param <bin_path>"}"
  start_type="${4?"ERROR: create_service() missing required param <start_type>"}"
  dependencies="${5?"ERROR: create_service() missing required param <dependencies>"}"

  echo 'Creating service ['$svc_name']...'
  rc=$(sc \\\\$mvs_host create $svc_name binPath= \""$bin_path"\" depend= "$dependencies" start= $start_type DisplayName= "$display_name" type= own 2>&1)
  if $(print "$rc" | grep -iq -e "CreateService failed" -e "OpenSCManager failed"); then;
    msg_no=$(print "$rc" | awk '{print $NF}')
    if [ $msg_no -ne 1073 ]; then; #1073= The specified service already exists.
      print "FAILURE: $rc"
      NET HELPMSG $msg_no
    else
      echo 'Updating ['$svc_name'] as already existed'
      rc=$(sc \\\\$mvs_host config $svc_name binPath= \""$bin_path"\" depend= "$dependencies" start= $start_type DisplayName= "$display_name" type= own 2>&1)
      if $(print "$rc" | grep -iq  -e "OpenSCManager failed" -e "failed"); then;
        msg_no=$(print "$rc" | awk '{print $NF}')
        print "FAILURE: $rc"
        NET HELPMSG $msg_no
      fi
      if $(print "$rc" | grep -iq  -e "error" -e "failure" -e "SYNTAX"); then;
        print "FAILURE: $rc"
      fi
    fi
  fi
}

create_TPNAME_on_SNABASE_service()
{
  if [ $bInstall_LS -ne 1 ]; then;
    return
  fi

  MVS_HOST_LOCALDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n -m '\\'$mvs_host "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\$//g')
  print 'Local Install directory of MVS host = ['"$MVS_HOST_LOCALDIR"']'

  # check for existance to SNABASE
  sclist $mvs_host | grep -iq SNABASE;
  if [ $? -eq 1 ]; then
    echo 'SNABASE service is not installed; not configuring TPSTART.'
    return
  fi

  rm -f "$0.reg" 2> nul:

  # SNABASE service registry key
  if [ $bInstall_LS -eq 1 ]; then;
    echo 'Adding TPSTART configuration settings to SNABASE service...'
    echo 'HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\SnaBase\Parameters\TPs\ACC1RCV
      Parameters
        SNAServiceType = REG_DWORD 0x00000006
        ConversationSecurity = no
        AlreadyVerified = no
        LocalLU = REG_SZ "'"${LOCAL_LUNAME}"'"
        Timeout = REG_DWORD 0xFFFFFFFF
        Parameters = REG_SZ "-nASIMVSListenerSvc -c"
        PathName = REG_SZ "'"${MVS_HOST_LOCALDIR}"\\\\'bin\ASIServiceApp.exe"
    ' >> "$0.reg"
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
}

main()
{
  process_component_listing

  check_for_required_parameters

  validate_os_path

  # remove components if specified
  if [ $bRemoveComponents -eq 1 ]; then;
    remove_mvs_components "$BASENAME" "$OSPATH"
    exit
  fi

  populate_MVSSenderSvcMap_table

  create_TPNAME_on_SNABASE_service

  # Install the registry settings
  create_registry_settings
  if [ $bForceDatabaseSettings -eq 1 ]; then;
    create_db_registry_settings
  fi

  # Copy binaries
  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Copying the Event Handler service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASIEH_$BASENAME.exe"
  fi
  if [ $bInstall_SS -eq 1 ]; then;
    echo 'Copying the Sender Service service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASISS_$BASENAME.exe"
  fi
  if [ $bInstall_FR -eq 1 ]; then;
    echo 'Copying the File Receiver service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASIFR_$BASENAME.exe"
  fi

  # Install services
  if [ $bInstall_EH -eq 1 ]; then;
    create_service "ASIMVSEventHandlerSvc-$BASENAME" "Tivoli BSM MVSEventHandlerSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIEH_$BASENAME.exe" "auto" "ASIDBValidater RPCSS RPCLOCATOR"
  fi
  if [ $bInstall_SS -eq 1 ]; then;
    create_service "ASIMVSSenderSvc-$BASENAME" "Tivoli BSM MVSSenderSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASISS_$BASENAME.exe" "auto" "SNABASE RPCSS RPCLOCATOR"
  fi
  if [ $bInstall_FR -eq 1 ]; then;
    create_service "ASIFileReceiverSvc-$BASENAME" "Tivoli BSM FileReceiverSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIFR_$BASENAME.exe" "demand" "RPCSS RPCLOCATOR"
  fi

  echo 'Done.'
}

main
