# $Header:   //ARCHIVE/Archive_NLS/scripts/opcload_remotesetup.ksv   1.0   Aug 01 2001 14:07:22   BRENT  $
#
# opcload_remotesetup.ksh
#
# Module Description
#   Script to configure remote database for running OPC load front-end



pgm=$0
pgmbase=$(basename $pgm)

function notify {
    echo "$(date +%H:%M:%S): $*"
}

function debug {
  if [ $verbose -ne 0 ]
  then
    echo "$*"
  fi
}

function usage {
  (
    echo "Syntax: $pgm -E -S central_server -U central_sql_user -P central_sql_password -D central_directory -W central_system_directory -e -u opcload_sql_user -p opcload_sql_password -d opcload_directory -v"
    echo "-E is the SQL trusted login option for the central server"
    echo "-e is the SQL trusted login option for the local OPC load server"
    echo "-v is the verbose option"
  ) >&2
}

function scstatus {
  sclist | awk '
    function trim(str) {
      x = gsub(" *$", "", str)
      return str
    }

    { 
      if ($2 == "'$1'")
        print trim(substr($0,1,17)) 
    }'
}

function scstop {
  status=$(scstatus $1)
  if [ "X$status" == "Xrunning" ]
  then
    sc stop $1

    while [ "X$(scstatus $1)" != "Xstopped" ]
    do
      notify "Waiting for $1 to stop"
      sleep 3
    done
  elif [ "X$status" = "Xstopped" ]
  then
    notify "$1 is already stopped"
  else
    notify "$1 is $status"
  fi
}

function scstart {
  status=$(scstatus $1)
  if [ "X$status" == "Xstopped" ]
  then
    sc start $1

    while [ "X$(scstatus $1)" != "Xrunning" ]
    do
      notify "Waiting for $1 to start"
      sleep 3
    done
  elif [ "X$status" = "Xrunning" ]
  then
    notify "$1 is already running"
  else
    notify "$1 is $status"
  fi
}

function masscp {
  from=$1
  to=$2
  shift 2
  result=true

  while [ $# -gt 0 -a $result = "true" ]
  do
    file=$1
    if [ $from/$file -nt $to/$file ]
    then
      notify "Copying $file from $from to $to"
      if ! cp -r -m $from/$file $to/$file
      then
        result=false
      fi
    else
      notify "No need to copy $file from $from to $to"
    fi
    shift
  done
  eval $result
}

function cleanup {
  if [ "X$bcp_file" != X ]
  then
    rm -f $bcp_file
  fi

  if [ "X$out_file" != X ]
  then
    rm -f $out_file
  fi

  if [ "X$regFile" != X ]
  then
    rm -f $regFile
  fi

  notify "Stop processing"
}

function main {
  notify "Start processing"
  trap cleanup EXIT

  notify "Stopping SQL services"
  scstop MSDTC
  scstop SQLServerAgent
  scstop MSSQLServer

  notify "Creating local directories"
  if ! mkdir -p $to_dir/sql $to_dir/bin
  then
    exit 1
  fi
  
  if ! masscp $from_dir/sql $to_dir/sql MasterDBChanges.sql SchemaTypes.sql OPCLoad_OPCLoadServer.sql _LoadOPCPlan.sql OPC_Check_for_new_daily_plan.job
  then
    exit 1
  fi
  
  if ! masscp $from_dir/bin $to_dir/bin TbsmOpcCopyFromCentral.ksh TbsmOpcCopyToCentral.ksh TbsmTableCopy.ksh check_opc_plan.ksh OPCParseDailyPlan.ksh ASIDBXp.dll ASITMExp.dll ASITSDXp.dll BCYSXp.dll TMXpReplaceFields.dll reqsvrclient.dll asirpc.dll asidb.dll asiutil.dll tls7012d.dll tsdc110.dll
  then
    exit 1
  fi

  if ! masscp $from_sys_dir $to_dir/bin MSVCP60.DLL
  then
    exit 1
  fi

  if ! masscp $from_dir $to_dir Lang
  then
    exit 1
  fi
  
  notify "Restarting SQL server"
  scstart MSSQLServer

  notify "Executing SchemaTypes.sql"
  if ! isql -d master $to_isql_opts -i $to_dir/sql/SchemaTypes.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  if ! isql -d model $to_isql_opts -i $to_dir/sql/SchemaTypes.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  if ! isql -d tempdb $to_isql_opts -i $to_dir/sql/SchemaTypes.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Executing MasterDBChanges.sql"
  if ! isql $to_isql_opts -i $to_dir/sql/MasterDBChanges.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Stopping SQL service"
  scstop MSSQLServer

  notify "Restarting SQL server"
  scstart MSSQLServer

  notify "Creating and populating OPCLoad database"
  if ! isql $to_isql_opts -i $to_dir/sql/OPCLoad_OPCLoadServer.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Creating discovery procedure"
  if ! isql $to_isql_opts -i $to_dir/sql/_LoadOPCPlan.sql > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Copying opc_app_instance rows from central server"
  bcp_file=${TMPDIR:-.}/${pgmbase#.ksh}.$(date '+%Y%m%d.%H%M%S').$$.bcp  
  if ! bcp 'SELECT plan_name, app_name, ctime, rtime, 1 FROM Object..opc_app_instance' QUERYOUT $bcp_file -c -t'|' $from_bcp_opts > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Clearing out OPCLoad..opc_app_instance"
  if ! isql $to_isql_opts -Q "delete OPCLoad..opc_app_instance" > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Loading OPCLoad..opc_app_instance"
  if ! bcp OPCLoad..opc_app_instance IN $bcp_file -c -t'|' $to_bcp_opts > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  notify "Defining 'OPC Check for new daily plan' job"
  if ! isql $to_isql_opts -i $to_dir/sql/OPC_Check_for_new_daily_plan.job > $out_file 2>&1
  then
    cat $out_file >&2
    exit 1
  fi
  if [ $verbose -ne 0 ]
  then
    cat $out_file
  fi

  if [ "X$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>/dev/null)" = X ]
  then
    notify "Saving \"install directory\" in registry"
    regFile=${TMPDIR:-.}/$pgmbase.$(date '+%Y%m%d.%H%M%S').$$.reg
    cat > $regFile <<EOF
\Registry\Machine
  SOFTWARE
    Accessible Software, Inc.
      Access1
        1.0
          InstallDirectory = REG_SZ $to_dos_dir\
EOF
    if ! regini $regFile > $out_file 2>&1
    then
      cat $out_file >&2
      exit 1
    fi
    if [ $verbose -ne 0 ]
    then
      cat $out_file
    fi
  fi
}

from_dir=""
from_server=""
from_user=""
from_pw=""
from_db=""
from_sys_dir=""

to_dir=""
to_server=""
to_user=""
to_pw=""
to_db=""

from_trusted=0
to_trusted=0
verbose=0

set -- $(getopt -c $0 'W:ES:U:P:D:eu:p:d:v?' "$@")
if [ $? -ne 0 ]
then
  usage
  exit 1
fi

from_opts=0
to_opts=0

# ops bits:
#      1  server
#      2  user
#      4  password
#      8  TivoliManager directory
#     16  trusted mode
#     32  Windows directory

while [ $# -gt 0 ]
do
  if [ "$1" = -S ]
  then
    shift
    from_server="$1"
    let 'from_opts = from_opts | 1'
  elif [ "$1" = -U ]
  then
    shift
    from_user="$1"
    let 'from_opts = from_opts | 2'
  elif [ "$1" = -P ]
  then
    shift
    from_pw="$1"
    let 'from_opts = from_opts | 4'
  elif [ "$1" =  -D ]
  then
    shift
    from_dir=$(echo "$1" | tr '\\' '/')
    from_dos_dir=$(echo "$1" | tr '/' '\\')
    let 'from_opts = from_opts | 8'
  elif [ "$1" =  -W ]
  then
    shift
    from_sys_dir="$1"
    let 'from_opts = from_opts | 32'
  elif [ "$1" =  -u ]
  then
    shift
    to_user="$1"
    let 'to_opts = to_opts | 2'
  elif [ "$1" =  -p ]
  then
    shift
    to_pw="$1"
    let 'to_opts = to_opts | 4'
  elif [ "$1" =  -d ]
  then
    shift
    to_dir=$(echo "$1" | tr '\\' '/')
    to_dos_dir=$(echo "$1" | tr '/' '\\')
    let 'to_opts = to_opts | 8'
  elif [ "$1" =  -v ]
  then
    if [ $verbose -eq 0 ]
    then
      verbose=1
    else
      verbose=0
    fi
  elif [ "$1" =  -E ]
  then
    if [ $from_trusted -eq 0 ]
    then
      from_trusted=1
      let 'from_opts = from_opts | 16'
    else
      from_trusted=0
      let 'from_opts = from_opts ^ 16'
    fi
  elif [ "$1" =  -e ]
  then
    if [ $to_trusted -eq 0 ]
    then
      to_trusted=1
      let 'to_opts = to_opts | 16'
    else
      to_trusted=0
      let 'to_opts = to_opts ^ 16'
    fi
  elif [ "$1" = -? ]
  then
    usage
    exit 0
  fi
  shift
done

# from opts must be server,user,ps,dir or server,trusted,dir
# to opts must be user,pw,dir or trusted,dir

debug "from_opts=$from_opts, to_opts=$to_opts"
if [ \( $from_opts -ne 47 -a $to_opts -ne 57 \) -o \( $to_opts -ne 14 -a $to_opts -ne 24 \) ]
then
  echo "One or more options invalid.  from_opts=$from_opts, to_opts=$to_opts" >&2
  echo "" >&2
  usage
  exit 1
fi

if ! [ -d $from_dir/sql -a -d $from_dir/bin -a -d $from_sys_dir ]
then
  echo "Cannot locate $from_dir/sql, $from_dir/bin, and/or $from_sys_dir"
  exit 1
fi

from_bcp_opts="-b1000 -S $from_server"

if [ $from_trusted -ne 0 ]
then
  from_bcp_opts="$from_bcp_opts -T"
else
  from_bcp_opts="$from_bcp_opts -U $from_user -P $from_pw"
fi

to_isql_opts=""
to_bcp_opts=""

if [ $to_trusted -ne 0 ]
then
  to_isql_opts="$to_isql_opts -E"
  to_bcp_opts="$to_bcp_opts -T"
else
  to_isql_opts="$to_isql_opts -U $to_user -P $to_pw"
  to_bcp_opts="$to_bcp_opts -U $to_user -P $to_pw"
fi

out_file=${TMPDIR:-.}/${pgmbase#.ksh}.$(date '+%Y%m%d.%H%M%S').$$.bcp  
logname="./$(basename ${pgmbase%.ksh}).log"
main 2>&1 | tee $logname
