# $Header:   //ARCHIVE/Archive_NLS/scripts/tbsmopccopyfromcentral.ksv   1.0   Aug 01 2001 14:07:26   BRENT  $
#
# TBSMOPCCopyFromCentral.ksh
#
# Module Description
#   Script to copy OPC tables from central server to remote server.
#   Invoked by the _LoadOPCPlan stored procedure running on the
#   remote server.


function usage {
  (
    echo "Syntax: $pgm -s server [-t | -u user -p password] [-v]"
  ) >&2
}

pgm=$0
pgmbase=$(basename $pgm)

server=""
user=""
pw=""
trusted=0
verbose=0

while getopts s:u:p:tv o
do   case "$o" in
     s)   server="$OPTARG";;
     u)   user="$OPTARG";;
     p)   pw="$OPTARG";;
     t)   if [ $trusted -eq 0 ]
          then
            trusted=1
          else
            trusted=0
          fi;;
     v)   if [ $verbose -eq 0 ]
          then
            verbose=1
          else
            verbose=0
          fi;;
     [?]) usage
          exit 1;;
     esac
done
shift $OPTIND-1

if [ "X$server" = X ]
then
  echo "No server specified" >&2
  usage
  exit 1
fi

from_opts="-S $server"

if [ $trusted -eq 0 ]
then
  if [ "X$user" = X -o "X$pw" = X ]
  then
    echo "If trusted mode is not used, both userid and password must be used" >&2
    usage
    exit 1
  fi
  from_opts="$from_opts -U $user -P $pw"
else
  if [ "X$user" != X -o "X$pw" != X ]
  then
    echo "If trusted mode is used, neither userid nor password may be used" >&2
    usage
    exit 1
  fi
  from_opts="$from_opts -E"
fi

if [ $verbose -ne 0 ]
then
  from_opts="$from_opts -v"
fi

function debug {
  if [ $verbose -ne 0 ]
  then
    echo $*
  fi
}

function doit {
  debug "Copying $1"
  TbsmTableCopy $from_opts -D Object -T $1 -e -d OPCLoad
}

function fakebcp {
  debug "Trying 'fake bcp', which may timeout"
  if [ $trusted -eq 0 ]
  then
    opts="-S $server -U $user -P $pw"
  else
    opts="-S $server -T"
  fi
  outfile="${TMPDIR:-.}/${pgmbase%.ksh}.$(date '+%Y%m%d.%H%M%S').$$.fakebcp.out"
  bcpfile="${TMPDIR:-.}/${pgmbase%.ksh}.$(date '+%Y%m%d.%H%M%S').$$.fakebcp.bcp"
  trap "rm -f $outfile $bcpfile" EXIT
  if [ $verbose -ne 0 ]
  then
    echo bcp 'select 1 where 1=2' QUERYOUT $bcpfile $opts -c -t '|'
  fi
  bcp 'select 1 where 1=2' QUERYOUT $bcpfile $opts -c -t '|' > $outfile 2>&1
  bcp_rc=$?
  if [ $verbose -ne 0 ]
  then
    echo "fake bcp rc=$bcp_rc"
    cat $outfile
  fi
}

fakebcp

if doit dyn_obj_path
then
  if doit dyn_obj_path_detail
  then
    if doit opc_app_pattern
    then
      if ! doit opc_op_pattern
      then
        exit 1
      fi
    else
      exit 1
    fi
  else
    exit 1
  fi
else
  exit 1
fi
