# $Header:   //ARCHIVE/Archive_NLS/scripts/tbsmopccopytocentral.ksv   1.0   Aug 01 2001 14:07:30   BRENT  $
#
# TBSMOPCCopyToCentral.ksh
#
# Module Description
#   Script to copy OPC tables from remote server to central server.
#   Invoked by the _LoadOPCPlan stored procedure running on the
#   remote server after plan has been processed and before executing
#   the back-end procedure (_LoadOPCPlanBackend) on the central server.

function usage {
  (
    echo "Syntax: $pgm -i planid -s server [-t | -u user -p password] [-v]"
  ) >&2
}

pgm=$0
pgmbase=$(basename $pgm)

server=""
user=""
pw=""
trusted=0
verbose=0
planid=""

while getopts i:s:u:p:tv o
do   case "$o" in
     i)   planid="$OPTARG";;
     s)   server="$OPTARG";;
     u)   user="$OPTARG";;
     p)   pw="$OPTARG";;
     t)   if [ $trusted -eq 0 ]
          then
            trusted=1
          else
            trusted=0
          fi;;
     v)   if [ $verbose -eq 0 ]
          then
            verbose=1
          else
            verbose=0
          fi;;
     [?]) usage
          exit 1;;
     esac
done
shift $OPTIND-1

if [ "X$planid" = X ]
then
  echo "No plan id specified" >&2
  usage
  exit 1
fi

if [ "X$server" = X ]
then
  echo "No server specified" >&2
  usage
  exit 1
fi

to_opts="-s $server"

if [ $trusted -eq 0 ]
then
  if [ "X$user" = X -o "X$pw" = X ]
  then
    echo "If trusted mode is not used, both userid and password must be used" >&2
    usage
    exit 1
  fi
  to_opts="$to_opts -u $user -p $pw"
else
  if [ "X$user" != X -o "X$pw" != X ]
  then
    echo "If trusted mode is used, neither userid nor password may be used" >&2
    usage
    exit 1
  fi
  to_opts="$to_opts -e"
fi

if [ $verbose -ne 0 ]
then
  to_opts="$to_opts -v"
fi

function debug {
  if [ $verbose -ne 0 ]
  then
    echo $*
  fi
}

function doit {
  debug "Copying $1"
  if [ "X$2" = X ]
  then
    whereClause=""
  else
    whereClause="-w '$2'"
  fi
  eval "TbsmTableCopy -E -D OPCLoad -T $1 $to_opts -d Object $whereClause"
}

function fakebcp {
  debug "Trying 'fake bcp', which may timeout"
  if [ $trusted -eq 0 ]
  then
    opts="-S $server -U $user -P $pw"
  else
    opts="-S $server -T"
  fi
  outfile="${TMPDIR:-.}/${pgmbase%.ksh}.$(date '+%Y%m%d.%H%M%S').$$.fakebcp.out"
  bcpfile="${TMPDIR:-.}/${pgmbase%.ksh}.$(date '+%Y%m%d.%H%M%S').$$.fakebcp.bcp"
  trap "rm -f $outfile $bcpfile" EXIT
  if [ $verbose -ne 0 ]
  then
    echo bcp 'select 1 where 1=2' QUERYOUT $bcpfile $opts -c -t '|'
  fi
  bcp 'select 1 where 1=2' QUERYOUT $bcpfile $opts -c -t '|' > $outfile 2>&1
  bcp_rc=$?
  if [ $verbose -ne 0 ]
  then
    echo "fake bcp rc=$bcp_rc"
    cat $outfile
  fi
}

fakebcp

if doit opc_app_new_instance
then
  if doit opc_app_path_aggr
  then
    if doit opc_app_path_detail
    then
      if doit opc_application "plan_id = $planid"
      then
        if doit opc_daily_plan "id = $planid"
        then
          if ! doit opc_operation "plan_id = $planid"
          then
            exit 1
          fi
        else
          exit 1
        fi
      else
        exit 1
      fi
    else
      exit 1
    fi
  else
    exit 1
  fi
else
  exit 1
fi
