# $Header:   //ARCHIVE/Archive_NLS/scripts/tbsmtablecopy.ksv   1.0   Aug 01 2001 14:07:32   BRENT  $
#
# TBSMTableCopy.ksh
#
# Module Description
#   Generic script to copy one SQL table from a source server
#   to another server.

bcp_file=""
bcp_clean=0

from_server=""
from_user=""
from_pw=""
from_db=""
from_table=""

to_server=""
to_user=""
to_pw=""
to_db=""
to_table=""
whereClause=""

from_trusted=0
to_trusted=0
verbose=0
append=0

typeset -L1 char1

function usage {
  (
  echo "Syntax: $pgm [-v] [-a] -S server -E -U user -P pw -D db -T table -s server -e -u user -p pw [ -d db ] [ -t table ] -w whereClause -b file.bcp -c"
  echo ""
  echo "-S, -E -U, -P, -D, and -T refer to the SQL server you're copying from"
  echo ""
  echo "-s, -e, -u, -p, -d, and -t refer to the SQL server you're copying to"
  echo ""
  echo "-b specifies a bcp file that takes the place of the 'from' SQL information'"
  echo ""
  echo "-c specifies that the bcp file should be deleted upon completion"
  echo ""
  echo "-E and -e are trusted mode flags for their respective servers"
  echo ""
  echo "-v is verbose mode"
  echo ""
  echo "-a is append mode, the default is to delete all rows of the table in the target database before copying"
  echo ""
  echo "Usage notes:"
  echo ""
  echo " - Omitting the 'to' database or table will default to the respective 'from' parameter"
  echo ""
  echo " - Omitting either server flag will default to the local server"
  echo ""
  echo " - The trusted flag is mutually exclusive with userid but one or the other must be specified"
  echo ""
  echo " - Userid and password must be used together"
  ) >&2
}

function debug {
  if [ $verbose -ne 0 ]
  then
    echo "$*"
  fi
}

function dump {
  if [ $verbose -ne 0 ]
  then
    cat $1
  fi
}

pgm=$0

set -- $(getopt -c $0 'b:cES:U:P:D:T:es:u:p:d:t:vaw:?' "$@")
if [ $? -ne 0 ]
then
  usage
  exit 1
fi

from_opts=0
to_opts=0

# ops bits:
#      1  server
#      2  user
#      4  password
#      8  database
#     16  table
#     32  trusted mode
#     64  bcp file used
#    128  clean up bcp file

while [ $# -gt 0 ]
do
  if [ "$1" = -b ]
  then
    shift
    bcp_file="$1"
    let 'from_opts = from_opts | 64'
  elif [ "$1" = -S ]
  then
    shift
    from_server="$1"
    let 'from_opts = from_opts | 1'
  elif [ "$1" = -U ]
  then
    shift
    from_user="$1"
    let 'from_opts = from_opts | 2'
  elif [ "$1" = -P ]
  then
    shift
    from_pw="$1"
    let 'from_opts = from_opts | 4'
  elif [ "$1" =  -D ]
  then
    shift
    from_db="$1"
    let 'from_opts = from_opts | 8'
  elif [ "$1" =  -T ]
  then
    shift
    from_table="$1"
    let 'from_opts = from_opts | 16'
  elif [ "$1" =  -s ]
  then
    shift
    to_server="$1"
    let 'to_opts = to_opts | 1'
  elif [ "$1" =  -u ]
  then
    shift
    to_user="$1"
    let 'to_opts = to_opts | 2'
  elif [ "$1" =  -p ]
  then
    shift
    to_pw="$1"
    let 'to_opts = to_opts | 4'
  elif [ "$1" =  -d ]
  then
    shift
    to_db="$1"
    let 'to_opts = to_opts | 8'
  elif [ "$1" =  -t ]
  then
    shift
    to_table="$1"
    let 'to_opts = to_opts | 16'
  elif [ "$1" =  -v ]
  then
    if [ $verbose -eq 0 ]
    then
      verbose=1
    else
      verbose=0
    fi
  elif [ "$1" =  -a ]
  then
    if [ $append -eq 0 ]
    then
      append=1
    else
      append=0
    fi
  elif [ "$1" =  -c ]
  then
    if [ $bcp_clean -eq 0 ]
    then
      bcp_clean=1
      let 'from_opts = from_opts | 128'
    else
      bcp_clean=0
      let 'from_opts = from_opts ^ 128'
    fi
  elif [ "$1" =  -E ]
  then
    if [ $from_trusted -eq 0 ]
    then
      from_trusted=1
      let 'from_opts = from_opts | 32'
    else
      from_trusted=0
      let 'from_opts = from_opts ^ 32'
    fi
  elif [ "$1" =  -e ]
  then
    if [ $to_trusted -eq 0 ]
    then
      to_trusted=1
      let 'to_opts = to_opts | 32'
    else
      to_trusted=0
      let 'to_opts = to_opts ^ 32'
    fi
  elif [ "$1" = -w ]
  then
    done=0
    while [ $done -eq 0 ]
    do
      shift
      whereClause="$whereClause $1"
      char1="$2"
      if [ "X$char1" = X- ]
      then
        done=1
      fi
    done
  elif [ "$1" = -? ]
  then
    usage
    exit 0
  fi
  shift
done

if [ "X$to_db" = X -a "X$from_db" != X ]
then
  to_db="$from_db"
  let 'to_opts = to_opts | 8'
fi

if [ "X$to_table" = X -a "X$from_table" != X ]
then
  to_table="$from_table"
  let 'to_opts = to_opts | 16'
fi

# database and table (8 + 16 = 24) are always required. 
# either both userid and password (2 + 4 = 6) or trusted (32) must be present.
# userid/password and trusted are mutually exclusive.
# the server name (1) is optional.

debug "from_opts=$from_opts, to_opts=$to_opts"
if [ \( $from_opts -ne 64 -a $from_opts -ne 31 -a $from_opts -ne 57 -a $from_opts -ne 30 -a $from_opts -ne 56 \) -o \( $to_opts -ne 31 -a $to_opts -ne 57 -a $to_opts -ne 30 -a $to_opts -ne 56 \) ]
then
  echo "One or more options invalid.  from_opts=$from_opts, to_opts=$to_opts" >&2
  echo "" >&2
  usage
  exit 1
fi

from_bcp_opts="-b1000"

if [ "X$bcp_file" = X ]
then
  if [ "X$from_server" != X ]
  then
    from_bcp_opts="$from_bcp_opts -S $from_server"
  fi

  if [ $from_trusted -ne 0 ]
  then
    from_bcp_opts="$from_bcp_opts -T"
  else
    from_bcp_opts="$from_bcp_opts -U $from_user -P $from_pw"
  fi
fi

to_isql_opts=""
to_bcp_opts="-b1000"

if [ "X$to_server" != X ]
then
  to_isql_opts="-S $to_server"
  to_bcp_opts="$to_bcp_opts -S $to_server"
fi

if [ $to_trusted -ne 0 ]
then
  to_isql_opts="$to_isql_opts -E"
  to_bcp_opts="$to_bcp_opts -T"
else
  to_isql_opts="$to_isql_opts -U $to_user -P $to_pw"
  to_bcp_opts="$to_bcp_opts -U $to_user -P $to_pw"
fi

filename_base=${TMPDIR:-.}/${from_db}.${from_table}.$(date '+%Y%m%d.%H%M%S').$$
out_file=${filename_base}.out

if [ "X$bcp_file" = X ]
then
  data_file=${filename_base}.bcp
  trap "rm -f $out_file $data_file" EXIT
else
  data_file=$(echo $bcp_file | tr '\\' '/')
  if [ $bcp_clean -eq 0 ]
    trap "rm -f $out_file" EXIT
  then
  else
    trap "rm -f $out_file $data_file" EXIT
  fi
fi

if [ "X$bcp_file" = X ]
then
  if [ "X$whereClause" = X ]
  then
    selectStatement="SELECT * FROM ${from_db}..${from_table}"
  else
    selectStatement="SELECT * FROM ${from_db}..${from_table} WHERE ${whereClause}"
  fi

  debug "from: bcp '$selectStatement' QUERYOUT $data_file ${from_bcp_opts} -c -t'|'"
  if bcp "$selectStatement" QUERYOUT $data_file ${from_bcp_opts} -b1000 -c -t'|' > $out_file 2>&1
  then
    dump $out_file
  else
    if [ $verbose -eq 0 ]
    then
      echo "from bcp of $from_table failed"
    fi
    cat $out_file
    exit 1
  fi
fi

if [ $append -ne 1 ]
then
  debug "'to' DELETE"
  if ! isql ${to_isql_opts} -Q "USE ${to_db} DELETE ${to_table}" > $out_file 2>&1
  then
    if [ $verbose -eq 0 ]
    then
      echo "to isql for $to_table failed"
    fi
    cat $out_file
    exit 1
  else
    dump $out_file
  fi
fi

debug "to: bcp ${to_db}..${to_table} IN $data_file ${to_bcp_opts} -c -t'|'"
if bcp ${to_db}..${to_table} IN $data_file ${to_bcp_opts} -b1000 -c -t'|' > $out_file 2>&1
then
  dump $out_file
  exit 0
else
  if [ $verbose -eq 0 ]
  then
    echo "to bcp of $to_table failed"
  fi
  cat $out_file
  exit 1
fi
