-- $header $
-- DEFECT 79561
USE 
Object
GO

---------------------------------------------------------------------------------------------------
PRINT 'Cleanup old ca7 procedures...'
GO

-- asisp_setCA7bulkloadStatus is replaced by asisp_setBJBulkloadStatus
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setCA7bulkloadStatus' AND type = 'P')
  DROP PROCEDURE asisp_setCA7bulkloadStatus
GO

-- _LoadCA7DiscoveryBatch is replaced by
-- _LoadJobDiscoveryBatch 10, 'sh ca7.parse.ksh -d', 'ca7_bcp', '.ca7.bcp'
IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadCA7DiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadCA7DiscoveryBatch
GO

-- asisp_CA7createBulkBATC is replaced by asisp_bulkcreateBATC
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_CA7createBulkBATC' AND type = 'P')
  DROP PROCEDURE asisp_CA7createBulkBATC
GO

-- _LoadCA7Plan_CreateBTCY is replaced by asisp_bulkcreateBTCY
IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadCA7Plan_CreateBTCY' AND type = 'P')
  DROP PROCEDURE _LoadCA7Plan_CreateBTCY
GO

-- asisp_setGlobals 'CA7', 'autoDiscoveryFlag', '1', 'INT' 
-- default value 1 for auto discovery.
-- asisp_setGlobals 'CA7', 'keep_unknown', '0', 'INT' 
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- ca7_audit_events
PRINT 'Creating ca7_audit_events table....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_audit_events' AND type = 'U')
  DROP TABLE ca7_audit_events
GO

CREATE TABLE ca7_audit_events
  (src_cid ClassID, 
   src_id ObjID, 
   ctime DATETIME NULL, 
   MESSAGE VARCHAR(800) NULL) 

GO
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- Create BJDiscoveryFilter_Lookup for filter lookup
PRINT 'Creating table BJDiscoveryFilter_Lookup....'
GO
   
IF NOT EXISTS( SELECT * FROM sysobjects WHERE name = 'BJDiscoveryFilter_Lookup' AND type = 'U')
  EXEC('CREATE TABLE BJDiscoveryFilter_Lookup(
       name_schedule_set		ObjName DEFAULT ''ALL'',
       name_schedule			ObjName DEFAULT ''ALL'',
       name_job				ObjName DEFAULT ''ALL'',
       data_source			ObjName)'
      )
GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- Create Disc_jobs for all bulk create batch jobs for cleanup only
PRINT 'Creating table Disc_jobs....'
GO
   
IF NOT EXISTS( SELECT * FROM sysobjects WHERE name = 'Disc_jobs' AND type = 'U')
  EXEC('CREATE TABLE Disc_jobs(
	name		ObjName PRIMARY KEY (name, parent_cid, parent_id, sched_id, data_source, bulk_load),
	parent_cid 	ClassID,
	parent_id 	ObjID,
	sched_id	TINYINT DEFAULT NULL,
	data_source	ObjName,
	bulk_load	TINYINT
	)'
      )
GO 

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- Create Disc_job_lookups for all bulk create batch jobs for cleanup only
PRINT 'Creating table Disc_job_lookups....'
GO
   
IF NOT EXISTS( SELECT * FROM sysobjects WHERE name = 'Disc_job_lookups' AND type = 'U')
  EXEC('CREATE TABLE Disc_job_lookups(
	btcy_id 	ObjID PRIMARY KEY (btcy_id, batc_id, data_source),
	batc_id 	ObjID,
	data_source	ObjName)'
      )
GO 

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- Create Disc_schedules for all bulk create batch schedules
PRINT 'Creating table Disc_schedules....'
GO
   
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'Disc_schedules' AND type = 'U')
BEGIN
  EXEC('CREATE TABLE Disc_schedules(
	obj_id 			ObjID NULL,
        name 			ObjName PRIMARY KEY(name, name_schedule_set, bulk_load, data_source),
	name_schedule_set	ObjName,
	bcys_id 		ObjID NULL,
	bulk_load		INT,
	data_source		ObjName)
	')
END

GO
-- transfer data from ca7_schedules to Disc_schedules
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_schedules' AND type = 'U')
BEGIN
  DELETE Disc_schedules
  INSERT Disc_schedules(obj_id, name, name_schedule_set, bulk_load, data_source)
    SELECT obj_id, name, name_schedule_set, bulk_load, 'CA7' FROM ca7_schedules 
  DROP TABLE ca7_schedules 
END
GO

---------------------------------------------------------------------------------------------------
-- asisp_setBJBulkloadStatus
PRINT 'Creating PROCEDURE asisp_setBJBulkloadStatus....'
GO
 
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setBJBulkloadStatus' AND type = 'P')
  DROP PROCEDURE asisp_setBJBulkloadStatus
GO 

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- pstep AND pstat of job autodiscovery process are setup IN the database AS:
-- @pstep = 0, @pstat = 0  	ASIIPListenerSvc get ready file FOR the schedule SET.
-- @pstep = 1, @pstat = 1  	parsing forecast file;
-- @pstep = 1, @pstat = 0 	*.bcp file ready to process;
-- @pstep = 2, @pstat = 1 	start to load *.bcp file INTO db;
-- @pstep = 2, @pstat = 0 	finish to load *.bcp file INTO db;
-- @pstep = 3, @pstat = 1 	start to process data IN db;
-- entries will be deleted after process successfully
-- we will USE error AND errtext to atract error FOR each step

CREATE PROCEDURE asisp_setBJBulkloadStatus
  @FormatType INT,
  @file_path VARCHAR(256),
  @pstep TINYINT = NULL,
  @pstat TINYINT = NULL,
  @batchid INT = NULL,
  @local_codepage INT = NULL
AS

  RAISERROR('Loading Batch Job Discovery Batch....', 0, 1)
  DECLARE @pstep_cur TINYINT, @pstat_cur TINYINT

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_setBJBulkloadStatus
  ELSE
    SAVE TRAN asisp_setBJBulkloadStatus
  IF @batchid IS NULL
    SELECT @batchid = batchid FROM DiscoveryBatch 
      WHERE FormatType = @FormatType 
      AND FilePath = @file_path
      AND CodePage = @local_codepage
      AND pstep = @pstep
      AND pstat = @pstat
  IF @batchid IS NULL
    INSERT DiscoveryBatch(ctime, FormatType, FilePath, CodePage, pstep, pstat)
      SELECT GETDATE(), @FormatType, @file_path, @local_codepage, @pstep, @pstat
  ELSE
    UPDATE DiscoveryBatch SET pstep = @pstep, pstat = @pstat
      WHERE FormatType = @FormatType AND batchid = @batchid

  IF @tcount = 0
    COMMIT TRAN asisp_setBJBulkloadStatus
  RETURN 0

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_setBJDiscoverFilter
PRINT 'Creating procedure asisp_setBJDiscoverFilter....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setBJDiscoverFilter' AND type = 'P')
  DROP PROCEDURE asisp_setBJDiscoverFilter
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
CREATE PROCEDURE asisp_setBJDiscoverFilter
  @name_schedule_set	ObjName = 'ALL', 
  @name_schedule	ObjName = 'ALL', 
  @name_job		ObjName = 'ALL', 
  @data_source		ObjName = 'ALL',
  @add_entry		TINYINT = NULL
AS

  RAISERROR('Set batch discover lookup with asisp_setDiscoverLookup', 0, 1) 
  SET NOCOUNT ON 

  IF @add_entry IS NULL
  BEGIN
    PRINT '========================'
    PRINT 'Current excluding lists:'
    PRINT '========================'
    SELECT * FROM BJDiscoveryFilter_Lookup
  END
  ELSE IF @add_entry = 1
    INSERT BJDiscoveryFilter_Lookup SELECT @name_schedule_set, @name_schedule, @name_job, @data_source
  ELSE IF @add_entry = 0
      DELETE BJDiscoveryFilter_Lookup WHERE name_schedule_set = @name_schedule_set
        AND name_schedule = @name_schedule
        AND name_job = @name_job
        AND data_source = @data_source
  SET NOCOUNT OFF

GO

--------------------------------------------------------------------------------------------------
-- _LoadJobDiscoveryBatch
PRINT 'Creating PROCEDURE _LoadJobDiscoveryBatch....'
GO
 
IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadJobDiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadJobDiscoveryBatch
GO 

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- bulk load *.bcp files INTO ca7_bcp TABLE

CREATE PROCEDURE _LoadJobDiscoveryBatch
  @FormatType 	INT, 
  @ParseCode 	ObjName,	   -- parsing command such as 'sh ca7.parse.ksh -d', 
				   -- the file name will appended on this parsing code.  
  @dstTable	ObjName,           -- bulk insert table to hold data from *.bcp files.
  @bcpExtension VARCHAR(20) = ''  -- the extension added on output file name after
                                   -- parsing the original data into bcp files
AS

  RAISERROR('Processing _LoadJobDiscoveryBatch where @FormatType = %d, @ParseCode = %s, 
            @dstTable = %s, @bcpExtension = %s.', 0, 1, @FormatType, @ParseCode, @dstTable, @bcpExtension)

  -- stardard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _LoadJobDiscoveryBatch
  ELSE
    SAVE TRAN _LoadJobDiscoveryBatch

  DECLARE @ProcessName ObjName, @batchid ObjID, @file_path VARCHAR(225), @CodePage VARCHAR(20)
  SELECT @ProcessName = NAME from FORMAT_TYPE WHERE FORMAT_TYPE = @FormatType

  -- check load process status
  RAISERROR('Checking bulk load status for %s', 0,1, @ProcessName)
  IF EXISTS(SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstat = 1)  
  BEGIN
    RAISERROR('Other %s autodiscovery process IS running.', 0, 1, @ProcessName )
    RETURN -30000
  END
  -- cleanup FOR multiple entries IN DiscoveryBatch. Since new forecast file IS here, 
  -- we don't need old any more even the one IS IN step 2. How ever, we keep the entries 
  -- FOR old files was loaded AND new one IS NOT here yet 
  DELETE b1 FROM DiscoveryBatch AS b1 
    JOIN DiscoveryBatch AS b2 ON b1.FilePath = b2.FilePath
    AND b1.FormatType = b2.FormatType
    AND b1.batchid < b2.batchid
    WHERE b1.FormatType = @FormatType 
  
  -- UPDATE the status
  UPDATE DiscoveryBatch SET pstat = 1, pstep = 1
    WHERE FormatType = @FormatType AND pstat = 0 AND (error IS NULL OR error = 0) 

  -- insert all the file names in the temp table 
  RAISERROR('Extract file names for parsing', 0,1)
  CREATE TABLE #FilePath(FilePath ObjName) 
  INSERT #FilePath SELECT FilePath FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstep = 1
  IF @@ROWCOUNT > 0
  BEGIN
    UPDATE #FilePath SET FilePath = SUBSTRING(REVERSE(FilePath), 1, CHARINDEX('/', REVERSE(FilePath))-1)
    UPDATE #FilePath SET FilePath = REVERSE(FilePath)
    -- CA7 specific 
    IF @ProcessName = 'CA7'
    BEGIN
      IF EXISTS (SELECT * FROM #FilePath WHERE FilePath IN (SELECT short_name FROM ca7_bcys_name_lookup))
        UPDATE f SET f.FilePath = l.full_name FROM #FilePath AS f 
          JOIN ca7_bcys_name_lookup AS l ON f.FilePath = l.short_name 
      DELETE ca7_bcp WHERE name_schedule_set IN (SELECT FilePath FROM #FilePath)
    END
  END 
  ELSE
  BEGIN
    RAISERROR('No file is waiting to be processed', 0, 1)
    IF @tcount = 0
      COMMIT TRAN _LoadJobDiscoveryBatch
    RETURN  
  END

  -- loop all the ready files IN DiscoveryBatch TABLE FOR CA7 even the files were loaded before. 
  RAISERROR('Start bulk load processes.....', 0,1)
  DECLARE @num_schedule_set TINYINT
  SELECT @num_schedule_set = 0
  DECLARE cc CURSOR FOR SELECT batchid, FilePath, CodePage FROM DiscoveryBatch 
                          WHERE FormatType = @FormatType 
                          AND (error IS NULL OR error = 0) AND pstep <> 2 AND pstat =1
  OPEN cc
  FETCH NEXT FROM cc INTO @batchid, @file_path, @CodePage
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    DECLARE @cmd VARCHAR(1023), @xpResult INT
    SELECT @cmd = @ParseCode + @file_path 
    RAISERROR('Parsing on ''%s''....', 0,1, @cmd)
    EXEC @xpResult=master..xp_cmdshell @cmd

    IF (@xpResult != 0)
    BEGIN
      UPDATE DiscoveryBatch SET error = @@ERROR, 
		errtext = 'Failed ON parse.',
                pstep = 1,
		pstat = 0
                WHERE FormatType = @FormatType AND FilePath = @file_path
      FETCH NEXT FROM cc INTO @batchid, @file_path, @CodePage
      CONTINUE
    END 
    EXEC asisp_setBJBulkloadStatus @FormatType, @file_path, 2, 1, @batchid
    -- bulk INSERT data INTO db
    RAISERROR('Bulk insert file %s%s into %s with codepage = %n....', 0,1, @file_path, @bcpExtension, @CodePage)
    EXEC('BULK INSERT ' + @dstTable + ' FROM ''' + @file_path + @bcpExtension + '''
    WITH (
      FIELDTERMINATOR = ''|'',
      CODEPAGE = ' + @CodePage + ', 
      DATAFILETYPE = ''WIDECHAR'',
      FIRSTROW = 2
    )')
    IF @@ERROR <> 0
    BEGIN
      UPDATE DiscoveryBatch SET error = @@ERROR, 
		errtext = 'Failed ON bulk INSERT.',
                pstep = 2,
		pstat = 0
                WHERE FormatType = @FormatType AND FilePath = @file_path
      FETCH NEXT FROM cc INTO @batchid, @file_path, @CodePage
      CONTINUE
    END
    EXEC asisp_setBJBulkloadStatus @FormatType, @file_path, 2, 0, @batchid
    SELECT @num_schedule_set = @num_schedule_set + 1 
    FETCH NEXT FROM cc INTO @batchid, @file_path, @CodePage
  END
  DEALLOCATE cc

  IF @tcount = 0
    COMMIT TRAN _LoadJobDiscoveryBatch

  IF EXISTS (SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND error <> 0 AND pstep = 1)
  BEGIN
    RAISERROR('Please check DB error IN DiscoveryBatch TABLE', 0, 1 )
    RAISERROR('Total schedule SET processed = %d', 0, 1, @num_schedule_set)
    RETURN -30000
  END
  ELSE
  BEGIN
    RAISERROR('Total schedule SET processed = %d', 0, 1, @num_schedule_set)
    RETURN 0
  END

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_setGlobals
PRINT 'Creating procedure asisp_setGlobals....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setGlobals' AND type = 'P')
  DROP PROCEDURE asisp_setGlobals
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
CREATE PROCEDURE asisp_setGlobals
  @groupname VARCHAR(50), 
  @propertyname VARCHAR(50), 
  @propertyvalue VARCHAR(50), 
  @propertydatatype VARCHAR(50)
AS

  RAISERROR('SET Global variable %s FOR %s', 0, 1, @propertyname, @groupname) 
  SET NOCOUNT ON 

  IF NOT EXISTS (SELECT * FROM SystemConfiguration  WHERE groupname = @groupname AND propertyname = @propertyname)
    INSERT SystemConfiguration(groupname, propertyname, propertyvalue, propertydatatype)
     SELECT @groupname, @propertyname, @propertyvalue, @propertydatatype
  ELSE 
    UPDATE SystemConfiguration SET groupname = @groupname, 
      propertyname = @propertyname, 
      propertyvalue = @propertyvalue, 
      propertydatatype = @propertydatatype
      WHERE groupname = @groupname AND propertyname = @propertyname

  SET NOCOUNT OFF

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_processca7SIRD11Events
PRINT 'Creating procedure asisp_processca7SIRD11Events....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processca7SIRD11Events' AND type = 'P')
  DROP PROCEDURE asisp_processca7SIRD11Events
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Process ca7 SIRD-11 events AND events with schedule name. 
-- It IS required SIRD-11 event has schedule name 
CREATE PROCEDURE asisp_processca7SIRD11Events
  @autoDiscoveryFlag TINYINT = 1
AS

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processca7SIRD11Events
  ELSE
    SAVE TRANSACTION asisp_processca7SIRD11Events

  -- UPDATE btcy_id FOR existing schedules
  UPDATE s SET s.btcy_id = b.id FROM StagedCA7InProgress AS s
    JOIN BatchCycle_V AS b ON s.SYSTEMNAME = b.name AND MSG = 'SIRD-11'
    JOIN link AS l ON l.dst_id = b.id AND l.src_id = s.bcys_id 
    WHERE l.src_cid = 'BCYS' 
    AND l.dst_cid = 'BTCY' 
    AND link_type = 'AGGC' 
    AND l.deleted = 0
    AND b.userid = 'CA7' 

  -- CREATE non existing schedules if CA7_autodisflag = 1
  DECLARE @rows INT
  IF @autoDiscoveryFlag = 1
  BEGIN
    DELETE Disc_schedules WHERE bulk_load = 0 and data_source = 'CA7'
    -- get a list of the batch schedule INTO Disc_schedules TABLE with _ParentID
    INSERT INTO Disc_schedules (name, bcys_id, name_schedule_set, bulk_load, data_source)
      SELECT DISTINCT SYSTEMNAME, bcys_id, '', 0, 'CA7'
      FROM StagedCA7InProgress 
      WHERE MSG = 'SIRD-11' 
      AND SYSTEMNAME IS NOT NULL 
      AND SYSTEMNAME <> '' 
      AND btcy_id = 0
    IF EXISTS (SELECT * FROM Disc_schedules WHERE bulk_load = 0 AND data_source = 'CA7') 
    BEGIN
      EXEC @rows = asisp_bulkcreateBTCY NULL, NULL, 0, @autoDiscoveryFlag, 'CA7'
      IF @rows < 0
      BEGIN
        RAISERROR('Bulk CREATE batch schedules failed', 0, 1)
        ROLLBACK TRAN asisp_processca7SIRD11Events
        RETURN -30000
      END
      -- UPDATE btcy_id FOR new created objects IN StagedCA7InProgress
      UPDATE s SET s.btcy_id = b.id FROM StagedCA7InProgress AS s
        JOIN BatchCycle_V AS b ON s.SYSTEMNAME = b.name
        JOIN link AS l ON l.dst_id = b.id AND l.src_id = s.bcys_id 
        AND l.src_cid = 'BCYS' AND l.dst_cid = 'BTCY' AND l.link_type = 'AGGC' AND l.deleted = 0
        WHERE s.btcy_id = 0 AND b.userid = 'CA7'
    END
  END
  -- UPDATE batc_id of SIRD-11 events
  UPDATE s SET s.batc_id = b.id FROM  StagedCA7InProgress AS s 
    JOIN BATCH_V AS b ON s.JOBNAME = b.name AND s.MSG = 'SIRD-11'
    AND s.SCHEDID = b._SCHED_PACK
    JOIN link AS l ON b.id = l.dst_id 
    AND l.src_id = s.btcy_id 
    WHERE l.src_cid = 'BTCY' 
    AND l.dst_cid = 'BATC' 
    AND l.deleted = 0
    AND b.userid = 'CA7'

  -- discover non-exist BATC objects if @autoDiscoveryFlag = 1, FOR jobs run multiple times a day, 
  -- USE event start time AND earlest DUEOUT 
  IF @autoDiscoveryFlag = 1
  BEGIN
    DELETE ca7_jobs WHERE bulk_load = 0
    CREATE TABLE #temp_job(
      JOBNAME 	ObjName,
      SYSTEMNAME 	ObjName,
      btcy_id 	ObjID,
      SCHEDID	ObjID,
      start_year 	INT,
      start_day	INT,
      start_time	CHAR(5),
      end_year	INT,
      end_day	INT,
      end_time	CHAR(5)
    )

    INSERT #temp_job 
      SELECT JOBNAME, 
      	SYSTEMNAME, 
      	btcy_id, 
	SCHEDID,
        DATEPART(yy, TIME) AS start_year,
        DATEPART(dy, TIME) AS start_day,
        CONVERT(CHAR(5), TIME, 108) AS start_time,                
        2000 + CONVERT(INT, SUBSTRING(DUEOUT, 1, 2)) AS end_year,
        CONVERT(INT, SUBSTRING(DUEOUT, 4, 3)) AS end_day,
        SUBSTRING(DUEOUT, CHARINDEX(' ', DUEOUT)+1, 5) AS end_time          
    FROM StagedCA7InProgress 
      WHERE MSG = 'SIRD-11' AND batc_id = 0 AND SYSTEMNAME IS NOT NULL AND SYSTEMNAME <> ''

    DELETE t1 FROM #temp_job AS t1 
      JOIN #temp_job AS t2 ON t1.JOBNAME = t2.JOBNAME
      AND t1.SYSTEMNAME = t2.SYSTEMNAME
      AND t1.btcy_id = t2.btcy_id
      AND t1.SCHEDID = t2.SCHEDID
      AND (t1.end_day < t2.end_day OR CONVERT(DATETIME, t1.end_time) < CONVERT(DATETIME, t2.end_time))
    
    INSERT INTO ca7_jobs (name, cycle_name, btcy_id, sched_id, flag_keyjob, year_start, day_start, time_start, year_stop, day_stop, time_stop, bulk_load)
      SELECT DISTINCT JOBNAME, 
      	SYSTEMNAME, 
      	btcy_id, 
	SCHEDID,
	0,
        start_year,
        start_day,
        start_time,
        end_year,
        end_day,
        end_time, 
        0
      FROM #temp_job
    IF EXISTS (SELECT * FROM ca7_jobs WHERE bulk_load = 0)
    BEGIN
      EXEC @rows = _CA7CreateBATC 0, @autoDiscoveryFlag, 'CA7'
      IF @rows < 0
      BEGIN
        RAISERROR('Bulk CREATE batch jobs failed WHEN loading events', 0, 1) 
        ROLLBACK TRAN asisp_processca7SIRD11Events
        RETURN -30000
      END
      -- UPDATE batc_id FOR new created objects
      UPDATE s SET s.batc_id = b.id FROM  StagedCA7InProgress AS s 
        JOIN BATCH_V AS b ON s.JOBNAME = b.name 
        AND s.SCHEDID = b._SCHED_PACK AND b.userid = 'CA7'
        JOIN link AS l ON b.id = l.dst_id 
      AND l.src_id = s.btcy_id 
      AND l.src_cid = 'BTCY' 
      AND l.dst_cid = 'BATC'
      AND l.deleted = 0
      WHERE s.batc_id = 0 AND s.MSG = 'SIRD-11'
    END
  END

  -- Unset ca7_SIRD11_events
  DELETE c FROM ca7_SIRD11_events AS c
    JOIN StagedCA7InProgress AS s ON c.bcys_id = s.bcys_id AND c.job_num = s.JOBNUM AND c.job_name = s.JOBNAME
    WHERE s.MSG = 'SIRD-11'

  -- SET ca7_SIRD11_events FOR new SIRD-11 events
  INSERT ca7_SIRD11_events 
    SELECT bcys_id, batc_id, JOBNAME, JOBNUM
    FROM StagedCA7InProgress 
    WHERE MSG = 'SIRD-11'
  
  -- UPDATE mtime IN BATCH_C FOR DELETE old objects
  UPDATE BATCH_C SET mtime = GETDATE() 
    WHERE id IN (SELECT DISTINCT batc_id FROM StagedCA7InProgress WHERE MSG = 'SIRD-11' AND batc_id <> 0)

  IF @tcount = 0
    COMMIT TRAN asisp_processca7SIRD11Events
GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_processStagedCA7Events
PRINT 'Creating procedure asisp_processStagedCA7Events....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedCA7Events' AND type = 'P')
  DROP PROCEDURE asisp_processStagedCA7Events
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Processes events in the StagedCA7Event table
CREATE PROCEDURE asisp_processStagedCA7Events
  @maxrows INT = 0,
  @verbose TINYINT = 1
AS

  SET NOCOUNT ON
  TRUNCATE TABLE StagedCA7InProgress
  DECLARE @err INT, @rc INT, @step INT
  SET @step = 1
  SET ROWCOUNT @maxrows

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating StagedCA7InProgress'
  SET @step = @step + 1

  DECLARE @nEvents INT
  INSERT INTO StagedCA7InProgress (timestamp_col, ctime, src_cid, src_id, TIME, SEQ_NO, MESSAGE)
  SELECT * FROM StagedCA7Event
  SELECT @nEvents = @@ROWCOUNT

  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': Inserted ' + CAST(@nEvents AS VARCHAR(9)) + ' row(s)' 
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1

  SET ROWCOUNT 0

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedCA7Events
  ELSE
    SAVE TRANSACTION asisp_processStagedCA7Events

  -- find out schedule SET id based ON OS name, It IS required that IN LOB "CA7 COMPLEXES"
  -- was created AND schedule sets of CA7 were dragged FROM physical tree under "CA7 COMPLEXES",
  -- Under each schedule SET will be the OSs that the CA7 schedule SET could run.  
  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': lookup schedule SET id based ON os_id'
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1

  EXEC asisp_lookupCA7BCYSID 

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': processing message fields.'
  SET @step = @step + 1

  -- We now iterate through each message AND find the best matching CA7EventMap entry based
  -- ON the CA7EventMap.WhereClause column. IN the CASE of multiple WhereClause matches we 
  -- USE the CA7EventMap.MapPriority column to determine which one applies. IF more than
  -- one row with a matching WhereClause AND a given MapPriority are found THEN which IS
  -- chosen will be indeterminate.

  -- CREATE a regular expression context handle. This will be reused FOR each SearchExp
  DECLARE @regex INT, @hr INT
  EXEC @hr = sp_OACreate 'TMRegExp.RegExContext', @regex OUT
  IF @hr <> 0
  BEGIN
    DECLARE @errtext VARCHAR(255)
    EXEC sp_OAGetErrorInfo NULL, NULL, @errtext OUT
    RAISERROR('%s', 3, 1, @errtext)
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  DECLARE @MESSAGE VARCHAR(500), @SearchExp VARCHAR(255), @ReplaceExp VARCHAR(255)
  DECLARE cursor_asisp_processStagedCA7Events CURSOR FOR 
    SELECT MESSAGE FROM StagedCA7InProgress
  OPEN cursor_asisp_processStagedCA7Events
  FETCH NEXT FROM cursor_asisp_processStagedCA7Events INTO @MESSAGE
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- Find the best matching CA7EventMap row
    DECLARE @priority INT
    SELECT @priority = MAX(MapPriority) FROM CA7EventMap WHERE @MESSAGE LIKE WhereClause AND Enabled = 1
    SELECT @SearchExp = SearchExp, @ReplaceExp = ReplaceExp FROM CA7EventMap WHERE @MESSAGE LIKE WhereClause AND Enabled = 1 AND MapPriority = @priority
    IF @@ROWCOUNT <> 1
    BEGIN
      RAISERROR('No mapping FOR message NTEXT: %s, aborting.', 3, 1, @MESSAGE)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Compile the search regular expression
    EXEC @hr = sp_OAMethod @regex, 'Compile', NULL, @SearchExp, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Process the regular expression ON the message NTEXT
    EXEC @hr = sp_OAMethod @regex, 'EXEC', NULL, @MESSAGE, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Replace the matched fields FROM the regular expression using the replace expression
    -- IN order to CREATE the UPDATE statement which will SET the various fields of the
    -- current row
    -- **** WARNING ****
    -- Output Parameters from OLE calls MUST be VARCHAR they will return NULL if they are VARCHAR
    -- **** WARNING ****
    DECLARE @processing_sql VARCHAR(2000), @update_sql VARCHAR(1000)
    EXEC @hr = sp_OAMethod @regex, 'Replace', @update_sql OUT, @ReplaceExp
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- USE dynamic SQL to UPDATE the current row of the CURSOR
    SELECT @processing_sql = "UPDATE StagedCA7InProgress SET " + @update_sql + " WHERE CURRENT OF cursor_asisp_processStagedCA7Events"
    EXEC(@processing_sql)
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('asisp_processStagedCA7Events: Error executing SQL [%s]', 11, 1, @processing_sql)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END
    UPDATE StagedCA7InProgress SET SYSTEMNAME = NULL WHERE SYSTEMNAME = ' '

    FETCH NEXT FROM cursor_asisp_processStagedCA7Events INTO @MESSAGE
  END
  DEALLOCATE cursor_asisp_processStagedCA7Events

  -- handle evnet does NOT match any format we defined, INSERT events INTO ca7_audit_events TABLE
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': INSERT unmapped events INTO audit TABLE.'
  SET @step = @step + 1
  EXEC asisp_ca7auditevents @before_lookup = 1

  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': get global varibles.'
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1

  -- Two global varibles @keep_unknown and @autoDiscoveryFlag.
  -- @keep_unknown: look up tables(ca7_discarded_SIRD11 AND ca7_SIRD11_events) FOR SIRD-11 events discarded events
  -- AND non-discarded evnets. IF @keep_unknown =1, all jobs without systemane will be created under unkmown
  -- events will be loaded INTO db, OR ELSE objects will NOT be created, events will be discarded. 
  -- @autoDiscoveryFlag: indicate if the system is setup for creating object by events. 
  DECLARE @keep_unknown TINYINT, @autoDiscoveryFlag TINYINT
  SELECT @keep_unknown = CONVERT(TINYINT, propertyvalue) 
    FROM SystemConfiguration WHERE groupname = 'CA7' AND propertyname = 'keep_unknown'
  IF @keep_unknown = 1
  -- UPDATE NULL SYSTEMNAME with '*UNKNOWN*' FOR SIRD-11 events. IF batch job does NOT exist, CREATE it
    UPDATE StagedCA7InProgress SET SYSTEMNAME = '*UNKNOWN*' 
      WHERE MSG = 'SIRD-11' AND SYSTEMNAME IS NULL

  SELECT @autoDiscoveryFlag = CONVERT(TINYINT, propertyvalue) FROM SystemConfiguration 
    Where groupname = 'CA7' AND propertyname = 'autoDiscoveryFlag'

  -- process ca7 SIRD-11 events
  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': handle ca7 SIRD-11 events'
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1
  DECLARE @r INT
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE MSG = 'SIRD-11')
  BEGIN
    EXEC @r = asisp_processca7SIRD11Events @autoDiscoveryFlag
    IF @r < 0
    BEGIN
      ROLLBACK TRAN asisp_processStagedCA7Events
      RETURN @r
    END
  END 

  -- process non SIRD-11 events without SIRD-11 event IN this bacth
  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': Process non SIRD-11 evnets.'
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE MSG <> 'SIRD-11')
  BEGIN
    EXEC @r = asisp_processca7nonSIRD11Events
    IF @r < 0
    BEGIN
      ROLLBACK TRAN asisp_processStagedCA7Events
      RETURN @r
    END
  END

  IF @autoDiscoveryFlag = 0
    EXEC asisp_ca7auditevents @before_lookup = 0

  -- IF any objects here does NOT have id, there IS an error
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE batc_id = 0)
  BEGIN
    ROLLBACK TRAN asisp_processStagedCA7Events
    RAISERROR('ERROR: NOT all object id IS mapped.', 5, 1)
    RETURN -30000
  END

  -- CREATE messages
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating MESGs'
  SET @step = @step + 1
  UPDATE StagedCA7InProgress SET MESSAGEDETAIL = 'CA7 job_id(' + JOBNUM + '), SCHID=' + CONVERT(CHAR(3), SCHEDID) + ', '  + MESSAGEDETAIL
    WHERE MSG = 'SIRD-11' 
  UPDATE StagedCA7InProgress SET MESSAGEDETAIL = 'CA7 job_id(' + JOBNUM + '), ' + MESSAGEDETAIL
    WHERE MSG <> 'SIRD-11' 

  INSERT INTO StagedMESG
  (
    parent_cid, 
    parent_id, 
    TIME, 
    SEQ_NO, 
    OBJECT_NAME,
    NATIVE_KEY,
    MESSAGE, 
    REASON_CD, 
    DISCOVERY_IND,
    STATE
	)
  SELECT
    'BATC', 
    s.batc_id, 
    s.TIME, 
    s.SEQ_NO, 
    s.JOBNAME,
    '',
    s.MSG, 
    s.MESSAGEDETAIL, 
    'MON',
    t.State
  FROM StagedCA7InProgress AS s
    JOIN MessageDescription_T AS t ON s.MSG = t.MsgID AND t.obj_cid = 'BATC'
  WHERE s.batc_id <> 0 
  ORDER BY timestamp_col

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  -- CREATE exceptions
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating EXCPs'
  SET @step = @step + 1
  INSERT INTO StagedEXCP
  (
    parent_cid, 
	  parent_id, 
	  TIME, 
	  SEQ_NO, 
	  OBJECT_NAME, 
	  EXCP_NAME, 
	  EXCP_CD, 
	  AlertState, 
	  Priority, 
	  nEXCH
  )
  SELECT
    'BATC', 
    s.batc_id, 
    s.TIME, 
    s.SEQ_NO, 
    s.JOBNAME, 
    s.MSG, 
    'CA7 job_id(' + s.JOBNUM + ') ' + s.MESSAGEDETAIL, 
    e.AlertStateID, 
    e.PriorityID, 
    1
  FROM StagedCA7InProgress s 
  JOIN ExceptionType_T e ON e.Exception = s.MSG AND e.cid = 'BATC'
  WHERE s.batc_id IS NOT NULL AND s.batc_id <> 0

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  IF @tcount = 0
    COMMIT TRAN asisp_processStagedCA7Events

  DELETE s FROM StagedCA7Event AS s JOIN StagedCA7InProgress AS p ON s.timestamp = p.timestamp_col

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': processed ' + CAST(@nEvents AS VARCHAR(9)) + ' events'
  SET @step = @step + 1
  SET NOCOUNT OFF
  RETURN @nEvents

GO
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_ca7auditevents
PRINT 'Creating procedure asisp_ca7auditevents....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_ca7auditevents' AND type = 'P')
  DROP PROCEDURE asisp_ca7auditevents
GO


-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- handle ca7 events which cannot be mapped with current defination
CREATE PROCEDURE asisp_ca7auditevents
  @before_lookup INT = 1
AS

  RAISERROR('EXEC asisp_ca7auditevents.', 0, 1)

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_ca7auditevents
  ELSE
    SAVE TRANSACTION asisp_ca7auditevents

  -- INSERT unmatched event INTO audit TABLE
  DECLARE @WhereClause VARCHAR(255)
  IF @before_lookup = 1
  BEGIN
    SELECT @WhereClause = 'MSG = '''' OR MSG IS NULL'
    -- Filter events for objects in excluded table
    IF EXISTS (SELECT * FROM BJDiscoveryFilter_Lookup WHERE data_source = 'CA7')
    BEGIN
      SELECT scp.timestamp_col INTO #timestamp_col FROM StagedCA7InProgress AS scp
        JOIN BJDiscoveryFilter_Lookup AS lookup 
        ON scp.SYSTEMNAME like CASE lookup.name_schedule WHEN 'ALL' THEN '%' ELSE lookup.name_schedule END 
        AND scp.JOBNAME like CASE lookup.name_job WHEN 'ALL' THEN '%' ELSE lookup.name_job END
        JOIN BatchCycles_V AS bcys ON bcys.id = scp.bcys_id
        AND bcys.name like CASE lookup.name_schedule_set WHEN 'ALL' THEN '%' ELSE lookup.name_schedule_set END 

      IF EXISTS (SELECT * FROM #timestamp_col)
      BEGIN
        DELETE StagedCA7InProgress 
          WHERE timestamp_col IN (SELECT timestamp_col FROM #timestamp_col)
        DELETE StagedCA7Event 
          WHERE timestamp IN (SELECT timestamp_col FROM #timestamp_col)        
      END
    END
  END
  ELSE
    SELECT @WhereClause = 'batc_id = 0 OR batc_id IS NULL'
  EXEC('INSERT ca7_audit_events 
    SELECT src_cid, src_id, ctime, MESSAGE 
      FROM StagedCA7InProgress
      WHERE ' + @WhereClause)
  IF @@ROWCOUNT > 0
  BEGIN
    -- INSERT ProcessedStagedEvents
    EXEC('DELETE StagedCA7Event WHERE timestamp IN  
      (SELECT timestamp_col FROM StagedCA7InProgress
      WHERE ' + @WhereClause + ')')
    -- DELETE FROM StagedCA7InProgress
    EXEC('DELETE StagedCA7InProgress WHERE ' + @WhereClause)
  END
  IF @tcount = 0
    COMMIT TRAN asisp_ca7auditevents

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- _DiscoverCA7Objects
PRINT 'Creating procedure _DiscoverCA7Objects....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCA7Objects' AND type = 'P')
  DROP PROCEDURE _DiscoverCA7Objects
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- load ca7 plan FROM ca7_bcp TABLE. All schedule sets have to be created before auto discovery schedule AND batch job
-- objects. IF schedule SET was NOT created, RAISERROR will PRINT the missing schedule SET AND the whole TRANSACTION will
-- roll back.  
CREATE PROCEDURE _DiscoverCA7Objects
AS
  
  RAISERROR( '_DiscoverCA7Objects', 0, 1)
  SET NOCOUNT ON

  -- Standard TRANSACTION logic  
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _DiscoverCA7Objects
  ELSE
    SAVE TRAN _DiscoverCA7Objects

  -- get varibles
  DECLARE @FormatType TINYINT, @autoDiscoveryFlag TINYINT, @data_source ObjName
  SELECT @data_source = 'CA7'
  SELECT @FormatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = @data_source
  SELECT @autoDiscoveryFlag = CONVERT(TINYINT, propertyvalue) FROM SystemConfiguration 
    WHERE groupname = @data_source AND propertyname = 'autoDiscoveryFlag'
  -- if @autoDiscoveryFlag = 0, check lookup table for more filters  
  IF @autoDiscoveryFlag = 0
  BEGIN
    IF EXISTS (SELECT * FROM BJDiscoveryFilter_Lookup WHERE data_source = @data_source)
      DELETE ca7_bcp  
        FROM ca7_bcp AS ca7
        JOIN BJDiscoveryFilter_Lookup AS lookup ON 
        ca7.name_schedule_set like CASE lookup.name_schedule_set WHEN 'ALL' THEN '%' ELSE lookup.name_schedule_set END 
        AND ca7.name_system like CASE lookup.name_schedule WHEN 'ALL' THEN '%' ELSE lookup.name_schedule END 
        AND ca7.name_job like CASE lookup.name_job WHEN 'ALL' THEN '%' ELSE lookup.name_job END
  END

  -- Check IF other process with pstep > 1 running
  IF EXISTS ( SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstep >1 AND pstat = 1)
  BEGIN
    RAISERROR('Other loading OR process IS running.', 0, 1)
    RETURN -30000
  END

  -- SET pstep to 3, pstat = 1 FOR all data ready to process 
  UPDATE DiscoveryBatch SET pstep = 3, pstat = 1 WHERE FormatType = @FormatType AND pstep =2 AND pstat = 0

  -- clean up before process
  DECLARE @name_schedule_set ObjName, @id_schedule_set ObjID, @name_schedule ObjName, @id_schedule ObjID
  DECLARE @name_job ObjName, @id_job ObjID, @sched_id INT, @job_count INT, @like_clause VARCHAR(256)

  DELETE Disc_schedules WHERE bulk_load = 1 AND data_source = @data_source
  DELETE ca7_jobs WHERE bulk_load = 1
  -- handle objects with empty SYSTEM field AND non-executable jobs based ON global variable.
  -- These jobs will NOT be created IF @keep_unknown IS NOT SET to 1. 
  -- UPDATE SYSTEM = '*UNKNOWN*' FOR empty field IF @keep_unknown = 1 
  -- OR ELSE leave it AS NULL AND the objects under it will NOT be created
  DECLARE @keep_unknown TINYINT
  SELECT @keep_unknown = CONVERT(TINYINT, propertyvalue) 
    FROM SystemConfiguration 
    WHERE propertyname = 'keep_unknown' AND groupname = 'CA7'
  IF @keep_unknown = 1
    UPDATE ca7_bcp SET name_system = '*UNKNOWN*' WHERE name_system IS NULL OR name_system = ''
  ELSE 
     DELETE ca7_bcp WHERE name_system = '' OR name_system IS NULL
  -- Currently not handle this type object by desgin
  DELETE ca7_bcp WHERE flag_exec = 0

  -- DELETE multiple entries FOR the same job AND run ON different days
  DELETE c1 FROM ca7_bcp AS c1 
    JOIN ca7_bcp AS c2 ON c1.name_schedule_set = c2.name_schedule_set
    AND c1.name_system = c2.name_system
    AND c1.name_job = c2.name_job
    AND c1.sched_id = c2.sched_id
    AND (c1.day_start > c2.day_start OR c1.time_start > c2.time_start) 

  -- UPDATE schedule SET short name with full name 
  UPDATE b SET b.name_schedule_set = l.full_name FROM ca7_bcp AS b
    JOIN ca7_bcys_name_lookup AS l ON b.name_schedule_set = l.short_name

  -- loop ca7_bcp TABLE by schedule SET name AND created all schedules. INSERT all info FROM ca7_bcp 
  -- with _ParentCID AND _ParentID FOR all jobs.  
  DECLARE bcp CURSOR FOR SELECT DISTINCT name_schedule_set FROM ca7_bcp
  OPEN bcp 
  FETCH NEXT FROM bcp INTO @name_schedule_set
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @like_clause = '%' + short_name + '%' FROM ca7_bcys_name_lookup
      WHERE full_name = '@name_schedule_set'
    -- find schedule SET id, IF NOT EXISTS, RETURN error
    SELECT @id_schedule_set = id FROM BatchCycles_V WHERE deleted = 0 AND name = @name_schedule_set
    IF @id_schedule_set IS NULL
    BEGIN
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Schedule SET was NOT created.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      RAISERROR("Schedule SET was NOT created FOR %s.", 0, 1, @name_schedule_set)
      FETCH NEXT FROM bcp INTO @name_schedule_set
      CONTINUE
    END

    RAISERROR("Processing %s schedule SET.", 0, 1, @name_schedule_set)
    -- insert all schedules in Disc_schedule working table
    INSERT INTO Disc_schedules (name, name_schedule_set, bcys_id, data_source, bulk_load)
      SELECT DISTINCT name_system, name_schedule_set, @id_schedule_set, @data_source, 1
      FROM ca7_bcp WHERE name_schedule_set = @name_schedule_set 

    -- working on Disc_schedules table, update id if exists, else create objects
    DECLARE @rows INT 
    EXEC @rows = asisp_bulkcreateBTCY 
		@name_schedule_set, 
		@id_schedule_set, 
		1, 
		@autoDiscoveryFlag,
		@data_source 
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk CREATE batch schedules failed', 0, 1)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	  pstat = 0, 
	  error = -30000,
	  errtext = 'Bulk CREATE batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      FETCH NEXT FROM bcp INTO @name_schedule_set
      CONTINUE
    END
    -- make sure all monitored batch schedules were created under this schedule SET 
    IF EXISTS (SELECT * FROM ca7_bcp 
                 WHERE name_schedule_set = @name_schedule_set 
                 AND name_system IS NOT NULL AND name_system <> ''
                 AND name_system NOT IN 
                   (SELECT name FROM BatchCycle_V AS b 
                   JOIN BCYS_BTCY_AGGC AS l ON b.id = l.dst_id 
                   AND b.deleted = 0 
                   AND l.src_id = @id_schedule_set))
    BEGIN
      RAISERROR("Batch schedule %s was NOT created.", 0, 1, @name_schedule)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Bulk CREATE batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
    END
    -- copy monitored jobs out of the ca7_bcp TABLE INTO ca7_jobs AND UPDATE btcy_id AND id based ON 
    -- bcys_id, system_name, job name AND sched_id
    INSERT INTO ca7_jobs (
		name, 
	        sched_id,
		cycle_name, 
		flag_keyjob, 
		year_start, 
		day_start, 
		time_start, 
		time_duration, 
		year_stop, 
		day_stop, 
		time_stop,
		btcy_id,
        	bulk_load)
            SELECT c.name_job,
	        c.sched_id,
		c.name_system,
		c.flag_keyjob,
		c.year_start,
		c.day_start,
		c.time_start, 
		c.time_duration, 
		c.year_stop, 
		c.day_stop, 
		c.time_stop,
		b.id,
        	1
      FROM ca7_bcp AS c JOIN BatchCycle_V AS b ON c.name_system = b.name AND b.deleted = 0
      JOIN BCYS_BTCY_AGGC AS l ON b.id = l.dst_id AND l.src_id = @id_schedule_set AND l.deleted = 0
      WHERE  c.name_schedule_set = @name_schedule_set AND c.name_system <> '' 
      AND c.name_system IS NOT NULL

    FETCH NEXT FROM bcp INTO @name_schedule_set
  END 
  DEALLOCATE bcp 

  -- CREATE batch job IF NOT EXISTS, THEN UPDATE the plan info FOR jobs using ca7_jobs TABLE
  EXEC @rows = _CA7CreateBATC 1, @autoDiscoveryFlag, @data_source
  IF @rows < 0
  BEGIN
    RAISERROR('Bulk CREATE batch job failed', 0, 1)
    ROLLBACK TRAN _DiscoverCA7Objects
    RETURN -30000
  END

  -- clean up non-monitored batch schedules and batch jobs
  IF @autoDiscoveryFlag = 0
    EXEC asisp_cleanupAutoDiscJob @data_source

  -- Finish TRANSACTION
  IF @tcount = 0
    COMMIT TRAN _DiscoverCA7Objects

  SET NOCOUNT OFF  

  IF NOT EXISTS (SELECT * FROM DiscoveryBatch 
    WHERE FormatType = @FormatType AND pstep = 3 AND error <> 0 AND error IS NOT NULL)
  BEGIN
    -- SET pstep to 3, pstat = 1 FOR all data ready to process 
    DELETE DiscoveryBatch WHERE FormatType = @FormatType AND pstep = 3 AND pstat = 1 
    RETURN 0
  END
  ELSE
  BEGIN
    RAISERROR('Please check DiscoveryBatch TABLE FOR error messages', 0, 1)
    RETURN -30000
  END

GO
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_cleanupAutoDiscJob
PRINT 'Creating procedure asisp_cleanupAutoDiscJob....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_cleanupAutoDiscJob' AND type = 'P')
  DROP PROCEDURE asisp_cleanupAutoDiscJob
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Creates the BTCY objects FOR the CA7 plan AS loaded IN the ca7_bcp TABLE
CREATE PROCEDURE asisp_cleanupAutoDiscJob
  @data_source ObjName
AS
  
  RAISERROR('Excuting asisp_cleanupAutoDiscJob @data_source = %s.', 0, 1, @data_source)

  -- Create a temp table for non-monitored schedules
  CREATE TABLE #old_schedules(bcys_id ObjID, btcy_id ObjID)

  -- clean up batch schedules 
  INSERT #old_schedules SELECT c.src_id, btcy.id 
      FROM BatchCycle_V AS btcy JOIN containment AS c ON c.dst_cid = 'BTCY'
  	AND c.src_cid = 'BCYS' 
        AND c.deleted = 0
	AND btcy.id = c.dst_id
        AND btcy.userid = @data_source
        AND btcy._AlertStateID = 1
        AND btcy.id NOT IN (SELECT obj_id FROM Disc_schedules 
				WHERE bulk_load = 1 AND 
				data_source = @data_source)

  UPDATE BatchCycle_V SET deleted = 1 WHERE id IN (SELECT btcy_id FROM #old_schedules)
  UPDATE c SET c.deleted = 1 FROM containment AS c 
    JOIN #old_schedules AS old ON c.src_id = old.bcys_id
    AND c.dst_id = old.btcy_id
    WHERE c.src_cid = 'BCYS' AND c.dst_cid = 'BTCY'

  UPDATE l SET l.deleted = 1 FROM link AS l 
    JOIN #old_schedules AS old ON l.src_id = old.bcys_id
    AND l.dst_id = old.btcy_id
    WHERE l.src_cid = 'BCYS' AND l.dst_cid = 'BTCY'

  UPDATE l SET l.deleted = 1 FROM BCYS_BTCY_AGGC AS l 
    JOIN #old_schedules AS old ON l.src_id = old.bcys_id
    AND l.dst_id = old.btcy_id

  -- delete batch jobs which are not monitored any more
  DECLARE @btcy_id ObjID
  DECLARE cc CURSOR FOR SELECT DISTINCT btcy_id FROM Disc_job_lookups WHERE data_source = @data_source
  OPEN cc
  FETCH NEXT FROM cc INTO @btcy_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    UPDATE BATCH_V SET deleted = 1 
      WHERE _ParentCID = 'BTCY'
      AND _ParentID = @btcy_id
      AND id NOT IN (SELECT batc_id FROM Disc_job_lookups WHERE btcy_id = @btcy_id AND data_source = @data_source)
      AND deleted = 0
      AND userid = @data_source 
    FETCH NEXT FROM cc INTO @btcy_id
  END
  DEALLOCATE cc

GO
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_bulkcreateBTCY
PRINT 'Creating procedure asisp_bulkcreateBTCY....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_bulkcreateBTCY' AND type = 'P')
  DROP PROCEDURE asisp_bulkcreateBTCY
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Creates the BTCY objects FOR the CA7 plan AS loaded IN the ca7_bcp TABLE
CREATE PROCEDURE asisp_bulkcreateBTCY
  @name_schedule_set 	ObjName = NULL,
  @id_schedule_set 	ObjID = NULL,
  @useBCPTable 		TINYINT = 1,
  @autoDiscoveryFlag    TINYINT = 1,
  @data_source 		ObjName = NULL
AS

  SET NOCOUNT ON

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_bulkcreateBTCY
  ELSE
    SAVE TRAN asisp_bulkcreateBTCY

  IF @useBCPTable = 1
    -- map the obj_id column to actual ids
    UPDATE cyc SET cyc.obj_id = btcy.id
      FROM Disc_schedules AS cyc, BatchCycle_V AS btcy 
      JOIN containment AS c ON c.dst_cid = 'BTCY' AND c.dst_id = btcy.id
      WHERE c.deleted = 0 
      AND cyc.name = btcy.name 
      AND cyc.bcys_id = c.src_id 
      AND cyc.bulk_load = @useBCPTable
      AND cyc.bcys_id = @id_schedule_set
      AND btcy.userid = @data_source

  -- CREATE new instances
  CREATE TABLE #schedules_new
  (
    id ObjID IDENTITY(0,1),
    name ObjName NULL,
    parent_cid ClassID NULL,
    parent_id ObjID
  )

  INSERT INTO #schedules_new (name, parent_cid, parent_id)
  SELECT name, 'BCYS', bcys_id
    FROM Disc_schedules
    WHERE (obj_id = 0 OR obj_id IS NULL)
    AND bulk_load = @useBCPTable AND data_source = @data_source

  IF EXISTS (SELECT * FROM #schedules_new)
  BEGIN
    DECLARE @base_id ObjID, @rows INT
    SELECT @rows = COUNT(id) FROM #schedules_new
    IF @rows > 0
      EXEC @base_id = alloc_BTCY @rows
    ELSE
      SET @base_id = 0

    -- INSERT INTO _C
    INSERT BatchCycle_C (id, name, userid)
      SELECT id+@base_id, name, @data_source
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    -- INSERT INTO _A
    INSERT BatchCycle_A (id,_Desc,_ChildEventMatrixID,_ExceptionMatrixID,_PriorityID)
      SELECT id+@base_id, '', 0, 0, 6
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    -- INSERT links
    INSERT INTO BCYS_BTCY_AGGC (src_id, dst_id)
      SELECT parent_id, id + @base_id
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    -- update obj_id in Disc_schedules for cleanup 
    IF @autoDiscoveryFlag = 0
      UPDATE d set d.obj_id = n.id + @base_id 
        FROM Disc_schedules AS d 
        JOIN #schedules_new AS n ON d.bcys_id = n.parent_id
        AND d.name =  n.name  

  END

  IF @tcount = 0
    COMMIT TRAN asisp_bulkcreateBTCY
  
  SET NOCOUNT OFF

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- _CA7CreateBATC
PRINT 'Creating _CA7CreateBATC procedure....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CA7CreateBATC' AND type = 'P')
  DROP PROCEDURE _CA7CreateBATC
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Creates the BATC objects FOR the CA7 plan AS loaded IN the ca7_bcp TABLE
CREATE PROCEDURE _CA7CreateBATC
  @useBCPTable 		INT = 1,
  @autoDiscoveryFlag 	TINYINT = 1,
  @data_source		ObjName
AS

  SET NOCOUNT ON

  DECLARE @tcount INT, @rows INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _CA7CreateBATC
  ELSE
    SAVE TRAN _CA7CreateBATC

  -- fix times of 24:00 IN ca7_jobs
  UPDATE ca7_jobs
  SET time_start = '00:00', day_start = day_start + 1
  WHERE time_start = '24:00'
  UPDATE ca7_jobs
  SET time_stop = '00:00', day_stop = day_stop + 1
  WHERE time_stop = '24:00'

  -- map the obj_id column to actual ids FOR existing objects
  -- unique jobs are based ON parent, name, AND _SCHED_PACK
  IF @useBCPTable = 1
  BEGIN 
    UPDATE j SET j.obj_id = b.id
      FROM ca7_jobs AS j JOIN BATCH_C AS b ON b._ParentCID = 'BTCY'
        AND b._ParentID = j.btcy_id AND j.bulk_load = @useBCPTable 
        AND b.name = j.name
        AND b._SCHED_PACK = j.sched_id 
        AND b.userid = @data_source 
        AND b.deleted = 0
  END

  -- translate the CA7 time formats INTO SQL time formats
  -- the start_date_time AND end_date_time variables should be defaulted to 19000101,
  -- which should work IN any locale
  UPDATE ca7_jobs SET start_date_time = DATEADD(yyyy, year_start - DATEPART(yyyy, start_date_time), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(dy, day_start - 1, start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(yyyy, year_stop - DATEPART(yyyy, end_date_time), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(dy, day_stop - 1, end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET duration = CONVERT(INT, SUBSTRING(time_duration, 1, 2))*60 + CONVERT(INT, SUBSTRING(time_duration, 4, 5))

  -- bulk CREATE batch objects if not exists
  IF EXISTS (SELECT * FROM ca7_jobs WHERE obj_id = 0 AND bulk_load = @useBCPTable)
  BEGIN
    -- insert into work table for creation
    DELETE Disc_jobs WHERE bulk_load = @useBCPTable AND data_source = @data_source
    INSERT Disc_jobs 
      SELECT name, 'BTCY', btcy_id, sched_id, @data_source, @useBCPTable
      FROM ca7_jobs WHERE bulk_load = @useBCPTable

    -- create non-exist objects
    EXEC @rows = asisp_bulkcreateBATC @useBCPTable, @data_source 
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk CREATE batch jobs failed', 0, 1) 
      ROLLBACK TRAN _CA7CreateBATC
      RETURN -30000
    END
 
    -- update job id in ca7_jobs table for cleanup
    IF @autoDiscoveryFlag = 0
      UPDATE j SET j.obj_id = b.id
        FROM ca7_jobs AS j JOIN BATCH_C AS b ON b._ParentCID = 'BTCY'
        AND b._ParentID = j.btcy_id AND j.bulk_load = @useBCPTable 
        AND b.name = j.name
        AND b._SCHED_PACK = j.sched_id 
        AND b.deleted = 0
        AND b.userid = @data_source 
  END

  -- UPDATE BATCH_C attributes except PriorityID which will be updated by TRIGGER
  UPDATE b
    SET b._KeyStatus = j.flag_keyjob,
      b._SchedStartTime = j.start_date_time,
      b._SchedStopTime = j.end_date_time,
      b._SchedDuration = j.duration,
      b._SchedStatusFlags = 1
    FROM BATCH_C AS b JOIN ca7_jobs AS j ON b.name = j.name 
    AND b._SCHED_PACK = j.sched_id
    AND j.bulk_load = @useBCPTable
    JOIN link AS l ON j.btcy_id = l.src_id 
    AND l.src_cid = 'BTCY'
    AND b.id = l.dst_id 
    AND l.dst_cid = 'BATC' 
    AND l.deleted = 0

  -- for clean up batch jobs
  IF @autoDiscoveryFlag = 0
  BEGIN
    DELETE Disc_job_lookups WHERE data_source = @data_source
    INSERT Disc_job_lookups SELECT btcy_id, obj_id, @data_source FROM ca7_jobs WHERE bulk_load = @useBCPTable
  END

  IF @tcount = 0
    COMMIT TRAN _CA7CreateBATC
  
  IF @rows IS NULL
    RETURN 0
  ELSE
    RETURN @rows
  SET NOCOUNT OFF

GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_bulkcreateBATC
PRINT 'Creating asisp_bulkcreateBATC procedure....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_bulkcreateBATC' AND type = 'P')
  DROP PROCEDURE asisp_bulkcreateBATC
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Bulk creates the BATC objects with name, sched_id, _ParentCID, _ParentID attributes only
CREATE PROCEDURE asisp_bulkcreateBATC
  @useBCPTable INT = 1,
  @data_source ObjName
AS

  SET NOCOUNT ON

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_bulkcreateBATC
  ELSE
    SAVE TRAN asisp_bulkcreateBATC

  -- CREATE new TABLE FOR new instances with IDENTITY id column seeded AS (0,1) 
  CREATE TABLE #jobs_new
  (
    id ObjID IDENTITY(0,1),
    name ObjName NULL,
    sched_id INT NULL,
    parent_cid ClassID NULL,
    parent_id ObjID
  )
  -- INSERT all non existing job IN #ca7_jobs_new AND CREATE them IN bulk
  INSERT INTO #jobs_new (name, parent_cid, parent_id, sched_id)
    SELECT DISTINCT name, parent_cid, parent_id, sched_id 
    FROM Disc_jobs 
    WHERE bulk_load = @useBCPTable AND data_source = @data_source
  -- allocate new objects
  DECLARE @base_id ObjID, @rows INT
  SELECT @rows = COUNT(*) FROM #jobs_new
  IF @rows > 0
  BEGIN
    EXEC @base_id = alloc_BATC @rows
    -- INSERT INTO _C
    INSERT BATCH_C (id, name, _ParentID, _ParentCID, _SCHED_PACK, userid)
      SELECT id+@base_id, name, parent_id, parent_cid, sched_id, @data_source
      FROM #jobs_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBATC
      RETURN -30000
    END

    -- INSERT links
    INSERT INTO BTCY_BATC_PHYC (src_id, dst_id)
    SELECT parent_id, id + @base_id
    FROM #jobs_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBATC
      RETURN -30000
    END
  END
  -- finish up
  IF @tcount = 0
    COMMIT TRAN asisp_bulkcreateBATC

  RETURN @rows

  SET NOCOUNT OFF

GO
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_cleanupBJAuditEvent
PRINT 'Creating procedure asisp_cleanupBJAuditEvent....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_cleanupBJAuditEvent' AND type = 'P')
  DROP PROCEDURE asisp_cleanupBJAuditEvent
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7Update_1.1.1.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 27 2001 15:55:14  $
-- $Author:   Xiaoyan  $
-- Creates the BTCY objects FOR the CA7 plan AS loaded IN the ca7_bcp TABLE
CREATE PROCEDURE asisp_cleanupBJAuditEvent
  @keep_days INT = 7
AS
  
  RAISERROR('Excuting asisp_cleanupBJAuditEvent @keep_days = %d.', 0, 1, @keep_days)
  DECLARE @end_time DATETIME, @counter INT
  SELECT @end_time = DATEADD(dd, -7, getdate()) 
  SELECT @counter = COUNT(*) FROM ca7_audit_events WHERE ctime < @end_time
  WHILE @counter > 200000
  BEGIN
    SET ROWCOUNT 200000
    DELETE ca7_audit_events WHERE ctime < @end_time
    SELECT @counter = COUNT(*) FROM ca7_audit_events WHERE ctime < @end_time
  END

  SET ROWCOUNT 0
  DELETE ca7_audit_events WHERE ctime < @end_time

GO
-----------------------------------------------------------------------------------------------------
-- update scheduled tasks
USE msdb
GO

PRINT 'Update "CA7 Discovery Load" scheduled task....'
GO

IF EXISTS(SELECT * FROM msdb..sysjobs WHERE name = 'CA7 Discovery Load')
  EXEC sp_update_jobstep @job_name = 'CA7 Discovery Load', 
    @step_id = 1, 
    @command = '_LoadJobDiscoveryBatch 11, ''sh ca7.parse.ksh -d'', ''ca7_bcp'', ''.ca7.bcp''',
    @database_name = 'Object'
GO

-----------------------------------------------------------------------------------------------------
-- 
PRINT 'Creating scheduled task for CA7 cleanup....'
GO

IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = 'CA7 Cleanup Audit Events')
BEGIN
  BEGIN TRANSACTION            
    DECLARE @JobID BINARY(16)  
    DECLARE @ReturnCode INT, @server_name sysname
    SELECT @server_name = srvname FROM master..sysservers WHERE srvid = 0
    SELECT @ReturnCode = 0     
    EXECUTE @ReturnCode = msdb.dbo.sp_add_job @job_id = @JobID OUTPUT , @job_name = N'CA7 Cleanup Audit Events', @enabled = 1, @owner_login_name = N'sa', @description = N'', @category_name = N'[Uncategorized (Local)]', @notify_level_eventlog = 2, @delete_level= 0
    EXECUTE @ReturnCode = msdb.dbo.sp_add_jobstep @job_id = @JobID, @step_id = 1, @step_name = N'one', @command = N'asisp_cleanupBJAuditEvent ', @database_name = N'Object', @server = @server_name, @database_user_name = N'sa', @subsystem = N'TSQL', @cmdexec_success_code = 0, @flags = 0, @retry_attempts = 0, @retry_interval = 1, @output_file_name = N'', @on_success_step_id = 0, @on_success_action = 1, @on_fail_step_id = 0, @on_fail_action = 2
    EXECUTE @ReturnCode = msdb.dbo.sp_update_job @job_id = @JobID, @start_step_id = 1 
    EXECUTE @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id = @JobID, @name = N'one', @enabled = 1, @freq_type = 4, @active_start_date = 20010826, @active_start_time = 230000, @freq_interval = 1, @freq_subday_type = 1, @freq_subday_interval = 0, @freq_relative_interval = 0, @freq_recurrence_factor = 1, @active_end_date = 99991231, @active_end_time = 235959
    EXECUTE @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @JobID, @server_name = @server_name, @automatic_post = 0 
    SELECT @JobID 
  COMMIT TRANSACTION          
END
GO