--
-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/EventHistory.PART.03.sqv   1.1   Jun 20 2001 12:54:06   lin  $ 
--

SET NOCOUNT ON
GO

-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* in master database *******************************'
GO

USE master
GO

IF NOT EXISTS (SELECT * FROM master..sysdatabases WHERE name='Object')
BEGIN
  RAISERROR('*******************************************', 0, 11)
  RAISERROR('ERROR: Missing Required database: Object', 0, 11)
  RAISERROR('*******************************************', 0, 11)
END
GO

-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* IN Object database *******************************'
GO

USE Object
GO

------------------------------------------------------------------------------------------------------------------------------
-- tI_ExceptionHistory_A
PRINT 'Creating trigger tI_ExceptionHistory_A...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='TR' AND name='tI_ExceptionHistory_A')
  DROP TRIGGER tI_ExceptionHistory_A
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/EventHistory.PART.03.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jun 20 2001 12:54:06  $
-- $Author:   lin  $
CREATE TRIGGER tI_ExceptionHistory_A ON ExceptionHistory_A FOR INSERT
AS
  --First INSERT FROM exceptions with valid OS IN _SEQ_NO using os as agent
  INSERT EventHistory..event_table(agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state)
  SELECT
    'OS', os.id, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCH',
    o.name, a._TIME, a._EXCP_CD, ISNULL(e._AlertState,0), ISNULL(e._Priority,0), 0
  FROM inserted a, ExceptionHistory_C o, Exception_V e, OS_C os
  WHERE
    a.id=o.id and e.id=a._ParentID and os.name=substring(a._SEQ_NO,1,4)

  --Next INSERT FROM exceptions without valid os IN _SEQ_NO using parent of history as agent
  INSERT EventHistory..event_table(agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state)
  SELECT
    e._ParentCID, e._ParentID, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCH',
    o.name, a._TIME, a._EXCP_CD, ISNULL(e._AlertState,0), ISNULL(e._Priority,0), 0
  FROM inserted a, ExceptionHistory_C o, Exception_V e
  WHERE
    a.id=o.id and e.id=a._ParentID and a.id not IN (SELECT a.id FROM inserted a, OS_C os WHERE substring(a._SEQ_NO,1,4)=os.name)

  --Uncommenting these will enable DELETE of history
  --DELETE FROM ExceptionHistory_A WHERE id IN (SELECT id FROM inserted)
  --DELETE FROM ExceptionHistory_C WHERE id IN (SELECT id FROM inserted)
GO


------------------------------------------------------------------------------------------------------------------------------
-- tI_Message_A
PRINT 'Creating trigger tI_Message_A...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='TR' AND name='tI_Message_A')
  DROP TRIGGER tI_Message_A
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/EventHistory.PART.03.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jun 20 2001 12:54:06  $
-- $Author:   lin  $
CREATE TRIGGER tI_Message_A ON Message_A FOR INSERT
AS
  --First INSERT FROM messages with valid OS IN _SEQ_NO using os as agent
  INSERT EventHistory..event_table(agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state)
  SELECT
    'OS', os.id, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG',
    a._MESSAGE, a._TIME, a._REASON_CD, ISNULL(a._AlertState,0), ISNULL(a._Priority,0), ISNULL(a._StateID,0)
  FROM inserted a, OS_C os
  WHERE
    os.name=substring(a._SEQ_NO,1,4)

  --Next INSERT FROM messages without valid os IN _SEQ_NO using parent of message as agent
  INSERT EventHistory..event_table(agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state)
  SELECT
    a._ParentCID, a._ParentID, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG',
    a._MESSAGE, a._TIME, a._REASON_CD, ISNULL(a._AlertState,0), ISNULL(a._Priority,0), ISNULL(a._StateID,0)
  FROM inserted a
  WHERE
    a.id not IN (SELECT a.id FROM inserted a, OS_C os WHERE substring(a._SEQ_NO,1,4)=os.name)

  --Uncommenting this will enable DELETE of message
  --DELETE FROM Message_C WHERE id IN (SELECT id FROM inserted)
GO

------------------------------------------------------------------------------------------------------------------------------
-- tI_ChildEventHistory_A
PRINT 'Creating trigger tI_ChildEventHistory_A...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='TR' AND name='tI_ChildEventHistory_A')
  DROP TRIGGER tI_ChildEventHistory_A
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/EventHistory.PART.03.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jun 20 2001 12:54:06  $
-- $Author:   lin  $
CREATE TRIGGER tI_ChildEventHistory_A ON ChildEventHistory_A FOR INSERT
AS
  --Insert FROM child events using parent as agent
  INSERT EventHistory..event_table(agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state)
  SELECT
    c._ParentCID, c._ParentID, o.ctime, '', c._ParentCID, c._ParentID, 'CHVH',
    o.name, o.ctime, a._EventDetail, ISNULL(c._AlertState,0), ISNULL(c._Priority,0), 0
  FROM inserted a, ChildEventHistory_C o, ChildEvent_V c
  WHERE
    a.id=o.id and c.id=a._ParentID

  --Uncommenting these will enable DELETE of history
  --DELETE FROM ChildEventHistory_A WHERE id IN (SELECT id FROM inserted)
  --DELETE FROM ChildEventHistory_C WHERE id IN (SELECT id FROM inserted)
GO

----------------------------------------------------------------------------------
PRINT 'Done.'
GO
