-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/IMSDiscovery.sqv   1.9   Sep 07 2001 16:24:04   Forthman  $/Archive/DB/SQL/IMSDiscovery.sqv   1.1   Feb 27 2001 07:14:42   BRENT  $
--
-- IMSDiscovery.sql
--
-- Discovery procedures and supporting tables for IMS

USE Object
GO
SET NOCOUNT ON
GO

-- Drop old version of IMSDiscovery table if it exists
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'IMSDiscovery' AND type = 'U')
  AND NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('IMSDiscovery') AND name = 'IMS_SUBSYSTEM')
BEGIN
  PRINT 'Dropping old IMSDiscovery table'
  EXEC("
  DROP TABLE IMSDiscovery
  ")
END


-- Table IMSDiscovery
-- This table receives bulk-loaded discovery events for IMS resources
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'IMSDiscovery' AND type = 'U')
BEGIN
  PRINT 'Creating table IMSDiscovery'
  EXEC("
  CREATE TABLE IMSDiscovery
  (
    batchid                       INT,
    SystemId                      VARCHAR(40) NULL,
    obj_cid                       ClassID,
    IMS_SUBSYSTEM                 VARCHAR(40),
    IMS_SUBCOMPONENT              VARCHAR(40) NULL,
    IMS_SUBPROGRAM                VARCHAR(40) NULL,
    IMS_AREA                      VARCHAR(40) NULL,
    os_id                         ObjID NULL          CONSTRAINT IMSDiscovery_os_id_DF DEFAULT NULL,
    imsu_cid                      ClassID NULL        CONSTRAINT IMSDiscovery_imsu_cid_DF DEFAULT NULL,
    imsu_id                       ObjID NULL          CONSTRAINT IMSDiscovery_imsu_id_DF DEFAULT NULL,
    imsc_link_type                ClassID NULL        CONSTRAINT IMSDiscovery_imsc_link_type_DF DEFAULT NULL,
    imsc_cid                      ClassID NULL        CONSTRAINT IMSDiscovery_imsc_cid_DF DEFAULT NULL,
    imsc_id                       ObjID NULL          CONSTRAINT IMSDiscovery_imsc_id_DF DEFAULT NULL,
    imsp_cid                      ClassID NULL        CONSTRAINT IMSDiscovery_imsp_cid_DF DEFAULT NULL,
    imsp_id                       ObjID NULL          CONSTRAINT IMSDiscovery_imsp_id_DF DEFAULT NULL,
    imar_id                       ObjID NULL          CONSTRAINT IMSDiscovery_imar_id_DF DEFAULT NULL,
    obj_name                      ObjName NULL        CONSTRAINT IMSDiscovery_obj_name_DF DEFAULT NULL,
    obj_id                        ObjID NULL          CONSTRAINT IMSDiscovery_obj_id_DF DEFAULT NULL,
    obj_path                      VARCHAR(500) NULL   CONSTRAINT IMSDiscovery_obj_path_DF DEFAULT NULL,
    rediscovered                  TINYINT             CONSTRAINT IMSDiscovery_rediscovered_DF DEFAULT 0,
    DATE_TIME                     DATETIME NULL,
    SEQ_NO                        VARCHAR(40) NULL,
    MVS                           VARCHAR(40) NULL,
    DD                            VARCHAR(40) NULL,
    DSN                           VARCHAR(40) NULL,
    ASSOCMVS                      VARCHAR(40) NULL,
    ASSOCIATE                     VARCHAR(40) NULL,
    STATE                         VARCHAR(40) NULL,
    Event                         VARCHAR(40) NULL,
    EventText                     VARCHAR(255) NULL
  )
  ALTER TABLE IMSDiscovery ADD CONSTRAINT IMSDiscovery_batchid_FK FOREIGN KEY (batchid) REFERENCES Object.dbo.DiscoveryBatch (batchid)
  ALTER TABLE IMSDiscovery ADD CONSTRAINT 
    IMSDiscovery_instance_UQ UNIQUE (batchid, SystemId, obj_cid, IMS_SUBSYSTEM, IMS_SUBCOMPONENT, IMS_SUBPROGRAM, IMS_AREA)
  ")
END
GO


-- Drop old version of IMSDiscovery table if it exists
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ProcessedIMSDiscovery' AND type = 'U')
  AND NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('ProcessedIMSDiscovery') AND name = 'IMS_SUBSYSTEM')
BEGIN
  PRINT 'Dropping old ProcessedIMSDiscovery table'
  EXEC("
  DROP TABLE ProcessedIMSDiscovery
  ")
END

-- Table ProcessedIMSDiscovery
-- This table stores processed discovery records for IMS resources
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ProcessedIMSDiscovery' AND type = 'U')
BEGIN
  PRINT 'Creating table ProcessedIMSDiscovery'
  EXEC("
  CREATE TABLE ProcessedIMSDiscovery
  (
    ptime                         DATETIME,
    batchid                       INT,
    SystemId                      VARCHAR(40) NULL,
    obj_cid                       ClassID,
    IMS_SUBSYSTEM                 VARCHAR(40),
    IMS_SUBCOMPONENT              VARCHAR(40) NULL,
    IMS_SUBPROGRAM                VARCHAR(40) NULL,
    IMS_AREA                      VARCHAR(40) NULL,
    os_id                         ObjID NULL,
    imsu_cid                      ClassID NULL,
    imsu_id                       ObjID NULL,
    imsc_link_type                ClassID NULL,
    imsc_cid                      ClassID NULL,
    imsc_id                       ObjID NULL,
    imsp_cid                      ClassID NULL,
    imsp_id                       ObjID NULL,
    imar_id                       ObjID NULL,
    obj_name                      ObjName NULL,
    obj_id                        ObjID NULL,
    obj_path                      VARCHAR(500) NULL,
    rediscovered                  TINYINT,
    DATE_TIME                     DATETIME NULL,
    SEQ_NO                        VARCHAR(40) NULL,
    MVS                           VARCHAR(40) NULL,
    DD                            VARCHAR(40) NULL,
    DSN                           VARCHAR(40) NULL,
    ASSOCMVS                      VARCHAR(40) NULL,
    ASSOCIATE                     VARCHAR(40) NULL,
    STATE                         VARCHAR(40) NULL,
    Event                         VARCHAR(40) NULL,
    EventText                     VARCHAR(255) NULL
  )
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- asisp_recreatePartitionedClassView
PRINT 'Creating procedure asisp_recreatePartitionedClassView'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_recreatePartitionedClassView' AND type = 'P')
  DROP PROCEDURE asisp_recreatePartitionedClassView
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSDiscovery.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.9  $
-- $Date:   Sep 07 2001 16:24:04  $
-- $Author:   Forthman  $
-- (Re)creates a partitioned view of all non-abstract derived classed of the specified class.
-- The view is comprised of selecting the base columns from the class tables for each class derived
-- from the specified @base_cid. Also makes sure that each of tables has the cid column which has
-- been set to have both a DEFAULT and CHECK constraint that sets it to the cid of the class.
-- This allows for queries on the view to be efficiently handled by the query optimizer.
-- Specifically, if you do a select of the following form:
--   SELECT * FROM <view> WHERE cid IN (<cid1>, <cid2>)
-- then only the class tables for <cid1> and <cid2> will be touched even if the view spans more
-- tables -- pretty cool, eh? This is called a "partitioned view" -- for more information see the 
-- article "Using Views with Partitioned Data" in SQL Books Online.
CREATE PROCEDURE asisp_recreatePartitionedClassView
  @base_cid           ClassID,
  @additional_columns VARCHAR(255) = NULL
AS
  DECLARE @flag_abstract TINYINT
  SELECT @flag_abstract = flag_abstract FROM obj_class WHERE cid = @base_cid
  IF @@ROWCOUNT = 0
  BEGIN
    PRINT '"' + @base_cid + '" is not a valid class id'
    RETURN -30000
  END

  IF @flag_abstract = 0
  BEGIN
    PRINT '"' + @base_cid + '" is not an abstract class -- cannot create the partitioned view.'
    RETURN -30001
  END

  -- Make sure the additional columns, if specified, are valid
  DECLARE @additional_column_select VARCHAR(255), @column VARCHAR(30), @i INT
  SET @additional_column_select = ''
  WHILE DATALENGTH(@additional_columns) > 0
  BEGIN
    -- Extract column name
    SET @i = CHARINDEX(',', @additional_columns)
    IF @i = 0
    BEGIN
      SET @column = @additional_columns
      SET @additional_columns = NULL
    END
    ELSE
    BEGIN
      SET @column = SUBSTRING(@additional_columns, 1, @i-1)
      SET @additional_columns = SUBSTRING(@additional_columns, DATALENGTH(@column)+2, DATALENGTH(@additional_columns)-DATALENGTH(@column)-1)
    END

    SET @additional_column_select = @additional_column_select + ', c.' + @column
  END

  DECLARE @sql VARCHAR(8000)
  DECLARE classc INSENSITIVE CURSOR FOR
    SELECT o.cid, o.cname, o.flag_consolidated
    FROM isa_chain AS i JOIN obj_class AS o ON o.cid = i.cid
    WHERE i.base_cid = @base_cid AND o.flag_abstract = 0
  OPEN classc
  DECLARE @cid ClassID, @cname VARCHAR(24), @consolidated TINYINT, @n INT
  SET @sql = ''
  SET @n = 0
  FETCH NEXT FROM classc INTO @cid, @cname, @consolidated
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @n > 0
      SELECT @sql = @sql + '
  UNION
'
    SET @n = @n + 1
    SELECT @sql = @sql + '  SELECT c.cid, c.id, c.name, c.ctime, c.mtime, c.deleted, c._AlertStateID, c._PriorityID, c._CurrentStateID, c._DesiredStateID' + @additional_column_select + ' FROM ' + @cname + '_C AS c'

    -- Make sure the table has the cid column
    IF NOT EXISTS 
    (
      SELECT * FROM syscolumns AS c JOIN sysobjects AS o ON o.id = c.id
      WHERE o.name = @cname + '_C' AND o.type = 'U' and c.name = 'cid'
    )
    BEGIN
      PRINT 'Adding cid column to ' + @cname + '_C'
      EXEC("
      ALTER TABLE " + @cname + "_C ADD cid ClassID  DEFAULT '" + @cid + "' CHECK (cid = '" + @cid + "')
      ")
      EXEC("
      UPDATE " + @cname + "_C SET cid = '" + @cid + "'
      ")
    END

    FETCH NEXT FROM classc INTO @cid, @cname, @consolidated
  END
  DEALLOCATE classc

  DECLARE @base_cname VARCHAR(24)
  SELECT @base_cname = cname FROM obj_class WHERE cid = @base_cid

  -- Drop previous view
  IF EXISTS (SELECT * FROM sysobjects WHERE name = @base_cname + '_C' AND type = 'V')
    EXEC("
    DROP VIEW " + @base_cname + "_C
    ")

  -- Recreate view
  PRINT @base_cname + '_C select text is ' + CAST(len(@sql) AS VARCHAR(10)) + ' characters long'
  EXEC("
  -- Partitioned view for " + @base_cname + " class
  CREATE VIEW " + @base_cname + "_C AS
" + @sql)
GO

EXEC asisp_recreatePartitionedClassView @base_cid = 'IMBA', @additional_columns = '_DDName,_Dataset,_Path,description,_HasOwnershipNote'
EXEC asisp_recreatePartitionedClassView @base_cid = 'IMAS'
EXEC asisp_recreatePartitionedClassView @base_cid = 'IMSU'
EXEC asisp_recreatePartitionedClassView @base_cid = 'IMDR'
EXEC asisp_recreatePartitionedClassView @base_cid = 'IMSC'
EXEC asisp_recreatePartitionedClassView @base_cid = 'IMSP', @additional_columns = '_ParentCID,_ParentID'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'IMSSubComponent_Rediscovery' AND type = 'V')
  DROP VIEW IMSSubComponent_Rediscovery
GO

CREATE VIEW IMSSubComponent_Rediscovery AS
  SELECT cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM IMSSubComponent_C 
  UNION
  SELECT cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM IMSBMP_C
GO

---------------------------------------------------------------------------------------------------
-- _LoadIMSDiscoveryBatch
PRINT 'Creating procedure _LoadIMSDiscoveryBatch'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadIMSDiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadIMSDiscoveryBatch
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSDiscovery.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.9  $
-- $Date:   Sep 07 2001 16:24:04  $
-- $Author:   Forthman  $
-- Loads a IMSDiscovery batch from file path for the specified batch. 
-- The file is in native format -- see the definition of the #discovery temp table in the body of 
-- the procedure for the required columns.
CREATE PROCEDURE _LoadIMSDiscoveryBatch
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL
AS
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = MIN(batchid)
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      b.FormatType = 18
      AND b.ActionType = 1
      AND v.name = 'ENQUEUED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No IMS DiscoveryBatch is in a ENQUEUED state, nothing to do!'
      RETURN 0
    END
  END
  ELSE
    UPDATE DiscoveryBatch SET pstat = v.pstat
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'ENQUEUED'

  -- Create temp table for holding bulk-loaded results
  CREATE TABLE #discovery
  (
    FormatType        SMALLINT,
    ActionType        SMALLINT,
    DATE_TIME         VARCHAR(40) NULL, --  1
    SEQ_NO            VARCHAR(40) NULL, --  2
    OBJ_TYPE_ID       VARCHAR(40) NULL, --  3
    EventSource       VARCHAR(40) NULL, --  4
    SystemId          VARCHAR(40) NULL, --  5
    MVS               VARCHAR(40) NULL, --  6
    IMS_TMDB          VARCHAR(40) NULL, --  7
    IMS_DCCTL         VARCHAR(40) NULL, --  8
    IMS_DBCTL         VARCHAR(40) NULL, --  9
    IMS_IRLM          VARCHAR(40) NULL, -- 10
    IMS_CQS           VARCHAR(40) NULL, -- 11
    IMS_FDR           VARCHAR(40) NULL, -- 12
    IMS_ITOC          VARCHAR(40) NULL, -- 13
    IMS_MPR           VARCHAR(40) NULL, -- 14
    IMS_FPM           VARCHAR(40) NULL, -- 15
    IMS_BMP           VARCHAR(40) NULL, -- 16
    IMS_LOGS          VARCHAR(40) NULL, -- 17
    IMS_TRANSACTION   VARCHAR(40) NULL, -- 18
    IMS_PROGRAM       VARCHAR(40) NULL, -- 19
    IMS_DATABASE      VARCHAR(40) NULL, -- 20
    IMS_AREA          VARCHAR(40) NULL, -- 21
    IMS_ESAF_DB2      VARCHAR(40) NULL, -- 22
    IMS_ESAF_MQS      VARCHAR(40) NULL, -- 23
    DD                VARCHAR(40) NULL, -- 24
    DSN               VARCHAR(40) NULL, -- 25
    ASSOCMVS          VARCHAR(40) NULL, -- 26
    ASSOCIATE         VARCHAR(40) NULL, -- 27
    STATE             VARCHAR(40) NULL, -- 28
    Event             VARCHAR(40) NULL, -- 29
    EventText         VARCHAR(40) NULL  -- 30
  )

  -- Lookup file and codepage of the batch
  DECLARE @FilePath NVARCHAR(500), @CodePage VARCHAR(8)
  SELECT @FilePath = FilePath, @CodePage = CodePage FROM DiscoveryBatch WHERE batchid = @batchid
  IF @@ROWCOUNT = 0
  BEGIN
    RAISERROR('DiscoveryBatch id %d is not valid', 3, 1, @batchid)
    RETURN -30000
  END

  DECLARE @err INT, @rc INT

  UPDATE DiscoveryBatch SET pstat = v.pstat
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
  -- Perform bulk load
  DECLARE @sql VARCHAR(500)
  SELECT @sql ="
  BULK INSERT #discovery FROM '" + @FilePath + "'
  WITH
  (
    BATCHSIZE = 1000,
    CODEPAGE = '" + @CodePage + "',
    DATAFILETYPE = 'native'
  )"
  PRINT @sql
  EXEC(@sql)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %s', 3, 1, @FilePath)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @FilePath

  -- We are getting rows which are duplicates on everything but the DATE_TIME
  -- Workaround: Update the DATE_TIME column for the given batch to the MAX(DATE_TIME) value
  -- from the batch
  UPDATE #discovery SET DATE_TIME = (SELECT MAX(DATE_TIME) FROM #discovery)

  -- Insert the records from temp table into IMSDiscovery
  INSERT INTO IMSDiscovery
  (
    batchid,
    SystemId,
    obj_cid,
    IMS_SUBSYSTEM,
    IMS_SUBCOMPONENT,
    IMS_SUBPROGRAM,
    IMS_AREA,
    imsu_cid,
    imsc_cid,
    imsp_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    MVS,
    DD,
    DSN,
    ASSOCMVS,
    ASSOCIATE,
    STATE,
    Event,
    EventText
  )
  SELECT DISTINCT
    @batchid,
    SystemId,
    OBJ_TYPE_ID AS obj_cid,
    CASE
      WHEN IMS_TMDB IS NOT NULL THEN IMS_TMDB
      WHEN IMS_DCCTL IS NOT NULL THEN IMS_DCCTL
      WHEN IMS_DBCTL IS NOT NULL THEN IMS_DBCTL
    END AS IMS_SUBSYSTEM,
    CASE
      WHEN IMS_IRLM IS NOT NULL THEN IMS_IRLM
      WHEN IMS_CQS IS NOT NULL THEN IMS_CQS
      WHEN IMS_FDR IS NOT NULL THEN IMS_FDR
      WHEN IMS_ITOC IS NOT NULL THEN IMS_ITOC
      WHEN IMS_MPR IS NOT NULL THEN IMS_MPR
      WHEN IMS_FPM IS NOT NULL THEN IMS_FPM
      WHEN IMS_BMP IS NOT NULL THEN IMS_BMP
      WHEN IMS_LOGS IS NOT NULL THEN IMS_LOGS
      WHEN IMS_ESAF_DB2 IS NOT NULL THEN IMS_ESAF_DB2
      WHEN IMS_ESAF_MQS IS NOT NULL THEN IMS_ESAF_MQS
      WHEN IMS_PROGRAM IS NOT NULL THEN IMS_PROGRAM
    END AS IMS_SUBCOMPONENT,
    CASE 
      WHEN IMS_TRANSACTION IS NOT NULL THEN IMS_TRANSACTION
      WHEN IMS_DATABASE IS NOT NULL THEN IMS_DATABASE
    END AS IMS_SUBPROGRAM,
    IMS_AREA,
    CASE
      WHEN IMS_TMDB IS NOT NULL THEN 'IMSS'
      WHEN IMS_DCCTL IS NOT NULL THEN 'IMTM'
      WHEN IMS_DBCTL IS NOT NULL THEN 'IMDB'
    END AS imsu_cid,
    CASE
      WHEN IMS_IRLM IS NOT NULL THEN 'IMIR'
      WHEN IMS_CQS IS NOT NULL THEN 'IMQA'
      WHEN IMS_FDR IS NOT NULL THEN 'IMFD'
      WHEN IMS_ITOC IS NOT NULL THEN 'IMIT'
      WHEN IMS_MPR IS NOT NULL THEN 'IMMP'
      WHEN IMS_FPM IS NOT NULL THEN 'IMFP'
      WHEN IMS_BMP IS NOT NULL THEN 'IMBM'
      WHEN IMS_LOGS IS NOT NULL THEN 'IMLG'
      WHEN IMS_ESAF_DB2 IS NOT NULL THEN 'IMD2'
      WHEN IMS_ESAF_MQS IS NOT NULL THEN 'IMMQ'
      WHEN IMS_PROGRAM IS NOT NULL THEN 'IMPG'
    END AS imsc_cid,
    CASE
      WHEN IMS_TRANSACTION IS NOT NULL THEN 'IMTX'
      WHEN IMS_DATABASE IS NOT NULL THEN 'IMDA'
    END AS imsp_cid,
    CASE OBJ_TYPE_ID
      WHEN 'IMSS' THEN IMS_TMDB
      WHEN 'IMTM' THEN IMS_DCCTL
      WHEN 'IMDB' THEN IMS_DBCTL
      WHEN 'IMIR' THEN IMS_IRLM
      WHEN 'IMCQ' THEN IMS_CQS
      WHEN 'IMFD' THEN IMS_FDR
      WHEN 'IMIT' THEN IMS_ITOC
      WHEN 'IMMP' THEN IMS_MPR
      WHEN 'IMFP' THEN IMS_FPM
      WHEN 'IMBM' THEN IMS_BMP
      WHEN 'IMLG' THEN IMS_LOGS
      WHEN 'IMD2' THEN IMS_ESAF_DB2
      WHEN 'IMMQ' THEN IMS_ESAF_MQS
      WHEN 'IMPG' THEN IMS_PROGRAM
      WHEN 'IMTX' THEN IMS_TRANSACTION
      WHEN 'IMDA' THEN IMS_DATABASE
      WHEN 'IMAR' THEN IMS_AREA
    END AS obj_name,
    SUBSTRING(DATE_TIME,1,4)+'-'+SUBSTRING(DATE_TIME,5,2)+'-'+SUBSTRING(DATE_TIME,7,2)+' '+SUBSTRING(DATE_TIME,9,12),
    SEQ_NO,
    MVS, 
    DD,
    DSN,
    ASSOCMVS,
    ASSOCIATE,
    STATE,
    Event,
    EventText
  FROM #discovery
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @@ERROR <> 0
  BEGIN
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END

  UPDATE DiscoveryBatch SET pstat = v.pstat, numrows = @rc
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'LOADED'
GO

-- Drop obsolete procedures

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverIMSResourcesIMSU' AND type = 'P')
  DROP PROCEDURE _DiscoverIMSResourcesIMSU
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverIMSResourcesIMSC' AND type = 'P')
  DROP PROCEDURE _DiscoverIMSResourcesIMSC
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverIMSResourcesIMSP' AND type = 'P')
  DROP PROCEDURE _DiscoverIMSResourcesIMSP
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverIMSResourcesIMAR' AND type = 'P')
  DROP PROCEDURE _DiscoverIMSResourcesIMAR
GO


---------------------------------------------------------------------------------------------------
-- _DiscoverIMSResources
PRINT 'Creating procedure _DiscoverIMSResources'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverIMSResources' AND type = 'P')
  DROP PROCEDURE _DiscoverIMSResources
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSDiscovery.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.9  $
-- $Date:   Sep 07 2001 16:24:04  $
-- $Author:   Forthman  $
CREATE PROCEDURE _DiscoverIMSResources
  @cid          ClassID,
  @id           ObjID,
  @batchid      int = NULL,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  DECLARE @err INT, @rc INT

  SET NOCOUNT ON

  RAISERROR( '_DiscoverIMSResources @cid = ''%s'', @id = %d, @batchid = %d, @incremental = %d , @verbose = %d', 0, 1, @cid, @id, @batchid, @incremental, @verbose)

  -- If the batchid was not specified then pick the oldest one that is in a LOADED state
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = min(batchid) 
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      FormatType = 18
--      AND ActionType = 1
      AND ( ActionType = 1 OR ActionType = 12) -- HACK ActionType 12, remove 12 after fixing data upstream
      AND v.name = 'LOADED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No IMS DiscoveryBatch is in a LOADED state, nothing to do!'
      RETURN 0
    END
  END

  -- Garbage collection if not running in incremental mode
--  IF @incremental = 0
--  BEGIN
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Starting garbage collection'
    DECLARE linkc CURSOR FOR
      SELECT dol.link_type, RTRIM(dol.src_cid), RTRIM(dol.dst_cid), dst.cname
      FROM derived_obj_link AS dol
        JOIN obj_class AS src ON src.cid = dol.src_cid
        JOIN obj_class AS dst ON dst.cid = dol.dst_cid
        JOIN isa_chain AS dic ON dic.cid = dol.dst_cid
        JOIN link_type AS lt ON lt.link_type = dol.link_type
      WHERE
        src.flag_abstract = 0
        AND dst.flag_abstract = 0
        AND dic.base_cid = 'IMBA'
        AND lt.flag_containment = 1
    OPEN linkc
    DECLARE @link_type VARCHAR(4), @src_cid VARCHAR(4), @dst_cid VARCHAR(4), @dst_cname VARCHAR(24)
    FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Delete link records
      DECLARE @table sysname
      SET @table = @src_cid + '_' + @dst_cid + '_' + @link_type
      IF @verbose = 1
        PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on ' + @table
      EXEC("
      DELETE FROM il FROM " + @table + " AS il JOIN link AS l ON
        l.link_type = '" + @link_type + "'
        AND l.src_cid = '" + @src_cid + "'
        AND l.src_id = il.src_id
        AND l.dst_cid = '" + @dst_cid + "'
        AND l.dst_id = il.dst_id
      WHERE
        l.deleted = 1
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)
    
      -- Delete instance records
      IF @verbose = 1
        PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on ' + @dst_cname + '_C'
      EXEC("
      DELETE FROM " + @dst_cname + "_C WHERE id NOT IN (SELECT l.dst_id FROM link AS l WHERE l.dst_cid='" + @dst_cid + "' AND l.deleted=0)
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)

      FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    END
    DEALLOCATE linkc

    -- Garbage-collect the directly contained instances.
   
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on IMSProgram_C'
    DELETE FROM child FROM IMSProgram_C AS child WHERE NOT EXISTS
    (
      SELECT * FROM IMSSubsystem_C AS parent WHERE parent.cid = child._ParentCID AND parent.id = child._ParentID
    )
    SET @rc = @@ROWCOUNT
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)

    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on IMSDatabase_C'
    DELETE FROM child FROM IMSDatabase_C AS child WHERE NOT EXISTS
    (
      SELECT * FROM IMSProgram_C AS parent WHERE parent.cid = child._ParentCID AND parent.id = child._ParentID
    )
    SET @rc = @@ROWCOUNT
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)

    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on IMSTransaction_C'
    DELETE FROM child FROM IMSTransaction_C AS child WHERE NOT EXISTS
    (
      SELECT * FROM IMSProgram_C AS parent WHERE parent.cid = child._ParentCID AND parent.id = child._ParentID
    )
    SET @rc = @@ROWCOUNT
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)

    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Doing garbage collection on IMSArea_C'
    DELETE FROM child FROM IMSArea_C AS child WHERE NOT EXISTS
    (
      SELECT * FROM IMSDatabase_C AS parent WHERE parent.cid = child._ParentCID AND parent.id = child._ParentID
    )
    SET @rc = @@ROWCOUNT
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS VARCHAR)
--  END -- end of garbage collection

  -- Take care of any mismatches between discovery data obj_cid and obj_class.cid
  UPDATE IMSDiscovery SET obj_cid = 'IMQA' WHERE obj_cid = 'IMCQ'

  -- Rediscover IMSSubsystem objects
  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Rediscovering IMSSubsystem objects'
  UPDATE child SET
    os_id = p.id
  FROM
    IMSDiscovery AS child
    JOIN OS_V AS p ON p._SystemId = child.SystemId

  -- If we could not map any of the records from the batch to an operating system then
  -- there's something wrong.
  IF NOT EXISTS
  (
    SELECT * FROM IMSDiscovery WHERE batchid = @batchid AND os_id IS NOT NULL
  )
  BEGIN
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: batch ' + CAST(@batchid AS VARCHAR) + ' cannot be resolved to any operating system. The SystemId field of the discovery records does not match any known operating systems.'
    RETURN -30000
  END

  -- Rediscover Subsystems
  UPDATE child SET
    imsu_id = obj.id,
    rediscovered = 1
  FROM
    IMSDiscovery AS child
    JOIN IMSSubsystem_C AS obj ON          -- Find any existing IMSSubsystem objects by class,name
      obj.cid = child.imsu_cid
      AND obj.name = child.IMS_SUBSYSTEM
    JOIN link AS pc ON                     -- Lookup for parent links of subsystems to make sure
      pc.src_cid = 'OS'                    -- we're under the right operating system
      AND pc.src_id = child.os_id 
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
    JOIN isa_chain AS pic ON           -- Hook in base classes for parent class
      pic.cid = child.obj_cid
  WHERE
    child.batchid = @batchid
    AND pic.base_cid = 'IMSU'               -- Parent is derived from IMSSubsystem

  -- Compute the path of all objects. We can do this now because all operating systems have been looked up.
  DECLARE osc CURSOR FOR
    SELECT DISTINCT os_id FROM IMSDiscovery WHERE batchid = @batchid
  OPEN osc
  DECLARE @os_id ObjID, @Path VARCHAR(255)
  FETCH NEXT FROM osc INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_fullpath OS, @os_id, @Path OUTPUT
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Updating paths for objects under ' + @Path
    UPDATE IMSDiscovery SET obj_path = 
      @Path
      + '/' + IMS_SUBSYSTEM
      + CASE WHEN imsc_cid IS NOT NULL THEN '/' + IMS_SUBCOMPONENT END
      + CASE WHEN imsp_cid IS NOT NULL THEN '/' + IMS_SUBPROGRAM END
      + CASE WHEN obj_cid = 'IMAR' THEN '/' + IMS_AREA END
    WHERE
      batchid = @batchid
      AND os_id = @os_id
    FETCH NEXT FROM osc INTO @os_id
  END
  DEALLOCATE osc

  -- Create any required subsystems
  IF EXISTS (
  SELECT * FROM IMSDiscovery AS d
    INNER JOIN isa_chain AS isa ON d.obj_cid = isa.cid
    WHERE batchid = @batchid 
      AND imsu_id IS NULL 
      AND isa.base_cid = 'IMSU'
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'IMSDiscovery',
      @LinkTypeColumn  = '''PHYC''',
      @ParentCIDColumn = '''OS''',
      @ParentIDColumn  = 'os_id',
      @ChildCIDColumn  = 'imsu_cid',
      @ChildIDColumn   = 'imsu_id',
      @ChildNameColumn = 'IMS_SUBSYSTEM',
      @WhereClause     = 'JOIN isa_chain AS ic ON ic.cid = d.obj_cid WHERE ic.base_cid = ''IMSU'' AND d.os_id IS NOT NULL AND d.imsu_id IS NULL',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS VARCHAR) + ' subsystems'
    END
  END
  UPDATE IMSDiscovery SET obj_id = imsu_id WHERE obj_cid = imsu_cid

  -- Update the subsystem ids of all objects
  UPDATE d SET imsu_id = imsud.obj_id
  FROM
    IMSDiscovery AS d
    JOIN IMSDiscovery AS imsud ON
      imsud.obj_cid = d.imsu_cid
      AND imsud.obj_name = d.IMS_SUBSYSTEM
  WHERE
    d.obj_cid NOT IN (SELECT cid FROM isa_chain WHERE base_cid = 'IMSU')
      AND d.batchid = @batchid
      AND imsud.obj_id IS NOT NULL

  IF @verbose = 1
    SELECT 'IMSSubsystem', * FROM IMSDiscovery AS d JOIN isa_chain AS i ON i.cid = d.obj_cid
    WHERE batchid = @batchid AND i.base_cid = 'IMSS'

  -- Rediscover objects that live under the subsystems
  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Rediscovering subcomponents under subsystems'
  UPDATE child SET
    imsc_link_type = dol.link_type
  FROM
    IMSDiscovery AS child
    JOIN derived_obj_link AS dol ON    -- Lookup link_type for relationship
      dol.src_cid = child.imsu_cid
      AND dol.dst_cid = child.obj_cid

  UPDATE child SET
    imsc_id = obj.id,
    rediscovered = 1
  FROM
    IMSDiscovery AS child
    JOIN IMSSubComponent_Rediscovery AS obj ON
      obj.cid = child.imsc_cid
      AND obj.name = child.IMS_SUBCOMPONENT
    JOIN link AS pc ON                 -- Make sure we're identifying the right subcomponent
      pc.src_cid = child.imsu_cid      -- under the right subsystem.
      AND pc.src_id = child.imsu_id
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
      AND pc.link_type = child.imsc_link_type
    JOIN isa_chain AS pic ON           -- Hook in base classes for parent class
      pic.cid = child.imsu_cid
  WHERE
    child.batchid = @batchid
    AND child.IMS_SUBCOMPONENT IS NOT NULL
    AND pic.base_cid = 'IMSU'               -- Parent is derived from IMSSubsystem

  -- Create any required subcomponents
  IF EXISTS (SELECT * FROM IMSDiscovery WHERE batchid = @batchid AND imsu_id IS NOT NULL AND imsc_cid IS NOT NULL AND imsc_cid <> 'IMPG' AND imsc_id IS NULL)
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'IMSDiscovery',
      @LinkTypeColumn  = 'imsc_link_type',
      @ParentCIDColumn = 'imsu_cid',
      @ParentIDColumn  = 'imsu_id',
      @ChildCIDColumn  = 'imsc_cid',
      @ChildIDColumn   = 'imsc_id',
      @ChildNameColumn = 'IMS_SUBCOMPONENT',
      @WhereClause     = 'JOIN isa_chain AS ic ON ic.cid = d.obj_cid WHERE ic.base_cid IN ( ''IMBM'', ''IMSC'' ) AND d.imsu_id IS NOT NULL AND d.imsc_id IS NULL',
      @AdditionalDiscoveryColumns = ',  DD, DSN, obj_path',
      @AdditionalAttributeColumns = ',  _DDName, _Dataset, _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS VARCHAR) + ' subcomponents'
    END
  END

  -- Rediscover programs under the subsystems. These are directly contained and therefore
  -- not handled like the other subcomponents above.
  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Rediscovering programs under subsystems'
  UPDATE child SET
    imsc_id = obj.id,
    rediscovered = 1
  FROM
    IMSDiscovery AS child
    JOIN IMSProgram_C AS obj ON
      obj.cid = child.imsc_cid
      AND obj.name = child.IMS_SUBCOMPONENT
      AND obj._ParentCID = child.imsu_cid
      AND obj._ParentID = child.imsu_id
  WHERE
    child.imsc_cid = 'IMPG'

  -- Create any required programs
  IF EXISTS (SELECT * FROM IMSDiscovery WHERE batchid = @batchid AND imsu_id IS NOT NULL AND imsc_cid = 'IMPG' AND imsc_id IS NULL)
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'IMSDiscovery',
      @LinkTypeColumn  = 'NULL',
      @ParentCIDColumn = 'imsu_cid',
      @ParentIDColumn  = 'imsu_id',
      @ChildCIDColumn  = 'imsc_cid',
      @ChildIDColumn   = 'imsc_id',
      @ChildNameColumn = 'IMS_SUBCOMPONENT',
      @WhereClause     = 'WHERE d.obj_cid = ''IMPG'' AND d.imsu_id IS NOT NULL AND d.imsc_id IS NULL',
      @AdditionalDiscoveryColumns = ', imsu_cid, imsu_id, obj_path',
      @AdditionalAttributeColumns = ', _ParentCID, _ParentID, _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS VARCHAR) + ' programs'
    END
  END
  UPDATE IMSDiscovery SET obj_id = imsc_id WHERE obj_cid = imsc_cid

  -- Rediscover objects that live under programs
  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Rediscovering subprograms'
  UPDATE child SET
    imsp_id = obj.id,
    rediscovered = 1
  FROM
    IMSDiscovery AS child
    JOIN IMSSubProgram_C AS obj ON
      obj.cid = child.imsp_cid
      AND obj.name = child.IMS_SUBPROGRAM
      AND obj._ParentCID = child.imsc_cid
      AND obj._ParentID = child.imsc_id
  WHERE
    child.batchid = @batchid

  -- Create any required subprograms
  IF EXISTS (SELECT * FROM IMSDiscovery WHERE batchid = @batchid AND imsc_cid = 'IMPG' AND imsc_id IS NOT NULL AND imsp_cid IS NOT NULL AND imsp_id IS NULL)
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'IMSDiscovery',
      @LinkTypeColumn  = 'NULL',
      @ParentCIDColumn = '''IMPG''',
      @ParentIDColumn  = 'imsc_id',
      @ChildCIDColumn  = 'imsp_cid',
      @ChildIDColumn   = 'imsp_id',
      @ChildNameColumn = 'IMS_SUBPROGRAM',
      @WhereClause     = 'JOIN isa_chain AS ic ON ic.cid = d.obj_cid WHERE ic.base_cid = ''IMSP'' AND d.imsc_cid = ''IMPG'' AND d.imsc_id IS NOT NULL AND d.imsp_id IS NULL',
      @AdditionalDiscoveryColumns = ', DD, DSN, ''IMPG'', imsc_id, obj_path',
      @AdditionalAttributeColumns = ', _DDName, _Dataset, _ParentCID, _ParentID, _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS VARCHAR) + ' subprograms'
    END
  END
  UPDATE IMSDiscovery SET obj_id = imsp_id WHERE obj_cid = imsp_cid

  -- Rediscover database areas
  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Rediscovering IMSAreas'
  UPDATE child SET
    imar_id = obj.id,
    rediscovered = 1
  FROM
    IMSDiscovery AS child
    JOIN IMSArea_C AS obj ON
      obj.cid = child.obj_cid
      AND obj.name = child.IMS_AREA
      AND obj._ParentCID = child.imsp_cid
      AND obj._ParentID = child.imsp_id
  WHERE
    child.batchid = @batchid
    AND child.obj_cid = 'IMAR'  -- child is IMSArea

  -- Create any required areas
  IF EXISTS (SELECT * FROM IMSDiscovery WHERE batchid = @batchid AND imsp_cid = 'IMDA' AND imsp_id IS NOT NULL AND obj_cid = 'IMAR' AND imar_id IS NULL)
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'IMSDiscovery',
      @LinkTypeColumn  = 'NULL',
      @ParentCIDColumn = '''IMDA''',
      @ParentIDColumn  = 'imsp_id',
      @ChildCIDColumn  = '''IMAR''',
      @ChildIDColumn   = 'imar_id',
      @ChildNameColumn = 'IMS_AREA',
      @WhereClause     = 'WHERE d.obj_cid = ''IMAR'' AND d.imsp_cid = ''IMDA'' AND d.imsp_id IS NOT NULL AND d.imar_id IS NULL',
      @AdditionalDiscoveryColumns = ', DD, DSN, ''IMDA'', imsp_id, obj_path',
      @AdditionalAttributeColumns = ', _DDName, _Dataset, _ParentCID, _ParentID, _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS VARCHAR) + ' areas'
    END
  END
  UPDATE IMSDiscovery SET obj_id = imar_id WHERE obj_cid = 'IMAR'

GOTO EndOfProc
  -- If not running in incremental mode then delete objects that were not rediscovered.
  IF @incremental = 0
  BEGIN
    -- Table of objects to be deleted
    CREATE TABLE #delete
    (
      reason      VARCHAR(60),
      link_type   VARCHAR(4) NULL,
      src_cid     ClassID,
      src_id      ObjID,
      dst_cid     ClassID,
      dst_id      ObjID
    )

    -- Subsystems under the OS's in the discovery batch that were not rediscovered
    IF @verbose = 1
    BEGIN
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Unrediscovered subsystems'
      SELECT DISTINCT 'Unrediscovered subsystems',
        l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
      FROM link AS l
        JOIN IMSDiscovery AS dis ON l.src_cid = 'OS' AND dis.os_id = l.src_id
        JOIN link_type AS lt ON lt.link_type = l.link_type
      WHERE
        dis.batchid = @batchid
        AND lt.flag_containment = 1
        AND NOT EXISTS
        (
          SELECT * FROM IMSDiscovery AS redis
          WHERE
            redis.batchid = dis.batchid
            AND redis.os_id = dis.os_id
            AND redis.imsu_cid = dis.imsu_cid
            AND redis.imsu_id = dis.imsu_id
        )
    END
PRINT 'SKIPPING'
GOTO EndOfProc
    INSERT INTO #delete
      SELECT DISTINCT 'Unrediscovered subsystems',
        l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
      FROM link AS l
        JOIN IMSDiscovery AS dis ON l.src_cid = 'OS' AND dis.os_id = l.src_id
        JOIN link_type AS lt ON lt.link_type = l.link_type
      WHERE
        dis.batchid = @batchid
        AND lt.flag_containment = 1
        AND NOT EXISTS
        (
          SELECT * FROM IMSDiscovery AS redis
          WHERE
            redis.batchid = dis.batchid
            AND redis.os_id = dis.os_id
            AND redis.imsu_cid = dis.imsu_cid
            AND redis.imsu_id = dis.imsu_id
        )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING UNREDISCOVERED SUBSYSTEMS', 3, 1)
      GOTO EndOfProc
    END

    -- All programs under the unrediscovered subsystems
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Programs under unrediscovered subsystems'
    INSERT INTO #delete
      SELECT DISTINCT 'Programs under unrediscovered subsystems',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        #delete AS d
        JOIN IMSProgram_C AS p ON
          p._ParentCID = d.dst_cid
          AND p._ParentID = d.dst_id
      WHERE
        d.src_cid = 'OS'
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING PROGRAMS UNDER UNREDISCOVERED SUBSYSTEMS', 3, 1)
      GOTO EndOfProc
    END

    -- Previously known subcomponents that were not rediscovered
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Previously known subcomponents not rediscovered'
    INSERT INTO #delete
      SELECT DISTINCT 'Previously known subcomponents not rediscovered',
        l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
      FROM link AS l
        JOIN IMSDiscovery AS dis ON l.src_cid = dis.imsu_cid AND dis.imsu_id = l.src_id
        JOIN isa_chain AS ic ON ic.dst_cid = l.dst_cid
        JOIN link_type AS lt ON lt.link_type = l.link_type
      WHERE
        dis.batchid = @batchid
        AND lt.flag_containment = 1
        AND ic.base_cid =  'IMSC' -- we only want links to subcomponents
        AND dis.imsc_cid <> 'IMPG'  -- exclude programs since these are directly contained
        AND NOT EXISTS
        (
          SELECT * FROM IMSDiscovery AS redis
          WHERE
            redis.batchid = dis.batchid
            AND redis.imsu_cid = dis.imsu_cid
            AND redis.imsu_cid = dis.imsu_cid
            AND redis.imsc_cid = dis.imsc_cid
            AND redis.imsc_id = dis.imsc_id
            AND redis.imsc_id = l.dst_id
        )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING PREVIOUSLY KNOWN SUBCOMPONENTS NOT REDISCOVERED', 3, 1)
      GOTO EndOfProc
    END

    -- Unrediscovered subcomponents
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Unrediscovered subcomponents under rediscovered subsystems'
    INSERT INTO #delete
      SELECT DISTINCT 'Unrediscovered subcomponents under rediscovered subsystems',
        l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
      FROM
        #delete AS d
        JOIN link AS l ON l.src_cid = d.dst_cid AND l.src_id = d.dst_id
        JOIN link_type AS lt ON lt.link_type = l.link_type
      WHERE
        lt.flag_containment = 1
        AND NOT EXISTS
        (
          SELECT * FROM IMSDiscovery AS redis
          WHERE
            redis.batchid = d.batchid
            AND redis.os_id = d.os_id
            AND redis.imsu_cid = d.imsu_cid
            AND redis.imsu_id = d.imsu_id
        )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING UNREDISCOVERED SUBCOMPONENTS UNDER REDISCOVERED SUBSYSTEMS', 3, 1)
      GOTO EndOfProc
    END

    -- Previously known programs that were not rediscovered under subsystems that have programs in
    -- the discovery batch.
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Previously known programs under subsystems with programs'
    INSERT INTO #delete
      SELECT DISTINCT 'Previously known programs under subsystems with programs',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        IMSProgram_C AS obj
        JOIN IMSDiscovery AS pd ON
          pd.imsu_cid = obj._ParentCID
          AND pd.imsu_id = obj._ParentID
          AND pd.imsc_cid = 'IMPG'
      WHERE
        pd.batchid = @batchid
        AND NOT EXISTS
        (
          SELECT * FROM IMSDiscovery AS redis
          WHERE
            redis.batchid = pd.batchid
            AND redis.os_id = pd.os_id
            AND redis.imsu_cid = pd.imsu_cid
            AND redis.imsu_id = pd.imsu_id
            AND redis.imsc_cid = pd.imsc_cid
            AND redis.imsc_id = pd.imsc_id
            AND redis.imsc_id = obj.id
        )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING PREVIOUSLY KNOWN PROGRAMS UNDER SUBSYSTEMS WITH PROGRAMS', 3, 1)
      GOTO EndOfProc
    END

    -- SubPrograms under the unrediscovered programs
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Subprograms under unrediscovered programs'
    INSERT INTO #delete
      SELECT DISTINCT 'Subprograms under unrediscovered programs',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        #delete AS d
        JOIN IMSSubProgram_C AS sp ON
          sp._ParentCID = d.dst_cid
          AND sp._ParentID = d.dst_id
      WHERE
        d.dst_cid = 'IMPG'
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING SUBPROGRAMS UNDER UNREDISCOVERED PROGRAMS', 3, 1)
      GOTO EndOfProc
    END

    -- Previously known subprograms that were not rediscovered under programs that have subprograms in
    -- the discovery batch.
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Previously known subprograms not rediscovered'
    INSERT INTO #delete
      SELECT DISTINCT 'Previously known subprograms not rediscovered',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        #delete AS d
        JOIN IMSSubProgram_C AS sp ON
          sp._ParentCID = d.dst_cid
          AND sp._ParentID = d.dst_id
        JOIN IMSDiscovery AS pd ON
          pd.imsc_cid = sp._ParentCID
          AND pd.imsc_id = sp._ParentID
          AND pd.imsp_cid = sp.cid
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING PREVIOUSLY KNOWN SUBPROGRAMS NOT REDISCOVERED', 3, 1)
      GOTO EndOfProc
    END

    -- Areas under the unrediscovered subprograms of type IMSDatabase
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Areas under unrediscovered databases'
    INSERT INTO #delete
      SELECT DISTINCT 'Areas under unrediscovered databases',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        #delete AS d
        JOIN IMSArea_C AS sp ON
          sp._ParentCID = d.dst_cid
          AND sp._ParentID = d.dst_id
      WHERE
        d.dst_cid = 'IMDA'
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING AREAS UNDER UNREDISCOVERED DATABASES', 3, 1)
      GOTO EndOfProc
    END

    -- Previously known areas that were not rediscovered under databases that have areas in
    -- the discovery batch.
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Previously known areas not rediscovered'
    INSERT INTO #delete
      SELECT DISTINCT 'Previously known areas not rediscovered',
        NULL, _ParentCID, _ParentID, cid, id
      FROM
        #delete AS d
        JOIN IMSArea_C AS a ON
          a._ParentCID = d.dst_cid
          AND a._ParentID = d.dst_id
        JOIN IMSDiscovery AS pd ON
          pd.imsc_cid = a._ParentCID
          AND pd.imsc_id = a._ParentID
          AND pd.imsp_cid = a.cid
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DiscoverIMSResources: ERROR DELETING PREVIOUSLY KNOWN AREAS NOT REDISCOVERED', 3, 1)
      GOTO EndOfProc
    END

    IF @verbose = 1
      SELECT DISTINCT COUNT(*) AS n, link_type, src_cid, src_id, dst_cid, dst_id, reason
      FROM #delete
      GROUP BY link_type, src_cid, src_id, dst_cid, dst_id, reason
  END
    
EndOfProc:

  PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Moving records from IMSDiscovery into ProcessedIMSDiscovery for batch ' + CAST(@batchid AS VARCHAR)
  -- Move records from IMSDiscovery into ProcessedIMSDiscovery
  INSERT ProcessedIMSDiscovery SELECT GETDATE(), * 
  FROM IMSDiscovery WHERE batchid = @batchid
  IF @@ERROR = 0
    DELETE IMSDiscovery WHERE batchid = @batchid

  -- Mark discovery batch as completed
  UPDATE DiscoveryBatch SET pend = GETDATE(), pstat = v.pstat, error = 0
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'COMPLETED'

  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _DiscoverIMSResources: Finished'
GO

---------------------------------------------------------------------------------------------------
-- asisp_discoverIMSResource
PRINT 'Creating procedure asisp_discoverIMSResource'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_discoverIMSResource' AND type = 'P')
  DROP PROCEDURE asisp_discoverIMSResource
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSDiscovery.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.9  $
-- $Date:   Sep 07 2001 16:24:04  $
-- $Author:   Forthman  $
-- Creates and processes a discovery batch for DB2 using the specified parameters
CREATE PROCEDURE asisp_discoverIMSResource
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @DATE_TIME         VARCHAR(40) = NULL, --  1
  @SEQ_NO            VARCHAR(40) = NULL, --  2
  @OBJ_TYPE_ID       VARCHAR(40) = NULL, --  3
  @SystemId          VARCHAR(40) = NULL, --  4
  @MVS               VARCHAR(40) = NULL, --  5
  @IMS_TMDB          VARCHAR(40) = NULL, --  6
  @IMS_DCCTL         VARCHAR(40) = NULL, --  7
  @IMS_DBCTL         VARCHAR(40) = NULL, --  8
  @IMS_IRLM          VARCHAR(40) = NULL, --  9
  @IMS_CQS           VARCHAR(40) = NULL, -- 10
  @IMS_FDR           VARCHAR(40) = NULL, -- 11
  @IMS_ITOC          VARCHAR(40) = NULL, -- 12
  @IMS_MPR           VARCHAR(40) = NULL, -- 13
  @IMS_FPM           VARCHAR(40) = NULL, -- 14
  @IMS_BMP           VARCHAR(40) = NULL, -- 15
  @IMS_LOGS          VARCHAR(40) = NULL, -- 16
  @IMS_TRANSACTION   VARCHAR(40) = NULL, -- 17
  @IMS_PROGRAM       VARCHAR(40) = NULL, -- 18
  @IMS_DATABASE      VARCHAR(40) = NULL, -- 19
  @IMS_AREA          VARCHAR(40) = NULL, -- 20
  @IMS_ESAF_DB2      VARCHAR(40) = NULL, -- 21
  @IMS_ESAF_MQS      VARCHAR(40) = NULL, -- 22
  @DD                VARCHAR(40) = NULL, -- 23
  @DSN               VARCHAR(40) = NULL, -- 24
  @ASSOCMVS          VARCHAR(40) = NULL, -- 25
  @ASSOCIATE         VARCHAR(40) = NULL, -- 26
  @STATE             VARCHAR(40) = NULL, -- 27
  @Event             VARCHAR(40) = NULL, -- 28
  @EventText         VARCHAR(255) = NULL, -- 29
  @auto_create       TINYINT = 1,
  @obj_cid           ClassID OUTPUT,
  @obj_id            ObjID OUTPUT,
  @obj_name          ObjName OUTPUT,
  @verbose           TINYINT = 0
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_discoverIMSResource
  ELSE
    SAVE TRANSACTION asisp_discoverIMSResource

  -- Create a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, numrows) 
    SELECT @now, @FormatType, @ActionType, 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  DECLARE @batchid INT
  SELECT @batchid = batchid FROM DiscoveryBatch WHERE ctime = @now
  IF @batchid IS NULL
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  -- Set object name based on OBJ_TYPE_ID
  SELECT @obj_name =
    CASE @OBJ_TYPE_ID
      WHEN 'IMSS' THEN @IMS_TMDB
      WHEN 'IMTM' THEN @IMS_DCCTL
      WHEN 'IMDB' THEN @IMS_DBCTL
      WHEN 'IMIR' THEN @IMS_IRLM
      WHEN 'IMCQ' THEN @IMS_CQS
      WHEN 'IMFD' THEN @IMS_FDR
      WHEN 'IMIT' THEN @IMS_ITOC
      WHEN 'IMMP' THEN @IMS_MPR
      WHEN 'IMFP' THEN @IMS_FPM
      WHEN 'IMBM' THEN @IMS_BMP
      WHEN 'IMLG' THEN @IMS_LOGS
      WHEN 'IMD2' THEN @IMS_ESAF_DB2
      WHEN 'IMMQ' THEN @IMS_ESAF_MQS
      WHEN 'IMPG' THEN @IMS_PROGRAM
      WHEN 'IMTX' THEN @IMS_TRANSACTION
      WHEN 'IMDA' THEN @IMS_DATABASE
      WHEN 'IMAR' THEN @IMS_AREA
    END

  -- Set discovered object type to the type of object this will be after discovery
  -- This is to handle that IMCQ gets converted to IMQA during processing,
  -- Certain queries in this procedure need to be aware of this conversion 
  DECLARE @OBJ_TYPE_ID_DISCOVERED VARCHAR(40) 
  SELECT @OBJ_TYPE_ID_DISCOVERED =
    CASE @OBJ_TYPE_ID
      WHEN 'IMCQ' THEN 'IMQA'
      ELSE @OBJ_TYPE_ID
    END

  -- Create temp table that gets populated with the class and name of
  -- each object that must be created
  CREATE TABLE #object
  (
    obj_cid   ClassID,
    obj_name  ObjName,
    is_object TINYINT
  )

  -- Insert record to create the specified object
  INSERT INTO #object VALUES ( @OBJ_TYPE_ID, @obj_name, 1 )

  -- Insert parent records if necessary

  -- IMSAreas need a parent database record
  IF @OBJ_TYPE_ID = 'IMAR'
  BEGIN
    INSERT INTO #object
      VALUES ( 'IMDA', @IMS_DATABASE, 0 )
  END

  -- IMSSubprograms need IMSProgram record
  IF @OBJ_TYPE_ID IN
  (
    SELECT DISTINCT cid FROM isa_chain WHERE base_cid IN ('IMSP', 'IMAR')
  )
  BEGIN
    INSERT INTO #object
      VALUES ( 'IMPG', @IMS_PROGRAM, 0 )
  END

  -- IMSSubcomponent and IMSProgram objects need IMSSubsystem parent record
  IF @OBJ_TYPE_ID_DISCOVERED IN
  (
    SELECT DISTINCT cid FROM isa_chain WHERE base_cid IN ('IMSC', 'IMBM', 'IMPG', 'IMSP', 'IMAR')
  )
  BEGIN
    INSERT INTO #object
    SELECT
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN 'IMSS'
      WHEN @IMS_DCCTL IS NOT NULL THEN 'IMTM'
      WHEN @IMS_DBCTL IS NOT NULL THEN 'IMDB'
    END AS obj_cid,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN @IMS_TMDB
      WHEN @IMS_DCCTL IS NOT NULL THEN @IMS_DCCTL
      WHEN @IMS_DBCTL IS NOT NULL THEN @IMS_DBCTL
    END AS obj_name,
    0
  END

  -- Insert IMSDiscovery records based on the rows that have been entered into #object temp table
  INSERT INTO IMSDiscovery
  (
    batchid,
    SystemId,
    obj_cid,
    IMS_SUBSYSTEM,
    IMS_SUBCOMPONENT,
    IMS_SUBPROGRAM,
    IMS_AREA,
    imsu_cid,
    imsc_cid,
    imsp_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    MVS,
    DD,
    DSN,
    ASSOCMVS,
    ASSOCIATE,
    STATE,
    Event,
    EventText
  )
  SELECT DISTINCT
    @batchid,
    @SystemId,
    o.obj_cid,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN @IMS_TMDB
      WHEN @IMS_DCCTL IS NOT NULL THEN @IMS_DCCTL
      WHEN @IMS_DBCTL IS NOT NULL THEN @IMS_DBCTL
    END AS IMS_SUBSYSTEM,
    CASE
      WHEN o.obj_cid IN ('IMSS', 'IMTM', 'IMDB') THEN NULL
      WHEN @IMS_IRLM IS NOT NULL THEN @IMS_IRLM
      WHEN @IMS_CQS IS NOT NULL THEN @IMS_CQS
      WHEN @IMS_FDR IS NOT NULL THEN @IMS_FDR
      WHEN @IMS_ITOC IS NOT NULL THEN @IMS_ITOC
      WHEN @IMS_MPR IS NOT NULL THEN @IMS_MPR
      WHEN @IMS_FPM IS NOT NULL THEN @IMS_FPM
      WHEN @IMS_BMP IS NOT NULL THEN @IMS_BMP
      WHEN @IMS_LOGS IS NOT NULL THEN @IMS_LOGS
      WHEN @IMS_ESAF_DB2 IS NOT NULL THEN @IMS_ESAF_DB2
      WHEN @IMS_ESAF_MQS IS NOT NULL THEN @IMS_ESAF_MQS
      WHEN @IMS_PROGRAM IS NOT NULL THEN @IMS_PROGRAM
    END AS IMS_SUBCOMPONENT,
    CASE 
      WHEN o.obj_cid NOT IN ('IMDA', 'IMTX', 'IMAR') THEN NULL
      WHEN @IMS_TRANSACTION IS NOT NULL THEN @IMS_TRANSACTION
      WHEN @IMS_DATABASE IS NOT NULL THEN @IMS_DATABASE
    END AS IMS_SUBPROGRAM,
    CASE
      WHEN o.obj_cid NOT IN ('IMAR')THEN NULL
      ELSE @IMS_AREA
    END AS IMS_AREA,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN 'IMSS'
      WHEN @IMS_DCCTL IS NOT NULL THEN 'IMTM'
      WHEN @IMS_DBCTL IS NOT NULL THEN 'IMDB'
    END AS imsu_cid,
    CASE
      WHEN o.obj_cid IN ('IMSS', 'IMTM', 'IMDB') THEN NULL
      WHEN @IMS_IRLM IS NOT NULL THEN 'IMIR'
      WHEN @IMS_CQS IS NOT NULL THEN 'IMQA'
      WHEN @IMS_FDR IS NOT NULL THEN 'IMFD'
      WHEN @IMS_ITOC IS NOT NULL THEN 'IMIT'
      WHEN @IMS_MPR IS NOT NULL THEN 'IMMP'
      WHEN @IMS_FPM IS NOT NULL THEN 'IMFP'
      WHEN @IMS_BMP IS NOT NULL THEN 'IMBM'
      WHEN @IMS_LOGS IS NOT NULL THEN 'IMLG'
      WHEN @IMS_ESAF_DB2 IS NOT NULL THEN 'IMD2'
      WHEN @IMS_ESAF_MQS IS NOT NULL THEN 'IMMQ'
      WHEN @IMS_PROGRAM IS NOT NULL THEN 'IMPG'
    END AS imsc_cid,
    CASE
      WHEN o.obj_cid NOT IN ('IMDA', 'IMTX', 'IMAR') THEN NULL
      WHEN @IMS_TRANSACTION IS NOT NULL THEN 'IMTX'
      WHEN @IMS_DATABASE IS NOT NULL THEN 'IMDA'
    END AS imsp_cid,
    o.obj_name,
    SUBSTRING(@DATE_TIME,1,4)+'-'+SUBSTRING(@DATE_TIME,5,2)+'-'+SUBSTRING(@DATE_TIME,7,2)+' '+SUBSTRING(@DATE_TIME,9,12),
    @SEQ_NO,
    @MVS,
    CASE WHEN o.is_object = 1 THEN @DD ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @DSN ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @ASSOCMVS ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @ASSOCIATE ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @STATE ELSE NULL END,
    @Event,
    @EventText
  FROM
    #object AS o
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  DECLARE @r INT
  EXEC @r = _DiscoverIMSResources ROOT, 1, @batchid, @incremental=1, @verbose=@verbose

  SELECT @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name
  FROM ProcessedIMSDiscovery
  WHERE batchid = @batchid AND obj_cid = @OBJ_TYPE_ID_DISCOVERED AND obj_name = @obj_name

  IF @tcount = 0
    COMMIT TRANSACTION asisp_discoverIMSResource

  RETURN @r
GO

-----------------------------------------------------------------------------------------------------------------------------------
-- Scheduled Task: 'IMS Discovery Load'
--
--exec msdb..sp_delete_job @job_name='IMS Discovery Load'
DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT
SET @job_name = 'IMS Discovery Load'
IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  PRINT 'Use SQL Enterprise Manager to define a schedule for this job.'
  EXEC @ret = msdb..sp_add_job
    @job_name = @job_name,
    @description = 'Loading job for IMS discovery batches.',
    @job_id = @job_id OUTPUT
  IF @ret = 0
  BEGIN
    EXEC msdb..sp_add_jobstep
      @job_id = @job_id,
      @subsystem = 'TSQL',
      @command = '_LoadIMSDiscoveryBatch ROOT, 0',
      @database_name = 'Object',
      @step_name = 'Load'
  END
END
GO

-----------------------------------------------------------------------------------------------------------------------------------
-- Scheduled Task: 'IMS Discovery Process'
--
--exec msdb..sp_delete_job @job_name='IMS Discovery Process'
DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT
SET @job_id = NULL
SET @job_name = 'IMS Discovery Process'
IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  EXEC @ret = msdb..sp_add_job
    @job_name = @job_name,
    @description = 'Processing job for IMS discovery batches.',
    @job_id = @job_id OUTPUT
  IF @ret = 0
  BEGIN
   EXEC msdb..sp_add_jobstep
      @job_id = @job_id,
      @subsystem = 'TSQL',
      @command = '_DiscoverIMSResources ROOT, 0',
      @database_name = 'Object',
      @step_name = 'Load'
 
    -- associate the job with a server
    EXEC msdb..sp_add_jobserver @job_name=@job_name, @server_name=@@SERVERNAME

    -- set up the schedule for which to run the job
    EXEC msdb..sp_add_jobschedule 
      @name='IMSDiscoveryProcessSchedule',
      @job_name=@job_name,
      @enabled=1,
      @freq_type=4,              -- run daily
      @freq_interval=1,          -- run every day
      @freq_subday_type=4,       -- run by minute
      @freq_subday_interval=1    -- every minute
  END
END
GO

-----------------------------------------------------------------------------------------------------------------------------------
PRINT 'Done.'
GO
