-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/IMSResourceViewDb.sqv   1.0   Sep 07 2001 16:49:38   Forthman  $
--
-- IMSResourceView.sqi
--
-- Defines the GetIMSObjects method, and its supporting references

SET NOCOUNT ON
GO

--------------------------------------------------------------------------------------------------------
-- asisp_updateManagedObjectTypesEnum
PRINT 'Creating procedure asisp_updateManagedObjectTypesEnum'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_updateManagedObjectTypesEnum' AND type = 'P')
  DROP PROCEDURE asisp_updateManagedObjectTypesEnum
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSResourceViewDb.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:49:38  $
-- $Author:   Forthman  $
CREATE PROCEDURE asisp_updateManagedObjectTypesEnum
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN updateManagedObjectTypesEnum
  ELSE
    SAVE TRAN updateManagedObjectTypesEnum

  DECLARE @eid INT
  SELECT @eid = eid FROM enumeration WHERE name = 'ManagedObjectTypes'
  IF @eid IS NULL
  BEGIN
    ROLLBACK TRAN updateManagedObjectTypesEnum
    RETURN -30000
  END

  DELETE enum_value WHERE eid = @eid
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateManagedObjectTypesEnum
    RETURN -30000
  END

  CREATE TABLE #ordered_cid (cid ClassID, row INT IDENTITY(1,1))
  INSERT #ordered_cid(cid)
    SELECT i.cid
    FROM isa_chain AS i JOIN obj_class AS c ON c.cid = i.cid
    WHERE i.base_cid = 'MNO2' AND c.flag_abstract = 0
    AND i.cid NOT IN ('IMPG', 'IMTX', 'IMDA', 'IMAR', 'IMBM')
    ORDER BY c.label
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateManagedObjectTypesEnum
    RETURN -30000
  END

  INSERT enum_value(eid, name, label, value, val_cid)
    SELECT e.eid, c.cname, c.label, o.row, o.cid
    FROM
      enumeration AS e,
      #ordered_cid AS o
        JOIN obj_class AS c ON c.cid = o.cid
    WHERE
      e.name = 'ManagedObjectTypes'
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateManagedObjectTypesEnum
    RETURN -30000
  END

  UPDATE Meta..DatabaseVersion SET verno = verno + 1

  IF @tcount = 0
    COMMIT TRAN updateManagedObjectTypesEnum
GO


--------------------------------------------------------------------------------------------------------
-- Update ManagedObjectType Enumeration
PRINT 'Updating ManagedObjectType Enumeration'
GO

EXEC asisp_updateManagedObjectTypesEnum
GO

--------------------------------------------------------------------------------------------------------
-- asisp_updateIMSObjectTypesEnum
PRINT 'Creating procedure asisp_updateIMSObjectTypesEnum'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_updateIMSObjectTypesEnum' AND type = 'P')
  DROP PROCEDURE asisp_updateIMSObjectTypesEnum
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSResourceViewDb.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:49:38  $
-- $Author:   Forthman  $
CREATE PROCEDURE asisp_updateIMSObjectTypesEnum
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN updateIMSObjectTypesEnum
  ELSE
    SAVE TRAN updateIMSObjectTypesEnum

  DECLARE @eid INT
  SELECT @eid = eid FROM enumeration WHERE name = 'IMSObjectTypes'
  IF @eid IS NULL
  BEGIN
    ROLLBACK TRAN updateIMSObjectTypesEnum
    RETURN -30000
  END

  DELETE enum_value WHERE eid = @eid
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateIMSObjectTypesEnum
    RETURN -30000
  END

  CREATE TABLE #ordered_cid (cid ClassID, row INT IDENTITY(1,1))
  INSERT #ordered_cid(cid)
    SELECT i.cid
    FROM isa_chain AS i JOIN obj_class AS c ON c.cid = i.cid AND i.base_cid = 'IMBA'
    WHERE c.flag_abstract = 0 AND c.cid <> 'IMSY'
    ORDER BY c.label
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateIMSObjectTypesEnum
    RETURN -30000
  END

  INSERT enum_value(eid, name, label, value, val_cid)
    SELECT e.eid, c.cname, c.label, o.row, o.cid
    FROM
      enumeration AS e,
      #ordered_cid AS o
        JOIN obj_class AS c ON c.cid = o.cid
    WHERE
      e.name = 'IMSObjectTypes'
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN updateIMSObjectTypesEnum
    RETURN -30000
  END

  UPDATE Meta..DatabaseVersion SET verno = verno + 1

  IF @tcount = 0
    COMMIT TRAN updateIMSObjectTypesEnum
GO

------------------------------------------------------------------------------------
-- Update IMSObjectType Enumeration
PRINT 'Update IMSObjectType Enumeration'
GO

EXEC asisp_updateIMSObjectTypesEnum
GO

------------------------------------------------------------------------------------
-- _GetIMSObjects
PRINT "Creating PROCEDURE _GetIMSObjects"
IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetIMSObjects' AND type = 'P')
  DROP PROCEDURE _GetIMSObjects 
GO

CREATE PROCEDURE _GetIMSObjects
-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/IMSResourceViewDb.sqv  $
-- $Revision:   1.0  $ 
-- $Date:   Sep 07 2001 16:49:38  $
-- $Author:   Forthman  $
  @cid               ClassID,
  @id                ObjID,
  @OrLocationPattern1  varchar(255) = null,
  @OrLocationPattern2  varchar(255) = null,
  @OrLocationPattern3  varchar(255) = null,
  @NamePattern       varchar(255) = null,
  @MinAlert          int = 0,
  @MinPriority       int = 0,
  @OpenNotes         int = 2,
  @ObjectType        int = 0, 
  @DDNamePattern     VARCHAR(255) = NULL,
  @DatasetPattern    VARCHAR(255) = NULL,
  @debug              int = 0
AS
  RAISERROR( '_GetIMSObjects: Called as _GetIMSObjects @cid=%s, @id=%d, @OrLocationPattern1=%s, @OrLocationPattern2=%s, @OrLocationPattern3=%s, @NamePattern=%s, @MinAlert=%d, @MinPriority=%d, @OpenNotes=%d, @ObjectType=%d, @DDNamePattern=%s, @DatasetPattern=%s,  @debug=%d', 0, 1, @cid, @id, @OrLocationPattern1, @OrLocationPattern2, @OrLocationPattern3, @NamePattern, @MinAlert, @MinPriority, @OpenNotes, @ObjectType, @DDNamePattern, @DatasetPattern, @debug)  

  SET NOCOUNT ON
  SET ROWCOUNT 0

  DECLARE @startdate datetime
  DECLARE @nowdate   datetime
  DECLARE @stepdate  datetime
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  VARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             (ms_step/ms_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  DECLARE @maxRowCount AS INT
  SELECT @maxRowCount = 2000

  -- Get list of IMS objects under the calling object using object linking 
  CREATE TABLE #descendent
  (
    cid   ClassID PRIMARY KEY (cid, id),
    id    ObjID,
    gen   INT
  )

  DECLARE @where VARCHAR(255)
  SET @where = 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''IMBA'')'

  DECLARE @ObjectTypeCid VARCHAR(4)
  SELECT @ObjectTypeCid = NULL

  -- Modify the where clause based on object type requested to get the bare minumim
  -- Either all, just itself, or just the classes need for the physical containments
  -- If no object type was specified leave the where clause alone
  IF @ObjectType <> 0 
  BEGIN
    SELECT  @ObjectTypeCid = ev.val_cid FROM enum_value AS ev 
      JOIN enumeration AS e ON ev.eid = e.eid
      WHERE e.name = 'IMSObjectTypes' AND ev.value = @ObjectType
      -- If the object type was ('IMPG', 'IMDA', 'IMTX', 'IMAR'), 
      --  then restrict the where clause to ('IMSS','IMTM','IMDB')
      IF @ObjectTypeCid IN ('IMPG', 'IMDA', 'IMTX', 'IMAR')
      BEGIN
         SELECT @where = 'IN (''IMSS'',''IMTM'',''IMDB'')'        
      END
      -- Else the object type is not zero and not ('IMPG', 'IMDA', 'IMTX', 'IMAR')
      --  then restrict to the requested object type
      ELSE
      BEGIN
        SELECT @where = 'IN (''' + ev.val_cid + ''')'
          FROM enum_value AS ev JOIN enumeration AS e ON ev.eid = e.eid
          WHERE e.name = 'IMSObjectTypes' AND ev.value = @ObjectType
      END
  END

  IF @debug >=2
  BEGIN
    PRINT 'Before EXEC asisp_descendents @cid, @id, @where call'
    SELECT @cid, @id, @where
  END

  -- populate #descendents table with those objects with link dependencies
  EXEC asisp_descendents @cid, @id, @where

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'asisp_descendents'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  IF @debug >=2
  BEGIN
    PRINT '# of rows in #descendent: ' 
    SELECT COUNT(*) FROM #descendent
    PRINT 'SELECT * FROM #descendent: '
    SELECT * FROM #descendent
  END

  -- Populate #obj table with all attributes of objects who should be returned
  CREATE TABLE #obj
  (
    src_cid            ClassID NULL,
    src_id             ObjID NULL,
    cid                ClassID,
    id                 ObjID,
    name               ObjName NULL,
    Location           VARCHAR(1000) NULL,
    Type               VARCHAR(40) NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        VARCHAR(255) NULL,
    DDName             VARCHAR(255) NULL,
    DatasetName        VARCHAR(255) NULL
  )
  -- Populate #obj table with all objects known by descendents links
  INSERT #obj
    SELECT c.src_cid, c.src_id, dst_cid, dst_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM #descendent AS d 
      JOIN link AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
      JOIN link_type AS lt ON lt.link_type = c.link_type
    WHERE
      (lt.flag_containment=1 OR lt.flag_parent_child=1)

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate by link'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @debug >=2
  BEGIN
    PRINT '# of rows in #obj: ' 
    SELECT COUNT(*) FROM #obj
  END

  -- If proc called on LOB, Populate #obj table with all objects known by phyiscal containment
  -- ('IMPG', 'IMDA', 'IMTX', 'IMAR')
  IF @cid LIKE 'LOB'
  BEGIN
  INSERT #obj
    SELECT NULL, NULL, d.cid, d.id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM #descendent AS d
    WHERE d.cid IN ('IMPG', 'IMDA', 'IMTX', 'IMAR')
  END

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate physical containment by lob'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @debug >=2
  BEGIN
  PRINT '# of rows in #obj: After insertions from LOB' 
  SELECT COUNT(*) FROM #obj
  END

  -- If proc called on physically contained object populate with self
  IF @cid IN ('IMPG', 'IMDA', 'IMTX', 'IMAR')
  BEGIN 
  INSERT #obj
    SELECT NULL, NULL, @cid, @id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
  END

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate by self'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  IF @debug >=2
  BEGIN
  PRINT '# of rows in #obj: After the insertion due from physically contained parent IMPG, IMDA, IMTX, IMAR' 
  SELECT COUNT(*) FROM #obj
  END

  -- Populate #obj with physically contained IMPGs if they are needed
  IF @ObjectType = 0 OR @ObjectTypeCid IN ('IMPG', 'IMTX', 'IMDA', 'IMAR')
  BEGIN 
    INSERT #obj
      SELECT p._ParentCID, p._ParentID, p.cid, p.id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
      FROM IMSProgram_C AS p 
        JOIN #obj AS o ON o.cid = p._ParentCID AND o.id = p._ParentID AND p.deleted = 0 
        WHERE @ObjectType = 0
          OR  (@ObjectTypeCid IN ('IMTX', 'IMDA', 'IMAR')
          OR  (@NamePattern IS NULL 
          OR  p.name LIKE @NamePattern))
--          A OR (B OR (C OR D))
  END

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate IMPG'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Populate #obj with physically contained IMTX and IMDA if the are needed
  IF @ObjectType = 0 OR @ObjectTypeCid IN ('IMTX', 'IMDA', 'IMAR')
  BEGIN
  INSERT #obj
    SELECT s._ParentCID, s._ParentID, s.cid, s.id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM IMSSubProgram_C AS s 
      JOIN #obj AS o ON o.cid = s._ParentCID AND o.id = s._ParentID AND s.deleted = 0 
        WHERE @ObjectType = 0
          OR  (@ObjectTypeCid IN ('IMAR')
          OR  (@NamePattern IS NULL 
          OR  s.name LIKE @NamePattern))
--          A OR (B OR (C OR D))
  END

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate IMDA, IMTX'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Populate #obj with physically contained IMAR if the are needed
  IF @ObjectType = 0 OR @ObjectTypeCid IN ('IMAR')
  BEGIN
  INSERT #obj
    SELECT a._ParentCID, a._ParentID, a.cid, a.id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM IMSArea_C AS a
      JOIN #obj AS o ON o.cid = a._ParentCID AND o.id = a._ParentID AND a.deleted = 0
        WHERE @ObjectType = 0
          OR  (@NamePattern IS NULL 
          OR  a.name LIKE @NamePattern)
--          A OR (B OR C)
  END

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'populate IMAR'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @debug >=2
  BEGIN
    SELECT @cid, @id 
    PRINT '# of rows in #obj: After object type filtering.' 
    SELECT COUNT(*) FROM #obj
  END


  SELECT i.cid, i.id, i.name, Location = i._Path, Type = oc.label, i._AlertStateID, i._PriorityID, i._CurrentStateID, i._DesiredStateID, i._HasOwnershipNote, Description = i.description, DDName = i._DDName, DatasetName = i._Dataset
    INTO #output 
      FROM IMSBase_C AS i
      INNER JOIN #obj AS o ON o.cid = i.cid AND o.id = i.id
      INNER JOIN obj_class AS oc ON oc.cid = i.cid
      WHERE (@ObjectType = 0         OR i.cid = @ObjectTypeCid)
        AND (@MinAlert = 0           OR i._AlertStateID >= @MinAlert)
        AND (@NamePattern IS NULL    OR i.name LIKE @NamePattern)
        AND (@MinPriority = 0        OR i._PriorityID <= @MinPriority) -- yes, priorities work in reverse!
        AND (@OpenNotes = 2          OR i._HasOwnershipNote = @OpenNotes)
        AND ( (@OrLocationPattern1 IS NULL AND @OrLocationPattern2 IS NULL AND @OrLocationPattern3 IS NULL )
          OR  (    i._Path LIKE @OrLocationPattern1
                OR i._Path LIKE @OrLocationPattern2
                OR i._Path LIKE @OrLocationPattern3
              )
            )
        AND (@DDNamePattern IS NULL  OR i._DDName LIKE @DDNamePattern)
        AND (@DatasetPattern IS NULL OR i._Dataset LIKE @DatasetPattern)
       
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'SELECT INTO #output'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
	   
  -- Output columns based on final result set
  DECLARE @countRow INT
  SELECT @countRow = COUNT(*) FROM #output

  -- Output error message
  IF @countRow > @maxRowCount 
  BEGIN 
    CREATE TABLE #errortext
    (
      _AlertStateID      INT,
      Error              VARCHAR(100)
    )
    INSERT #errortext
      SELECT _AlertStateID = 3,
      Error = 'The query returned ' + CONVERT( VARCHAR(8), @countRow) + ' rows, please submit query again with more restrictive criteria.'
    SELECT 
      e._AlertStateID, e.Error
    FROM 
      #errortext AS e
  END
  ELSE
  BEGIN

  -- Return Good Results
  SELECT
    obj._AlertStateID,
    obj._HasOwnershipNote,
    obj.Location,
    obj.Type,
    obj.name,
    obj._PriorityID,
    obj._CurrentStateID,
    obj._DesiredStateID,
    obj.Description,
    obj.DDName,
    obj.DatasetName,
    obj.cid,
    obj.id
  FROM
    #output AS obj
  ORDER BY
    obj.Location
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Result Output'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
  DROP TABLE #obj
  DROP TABLE #descendent
GO

SET NOCOUNT OFF
GO

-----------------------------------------------------------------------------------------------------------------------
PRINT 'Done.'
GO
