-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/OPCLoad_CentralServer.sqv   1.3   Aug 09 2001 10:46:50   BRENT  $
--
-- OPCLoad_CentralServer.sql
--
-- Module Description
--   Creates OPC related tables in Object database

USE Object
GO


IF EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_plan_FK' AND type = 'F')
BEGIN
  PRINT 'Dropping foreign key opc_app_plan_FK from opc_application table'
  ALTER TABLE opc_application DROP CONSTRAINT opc_app_plan_FK
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_op_app_FK' AND type = 'F')
BEGIN
  PRINT 'Dropping foreign key opc_op_app_FK from opc_operation table'
  ALTER TABLE opc_operation DROP CONSTRAINT opc_op_app_FK
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_predecessor_succ_op_FK' AND type = 'F')
BEGIN
  PRINT 'Dropping foreign key opc_predecessor_succ_op_FK from opc_predecessor table'
  ALTER TABLE opc_predecessor DROP CONSTRAINT opc_predecessor_succ_op_FK
END
GO

PRINT 'Creating OPC tables in Object database'

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_detail' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_path_detail'
  CREATE TABLE opc_app_path_detail
  (
    plan_id INT NOT NULL ,
    app_name NVARCHAR (20) NOT NULL ,
    path_id INT NOT NULL ,
    gen INT NOT NULL ,
    obj_cid ClassID NOT NULL ,
    obj_name NVARCHAR (255) NULL ,
    obj_desc NVARCHAR (255) NULL ,
    obj_id ObjID DEFAULT 0 NOT NULL 
  )
  ALTER TABLE opc_app_path_detail ADD CONSTRAINT opc_app_path_det_PK PRIMARY KEY CLUSTERED(plan_id, app_name, path_id, gen)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_aggr' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_path_aggr'
  CREATE TABLE opc_app_path_aggr
  (
    plan_id INT NOT NULL ,
    app_name NVARCHAR (20) NOT NULL ,
    path_id INT NOT NULL ,
    gen INT NOT NULL ,
    obj_cid ClassID NOT NULL ,
    obj_id ObjID NULL ,
    obj_name NVARCHAR (255) NULL
  )
END
GO

-- Drop old opc_daily_plan table if it exists. We tell this by the presence of the identity column.
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_daily_plan' AND type = 'U')
  AND EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('opc_daily_plan') AND autoval IS NOT NULL)
BEGIN
  PRINT 'Dropping old opc_daily_plan table'
  EXEC("DROP TABLE opc_daily_plan")
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_daily_plan' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_daily_plan'
  CREATE TABLE opc_daily_plan
  (
    id INT NOT NULL ,
    name NVARCHAR (255) NULL ,
    start_date DATETIME NULL ,
    end_date DATETIME NULL 
  )
  ALTER TABLE opc_daily_plan ADD CONSTRAINT opc_daily_plan_PK PRIMARY KEY CLUSTERED(id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_application' AND type = 'U')
BEGIN
  PRINT 'Creating table table opc_application'
  CREATE TABLE opc_application
  (
    plan_id INT NOT NULL ,
    app_id INT NOT NULL ,
    app_name NVARCHAR (20) NULL ,
    app_owner NVARCHAR (40) NULL ,
    start_day SMALLINT NULL ,
    start_day_mod SMALLINT NULL ,
    start_hour SMALLINT NULL ,
    stop_day SMALLINT NULL ,
    stop_day_mod SMALLINT NULL ,
    stop_hour SMALLINT NULL ,
    app_desc NVARCHAR (255) NULL ,
    est_start DATETIME NULL ,
    est_stop DATETIME NULL 
  )
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_id_UQ PRIMARY KEY CLUSTERED(plan_id, app_id)
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_start_day_mod_CK CHECK(([start_day_mod] >= (-1) and [start_day_mod] <= 1))
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_stop_day_mod_CK CHECK(([stop_day_mod] >= (-1) and [stop_day_mod] <= 1))
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_operation' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_operation'
  CREATE TABLE opc_operation
  (
    plan_id INT NOT NULL ,
    app_id INT NULL ,
    app_name NVARCHAR (20) NULL ,
    op_id INT NOT NULL ,
    op_name NVARCHAR (8) NULL ,
    job_name NVARCHAR (8) NULL ,
    status nchar (1) NULL ,
    start_day SMALLINT NULL ,
    start_day_mod SMALLINT NULL ,
    start_hour SMALLINT NULL ,
    duration SMALLINT NULL ,
    stop_day SMALLINT NULL ,
    stop_day_mod SMALLINT NULL ,
    stop_hour SMALLINT NULL ,
    op_desc NVARCHAR (255) NULL ,
    initpri INT NULL ,
    termpri INT NULL ,
    est_start DATETIME NULL ,
    est_stop DATETIME NULL ,
    op_type TINYINT NULL ,
    btcy_id ObjID NULL ,
    job_id INT NULL ,
    instanceno INT NULL 
  )
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_PK PRIMARY KEY CLUSTERED(plan_id, op_id)
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_start_day_mod_CK CHECK(([start_day_mod] >= (-1) and [start_day_mod] <= 1))
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_stop_day_mod_CK CHECK(([stop_day_mod] >= (-1) and [stop_day_mod] <= 1))
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_type_CK CHECK(([op_type] >= 0 and [op_type] <= 2))
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_predecessor' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_predecessor'
  CREATE TABLE opc_predecessor
  (
    plan_id INT NOT NULL ,
    succ_op_id INT NOT NULL ,
    pred_app_name NVARCHAR (20) NOT NULL ,
    pred_op_name NVARCHAR (8) NOT NULL ,
    pred_op_id INT NULL 
  )
  ALTER TABLE opc_predecessor ADD CONSTRAINT opc_predecessor_PK PRIMARY KEY CLUSTERED(plan_id, succ_op_id, pred_app_name, pred_op_name)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_new_instance' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_new_instance'
  CREATE TABLE opc_app_new_instance
  (
    btcy_name ObjName PRIMARY KEY,
    btcy_desc NVARCHAR(255),
    row       INT NOT NULL
  )
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_aggr' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_path_aggr'
  CREATE TABLE opc_app_path_aggr
  (
    plan_id INT NOT NULL ,
    app_name NVARCHAR (20) NOT NULL ,
    path_id INT NOT NULL ,
    gen INT NOT NULL ,
    obj_cid ClassID NOT NULL ,
    obj_id ObjID NULL ,
    obj_name NVARCHAR (255) NULL
  )
  ALTER TABLE opc_app_path_aggr ADD CONSTRAINT opc_app_path_aggr_UQ UNIQUE (plan_id, app_name, path_id, gen, obj_cid, obj_name)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_new_instance' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_new_instance'
  CREATE TABLE opc_app_new_instance
  (
    btcy_name ObjName PRIMARY KEY,
    btcy_desc NVARCHAR(255),
    row       INT
  )
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_detail' AND type = 'U')
  AND (SELECT COUNT(*) FROM syscolumns WHERE id = OBJECT_ID('opc_app_path_detail')) <> 8
BEGIN
  PRINT 'Dropping old opc_app_path_detail table'
  DROP TABLE opc_app_path_detail
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_detail' AND type = 'U')
BEGIN
  PRINT 'Creating table opc_app_path_detail'
  CREATE TABLE opc_app_path_detail
  (
    plan_id                       int  NOT NULL CONSTRAINT opc_app_path_detail_plan_id_DF DEFAULT (0),
    app_name                      nvarchar(20) NOT NULL,
    path_id                       int  NOT NULL,
    gen                           int  NOT NULL,
    obj_cid                       ClassID  NOT NULL,
    obj_name                      nvarchar(255) NULL,
    obj_desc                      nvarchar(255) NULL,
    obj_id                        ObjID  NOT NULL CONSTRAINT opc_app_path_detail_obj_id_DF DEFAULT (0)
  )
  ALTER TABLE opc_app_path_detail ADD CONSTRAINT opc_app_path_det_PK PRIMARY KEY CLUSTERED(plan_id, app_name, path_id, gen)
END
GO
