-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/OPCLoad_OPCLoadServer.sqv   1.0   Aug 01 2001 13:34:38   BRENT  $
--
-- OPCLoad_OPCLoadServer.sql
--
-- Module Description
--   Creates OPC related tables in the OPCLoad database

IF NOT EXISTS (SELECT * FROM master..sysdatabases WHERE name = 'OPCLoad') BEGIN
  PRINT 'Creating OPCLoad database'
  CREATE DATABASE OPCLoad 
END
GO

-- create object tables that mirror views and tables in Object database

USE OPCLoad

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'dyn_obj_path' AND type = 'U')
BEGIN
  PRINT 'Creating dyn_obj_path'
  CREATE TABLE dyn_obj_path
  (
    id INT NOT NULL,
    name NVARCHAR (80) NULL,
    ctime DATETIME NULL,
    description NVARCHAR (255) NULL 
  )
  ALTER TABLE dyn_obj_path ADD CONSTRAINT dyn_obj_path_PK PRIMARY KEY CLUSTERED(id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'dyn_obj_path_detail' AND type = 'U')
BEGIN
  PRINT 'Creating dyn_obj_path_detail'
  CREATE TABLE dyn_obj_path_detail
  (
    path_id INT NOT NULL,
    gen INT NOT NULL,
    obj_cid ClassID NOT NULL,
    obj_name NVARCHAR (255) NULL,
    obj_desc NVARCHAR (255) NULL,
    obj_id ObjID NOT NULL 
  )
  ALTER TABLE dyn_obj_path_detail ADD CONSTRAINT dyn_obj_path_det_PK PRIMARY KEY CLUSTERED(path_id, gen)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'dyn_obj_path_hist' AND type = 'U')
BEGIN
  PRINT 'Creating dyn_obj_path_hist'
  CREATE TABLE dyn_obj_path_hist
  (
    path_id INT NOT NULL,
    ctime DATETIME NOT NULL,
    reason NVARCHAR (255) NULL 
  )
  ALTER TABLE dyn_obj_path_hist ADD CONSTRAINT dyn_obj_path_hist_PK PRIMARY KEY CLUSTERED(path_id, ctime)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_aggr' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_aggr'
  CREATE TABLE opc_app_aggr
  (
    plan_id INT NOT NULL,
    app_name NVARCHAR (20) NOT NULL,
    app_owner NVARCHAR (40) NULL,
    app_desc NVARCHAR (255) NULL,
    path_id INT NOT NULL 
  )
  ALTER TABLE opc_app_aggr ADD CONSTRAINT opc_app_aggr_PK PRIMARY KEY CLUSTERED(plan_id, app_name, path_id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_instance' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_instance'
  CREATE TABLE opc_app_instance
  (
    plan_name NVARCHAR (80) NOT NULL,
    app_name NVARCHAR (20) NOT NULL,
    ctime DATETIME DEFAULT GETDATE() NOT NULL,
    rtime DATETIME DEFAULT GETDATE() NOT NULL,
--        btcy_id ObjID NULL,
    existing TINYINT NOT NULL DEFAULT 0
  )
  ALTER TABLE opc_app_instance ADD CONSTRAINT opc_app_instance_PK PRIMARY KEY CLUSTERED(plan_name, app_name)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_aggr' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_path_aggr'
  CREATE TABLE opc_app_path_aggr
  (
    plan_id  INT NOT NULL,
    app_name NVARCHAR (20) NOT NULL,
    path_id  INT NOT NULL,
    gen      INT NOT NULL,
    obj_cid  ClassID NOT NULL,
    obj_id   ObjID NULL,
    obj_name NVARCHAR (255) NULL
  )
  ALTER TABLE opc_app_path_aggr ADD CONSTRAINT opc_app_path_aggr_UQ UNIQUE (plan_id, app_name, path_id, gen, obj_cid, obj_name)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_path_detail' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_path_detail'
  CREATE TABLE opc_app_path_detail
  (
    plan_id INT DEFAULT 0 NOT NULL,
    app_name NVARCHAR (20) NOT NULL,
    path_id INT NOT NULL,
    gen INT NOT NULL,
    obj_cid ClassID NOT NULL,
    obj_name NVARCHAR (255) NULL,
    obj_desc NVARCHAR (255) NULL,
    obj_id ObjID DEFAULT 0 NOT NULL 
  )
  ALTER TABLE opc_app_path_detail ADD CONSTRAINT opc_app_path_det_PK PRIMARY KEY CLUSTERED(plan_id, app_name, path_id, gen)
/*
  ALTER TABLE opc_app_path_detail ADD CONSTRAINT opc_app_path_det_app_FK FOREIGN KEY (plan_id, app_name, path_id) REFERENCES opc_app_aggr (plan_id, app_name, path_id)
  ALTER TABLE opc_app_path_detail ADD CONSTRAINT opc_app_path_det_gen_FK FOREIGN KEY (path_id, gen) REFERENCES dyn_obj_path_detail (path_id, gen)
*/
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_pattern' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_pattern'
  CREATE TABLE opc_app_pattern
  (
    id INT NOT NULL,
    plan_name_pat VARCHAR (128) NOT NULL,
    app_name_pat VARCHAR (100) NOT NULL,
    app_owner_pat VARCHAR (100) NOT NULL,
    app_desc_pat VARCHAR (255) NOT NULL,
    priority INT NULL,
    path_id INT NOT NULL 
  )
  ALTER TABLE opc_app_pattern ADD CONSTRAINT opc_app_pattern_PK PRIMARY KEY CLUSTERED(plan_name_pat, app_name_pat, app_owner_pat, app_desc_pat, path_id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_daily_plan' AND type = 'U')
BEGIN
  PRINT 'Creating opc_daily_plan'
  CREATE TABLE opc_daily_plan
  (
    id INT IDENTITY (1, 1) NOT NULL,
    name NVARCHAR (255) NULL,
    start_date DATETIME NULL,
    end_date DATETIME NULL 
  )
  ALTER TABLE opc_daily_plan ADD CONSTRAINT opc_daily_plan_PK PRIMARY KEY CLUSTERED(id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_application' AND type = 'U')
BEGIN
  PRINT 'Creating opc_application'
  CREATE TABLE opc_application
  (
    plan_id INT NOT NULL,
    app_id INT NOT NULL,
    app_name NVARCHAR (20) NULL,
    app_owner NVARCHAR (40) NULL,
    start_day SMALLINT NULL,
    start_day_mod SMALLINT NULL,
    start_hour SMALLINT NULL,
    stop_day SMALLINT NULL,
    stop_day_mod SMALLINT NULL,
    stop_hour SMALLINT NULL,
    app_desc NVARCHAR (255) NULL,
    est_start DATETIME NULL,
    est_stop DATETIME NULL 
  )
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_id_UQ PRIMARY KEY CLUSTERED(plan_id, app_id)
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_start_day_mod_CK CHECK(([start_day_mod] >= (-1) and [start_day_mod] <= 1))
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_stop_day_mod_CK CHECK(([stop_day_mod] >= (-1) and [stop_day_mod] <= 1))
  ALTER TABLE opc_application ADD CONSTRAINT opc_app_plan_FK FOREIGN KEY (plan_id) REFERENCES opc_daily_plan (id)
END
GO


IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_op_pattern' AND type = 'U')
BEGIN
  PRINT 'Creating opc_op_pattern'
  CREATE TABLE opc_op_pattern
  (
    name VARCHAR (100) NOT NULL,
    op_type TINYINT NOT NULL,
    priority INT NOT NULL,
    description VARCHAR (255) NULL,
    plan_name_pat VARCHAR (128) NOT NULL,
    app_name_pat VARCHAR (100) NOT NULL,
    app_owner_pat VARCHAR (100) NOT NULL,
    app_desc_pat VARCHAR (255) NOT NULL,
    op_operation_pat VARCHAR (100) NOT NULL,
    op_job_pat VARCHAR (100) NOT NULL,
    op_desc_pat VARCHAR (100) NOT NULL 
  )
  ALTER TABLE opc_op_pattern ADD CONSTRAINT opc_op_pattern_PK PRIMARY KEY CLUSTERED(name)
  ALTER TABLE opc_op_pattern ADD CONSTRAINT opc_op_pattern_type_CK CHECK(([op_type] >= 1 and [op_type] <= 2))
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_operation' AND type = 'U')
BEGIN
  PRINT 'Creating opc_operation'
  CREATE TABLE opc_operation
  (
    plan_id INT NOT NULL,
    app_id INT NULL,
    app_name NVARCHAR (20) NULL,
    op_id INT NOT NULL,
    op_name NVARCHAR (8) NULL,
    job_name NVARCHAR (8) NULL,
    status nchar (1) NULL,
    start_day SMALLINT NULL,
    start_day_mod SMALLINT NULL,
    start_hour SMALLINT NULL,
    duration SMALLINT NULL,
    stop_day SMALLINT NULL,
    stop_day_mod SMALLINT NULL,
    stop_hour SMALLINT NULL,
    op_desc NVARCHAR (255) NULL,
    initpri INT NULL,
    termpri INT NULL,
    est_start DATETIME NULL,
    est_stop DATETIME NULL,
    op_type TINYINT NULL,
    btcy_id ObjID NULL,
    job_id INT NULL,
    instanceno INT NULL 
  )
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_PK PRIMARY KEY CLUSTERED(plan_id, op_id)
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_start_day_mod_CK CHECK(([start_day_mod] >= (-1) and [start_day_mod] <= 1))
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_stop_day_mod_CK CHECK(([stop_day_mod] >= (-1) and [stop_day_mod] <= 1))
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_type_CK CHECK(([op_type] >= 0 and [op_type] <= 2))
  ALTER TABLE opc_operation ADD CONSTRAINT opc_op_app_FK FOREIGN KEY (plan_id, app_id) REFERENCES opc_application (plan_id, app_id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_predecessor' AND type = 'U')
BEGIN
  PRINT 'Creating opc_predecessor'
  CREATE TABLE opc_predecessor
  (
    plan_id INT NOT NULL,
    succ_op_id INT NOT NULL,
    pred_app_name NVARCHAR (20) NOT NULL,
    pred_op_name NVARCHAR (8) NOT NULL,
    pred_op_id INT NULL 
  )
  ALTER TABLE opc_predecessor ADD CONSTRAINT opc_predecessor_PK PRIMARY KEY CLUSTERED(plan_id, succ_op_id, pred_app_name, pred_op_name)
  ALTER TABLE opc_predecessor ADD CONSTRAINT opc_predecessor_succ_op_FK FOREIGN KEY (plan_id, succ_op_id) REFERENCES opc_operation (plan_id, op_id)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'opc_app_new_instance' AND type = 'U')
BEGIN
  PRINT 'Creating opc_app_new_instance'
  CREATE TABLE opc_app_new_instance
  (
    btcy_name ObjName PRIMARY KEY,
    btcy_desc NVARCHAR(255),
    row       INT IDENTITY(0,1)
  )
END
GO

---------------------------------------------------------------------------------------------------
-- asisp_reset_opc_daily_plan
PRINT 'Creating procedure asisp_reset_opc_daily_plan'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_reset_opc_daily_plan' AND type = 'P')
  DROP PROCEDURE asisp_reset_opc_daily_plan
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/OPCLoad_OPCLoadServer.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 01 2001 13:34:38  $
-- $Author:   BRENT  $
-- Resets the opc tables FOR the daily plan with the specified name AND start/END dates. The plan
-- IS CREATE IN the opc_daily_plan TABLE IF necessary.
-- The PROCEDURE selects the unique id of the p.
CREATE PROCEDURE asisp_reset_opc_daily_plan
  @name       NVARCHAR(255),
  @start_date DATETIME,
  @end_date   DATETIME
AS
  SET NOCOUNT ON
  IF NOT EXISTS (SELECT * FROM opc_daily_plan WHERE name = @name AND start_date = @start_date AND end_date = @end_date)
    INSERT opc_daily_plan(name, start_date, end_date) SELECT @name, @start_date, @end_date
  DECLARE @plan_id INT
  SELECT @plan_id = id FROM opc_daily_plan WHERE name = @name AND start_date = @start_date AND end_date = @end_date
  DELETE opc_predecessor WHERE plan_id = @plan_id
  DELETE opc_operation WHERE plan_id = @plan_id
  DELETE opc_application WHERE plan_id = @plan_id
  SELECT @plan_id AS id
GO


---------------------------------------------------------------------------------------------------
-- asisp_checkpointmessage
PRINT 'Creating procedure asisp_checkpointmessage'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_checkpointmessage' AND type = 'P')
  DROP PROCEDURE asisp_checkpointmessage
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/OPCLoad_OPCLoadServer.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 01 2001 13:34:38  $
-- $Author:   BRENT  $
CREATE PROCEDURE asisp_checkpointmessage
  @message  NVARCHAR(255),
  @level    INT = 0,
  @chkpt    INT = 1,
  @state    INT = 1
AS
  DECLARE @d NVARCHAR(40)
  SELECT @d = CONVERT(NVARCHAR(40), GETDATE(), 113)
  -- Disregarding passed IN @level, @state AS 
  -- ADO IS interpreting all message #50000 AS errors.
  RAISERROR('%s: #%d: %s', 0, 1, @d, @chkpt, @message)
GO
