-------------------------------------------------------------------------------
-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/Process_DB2_MESG_Event.sqv   1.0   Sep 07 2001 16:50:12   Forthman  $
-------------------------------------------------------------------------------

Use Object
GO
-------------------------------------------------------------------------------
-- Process_DB2_MESG_Event

PRINT 'create procedure Process_DB2_MESG_Event'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'Process_DB2_MESG_Event' AND type = 'P' )
  DROP PROCEDURE Process_DB2_MESG_Event
GO

-- Creates a state change message ON a resource managed by DB2.
--
-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/Process_DB2_MESG_Event.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:50:12  $
-- $Author:   Forthman  $
CREATE PROCEDURE Process_DB2_MESG_Event
  @SourceCID                        ClassID,
  @SourceID                         ObjID,
  @FormatType                       SMALLINT,
  @ActionType                       SMALLINT,
  @DATE_TIME                        NVARCHAR(30),
  @SEQ_NO                           NVARCHAR(50),
  @OBJ_TYPE_ID                      NVARCHAR(50),
  @EventSource                      NVARCHAR(50)=NULL,
  @SystemId                         NVARCHAR(50),
  @SUBSYSTEM                        NVARCHAR(50),
  @MasterAddressSpace               NVARCHAR(50)=NULL,
  @LockManager                      NVARCHAR(50)=NULL,
  @DatabaseServices                 NVARCHAR(50)=NULL,
  @DataDistributionFacility         NVARCHAR(50)=NULL,
  @StoredProcedureManager           NVARCHAR(50)=NULL,
  @BUFFER_POOLS                     NVARCHAR(50)=NULL,
  @EDM_POOL                         NVARCHAR(50)=NULL,
  @LOGS                             NVARCHAR(50)=NULL,
  @DB2_PM                           NVARCHAR(50)=NULL,
  @DATABASE                         NVARCHAR(50)=NULL,
  @TABLE_SPACE                      NVARCHAR(50)=NULL, 
  @TABLE                            NVARCHAR(50)=NULL,
  @INDEX                            NVARCHAR(50)=NULL,
  @TABLESPACE_PARTITION             NVARCHAR(50)=NULL,
  @INDEX_PARTITION                  NVARCHAR(50)=NULL,
  @DB2_VIEW                         NVARCHAR(50)=NULL,
  @Event                            NVARCHAR(255),
  @EventText                        NVARCHAR(255)=NULL
AS
  RAISERROR('Process_DB2_MESG_Event: @SourceCID=%s, @SourceID=%d, @FormatType=%d, @ActionType=%d, @DATE_TIME=%s, @SEQ_NO=%s, @OBJ_TYPE_ID=%s, @SystemId=%s, @SUBSYSTEM=%s', 0, 1, @SourceCID, @SourceID, @FormatType, @ActionType, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @SystemId, @SUBSYSTEM)

  SELECT @SystemId=ISNULL(@SystemId,'')

  -- Initialization FOR the timings of steps
  DECLARE @startdate DATETIME
  DECLARE @nowdate   DATETIME
  DECLARE @stepdate  DATETIME
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  NVARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT
  DECLARE @debug INT

  SELECT @debug = 0

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             ( s_step/ s_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN Process_DB2_MESG_Event
  ELSE
    SAVE TRAN Process_DB2_MESG_Event

  -- HACK ************************
  IF @OBJ_TYPE_ID='DBDD' AND @DataDistributionFacility IS NULL AND @LockManager IS NOT NULL
  BEGIN
    RAISERROR('Process_DB2_MESG_Event: *** HACK *** ObjectType=DBDD, DataDistributionFacility(20) data came IN LockManager(18) data type', 0, 1)
    SELECT @DataDistributionFacility=@LockManager
    SELECT @LockManager=NULL
  END
  -- HACK ************************

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'HACK'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @datetime DATETIME
  EXEC asisp_convertMVSDateTime @DATE_TIME, @datetime OUTPUT

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'convertMVSDateTime'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- obtain the cid,id FROM the discovery process
  RAISERROR('Process_DB2_MESG_Event: obtaining cid, id FROM the discovery process', 0, 1)
  DECLARE @rc INT, @obj_cid ClassID, @obj_id ObjID, @obj_name ObjName
  EXEC @rc=asisp_discoverDB2Resource
                  @FormatType,
                  @ActionType,
                  @DATE_TIME,
                  @SEQ_NO,
                  @OBJ_TYPE_ID,
                  @SystemId,
                  @SUBSYSTEM,
                  @MasterAddressSpace,
                  @LockManager,
                  @DatabaseServices,
                  @DataDistributionFacility,
                  @StoredProcedureManager,
                  @BUFFER_POOLS,
                  @EDM_POOL,
                  @LOGS,
                  @DB2_PM,
                  @DATABASE,
                  @TABLE_SPACE, 
                  @TABLE,
                  @INDEX,
                  @TABLESPACE_PARTITION,
                  @INDEX_PARTITION,
                  @DB2_VIEW,
                  @STATE=NULL,
                  @auto_create=1,               -- boolean whether/NOT to auto-CREATE the object IF NOT found
                  @obj_cid=@obj_cid OUTPUT,
                  @obj_id=@obj_id OUTPUT,
                  @obj_name=@obj_name OUTPUT
  IF @rc<0
  BEGIN
    RAISERROR('Process_DB2_MESG_Event: error finding cid, id FOR DB2 object', 11, 1)
    ROLLBACK TRAN Process_DB2_MESG_Event
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'asisp_discoverDB2Resource'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @obj_cid IS NULL OR @obj_id IS NULL
  BEGIN
    RAISERROR('Process_DB2_MESG_Event: could NOT find cid, id FOR DB2 object', 0, 1)
    IF @tcount = 0
      COMMIT TRAN Process_DB2_MESG_Event
    RETURN 0
  END
  RAISERROR('Process_DB2_MESG_Event: found @cid=%s, @id=%d', 0, 1, @obj_cid, @obj_id)

  -- obtain native key
  DECLARE @native_key NVARCHAR(10)
  EXEC asisp_idtonativekey @obj_cid, @obj_id, @native_key OUTPUT
  RAISERROR('Process_DB2_MESG_Event: obtained native_key=%s', 0, 1, @native_key)

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'obtain native key'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- CREATE the message description IF NOT defined
  IF NOT EXISTS (SELECT * FROM MessageDescription_T WHERE obj_cid=@obj_cid AND obj_id IN (0, @obj_id) AND MsgID=@Event)
  BEGIN
    RAISERROR('Process_DB2_MESG_Event: auto-creating message description [%s]=(%s)', 0, 1, @Event, @EventText)
    EXEC asisp_defineMessageDescription
      @cid                = @obj_cid,
      @MsgID              = @Event,
      @Description        = @EventText,
      @State              = 'Unknown',
      @AlertState         = 'Green',
      @Priority           = 'Ignore',
      @Flags              = 1, -- 1
      @id                 = 0
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'create message if not defined'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- INSERT INTO the staging TABLE
  RAISERROR('Process_DB2_MESG_Event: inserting event INTO StagedB2_MESG', 0, 1)
  INSERT StagedDB2_MESG(obj_cid, obj_id, obj_name, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, EventSource,
            SystemId, SUBSYSTEM, MasterAddressSpace, LockManager, DatabaseServices,
            DataDistributionFacility, StoredProcedureManager, BUFFER_POOLS, EDM_POOL,
            LOGS, DB2_PM, [DATABASE], TABLE_SPACE, [TABLE], [INDEX], TABLESPACE_PARTITION,
            INDEX_PARTITION, DB2_VIEW, Event, EventText, NATIVE_KEY)
    SELECT  @obj_cid, @obj_id, @obj_name, @datetime, @SEQ_NO, @OBJ_TYPE_ID, @EventSource,
            @SystemId, @SUBSYSTEM, @MasterAddressSpace, @LockManager, @DatabaseServices,
            @DataDistributionFacility, @StoredProcedureManager, @BUFFER_POOLS, @EDM_POOL,
            @LOGS, @DB2_PM, @DATABASE, @TABLE_SPACE, @TABLE, @INDEX, @TABLESPACE_PARTITION,
            @INDEX_PARTITION, @DB2_VIEW, @Event, @EventText, @native_key

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In StagedDB2_MESG'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN Process_DB2_MESG_Event

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Finish Process_DB2_MESG_Event'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
GO