-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/Process_EQQE036I.SQv   1.0   Aug 22 2001 19:34:26   BRENT  $
--
-- Process_EQQE036I.sql
--
-- Module Description
--   Defines the Process_EQQE036I stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- Process_EQQE036I
PRINT 'Creating procedure Process_EQQE036I'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_EQQE036I' AND type = 'P')
  DROP PROCEDURE Process_EQQE036I
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/Process_EQQE036I.SQv  $
-- $Revision:   1.0  $
-- $Date:   Aug 22 2001 19:34:26  $
-- $Author:   BRENT  $
--
-- Processes a EQQE036I event.
--
-- Events that have a blank job name are recorded in the RejectedOPCEvent table.
-- These need to be reviewed so that the regular expressions can be tuned.
--
-- If the PRTY is non-numeric and IA is blank then it is assumed that PRTY is
-- actually missing and so the parameters are shifted to the right such that
-- @IA = @WKSTNID @WKSTNID = @APPID, and @APPID = @PRTY.
CREATE PROCEDURE Process_EQQE036I
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @TIME               DATETIME,
  @SEQ_NO             NVARCHAR(50),
  @MESSAGE            NVARCHAR(16),
  @MESSAGE_TEXT       NVARCHAR(255),
  @JOBNAME            NVARCHAR(8),
  @OPID               NVARCHAR(10), -- this is really the job #, not the opid
  @EC                 NVARCHAR(8),
  @PRTY               NVARCHAR(16) = NULL,
  @APPID              NVARCHAR(16),
  @WKSTNID            NVARCHAR(8),
  @IA                 NVARCHAR(16)
AS
  RAISERROR('Process_EQQE036I: cid=%s, id=%d, jobname=%s, opid=%s, ec=%s, prty=%s, appid=%s, wksntid=%s, ia=%s', 0, 1, @SourceCID, @SourceID, @JOBNAME, @OPID, @EC, @PRTY, @APPID, @WKSTNID, @IA)

  -- Standard transaction logic
  DECLARE @tcount int
  SELECT @tcount = @@trancount
  IF @tcount > 0
    SAVE TRANSACTION Process_EQQE036I
  ELSE
    BEGIN TRANSACTION Process_EQQE036I

  -- Make sure job name is specified
  IF @JOBNAME = ''
  BEGIN
    INSERT RejectedOPCEvent(SourceCID, SourceID, TIME, SEQ_NO, MESSAGE, MESSAGE_TEXT)
      SELECT @SourceCID, @SourceID, @TIME, @SEQ_NO, @MESSAGE, @MESSAGE_TEXT
    
    -- Finish up transaction (NOT AN ERROR)
    IF @tcount = 0
      COMMIT TRANSACTION Process_EQQE036I
    RETURN
  END

  -- Map priority string to integer (blank/NULL==>0)
  DECLARE @PRTY_VAL INT
  IF ISNULL(@PRTY,'') = ''
    SET @PRTY_VAL = 0
  ELSE
    SET @PRTY_VAL = CAST(@PRTY AS INT)

  -- Lookup containing OS. Event is entered only if the calling object is a valid entry in the OS
  DECLARE @src_cid ClassID, @src_id ObjID
  SELECT @src_cid = src_cid, @src_id = src_id FROM containment AS c WHERE dst_cid = @SourceCID AND dst_id = @SourceID
  IF @src_cid = 'OS' AND @src_id IS NOT NULL
    INSERT StagedOPCEvent(src_cid, src_id, TIME, SEQ_NO, MESSAGE, MESSAGE_TEXT, APPID, WKSTNID, OPID, JOBNAME, PRTY, IA, EC)
      SELECT @src_cid, @src_id, @TIME, @SEQ_NO, @MESSAGE, @MESSAGE_TEXT, @APPID, @WKSTNID, NULL, @JOBNAME, @PRTY_VAL, @IA, @EC

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION Process_EQQE036I
GO
