-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/Process_EQQE039I.SQv   1.0   Aug 22 2001 19:34:26   BRENT  $
--
-- Process_EQQE039I.sql
--
-- Module Description
--   Defines the Process_EQQE039I stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- Process_EQQE039I
PRINT 'Creating procedure Process_EQQE039I'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_EQQE039I' AND type = 'P')
  DROP PROCEDURE Process_EQQE039I
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/Process_EQQE039I.SQv  $
-- $Revision:   1.0  $
-- $Date:   Aug 22 2001 19:34:26  $
-- $Author:   BRENT  $
-- Modified by EBH on 8/13/01 to ignore the @WKSTNID and @OPID parameters.
-- The @OPID parameter is actually the job number for the current run, not the
-- operation id. Without this information the @WKSTNID parameter is not useful
-- for determining the distinct operation.
CREATE PROCEDURE Process_EQQE039I
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @TIME               DATETIME,
  @SEQ_NO             NVARCHAR(50),
  @MESSAGE            NVARCHAR(16),
  @MESSAGE_TEXT       NVARCHAR(255),
  @JOBNAME            NVARCHAR(8),
  @OPID               NVARCHAR(10), -- this is really the job number, not the opid
  @APPID              NVARCHAR(16),
  @WKSTNID            NVARCHAR(8),
  @IA                 NVARCHAR(16)
AS
  RAISERROR('Process_EQQE039I: cid=%s, id=%d, jobname=%s, opid=%s, appid=%s, wksntid=%s, ia=%s', 0, 1, @SourceCID, @SourceID, @JOBNAME, @OPID, @APPID, @WKSTNID, @IA)

  -- Standard transaction logic
  DECLARE @tcount int
  SELECT @tcount = @@trancount
  IF @tcount > 0
    SAVE TRANSACTION Process_EQQE039I
  ELSE
    BEGIN TRANSACTION Process_EQQE039I

  -- Make sure job name is specified
  IF @JOBNAME = ''
  BEGIN
    INSERT RejectedOPCEvent(SourceCID, SourceID, TIME, SEQ_NO, MESSAGE, MESSAGE_TEXT)
      SELECT @SourceCID, @SourceID, @TIME, @SEQ_NO, @MESSAGE, @MESSAGE_TEXT
    
    -- Finish up transaction (NOT AN ERROR)
    IF @tcount = 0
      COMMIT TRANSACTION Process_EQQE039I
    RETURN
  END

  -- Lookup containing OS. Event is entered only if the calling object is a valid entry in the OS
  DECLARE @src_cid ClassID, @src_id ObjID
  SELECT @src_cid = src_cid, @src_id = src_id FROM containment AS c WHERE dst_cid = @SourceCID AND dst_id = @SourceID
  IF @src_cid = 'OS' AND @src_id IS NOT NULL
    INSERT StagedOPCEvent(src_cid, src_id, TIME, SEQ_NO, MESSAGE, MESSAGE_TEXT, APPID, WKSTNID, OPID, JOBNAME, IA)
      SELECT @src_cid, @src_id, @TIME, @SEQ_NO, @MESSAGE, @MESSAGE_TEXT, @APPID, NULL, NULL, @JOBNAME, @IA

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION Process_EQQE039I
GO
