-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/Process_Identification_Pump_TBSM_1.1.1_patch.sqv   1.0   Sep 07 2001 16:50:16   Forthman  $
--

---------------------------------------------------------------------------------------------------
-- Process_Identification_Pump
PRINT 'Creating procedure Process_Identification_Pump'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_Identification_Pump' AND type = 'P')
  DROP PROCEDURE Process_Identification_Pump
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/Process_Identification_Pump_TBSM_1.1.1_patch.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:50:16  $
-- $Author:   Forthman  $
CREATE PROCEDURE Process_Identification_Pump
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @DATE_TIME          VARCHAR(30) = NULL,
  @SEQ_NO             VARCHAR(50) = NULL,
  @OBJ_TYPE_ID        VARCHAR(4) = NULL,
  @OBJECT_NAME        VARCHAR(54) = NULL, -- SMFID
  @IPL_DATE           VARCHAR(10) = NULL,
  @IPL_TIME           VARCHAR(15) = NULL,
  @IP_ADDRESS         VARCHAR(15) = NULL,
  @IP_DNS_NAME        VARCHAR(255) = NULL,
  @QUALIFIED_NETNAME  VARCHAR(255) = NULL
AS
  RAISERROR('Process_Identification_Pump: @src_cid=%s, @src_id=%ld, @FormatType=%d, @ActionType=%d, @DATE_TIME=%s, @SEQ_NO=%s, @OBJ_TYPE_ID=%s, @OBJECT_NAME=%s, @IPL_DATE=%s, @IPL_TIME=%s, @IP_ADDRESS=%s, @IP_DNS_NAME=%s, @QUALIFIED_NETNAME=%s', 0, 1, @SourceCID, @SourceID, @FormatType, @ActionType, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @OBJECT_NAME, @IPL_DATE, @IPL_TIME, @IP_ADDRESS, @IP_DNS_NAME, @QUALIFIED_NETNAME)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN Process_Identification_Pump
  ELSE
    SAVE TRAN Process_Identification_Pump

  DECLARE @r INT, @datetime DATETIME, @ipl_datetime DATETIME

  -- convert date/time data
  SELECT @ipl_datetime=CONVERT(DATETIME, @IPL_DATE+' '+@IPL_TIME)  -- may be NULL
  EXEC asisp_convertMVSDateTime @DATE_TIME, @datetime OUTPUT

  -- ensure that OBJ_TYPE_ID is 'OS'
  IF @OBJ_TYPE_ID <> 'OS'
  BEGIN
    RAISERROR('Process_Identification_Pump: @OBJ_TYPE_ID must be ''OS''', 6, 1)
    ROLLBACK TRAN Process_Identification_Pump
    RETURN -30000
  END

  -- update the OS_A._IPLDateTime attribute with the @datetime (@IPL_DATE+@IPL_TIME)
  IF @datetime IS NOT NULL
    UPDATE OS_A
      SET _IPLDateTime=@datetime
      WHERE id=@SourceID AND ISNULL(_IPLDateTime,'')<>@datetime

  -- update the OS_A._SMFID attribute with the @OBJECT_NAME
  IF @OBJECT_NAME IS NOT NULL
    UPDATE OS_A
      SET _SMFID=@OBJECT_NAME
      WHERE id=@SourceID AND ISNULL(_SMFID,'')<>@OBJECT_NAME

  -- update the OS_A._IPAddress attribute with the @IP_ADDRESS
  IF @IP_ADDRESS IS NOT NULL
    UPDATE OS_A
      SET _IPAddress=@IP_ADDRESS
      WHERE id=@SourceID AND ISNULL(_IPAddress,'')<>@IP_ADDRESS

  -- update the OS_A._IPDNSName attribute with the @IP_DNS_NAME
  IF @IP_DNS_NAME IS NOT NULL
    UPDATE OS_A
      SET _IPDNSName=@IP_DNS_NAME
      WHERE id=@SourceID AND ISNULL(_IPDNSName,'')<>@IP_DNS_NAME

  -- update the OS_A._SystemId (netid.cpname) attribute with the @QUALIFIED_NETNAME
  IF @QUALIFIED_NETNAME IS NOT NULL
    UPDATE OS_A
      SET _SystemId=@QUALIFIED_NETNAME
      WHERE id=@SourceID AND ISNULL(_SystemId,'')<>@QUALIFIED_NETNAME

  -- create the message
  EXEC @r = asisp_createObjPumpIdentMsg @SourceCID, @SourceID, @OBJ_TYPE_ID, @OBJECT_NAME, @datetime, @SEQ_NO

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRAN Process_Identification_Pump

  RETURN @r
GO

-----------------------------------------------------------------------------------------------------------------------------------
PRINT 'Done.'
GO
