-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/RecordBatchCurrentStateChange.sqv   1.3   Aug 08 2001 18:29:46   BRENT  $
--
-- RecordBatchCurrentStateChange.sql
--
-- Module Description
--   Defines the RecordBatchCurrentStateChange stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- RecordBatchCurrentStateChange
PRINT 'Creating procedure RecordBatchCurrentStateChange'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RecordBatchCurrentStateChange' AND type = 'P')
  DROP PROCEDURE RecordBatchCurrentStateChange
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/RecordBatchCurrentStateChange.sqv  $
-- $Revision:   1.3  $
-- $Date:   Aug 08 2001 18:29:46  $
-- $Author:   BRENT  $
-- Called as a MessageDescription hook to update the StartTime and StopTime of the batch job
CREATE PROCEDURE RecordBatchCurrentStateChange
  @cid        ClassID,
  @id         ObjID,
  @newState   INT,
  @evt_cid    ClassID,
  @evt_id     ObjID,
  @MESSAGE    NVARCHAR(255),
  @SEQ_NO     NVARCHAR(50),
  @REASON_CD  NVARCHAR(255),
  @time       DATETIME=NULL
AS
  -- Filter LOBs
  IF @cid = 'LOB'
    RETURN

/*-- Diagnostic code for putting record into auditing table
  INSERT RecordBatchCurrentStateChangeAudit SELECT
    @id,
    @newState,
    @evt_cid,
    @evt_id,
    @MESSAGE,
    @SEQ_NO,
    @REASON_CD,
    @time
*/

--  RAISERROR('RecordBatchCurrentStateChange @id=%d, @newState=%d, @REASON_CD=''%s''', 0, 1, @id, @newState, @REASON_CD)
  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount > 0
    SAVE TRAN RecordBatchCurrentStateChange
  ELSE
    BEGIN TRAN RecordBatchCurrentStateChange

  -- Lookup current state values
  DECLARE @cs INT, @ds INT, @ParentCID ClassID, @ParentID ObjID, @job_name ObjName
  SELECT
    @job_name = name,
    @cs = _CurrentStateID,
    @ds = _DesiredStateID,
    @ParentCID = _ParentCID,
    @ParentID = _ParentID
  FROM BATCH_V
  WHERE id = @id

  -- Batch states
  DECLARE @Running INT, @Completed INT, @Abended INT
  SET @Running = 82
  SET @Completed = 83
  SET @Abended = 84

  -- If no time was specified then look it up from the message
  IF @time IS NULL
    SELECT @time = _TIME FROM Message_A WHERE id = @evt_id

  -- If there was no state change then we just update the start/stop times and total/good run counters
  IF @newState = @cs
  BEGIN
    IF @newState = @Running
      UPDATE BATCH_V SET _StartTime = @time, _StopTime = NULL, _TotalRuns = ISNULL(_TotalRuns,0) + 1 WHERE id = @id
    ELSE IF @newState = @Completed
      UPDATE BATCH_V SET _StopTime = @time, _GoodRuns = ISNULL(_GoodRuns,0) + 1 WHERE id = @id

    IF @tcount = 0
      COMMIT TRAN RecordBatchCurrentStateChange
    RETURN
  END

  -- If the new state is Completed and the current state is Abended then nothing is
  -- recorded since there will not actually be a state change
  IF @newState = @Completed
  BEGIN
    IF @cs = @Abended
    BEGIN
      IF @tcount = 0
        COMMIT TRAN RecordBatchCurrentStateChange
      RETURN
    END
  END

  -- BatchCycle states
  DECLARE @BTCY_Pending INT, @BTCY_Inactive INT, @BTCY_Running INT, @BTCY_Completed INT
  SET @BTCY_Pending   = 90
  SET @BTCY_Inactive  = 91
  SET @BTCY_Running   = 92
  SET @BTCY_Completed = 93

  DECLARE @ParentState INT, @ParentName ObjName
  SELECT @ParentState = _CurrentStateID, @ParentName = name FROM BatchCycle_V WHERE id = @ParentID

  -- Update the StartTime and/or StopTime fields of the job
  DECLARE @STATE NVARCHAR(16)
  IF @newState = @Running
  BEGIN
    UPDATE BATCH_V SET _StartTime = @time, _StopTime = NULL, _TotalRuns = ISNULL(_TotalRuns,0) + 1 WHERE id = @id
    IF @ParentCID = 'BTCY' AND @ParentState IS NOT NULL
    BEGIN
      IF @ParentState <> @BTCY_Running
        SELECT @STATE = 'Running'
    END
  END
  ELSE IF @newState = @Completed
  BEGIN
    UPDATE BATCH_V SET _StopTime = @time, _GoodRuns = ISNULL(_GoodRuns,0) + 1 WHERE id = @id
  END
  ELSE IF @newState = @Abended
  BEGIN
    UPDATE BATCH_V SET _StopTime = @time WHERE id = @id
  END

  -- If @STATE is set this indicates that the message must be reflected up to the parent schedule
  IF @STATE IS NOT NULL
  BEGIN
    -- If there is no text in the message, then make it more informative by putting in the
    -- schedule name, job name, and its start time
    IF ISNULL(@REASON_CD,'') = ''
      SET @REASON_CD = FORMATMESSAGE(58002, @ParentName, @job_name, CONVERT(NVARCHAR(16), @time, 120))

    -- Create the message
    EXEC asisp_createMESG
      @MESSAGE,
      @ParentCID,
      @ParentID,
      @time,
      @time,
      0,
      @SEQ_NO,
      'BTCY',
      @ParentName,
      @NATIVE_KEY = '',
      @STATE = @STATE,
      @MESSAGE = @MESSAGE,
      @REASON_CD = @REASON_CD

    -- Update the attributes on the parent. In particular the _CurrentStateID column should be set so
    -- if another job starts before the parent's message has been processed we will not generate
    -- another message.
    UPDATE BatchCycle_V SET
      _CurrentStateID = @BTCY_Running,
      _StartTime = @time,
      _StopTime = NULL,
      _TotalRuns = ISNULL(_TotalRuns,0) + 1
    WHERE id = @ParentID
    -- Send notifications of the change for CurrentStateID and CurrentState attributes
    EXEC asisp_notifyObjAttrUpdated @ParentCID, @ParentID, @BTCY_Running, 'CurrentStateID'
    DECLARE @state_str NVARCHAR(60)
    SET @state_str = RTRIM(@cid) + '/STCR-1/STTP-' + CAST(@BTCY_Running AS NVARCHAR) + '/T/State'
    EXEC asisp_notifyObjAttrUpdated @ParentCID, @ParentID, @state_str, "CurrentState"
  END

/*
  -- Convert time to integer representation
  DECLARE @stime SMALLDATETIME, @itime INT
  SELECT @stime = CONVERT(SMALLDATETIME, @time)
  EXEC @itime = sp_date2int @stime

  -- Send state change message to rule engine
  INSERT PendingBatchRule(type, fields)
    SELECT 0, 
      '(assert (state_change_event ' + 
        '(cid BATC) ' + 
        '(id ' + CONVERT(NVARCHAR(9), @id) + ') ' + 
        '(mesg_id ' + CONVERT(NVARCHAR(9), @evt_id) + ') ' + 
        '(state ' + s._State + ') ' + 
        '(time ' + CONVERT(NVARCHAR(12), @itime) + ') ' + 
      '))'
    FROM StateType_V AS s
    WHERE id = @newState
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN RecordBatchCurrentStateChange
    RETURN -30000
  END

  INSERT PendingBatchRule(type, fields)
    SELECT 0, '(run)'
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN RecordBatchCurrentStateChange
    RETURN -30000
  END
*/

  -- Standard transaction logic
  IF @tcount = 0
    COMMIT TRAN RecordBatchCurrentStateChange
GO
