-- File: archive\db\sql\resetbatchclasses.sql
-- SID: 1.4
-- Date Modified: 01/11/19 15:45:21
--
-- Module Description
--   Defines the ResetBatchClasses stored procedure

USE Object
GO

------------------------------------------------------------------------------------------------
-- ResetBatchClasses
PRINT 'Create procedure ResetBatchClasses'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'ResetBatchClasses' AND type = 'P')
  DROP PROCEDURE ResetBatchClasses
GO

-- File: archive\db\sql\resetbatchclasses.sql
-- SID: 1.4
-- Date Modified: 01/11/19 15:45:21
-- Resets batch-related classes
CREATE PROCEDURE ResetBatchClasses
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION ResetBatchClasses
  ELSE
    SAVE TRANSACTION ResetBatchClasses

  -- Remove batch jobs from LOBs

  DELETE FROM lob
    FROM LineOfBusiness_C AS lob JOIN LOB_BCYS_LOBL AS lbl ON lob.id = lbl.src_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM lob
    FROM LineOfBusiness_C AS lob JOIN LOB_BTCY_LOBL AS lbl ON lob.id = lbl.src_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM lob
    FROM LineOfBusiness_C AS lob JOIN LOB_BATC_LOBL AS lbl ON lob.id = lbl.src_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM lbc
    FROM LOB_LOB_LOBC AS lbc
    WHERE NOT EXISTS (SELECT id FROM LineOfBusiness_C AS lob WHERE lob.id = lbc.dst_id)
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  -- Remove OPC processing records

  DELETE FROM opc_predecessor
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM opc_operation
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM opc_application
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM opc_daily_plan
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM opc_app_instance
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  -- Delete records from the OPCLoad database

  DELETE FROM OPCLoad..opc_predecessor
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM OPCLoad..opc_operation
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM OPCLoad..opc_application
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM OPCLoad..opc_daily_plan
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END

  DELETE FROM OPCLoad..opc_app_instance
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION ResetBatchClasses
    RETURN -30000
  END
  
  IF @tcount = 0
    COMMIT TRANSACTION ResetBatchClasses

  -- Clear out the classes
  EXEC asisp_resetclass BATC, BATCH
  EXEC asisp_resetclass BTCY, BatchCycle
  EXEC asisp_resetclass BCYS, BatchCycles
  
  -- Clear out the events
  DELETE FROM c FROM Message_C AS c JOIN Message_A AS a ON a.id = c.id WHERE a._ParentCID = 'BATC'
  DELETE FROM c FROM Exception_C AS c JOIN Exception_A AS a ON a.id = c.id WHERE a._ParentCID = 'BATC'
  DELETE FROM c FROM ExceptionHistory_C AS c JOIN ExceptionHistory_A AS a ON a.id = c.id 
    WHERE a._ParentID NOT IN (SELECT id FROM Exception_C)
  DELETE FROM c FROM BatchCycleJobHist AS c
GO
