-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/_CreateDiscoveredObjects.sqv   1.0   Sep 07 2001 16:50:22   Forthman  $
--
-- _CreateDiscoveredObjects.sql

USE Object
GO

---------------------------------------------------------------------------------------------------
-- _CreateDiscoveredObjects
PRINT 'Creating procedure _CreateDiscoveredObjects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateDiscoveredObjects' AND type = 'P')
  DROP PROCEDURE _CreateDiscoveredObjects
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/_CreateDiscoveredObjects.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:50:22  $
-- $Author:   Forthman  $
CREATE PROCEDURE _CreateDiscoveredObjects
  @cid                ClassID,
  @id                 ObjID,
  @batchid            INT,      -- Batch id. Can be NULL if discovery table does not have batchid.
  @DiscoveryTable     sysname,  -- Discovery table name. Gets aliased to 'd'
  @LinkTypeColumn     sysname,  -- Column specifying parent link_type. Can be a column name from the specified discovery table , the string 'NULL', or a literal surrounded by quotes (e.g. '''PHYC''')
  @ParentCIDColumn    sysname,  -- Column specifying the parent class id. Can be a column name from the specified discovery table or a literal string surrounded by quotes (e.g. '''OS''')
  @ParentIDColumn     sysname,  -- Column name specifying the parent object id. Can be a column name from the specified discovery table, or a literal integer value
  @ChildCIDColumn     sysname,  -- Column name specifying the child class id. Can be a column name from the specified discovery table, or a literal string surrounded by quotes (e.g. '''CICS''')
  @ChildIDColumn      sysname,  -- Column name specifying the child instance id. This MUST be a integer column from the specified discovery table. It also cannot be prefixed by the table alias.
  @ChildNameColumn    sysname,  -- Column name specifying the child instance name. Can be a column name from the specified discovery table, or a literal string surrounded by quotes (e.g. '''CICS0001''')
  @WhereClause        VARCHAR(500), -- WHERE clause (including the WHERE keyword) specifying which rows from the discovery table are records for objects to create.
  @AdditionalDiscoveryColumns  VARCHAR(500) = '', -- Optional string specifying a comma-separated list of columns from the discovery table that are to be selected for insertion into the new instance records.
                                                  -- If specified, must begin with a comma.
  @AdditionalAttributeColumns  VARCHAR(500) = '', -- Optional string specifying a comma-separated list of columns to be inserted into the object instance table.
                                                  -- This list must match in number the number of column values specified by the @AdditionalDiscoveryColumns parameter.
                                                  -- If specified, must begin with a comma.
  @verbose                     TINYINT = 0
AS
  DECLARE @rc INT
  SET @rc = 0

  IF @batchid IS NOT NULL
    SELECT @WhereClause = @WhereClause + ' AND d.batchid = ' + CAST(@batchid AS VARCHAR)

  DECLARE @sql VARCHAR(4000)

  CREATE TABLE #new_instance
  (
    link_type   VARCHAR(4) NULL,
    parent_cid  ClassID,
    parent_id   ObjID,
    child_cid   ClassID,
    child_name  ObjName,
    child_id    ObjID NULL
  )

  CREATE TABLE #new_id
  (
    link_type   VARCHAR(4) NULL,
    parent_cid  ClassID,
    parent_id   ObjID,
    child_cid   ClassID,
    child_name  ObjName,
    rowno       INT IDENTITY(0,1)    
  )

  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _CreateDiscoveredObjects: Populating table of new records'
  SELECT @sql = "
  INSERT INTO #new_instance(link_type, parent_cid, parent_id, child_cid, child_name, child_id)
  SELECT
    " + @LinkTypeColumn + ",
    " + @ParentCIDColumn + ",
    " + @ParentIDColumn + ",
    " + @ChildCIDColumn + ",
    " + @ChildNameColumn + ",
    NULL
  FROM
    " + @DiscoveryTable + " AS d
  " + @WhereClause
  IF @verbose = 1
    PRINT @sql
  EXEC(@sql)

  DECLARE newobjc CURSOR FOR
    SELECT DISTINCT n.child_cid, oc.cname
    FROM
      #new_instance AS n
      JOIN obj_class AS oc ON oc.cid = n.child_cid
  OPEN newobjc
  DECLARE @child_cid ClassID, @child_cname VARCHAR(24)
  FETCH NEXT FROM newobjc INTO @child_cid, @child_cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    DECLARE @nNew VARCHAR(9)
    SELECT @nNew = CAST(COUNT(*) AS VARCHAR) FROM #new_instance WHERE child_cid = @child_cid

    TRUNCATE TABLE #new_id
    INSERT INTO #new_id(link_type, parent_cid, parent_id, child_cid, child_name)
      SELECT link_type, parent_cid, parent_id, child_cid, child_name
      FROM #new_instance
      WHERE child_cid = @child_cid

    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _CreateDiscoveredObjects: Creating ' + @nNew + ' object(s) of class ' + @child_cname + ' (cid=' + @child_cid + ')'

    SELECT @sql = "
    -- Allocate ids
    DECLARE @base_id INT
    EXEC @base_id = alloc_" + @child_cid + " " + @nNew + "

    -- Create object records
    INSERT " + @child_cname + "_C(id, name" + @AdditionalAttributeColumns + ")
      SELECT @base_id + n.rowno, n.child_name" + @AdditionalDiscoveryColumns + "
      FROM
        #new_id AS n
        JOIN " + @DiscoveryTable + " AS d ON
          " + @ParentCIDColumn + " = n.parent_cid
          AND " + @ParentIDColumn + " = n.parent_id
          AND " + @ChildCIDColumn + " = n.child_cid
          AND " + @ChildNameColumn + " = n.child_name
      " + @WhereClause + "
         AND " + @ChildCIDColumn + " = '" + @child_cid + "'

    -- Update the child id column in the discovery table
    UPDATE d SET " + @ChildIDColumn + " = @base_id + n.rowno
    FROM
      #new_id AS n
      JOIN " + @DiscoveryTable + " AS d ON
        " + @ParentCIDColumn + " = n.parent_cid
        AND " + @ParentIDColumn + " = n.parent_id
        AND " + @ChildCIDColumn + " = n.child_cid
        AND " + @ChildNameColumn + " = n.child_name

    UPDATE inst SET child_id = @base_id + id.rowno 
    FROM
      #new_instance AS inst
      JOIN #new_id AS id ON
        id.parent_cid = inst.parent_cid
        AND id.parent_id = inst.parent_id
        AND id.child_cid = inst.child_cid
        AND id.child_name = inst.child_name
    "
    IF @verbose = 1
      PRINT @sql
    EXEC(@sql)
    SET @rc = @rc + @@ROWCOUNT

    FETCH NEXT FROM newobjc INTO @child_cid, @child_cname
  END
  DEALLOCATE newobjc

  -- If @ParentLinkType was specified, then we insert records into the appropriate link table(s)
  IF UPPER(@LinkTypeColumn) <> 'NULL'
  BEGIN
    DECLARE newlinkc CURSOR FOR
      SELECT DISTINCT RTRIM(link_type), RTRIM(parent_cid), RTRIM(child_cid) FROM #new_instance
    OPEN newlinkc
    DECLARE @link_type VARCHAR(4), @parent_cid VARCHAR(4)
    FETCH NEXT FROM newlinkc INTO @link_type, @parent_cid, @child_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      IF @verbose = 1
        PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _CreateDiscoveredObjects: Creating ' + @parent_cid + '_' + @child_cid + '_' + @link_type + ' record(s)'
      EXEC("
      INSERT " + @parent_cid + "_" + @child_cid + "_" + @link_type + "(src_id, dst_id)
      SELECT parent_id, child_id
      FROM #new_instance AS n
      WHERE link_type = '" + @link_type + "' AND parent_cid = '" + @parent_cid + "' AND child_cid = '" + @child_cid + "'
      ")
      FETCH NEXT FROM newlinkc INTO @link_type, @parent_cid, @child_cid
    END
    DEALLOCATE newlinkc

    -- Insert notifications of new links
    IF @verbose = 1
      PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _CreateDiscoveredObjects: Creating PendingNotification record(s)'
    INSERT PendingNotification(type, fields, detail)
      SELECT
        et.type,
        CAST(child.cno AS VARCHAR)  + '|' +
        CAST(n.child_id AS VARCHAR) + '|' + 
        n.child_cid                 + '|' + 
        n.link_type                 + '|' +
        CAST(parent.cno AS VARCHAR) + '|' + 
        CAST(n.parent_id AS VARCHAR) AS fields,
        n.child_name AS detail
      FROM
        #new_instance AS n,
        EventType AS et,
        obj_class AS child,
        obj_class AS parent
      WHERE
        et.name = 'ObjectCreated'
        AND child.cid = n.child_cid
        AND parent.cid = n.parent_cid
        AND n.link_type IS NOT NULL
  END

  IF @verbose = 1
    PRINT CONVERT(VARCHAR(40), GETDATE(), 121) + ': _CreateDiscoveredObjects: Finished'

  RETURN @rc
GO
