-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/_DeleteBatchJobs.sqv   1.1   Aug 22 2001 19:46:04   BRENT  $
--
-- _DeleteBatchJobs.sql
--
-- Module Description
--   Defines the _DeleteBatchJobs stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _DeleteBatchJobs
PRINT 'Creating procedure _DeleteBatchJobs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DeleteBatchJobs' AND type = 'P')
  DROP PROCEDURE _DeleteBatchJobs
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_DeleteBatchJobs.sqv  $
-- $Revision:   1.1  $
-- $Date:   Aug 22 2001 19:46:04  $
-- $Author:   BRENT  $
-- Deletes batch jobs matching a specified where-clause.
CREATE PROCEDURE _DeleteBatchJobs
  @cid          ClassID,
  @id           ObjID,
  @WhereClause  NVARCHAR(255)  -- where-clause for determining matching jobs. Includes the WHERE keyword.
                               -- Use the alias 'b' for select rows from BATCH_V.
AS
  DECLARE @count INT, @total INT

  -- Create temp table of batch jobs to be deleted
  CREATE TABLE #delete
  (
    id    ObjID NOT NULL PRIMARY KEY (id)
  )

  EXEC("
  INSERT INTO #delete
  SELECT DISTINCT b.id FROM BATCH_V AS b
  " + @WhereClause
  )
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  SELECT @count = COUNT(*) FROM #delete
  IF @count = 0
  BEGIN
    PRINT 'No matching rows to be deleted'
    RETURN 0
  END

  PRINT 'Deleting ' + CAST(@count AS NVARCHAR) + CASE WHEN @count=1 THEN ' row' ELSE ' rows' END
    + ' from BATCH_C'
  DECLARE @num_to_delete INT
  SET @num_to_delete = @count

/*
 Previously it was necessary to delete dependent records from opc_predecessor and
 opc_operation since they had foreign keys to BATCH_C. Now these tables are only
 used during the load process and the foreign keys have been removed.

  -- Delete opc_predecessor 
  SET NOCOUNT ON
  SET @count = 1
  SET @total = 0
  WHILE @count > 0
  BEGIN
    SET ROWCOUNT 100
    DELETE FROM p FROM opc_predecessor AS p 
      JOIN opc_operation AS o ON o.op_id = p.succ_op_id
      JOIN #delete AS d ON d.id = o.job_id
    SET @count = @@ROWCOUNT
    SET @total = @total + @count
    PRINT 'Deleted ' + CAST(@count AS NVARCHAR) + ' row(s) from opc_predecessor'
    SET ROWCOUNT 0
  END
  PRINT 'Deleted total of ' + CAST(@total AS NVARCHAR) + ' row(s) from opc_predecessor'

  SET NOCOUNT ON
  SET @count = 1
  SET @total = 0
  WHILE @count > 0
  BEGIN
    SET ROWCOUNT 1000
    DELETE FROM o FROM opc_operation AS o
      JOIN #delete AS d ON d.id = o.job_id
    SET @count = @@ROWCOUNT
    SET @total = @total + @count
    PRINT 'Deleted ' + CAST(@count AS NVARCHAR) + ' row(s) from opc_operation'
    SET ROWCOUNT 0
  END
  PRINT 'Deleted total of ' + CAST(@total AS NVARCHAR) + ' row(s) from opc_operation'
*/

  SET NOCOUNT ON
  SET @count = 1
  SET @total = 0
  WHILE @count > 0
  BEGIN
    SET ROWCOUNT 100
    DELETE FROM b FROM BATCH_C AS b
      JOIN #delete AS d ON d.id = b.id
    SET @count = @@ROWCOUNT
    SET @total = @total + @count
    RAISERROR('Deleted %d row(s) out of %d from BATCH_C', 0, 1, @total, @num_to_delete) WITH NOWAIT
    SET ROWCOUNT 0
  END
  PRINT 'Deleted total of ' + CAST(@total AS NVARCHAR) + ' row(s) from BATCH_C'
GO
