-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/_DiscoverDB2Resources.sqv   1.0   Sep 07 2001 16:50:28   Forthman  $
--
-- Module Description:
--   creates stored procedure _DiscoverDB2Resources

Use Object
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2Resources
PRINT 'Creating procedure _DiscoverDB2Resources'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2Resources' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2Resources
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/_DiscoverDB2Resources.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:50:28  $
-- $Author:   Forthman  $
CREATE PROCEDURE _DiscoverDB2Resources
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  DECLARE @rc INT

  SET NOCOUNT ON

  RAISERROR( '_DiscoverDB2Resources @cid = ''%s'', @id = %d, @batchid = %d, @incremental = %d , @verbose = %d', 0, 1, @cid, @id, @batchid, @incremental, @verbose)


  -- Initialization FOR the timings of steps
  DECLARE @startdate DATETIME
  DECLARE @nowdate   DATETIME
  DECLARE @stepdate  DATETIME
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  NVARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT
  DECLARE @debug INT

  SELECT @debug = 0

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             ( s_step/ s_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- IF the batchid was NOT specified THEN pick the oldest one that IS IN a LOADED state
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = min(batchid) 
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      FormatType = 17
      AND ActionType IN ( 1, 8, 12 )-- HACK ActionType 12, remove 12 after fixing data upstream
      AND v.name = 'LOADED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No DB2 DiscoveryBatch IS IN a LOADED state, nothing to do!'
      RETURN 0
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Get Batchid'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


-- Garbage collection eventually:IF NOT running IN incremental mode
--  IF @incremental = 0
--  BEGIN

/*
    -- This should done as a daily job; it appears that this processing is not necessary for discovery

    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Starting garbage collection'
    DECLARE linkc CURSOR FOR
      SELECT dol.link_type, RTRIM(dol.src_cid), RTRIM(dol.dst_cid), dst.cname
      FROM derived_obj_link AS dol
        JOIN obj_class AS src ON src.cid = dol.src_cid
        JOIN obj_class AS dst ON dst.cid = dol.dst_cid
        JOIN isa_chain AS dic ON dic.cid = dol.dst_cid
        JOIN link_type AS lt ON lt.link_type = dol.link_type
      WHERE
        src.flag_abstract = 0
        AND dst.flag_abstract = 0
        AND dic.base_cid IN ( 'D2SC', 'D2DB', 'D2VF', 'D2VW', 'D2TS', 'D2TB', 'D2TP', 'D2IX', 'D2IP'  )
        AND lt.flag_containment = 1
    OPEN linkc
    DECLARE @link_type LinkType, @src_cid ClassID, @dst_cid ClassID, @dst_cname NVARCHAR(24)
    FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- DELETE link records
      DECLARE @table sysname
      SET @table = @src_cid + '_' + @dst_cid + '_' + @link_type
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Doing garbage collection ON ' + @table
      EXEC("
      DELETE FROM il FROM " + @table + " AS il JOIN link AS l ON
        l.link_type = '" + @link_type + "'
        AND l.src_cid = '" + @src_cid + "'
        AND l.src_id = il.src_id
        AND l.dst_cid = '" + @dst_cid + "'
        AND l.dst_id = il.dst_id
      WHERE
        l.deleted = 1
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS NVARCHAR)

      -- Timing of Step
      IF @debug >= 1
      BEGIN
        SELECT @stepname = 'De ' + @table 
        SELECT @nowdate    = GETDATE()
        SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
        SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
        RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
        SELECT @stepdate   = @nowdate
      END

      -- DELETE instance records
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Doing garbage collection ON ' + @dst_cname + '_C'
      EXEC("
      DELETE FROM " + @dst_cname + "_C WHERE id NOT IN (SELECT l.dst_id FROM link AS l WHERE l.dst_cid='" + @dst_cid + "' AND l.deleted=0)
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS NVARCHAR)

      -- Timing of Step
      IF @debug >= 1
      BEGIN
        SELECT @stepname = 'De ' + @dst_cname + '_C'
        SELECT @nowdate    = GETDATE()
        SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
        SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
        RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
        SELECT @stepdate   = @nowdate
      END


      FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    END
    DEALLOCATE linkc
--  END -- END of garbage collection
*/

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Garbage Collection'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Take care of any mismatches between discovery data obj_cid AND obj_class.cid
  UPDATE DB2Discovery 
    SET obj_cid = 
      CASE obj_cid 
        WHEN 'DBBP' THEN 'D2BP'
        WHEN 'DBDB' THEN 'D2DB'
        WHEN 'DBDD' THEN 'D2DD'
        WHEN 'DBDS' THEN 'D2DS'
        WHEN 'DBEP' THEN 'D2EP'
        WHEN 'DBIP' THEN 'D2IP'
        WHEN 'DBIX' THEN 'D2IX'
        WHEN 'DBLG' THEN 'D2LG'
        WHEN 'DBLK' THEN 'D2LK'
        WHEN 'DBMS' THEN 'D2MS'
        WHEN 'DBPM' THEN 'D2PM'
        WHEN 'DBSP' THEN 'D2SP'
        WHEN 'DBSS' THEN 'D2SS'
        WHEN 'DBTB' THEN 'D2TB'
        WHEN 'DBTP' THEN 'D2TP'
        WHEN 'DBTS' THEN 'D2TS'
        WHEN 'DBVW' THEN 'D2VW'
        ELSE obj_cid
      END
    WHERE batchid = @batchid

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Update cid in DB2Discovery'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Map SystemId to os_id
  IF @verbose = 1
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Mapping SystemId to os_id'
  UPDATE child SET
    os_id = p.id
  FROM
    DB2Discovery AS child
    JOIN OS_V AS p ON p._SystemId = child.SystemId

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Map SystemId to os_id'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- IF we could NOT map any of the records FROM the batch to an operating system THEN
  -- there's something wrong.
  IF NOT EXISTS
  (
    SELECT * FROM DB2Discovery WHERE batchid = @batchid AND os_id IS NOT NULL
  )
  BEGIN
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: batch ' + CAST(@batchid AS NVARCHAR) + ' cannot be resolved to any operating system. The SystemId field of the discovery records does NOT match any known operating systems.'
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Error on no _SystemId'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Compute the path of all objects. We can do this now because all operating systems have been looked up.
  -- Note: because the D2VFs are NOT known at this time, D2VFs AND their children D2VWs will be WRONG, fixed below 
  DECLARE osc CURSOR FOR
    SELECT DISTINCT os_id FROM DB2Discovery WHERE batchid = @batchid
  OPEN osc
  DECLARE @os_id ObjID, @Path NVARCHAR(255)
  FETCH NEXT FROM osc INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_fullpath OS, @os_id, @Path OUTPUT
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Updating paths FOR objects under ' + @Path
    UPDATE DB2Discovery SET obj_path = 
      @Path
      + '/' + DB2_SUBSYSTEM
      + CASE WHEN dbsc_cid IS NOT NULL THEN '/' + DB2_SUBCOMPONENT END
      + CASE WHEN dbts_cid IS NOT NULL THEN '/' + DB2_TABLESPACE   END
      + CASE WHEN dbtb_cid IS NOT NULL THEN '/' + DB2_TABLE        END
      + CASE WHEN dbin_cid IS NOT NULL THEN '/' + DB2_INDEX        END
    WHERE
      batchid = @batchid
      AND os_id = @os_id
    FETCH NEXT FROM osc INTO @os_id
  END
  DEALLOCATE osc

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Compute Path of batch'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  DECLARE @flagDiscovery INT

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBSYSTEM IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2SS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN
  -- Rediscover Subsystems: Step 1, UPDATE dbsu_id IN DB2Discovery records 
  UPDATE child SET
    dbsu_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Subsystem_C AS obj ON          -- Find any existing DB2Subsystem objects by class,name
      obj.cid = child.dbsu_cid
      AND obj.name = child.DB2_SUBSYSTEM
    JOIN link AS pc ON                     -- Lookup FOR parent links of subsystems to make sure
      pc.src_cid = 'OS'                    -- we're under the right operating system
      AND pc.src_id = child.os_id 
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                   -- USE only Active objects
  WHERE
    child.batchid = @batchid               -- Reference the current batch

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2SS, Step1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Rediscover Subsystems: Step 2, UPDATE rediscovered, obj_id FOR D2SS objects that have dbsu_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsu_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid            -- USE the current batch
    AND d.obj_cid = 'D2SS'          -- Only UPDATE DB2 Subsystem class with a dbsu_id value
    AND dbsu_id IS NOT NULL     

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2SS, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- CREATE any subsystems that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsu_id IS NULL 
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''PHYC''',
      @ParentCIDColumn = '''OS''',
      @ParentIDColumn  = 'os_id',
      @ChildCIDColumn  = 'dbsu_cid',
      @ChildIDColumn   = 'dbsu_id',
      @ChildNameColumn = 'DB2_SUBSYSTEM',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL AND d.dbsu_id IS NULL AND d.obj_cid = ''D2SS''',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subsystems'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2SS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- UPDATE subsystem obj_id FOR those subsystems that were just created
  UPDATE d SET
    obj_id      = dbsu_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid          -- USE the current batch 
    AND d.obj_cid = 'D2SS'          -- Only UPDATE DB2 Subsystems class with a dbsu_id value that were NOT rediscovered
    AND dbsu_id IS NOT NULL
    AND rediscovered = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2SS obj_id'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Display DB2Subsystems
  IF @verbose = 1
    SELECT 'DB2Subsystem', * FROM DB2Discovery AS d
    WHERE batchid = @batchid 
      AND d.obj_cid = 'D2SS'

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Di D2SS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END


  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- The NEXT several paragraphs refer to DB2 Subcomponents, this really means everything but D2VWs because D2VWs are
  --   bound to D2VFs
  -- Rediscover Subcomponents: Step 1, UPDATE dbsc_id IN DB2Discovery records 
/*
  UPDATE child SET
    dbsc_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2SubComponent_Rediscovery AS obj ON   -- Find any existing DB2 Subcomponent objects by class,name
      obj.cid = child.dbsc_cid
      AND obj.name = child.DB2_SUBCOMPONENT
    JOIN link AS pc ON                           -- Restrict by parent subsystem
      pc.src_cid = child.dbsu_cid        
      AND pc.src_id = child.dbsu_id 
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                        -- USE only Active objects
    JOIN isa_chain AS pic ON                    -- Make parent classes known to WHERE clause
      pic.cid = child.dbsc_cid
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_SUBCOMPONENT IS NOT NULL        -- Only attempt to rediscovery DB2_SUBCOMPONENT with actual names
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS a instance OR a child of the these classes
    AND pc.dst_cid = child.dbsc_cid               -- Restrict match to the discovery record class
*/

  -- The above query relies on a multi view join that is expensive to create, the following goes after the 
  -- underlying tables directly one class at a time.

  DECLARE d2sc_cid CURSOR FOR
    SELECT DISTINCT dbsc_cid 
    FROM DB2Discovery AS child
    WHERE 
          child.batchid = @batchid 
      AND child.DB2_SUBCOMPONENT IS NOT NULL
  OPEN d2sc_cid 
  DECLARE @d2sc_cid ClassID
  DECLARE @d2sc_table NVARCHAR(80)
  DECLARE @sql NVARCHAR(1000)
  FETCH NEXT FROM d2sc_cid INTO @d2sc_cid
  WHILE @@FETCH_STATUS <> -1 
  BEGIN
    SELECT @d2sc_table = cname + '_C' FROM obj_class WHERE cid = @d2sc_cid
    SELECT @sql = '
    UPDATE child SET
      dbsc_id = obj.id
    FROM
      DB2Discovery AS child
      JOIN ' + @d2sc_table + ' AS obj ON   -- Find any existing DB2 Subcomponent objects by class,name
        obj.cid = child.dbsc_cid
        AND obj.name = child.DB2_SUBCOMPONENT
      JOIN link AS pc ON                           -- Restrict by parent subsystem
        pc.src_cid = child.dbsu_cid        
        AND pc.src_id = child.dbsu_id 
        AND pc.dst_cid = obj.cid
        AND pc.dst_id = obj.id
        AND pc.deleted = 0                         -- USE only Active objects
    WHERE
      child.batchid = ' + CONVERT( NVARCHAR(20), @batchid ) + '                      -- Reference the current batch
      AND child.DB2_SUBCOMPONENT IS NOT NULL        -- Only attempt to rediscovery DB2_SUBCOMPONENT with actual names'

--    PRINT @sql

    EXEC (@sql)
    FETCH NEXT FROM d2sc_cid INTO @d2sc_cid
  END
  CLOSE d2sc_cid 
  DEALLOCATE d2sc_cid 

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2SC, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Rediscover Subcomponents: Step 2, UPDATE rediscovered, obj_id FOR DB2 Subcomponent objects that have dbsc_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsc_id
  FROM 
    DB2Discovery AS d
    JOIN isa_chain AS pic ON        -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE 
    d.batchid = @batchid                          -- USE the current batch
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS a instance OR a child of the these classes
    AND dbsc_id IS NOT NULL                       -- WHEN there IS an actual dbsc_id value
    AND dbts_cid IS NULL                          -- WHEN record describes a subcomponent AND NOT a child of a subcomponent

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2SC, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- CREATE any subcomponents that are children of the D2SC class that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    JOIN isa_chain AS pic ON pic.cid = d.obj_cid  -- Make parent classes known to WHERE clause
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid IS NOT NULL                  -- Has a subcomponent
      AND pic.base_cid = 'D2SC'                   -- IS a child of the D2SC class
      AND d.dbsc_id IS NULL                       -- Subcomponent IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''PHYC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'JOIN isa_chain AS ic ON ic.cid = d.obj_cid 
                           WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.dbsc_id IS NULL 
                             AND ic.base_cid = ''D2SC''
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subcomponents'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL AND dbsc_cid = 'D2DB') 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2DB'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN


  -- CREATE any D2DB subcomponents that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid = 'D2DB'     -- IS a D2DB subcomponent
      AND d.dbsc_id IS NULL        -- Subcomponent IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2DB'' 
                             AND d.dbsc_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subcomponents (databases)'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2DB'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END


  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL AND dbsc_cid = 'D2VW' ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2VW'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- D2VF rediscovery
  -- IF there IS a subsystem that has D2VWs 
  --   THEN CREATE a DB2Discovery record FOR each D2VF that EXISTS

  INSERT INTO
    DB2Discovery 
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_VIEWFOLDER,
    os_id,
    dbsu_cid,
    dbsu_id,
    dbvf_cid,
    dbvf_id,
    obj_name,
    obj_id,
    rediscovered
  )
  SELECT DISTINCT
    @batchid,
    d.SystemId,
    l.dst_cid,
    d.DB2_SUBSYSTEM,
    obj.name,
    d.os_id,
    d.dbsu_cid,
    d.dbsu_id,
    l.dst_cid,
    obj.id,
    obj.name,
    obj.id,
    1
  FROM DB2Discovery AS d
    JOIN link AS l ON l.src_id = d.dbsu_id  -- JOIN link TABLE to provide list of D2VFs known to the DB2 Subsystem
      AND l.src_cid = d.dbsu_cid
      AND l.dst_cid = 'D2VF'
      AND l.deleted = 0                     -- USE only Active objects
    JOIN DB2ViewFolder_V AS obj             -- JOIN DB2ViewFolder_V to provide fields FOR SELECT
      ON obj.id = l.dst_id
  WHERE d.batchid = @batchid     -- Match ON batchid
    AND d.dbsu_id IS NOT NULL    -- Valid subsystem
    AND d.dbsc_cid = 'D2VW'      -- IS a D2VW record

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2VF in DB2Discovery'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- D2VW rediscovery
  -- IF subsystem -> D2VF -> D2VW 
  --  THEN mark D2VW AS rediscovered

  -- Rediscover DB2 Views: Step 1, UPDATE dbsc_id IN DB2Discovery records, AND its dbvf_cid AND dbvf_id since they are known
  UPDATE child SET
    dbsc_id        = obj.id,
    dbvf_cid       = pc.src_cid,
    dbvf_id        = pc.src_id,
    DB2_VIEWFOLDER = obj2.name
  FROM
    DB2Discovery AS child
    JOIN DB2View_V AS obj ON               -- Find any existing DB2 VIEW by name
          obj.name = child.DB2_SUBCOMPONENT
    JOIN link AS pc ON                     -- Lookup FOR parent links of db2 VIEW folder to make sure
          pc.src_cid = 'D2VF'              -- we're under the right folder, note the db2 VIEW folder's match floats (no id)
      AND pc.dst_cid = 'D2VW'
      AND pc.dst_id  = obj.id
      AND pc.deleted = 0                   -- USE only Active objects
    JOIN link AS pc2 ON                    -- Limit to D2VWs subsystem, by chaining with link TABLE
          pc2.src_cid = child.dbsu_cid
      AND pc2.src_id  = child.dbsu_id
      AND pc2.dst_cid = pc.src_cid
      AND pc2.dst_id  = pc.src_id
      AND pc2.deleted = 0                  -- USE only Active objects
    JOIN DB2ViewFolder_V AS obj2 ON        -- Hook IN attributes of VIEW Folder FOR SELECT statement
      obj2.id = pc.src_id
  WHERE
        child.batchid = @batchid           -- Reference the current batch
    AND child.obj_cid = 'D2VW'             -- record refers to a db2 VIEW
    AND obj.deleted   = 0                  -- Don't reference any deleted db2 VIEW folders

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2VF, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Rediscover DB2 Views: Step 2, UPDATE rediscovered, obj_id FOR D2VW objects that have dbsc_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsc_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid            -- USE the current batch
    AND d.obj_cid = 'D2VW'          -- Only UPDATE DB2 VIEW class with a dbsc_id value
    AND dbsc_id IS NOT NULL     

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2VF, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- IF there are D2VW that have NOT been rediscovered 
  --   AND there IS NOT a D2VF FOR the same subsystem 
  -- THEN
  --   CREATE a D2VF record FOR that subsystem

  INSERT INTO
    DB2Discovery 
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_VIEWFOLDER,
    os_id,
    dbsu_cid,
    dbsu_id,
    dbvf_cid,
    dbvf_id,
    obj_name,
    obj_id,
    rediscovered
  )
  SELECT DISTINCT
    @batchid,
    d.SystemId,
    'D2VF',
    d.DB2_SUBSYSTEM,
    '',
    d.os_id,
    d.dbsu_cid,
    d.dbsu_id,
    'D2VF',
    NULL,
    '',
    NULL,
    0
  FROM DB2Discovery AS d
  WHERE 
        d.batchid  = @batchid      -- Match ON batchid
    AND d.dbsc_cid = 'D2VW'        -- IS a DB2 VIEW
    AND d.rediscovered = 0         -- that has NOT been rediscovered
    AND NOT EXISTS ( SELECT * FROM DB2Discovery AS d2 WHERE
        d2.batchid  = d.batchid    -- Match ON batchid
    AND d2.dbsu_cid = d.dbsu_cid   -- Match ON subsystem cid
    AND d2.dbsu_id  = d.dbsu_id    -- Match ON subsystem id
    AND d2.dbvf_cid = 'D2VF'       -- There IS a DB2 VIEW Folder
    )

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2VF New Record'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- UPDATE undiscovered db2 views to have their db2 VIEW folder information match the a D2VF record
  SELECT dbsu_cid, dbsu_id, dbvf_cid, dbvf_id, DB2_VIEWFOLDER INTO #dbvf_detail 
  FROM DB2Discovery
  WHERE
        batchid = @batchid
    AND obj_cid = 'D2VF'

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Create #dbvf_detail'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  UPDATE d SET
    d.dbvf_cid       = d2.dbvf_cid,
    d.dbvf_id        = d2.dbvf_id,
    d.DB2_VIEWFOLDER = d2.DB2_VIEWFOLDER
  FROM DB2Discovery AS d, #dbvf_detail AS d2
  WHERE 
        d.batchid      = @batchid
    AND d.dbsu_cid     = d2.dbsu_cid
    AND d.dbsu_id      = d2.dbsu_id
    AND d.obj_cid      = 'D2VW'
    AND d.rediscovered = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up DB2Discovery from #dbvf_detail'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  DROP TABLE #dbvf_detail

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Drop #dbvf_detail'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
 
  -- IF there are D2VF that have NOT been rediscovered
  -- THEN CREATE the D2VF objects

  -- CREATE any D2VF objects that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbvf_cid = 'D2VF'     -- IS a D2VF object
      AND d.dbvf_id IS NULL       -- This IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbvf_cid',
      @ChildIDColumn   = 'dbvf_id',
      @ChildNameColumn = 'DB2_VIEWFOLDER',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2VF'' 
                             AND d.dbvf_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' VIEW folder(s)'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2VF'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- UPDATE discovered D2VF obj_id
  UPDATE d SET
    obj_id       = dbvf_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                          -- USE the current batch
    AND obj_cid = 'D2VF'                          -- Only D2VFs
    AND dbvf_id IS NOT NULL                       -- That have ids
    AND rediscovered = 0                          -- AND are newly discovered

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2VF'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- IF there IS a D2VW that has NOT been rediscovered
  -- THEN 
  --   assign the D2VF cid AND id FROM that subsystem (see D2VF records)
  -- This should be handled automatically by the _CreateDiscoveredObjects because it can match ON the name

  -- CREATE all D2VW that have NOT been rediscovered

  -- CREATE any D2VW objects that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid = 'D2VW'     -- IS a D2VW object
      AND d.dbsc_id IS NULL       -- This IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbvf_cid',
      @ParentIDColumn  = 'dbvf_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2VW'' 
                             AND d.dbvf_id IS NOT NULL 
                             AND d.dbsc_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' VIEW(s)'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2VW'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Compute the path of all D2VF AND D2VW records, since everything IS now known.
  DECLARE osc CURSOR FOR
    SELECT DISTINCT os_id FROM DB2Discovery WHERE batchid = @batchid
  OPEN osc
  FETCH NEXT FROM osc INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_fullpath OS, @os_id, @Path OUTPUT
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Updating paths FOR objects under ' + @Path
    UPDATE DB2Discovery SET obj_path = 
      @Path
      + '/' + DB2_SUBSYSTEM
      + CASE WHEN dbvf_cid IS NOT NULL THEN '/' + DB2_VIEWFOLDER   END
      + CASE WHEN dbsc_cid IS NOT NULL THEN '/' + DB2_SUBCOMPONENT END
    WHERE
      batchid = @batchid
      AND os_id = @os_id
      AND dbvf_id IS NOT NULL
    FETCH NEXT FROM osc INTO @os_id
  END
  DEALLOCATE osc

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Compute Path D2VF, D2VW'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- UPDATE subcomponent obj_id FOR those subcomponents that were just created
  UPDATE d SET
    d.obj_id      = dbsc_id
  FROM 
    DB2Discovery AS d
    JOIN isa_chain AS pic ON                      -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS an instance OR a child of the these classes
    AND d.dbsc_id IS NOT NULL                     -- dbsc_id has actual value
    AND DB2_TABLESPACE IS NULL                    -- UPDATE only Subcomponent records 
    AND d.rediscovered = 0                        -- Was NOT previously rediscovered

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2SC, D2DB, D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END 

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL AND dbvf_cid = 'D2VF' ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2VW'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- UPDATE the D2VF paths that have been discovered, doing it now AS a efficiency
  UPDATE DB2ViewFolder_C 
  SET _Path = d.obj_path
  FROM DB2ViewFolder_C AS c
    INNER JOIN DB2Discovery AS d ON c.id = d.obj_id
  WHERE 
        d.batchid      = @batchid
    AND d.obj_cid      = 'D2VF'
    AND d.rediscovered = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2VF Path'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- UPDATE the D2VW paths that have been discovered, doing it now AS a efficiency
  UPDATE DB2View_C 
  SET _Path = d.obj_path
  FROM DB2View_C AS c
    INNER JOIN DB2Discovery AS d ON c.id = d.obj_id
  WHERE 
        d.batchid      = @batchid
    AND d.obj_cid      = 'D2VW'
    AND d.rediscovered = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2VW Path'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END 

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_SUBCOMPONENT IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- Display DB2SubComponents
  IF @verbose = 1
    SELECT 'DB2SubComponents', * FROM DB2Discovery AS d
    JOIN isa_chain AS pic ON                      -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS an instance OR a child of the these classes

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Di D2SC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_TABLESPACE IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2TS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN


  -- Rediscover Tablespaces: Step 1, UPDATE dbts_id IN DB2Discovery records 
  UPDATE child SET
    dbts_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2TableSpace_C AS obj ON               -- Find any existing DB2 tablespace objects by class,name
      obj.name = child.DB2_TABLESPACE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbsc_cid        
      AND pc.src_id = child.dbsc_id 
      AND pc.dst_cid = 'D2TS'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLESPACE IS NOT NULL          -- Only attempt to rediscovery DB2_TABLESPACE with actual names

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TS, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Rediscover Tablespaces: Step 2, UPDATE rediscovered, obj_id FOR DB2 Subcomponent objects that have dbts_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbts_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbts_id        IS NOT NULL          -- Only UPDATE records WHERE the tablespace IS known 
    AND d.DB2_TABLE      IS NULL              -- Only UPDATE tablespace records (IF it has a DB2_TABLE record it IS child)

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TS, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- CREATE any tablespaces that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbts_cid IS NOT NULL   -- There IS a tablespace
      AND d.dbts_id  IS NULL       -- Tablespace IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsc_cid',
      @ParentIDColumn  = 'dbsc_id',
      @ChildCIDColumn  = 'dbts_cid',
      @ChildIDColumn   = 'dbts_id',
      @ChildNameColumn = 'DB2_TABLESPACE',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsc_id IS NOT NULL 
                             AND d.obj_cid = ''D2TS'' 
                             AND d.dbts_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' tablespaces'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2TS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- UPDATE tablespace obj_id FOR those tablespaces that were just created
  UPDATE d SET
    obj_id      = dbts_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND d.obj_cid = 'D2TS'                        -- IS a tablespace
    AND dbts_id IS NOT NULL                       -- dbts_id has actual value
    AND DB2_TABLE IS NULL                         -- UPDATE only tablespace records 
    AND rediscovered = 0                          -- Was NOT previously rediscovered

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2TS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Display DB2Tablespace
  IF @verbose = 1
    SELECT 'DB2Tablespace', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND d.obj_cid = 'D2TS'      -- IS an instance of D2TS

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Di D2TS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

/*
  -- Rediscover Tables: Step 1, UPDATE dbtb_id IN DB2Discovery records 
  UPDATE child SET
    dbtb_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Table_Rediscovery AS obj ON          -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbtb_cid
      AND obj.name = child.DB2_TABLE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbts_cid        
      AND pc.src_id = child.dbts_id 
      AND (pc.dst_cid = 'D2TB' OR pc.dst_cid = 'D2TP')
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLE IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbtb_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TB, Step 1 (D2TP)'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
*/

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_TABLE IS NOT NULL AND dbtb_cid = 'D2TB') 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2TB'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN


  -- Rediscover Tables: Step 1, UPDATE dbtb_id IN DB2Discovery records 
  UPDATE child SET
    dbtb_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Table_C AS obj ON          -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbtb_cid
      AND obj.name = child.DB2_TABLE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbts_cid        
      AND pc.src_id = child.dbts_id 
      AND pc.dst_cid = 'D2TB'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLE IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbtb_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TB, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_TABLE IS NOT NULL AND dbtb_cid = 'D2TP' ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2TP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN


  UPDATE child SET
    dbtb_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2TableSpacePartition_C AS obj ON      -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbtb_cid
      AND obj.name = child.DB2_TABLE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbts_cid        
      AND pc.src_id = child.dbts_id 
      AND pc.dst_cid = 'D2TP'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLE IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbtb_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TP, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END



  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_TABLE IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2TB D2TP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN



  -- Rediscover TABLE: Step 2, UPDATE rediscovered, obj_id FOR DB2 TABLE objects that have dbtb_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbtb_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbtb_id        IS NOT NULL          -- Only UPDATE records WHERE the TABLE IS known 
    AND d.DB2_INDEX      IS NULL              -- Only UPDATE TABLE records (IF it has a DB2_INDEX record it IS child)

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2TB, Step 2 (D2TB)'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- CREATE any tables that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbtb_cid IS NOT NULL   -- There IS a TABLE
      AND d.dbtb_id  IS NULL       -- TABLE IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbts_cid',
      @ParentIDColumn  = 'dbts_id',
      @ChildCIDColumn  = 'dbtb_cid',
      @ChildIDColumn   = 'dbtb_id',
      @ChildNameColumn = 'DB2_TABLE',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbts_id IS NOT NULL 
                             AND (d.obj_cid = ''D2TB'' OR d.obj_cid = ''D2TP'')
                             AND d.dbtb_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' tables'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2TB D2TP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- UPDATE TABLE obj_id FOR those tables that were just created
  UPDATE d SET
    obj_id      = dbtb_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2TB' OR d.obj_cid = 'D2TP') -- IS a TABLE (OR tablespace partition)
    AND dbtb_id IS NOT NULL                        -- dbtb_id has actual value
    AND DB2_INDEX IS NULL                          -- UPDATE only TABLE records 
    AND rediscovered = 0                           -- Was NOT previously rediscovered

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2TB D2TP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Display DB2 Tables (AND Partitions)
  IF @verbose = 1
    SELECT 'DB2Table', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2TB' OR d.obj_cid = 'D2TP') -- IS a TABLE (OR tablespace partition)

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Di D2TB D2TP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END
/*
  -- Rediscover Index: Step 1, UPDATE dbin_id IN DB2Discovery records 
  UPDATE child SET
    dbin_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Index_Rediscovery AS obj ON          -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbin_cid
      AND obj.name = child.DB2_INDEX
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbtb_cid        
      AND pc.src_id = child.dbtb_id 
      AND (pc.dst_cid = 'D2IX' OR pc.dst_cid = 'D2IP')
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_INDEX IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbin_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2IX, Step 1 (D2IP)'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
*/
  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_INDEX IS NOT NULL AND dbin_cid = 'D2IX' ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2IX'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- Rediscover Index: Step 1, UPDATE dbin_id IN DB2Discovery records 
  UPDATE child SET
    dbin_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2TableIndex_C AS obj ON               -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbin_cid
      AND obj.name = child.DB2_INDEX
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbtb_cid        
      AND pc.src_id = child.dbtb_id 
      AND pc.dst_cid = 'D2IX'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_INDEX IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbin_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2IX, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_INDEX IS NOT NULL AND dbin_cid = 'D2IP' ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2IP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN


  -- Rediscover Index: Step 1, UPDATE dbin_id IN DB2Discovery records 
  UPDATE child SET
    dbin_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2IndexPartition_C AS obj ON           -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbin_cid
      AND obj.name = child.DB2_INDEX
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbtb_cid        
      AND pc.src_id = child.dbtb_id 
      AND pc.dst_cid = 'D2IP'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_INDEX IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbin_cid               -- Restrict match to the discovery record class

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2IP, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END

  IF EXISTS ( SELECT * FROM DB2Discovery AS child WHERE batchid = @batchid AND DB2_INDEX IS NOT NULL ) 
    SELECT @flagDiscovery = 1
  ELSE
    SELECT @flagDiscovery = 0

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'flagDiscovery D2IX D2IP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF ( @flagDiscovery = 1 )
  BEGIN

  -- Rediscover Index: Step 2, UPDATE rediscovered, obj_id FOR DB2 Index objects that have dbin_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbin_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbin_id        IS NOT NULL          -- Only UPDATE records WHERE the TABLE IS known 

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Rd D2IX, Step 2 (D2IP)'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- CREATE any indexes that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbin_cid IS NOT NULL   -- There IS an index
      AND d.dbin_id  IS NULL       -- Index IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbtb_cid',
      @ParentIDColumn  = 'dbtb_id',
      @ChildCIDColumn  = 'dbin_cid',
      @ChildIDColumn   = 'dbin_id',
      @ChildNameColumn = 'DB2_INDEX',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbtb_id IS NOT NULL 
                             AND ( d.obj_cid = ''D2IX'' OR d.obj_cid = ''D2IP'' )
                             AND d.dbin_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' indexes'
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr D2IX D2IP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- UPDATE index obj_id FOR those indexes that were just created
  UPDATE d SET
    obj_id      = dbin_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2IX' OR d.obj_cid = 'D2IP') -- IS an index (FOR TABLE OR partition)
    AND dbin_id IS NOT NULL                        -- dbin_id has actual value
    AND rediscovered = 0                           -- Was NOT previously rediscovered

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up D2IX D2IP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Display DB2 Indexes (FOR Tables AND Partitions)
  IF @verbose = 1
    SELECT 'DB2Index', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2IX' OR d.obj_cid = 'D2IP') -- IS an index (FOR TABLE OR partition)

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Di D2IX D2IP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  END
  -- STATE: NOT assigning state information during discovery.

  -- IF records batch IS a VIEW TABLE relationship 17 08 THEN 
  -- IF records are D2TB INSERT those records INTO D2VW_D2TB_USEC FOR those that have NOT been seen before.
  -- to make this work, need to have the D2VW id be available IN the record.
  --   to make this happen the record needs  to be inserted INTO the discovery batch, before it arrives IN this stored proc.

  -- Special handling FOR 1708 records:
  -- Establish USEC link FROM D2VF to D2TB 

  IF EXISTS ( SELECT * FROM DiscoveryBatch WHERE batchid = @batchid AND ActionType = 8 )
  BEGIN

    SELECT batchid, obj_cid, obj_id, SystemId, DB2_SUBSYSTEM, DB2_SUBCOMPONENT, DB2_TABLESPACE, DB2_TABLE INTO #d2vw_detail 
    FROM DB2Discovery 
    WHERE 
          batchid = @batchid
      AND obj_cid IN ( 'D2VW', 'D2TB' )

    INSERT INTO D2VW_D2TB_USEC                             -- Establish the USEC relationship
    ( src_id, dst_id )
    SELECT d1.obj_id, d2.obj_id 
    FROM 
      #d2vw_detail  AS d1,
      #d2vw_detail  AS d2,
      DiscoveryBatch AS b, 
      DB2_VTRDIS   AS p  
    WHERE 
          b.batchid         = @batchid       -- match ON current batch
      AND d2.batchid        = @batchid
      AND p.batchid         = @batchid
      AND b.ActionType      = 8              -- Insert only on Table View relationship discoveries (1708)
      AND d1.obj_cid        = 'D2VW'         -- src record is the view
      AND d2.obj_cid        = 'D2TB'         -- dst record is the table
      AND p.SystemId        = d2.SystemId                     -- matching one record (p) to two (d#)
      AND p.SubsysName      = d2.DB2_SUBSYSTEM
      AND p.DATABASE_NAME   = d2.DB2_SUBCOMPONENT
      AND p.TABLESPACE_NAME = d2.DB2_TABLESPACE
      AND p.TABLE_NAME      = d2.DB2_TABLE
      AND p.SystemId        = d1.SystemId
      AND p.SubsysName      = d1.DB2_SUBSYSTEM
      AND p.VIEW_NAME       = d1.DB2_SUBCOMPONENT
      AND NOT EXISTS (                  -- but only IF the record has NOT been created before
        SELECT * FROM D2VW_D2TB_USEC AS l 
        WHERE 
              l.src_id = d1.obj_id 
          AND l.dst_id = d2.obj_id
      )
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Created ' + CAST(@@ROWCOUNT AS NVARCHAR) + ' D2VW to D2TB relationship(s)' 
    DROP TABLE #d2vw_detail 

    -- Timing of Step
    IF @debug >= 1
    BEGIN
      SELECT @stepname = '1708 Processing'
      SELECT @nowdate    = GETDATE()
      SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
      SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
      RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
      SELECT @stepdate   = @nowdate
    END
  END


  PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Moving records FROM DB2Discovery INTO ProcessedDB2Discovery FOR batch ' + CAST(@batchid AS NVARCHAR)
  -- Move records FROM IMSDiscovery INTO ProcessedIMSDiscovery
  INSERT ProcessedDB2Discovery SELECT GETDATE(), * 
  FROM DB2Discovery WHERE batchid = @batchid
  IF @@ERROR = 0
    DELETE DB2Discovery WHERE batchid = @batchid

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Move to ProcessedDB2Discovery'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Mark discovery batch AS completed
  UPDATE DiscoveryBatch SET pend = GETDATE(), pstat = v.pstat, error = 0
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'COMPLETED'

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Mark batch as complete'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @verbose = 1
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Finished'

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Exit _DiscoverDB2Resources'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

GO