-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/_GetBatchCycleJobHist.sqv   1.2   Aug 01 2001 11:37:04   BRENT  $
--
-- _GetBatchCycleJobHist.sql
--
-- Module Description
--   Defines the _GetBatchCycleJobHist stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _GetBatchCycleJobHist
PRINT 'Creating procedure _GetBatchCycleJobHist'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetBatchCycleJobHist' AND type = 'P')
  DROP PROCEDURE _GetBatchCycleJobHist
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/_GetBatchCycleJobHist.sqv  $
-- $Revision:   1.2  $
-- $Date:   Aug 01 2001 11:37:04  $
-- $Author:   BRENT  $
-- Implementation of the GetBatchCycleJobHist method.
CREATE PROCEDURE _GetBatchCycleJobHist
  @cid              ClassID       = NULL,
  @id               ObjID         = NULL,
  @keyJobIDExpr     NVARCHAR(255) = NULL, -- Expression to specify the ids of the key jobs to
                                          -- be selected. If specified must be in one of the 
                                          -- following forms:
                                          -- '= x'
                                          -- 'IN (...)'
                                          -- 'BETWEEN x AND y'
  @nonkeyJobIDExpr  NVARCHAR(255) = NULL, -- Expression to specify the ids of the nonkey jobs to
                                          -- be selected. Same form as @keyJobIDExpr.
  @alertStateExpr   NVARCHAR(255) = NULL, -- Expression to specify the alert state(s) of the events
                                          -- to filter by. Same form as @keyJobIDExpr.
  @currentStateExpr NVARCHAR(255) = NULL, -- Expression to specify the event current state value(s)
                                          -- to filter by. Same form as @keyJobIDExpr.
  @desiredStateExpr NVARCHAR(255) = NULL, -- Expression to specify the job desired state value(s)
                                          -- to filter by. Same form as @keyJobIDExpr. This filter
                                          -- applies to the desired state of the job as of the time
                                          -- of the event.
  @minTime          DATETIME      = NULL, -- If specified, the minimum time of the events to retrieve.
  @maxTime          DATETIME      = NULL, -- If specified, the maximum time of the events to retrieve.
  @strict           TINYINT       = 0,    -- If @strict is true then we are looking only for rows
                                          -- with one state record that matches the specified
                                          -- current and/or desired state. This option is not
                                          -- currently implemented.
  @jobTypeMask      INT           = 3     -- Bit values: 1=non-key, 2=key
AS
  SET NOCOUNT ON

/*
DECLARE @minTimeStr NVARCHAR(20)
SET @minTimeStr = CONVERT(NVARCHAR(20), @minTime, 121)
DECLARE @maxTimeStr NVARCHAR(20)
SET @maxTimeStr = CONVERT(NVARCHAR(20), @maxTime, 121)
RAISERROR('@minTime=''%s'', @maxTime=''%s''', 3, 1, @minTimeStr, @maxTimeStr)
*/
  -- Do not allow @maxTime to be specified without @minTime (we don't want to select back to the beginning of time!)
  IF @maxTime IS NOT NULL AND @minTime IS NULL
  BEGIN
    RAISERROR('_GetBatchCycleJobHist: Cannot specify a max time without a min time', 11, 1)
    RETURN -30000
  END

  -- Assemble the portion of the where-clause having to do with alert state
  DECLARE @alertStateClause NVARCHAR(255)
  IF @alertStateExpr IS NOT NULL
    SET @alertStateClause = ' AND h.alertstate' + @alertStateExpr

  -- Assemble the portion of the where-clause having to do with current state
  DECLARE @currentStateClause NVARCHAR(255)
  IF @currentStateExpr IS NOT NULL
    SET @currentStateClause = ' AND h.currentstate' + @currentStateExpr

  -- Assemble the portion of the where-clause having to do with desired state
  DECLARE @desiredStateClause NVARCHAR(255)
  IF @desiredStateExpr IS NOT NULL
    SET @desiredStateClause = ' AND h.desiredstate' + @desiredStateExpr

  -- Assemble the variable declarations for dynamic SQL

  DECLARE @now DATETIME
  SET @now = CURRENT_TIMESTAMP
  IF @minTime IS NULL
    SET @minTime = DATEADD(MONTH,-1,@now)
  IF @maxTime IS NULL
    SET @maxTime = @now

  DECLARE @timeRangeExpr NVARCHAR(255)
  SET @timeRangeExpr = ' h.agent_ctime BETWEEN ''' + CONVERT(NVARCHAR(40),@minTime,121) + '''' +
                       ' AND ''' + CONVERT(NVARCHAR(40),@maxTime,121) + ''''

  DECLARE @bcj_insert NVARCHAR(255)
  SET @bcj_insert = 'INSERT INTO #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID FROM BATCH_V AS b '

  DECLARE @keyJobSelect NVARCHAR(255)
  IF @keyJobIDExpr IS NOT NULL
    SET @keyJobSelect = @bcj_insert +
                        'WHERE b._KeyStatus = 1 AND b.id ' + @keyJobIDExpr

  DECLARE @nonkeyJobSelect NVARCHAR(255)
  IF @nonkeyJobIDExpr IS NOT NULL
    SET @nonkeyJobSelect = @bcj_insert +
                           'WHERE b._KeyStatus = 0 AND id ' + @nonkeyJobIDExpr +
                           ' AND b.id NOT IN (SELECT jobid FROM #bcj)'

  -- If both @keyJobSelect and @nonkeyJobSelect are NULL then we use the calling object's id
  IF @keyJobSelect IS NULL AND @nonkeyJobSelect IS NULL
  BEGIN
    IF @cid = 'BATC'
      SET @keyJobSelect = @bcj_insert +
                          'WHERE b.id =' + CAST(@id AS NVARCHAR)
    ELSE IF @cid = 'BTCY'
    BEGIN
      DECLARE @btcy_where NVARCHAR(255)
      SET @btcy_where = 'WHERE b._ParentCID = ''BTCY'' AND b._ParentID = ' + CAST(@id AS NVARCHAR) + ' '
      SET @keyJobSelect    = @bcj_insert + @btcy_where + ' AND b._KeyStatus = 1'
      SET @nonkeyJobSelect = @bcj_insert + @btcy_where + ' AND b._KeyStatus = 0'
    END
    ELSE IF @cid = 'BCYS'
    BEGIN
      DECLARE @bcys_where NVARCHAR(255)
      SET @bcys_where = 'JOIN BCYS_BTCY_AGGC AS link ON b._ParentCID = ''BTCY'' AND b._ParentID = link.dst_id ' +
                        'WHERE link.src_id = ' + CAST(@id AS NVARCHAR) + ' '
      SET @keyJobSelect    = @bcj_insert + @bcys_where + ' AND b._KeyStatus = 1'
      SET @nonkeyJobSelect = @bcj_insert + @bcys_where + ' AND b._KeyStatus = 0'
    END
    ELSE
    BEGIN
      RAISERROR('Attempt to call GetBatchCycleJobHist for unsupported object class %s', 11, 1, @cid)
      RETURN -30000
    END
  END

  DECLARE @jobTypeMaskExpr NVARCHAR(255)
  IF (@jobTypeMask & 3) = 3
    SET @jobTypeMaskExpr = ''
  ELSE
    SET @jobTypeMaskExpr = 'AND (b._KeyStatus & ' + CAST(@jobTypeMask AS NVARCHAR) + ') = ' + CAST(@jobTypeMask AS NVARCHAR)

/*
  RAISERROR('@alertStateExpr:  [%s]', 0, 1, @alertStateExpr)
  RAISERROR('@keyJobSelect:    [%s]', 0, 1, @keyJobSelect)
  RAISERROR('@nonkeyJobSelect: [%s]', 0, 1, @nonkeyJobSelect)
  RAISERROR('@jobTypeMaskExpr: [%s]', 0, 1, @jobTypeMaskExpr)
*/

  -- Standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _GetBatchCycleJobHist
  ELSE
    SAVE TRANSACTION _GetBatchCycleJobHist

  DECLARE @sql VARCHAR(8000)
--  EXEC("
  SET @sql = "
  -- Collect ids of all jobs into temp table
  CREATE TABLE #bcj
  (
    obj_cid             ClassID,        --   4
    obj_id              INT NULL,       --   5
    name                ObjName,        --  54
    jobid               INT,            --   4
    parent_cid          ClassID,        --   4
    parent_id           ObjID           --   4
  ) " +
  @keyJobSelect + " " + @jobTypeMaskExpr + " " +
  @nonkeyJobSelect + " " + @jobTypeMaskExpr + "

  IF EXISTS (SELECT * FROM #bcj)
  BEGIN
    -- Select history records for jobs
    SELECT
      cid='BCJH',
      h.id,
      j.name,
      j.parent_cid,
      j.parent_id,
      h.obj_cid,
      h.obj_id,
      h.currentstate AS currentStateID,
      h.desiredstate AS desiredStateID,
      h.eventcid AS eventCID,
      h.eventid AS eventID,
      h.name AS eventName,
      h.agent_ctime AS srctime,
      h.ctime,
      h.alertstate AS alertStateID,
      h.priority AS priorityID,
      h.agent_seqno AS seqno,
      h.detail AS eventDetail
    FROM
      #bcj AS j JOIN BatchCycleJobHist AS h ON h.obj_cid = 'BATC' AND h.obj_id = j.jobid
    WHERE " +
      @timeRangeExpr +
      @alertStateClause +
      @currentStateClause +
      @desiredStateClause + "
  END
  "
--  ")
/* Create this table and uncomment the insert statement below to trace the SQL that is executed
CREATE TABLE _GetBatchCycleJobHist_Trace
(
  calltime   DATETIME NOT NULL,
  sql        VARCHAR(8000) -- NOT NVARCHAR
)
*/
--INSERT INTO _GetBatchCycleJobHist_Trace VALUES ( CURRENT_TIMESTAMP, @sql )
EXEC(@sql)

  -- Standard transaction logic
  IF @tcount = 0
    COMMIT TRANSACTION _GetBatchCycleJobHist
GO
