-- File: archive\db\sql\_getbatchcyclesets.sql
-- SID: 1.3
-- Date Modified: 01/10/22 13:49:27
--
-- Module Description
--   Defines the _GetBatchCycleSets stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _GetBatchCycleSets
PRINT 'Creating procedure _GetBatchCycleSets'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetBatchCycleSets' AND type = 'P')
  DROP PROCEDURE _GetBatchCycleSets
GO

-- File: archive\db\sql\_getbatchcyclesets.sql
-- SID: 1.3
-- Date Modified: 01/10/22 13:49:27
-- Returns the list of matching BatchCycles objects under the calling object.
CREATE PROCEDURE _GetBatchCycleSets
  @cid                ClassID,
  @id                 ObjID, 
  @namePattern        NVARCHAR(255) = NULL, 
  @currentStateExpr   NVARCHAR(255) = NULL, 
  @desiredStateExpr   NVARCHAR(255) = NULL, 
  @alertStateExpr     NVARCHAR(255) = NULL, 
  @priorityExpr       NVARCHAR(255) = NULL, 
  @rowLimit           INT = 0, 
  @matchCount         INT = NULL OUTPUT, 
  @handle             INT = NULL OUTPUT
AS
  RAISERROR('_GetBatchCycleSets @cid=''%s'', @id=%d', 0, 1, @cid, @id)

  -- Assemble portion of the WHERE-clause from the filter expressions
  DECLARE @name_expr NVARCHAR(255)
  IF @namePattern IS NOT NULL
    IF CHARINDEX('%', @namePattern) <> 0 or CHARINDEX('_', @namePattern) <> 0
      SELECT @name_expr = ' AND bcys.name like ''' + @namePattern + ''''
    ELSE
      SELECT @name_expr = ' AND bcys.name=''' + @namePattern + ''''
  IF @currentStateExpr IS NOT NULL
    SELECT @currentStateExpr = ' AND bcys._CurrentStateID ' + @currentStateExpr
  IF @desiredStateExpr IS NOT NULL
    SELECT @desiredStateExpr = ' AND bcys._DesiredStateID ' + @desiredStateExpr
  IF @alertStateExpr IS NOT NULL
    SELECT @alertStateExpr = ' AND bcys._AlertStateID ' + @alertStateExpr
  IF @priorityExpr IS NOT NULL
    SELECT @priorityExpr = ' AND bcys._PriorityID ' + @priorityExpr

  CREATE TABLE #descendent
  (
    cid   ClassID,
    id    ObjID,
    gen   int
  )
  EXEC asisp_descendents @cid, @id, 'IN (''BCYS'')'

  IF NOT EXISTS (SELECT * FROM #descendent)
  BEGIN
    -- If the caller is an LOB and the LOB contains Batch Schedules instead of Batch Schedule Sets
    -- then we return the LOB as the proxy for the schedule sets
    IF @cid = 'LOB'
    BEGIN
      SELECT
        c.src_cid AS src_cid,
        c.src_id AS src_id,
        'LOB' AS cid,
        lob.id,
        lob.name AS scheduleset,
        lob.name,
        lob._CurrentStateID,
        lob._DesiredStateID,
        lob._AlertStateID,
        lob._PriorityID,
        lob._HasOwnershipNote
      FROM
        LineOfBusiness_V AS lob
        JOIN containment AS c ON c.dst_cid = @cid AND c.dst_id = lob.id
      WHERE
        lob.id = @id
    END
    RETURN
  END

  -- Execute the query dynamically
  EXEC("
    SELECT
      c.src_cid AS src_cid,
      c.src_id AS src_id,
      'BCYS' AS cid,
      bcys.id,
      bcys.name AS scheduleset,
      bcys.name,
      bcys._CurrentStateID,
      bcys._DesiredStateID,
      bcys._AlertStateID,
      bcys._PriorityID,
      bcys._HasOwnershipNote
    FROM
      BatchCycles_V AS bcys
        JOIN #descendent AS d ON d.cid = 'BCYS' AND d.id = bcys.id
        JOIN containment AS c ON c.dst_cid = 'BCYS' AND c.dst_id = bcys.id
    WHERE 1=1 " + 
      @name_expr +
      @currentStateExpr +
      @desiredStateExpr +
      @alertStateExpr +
      @priorityExpr
  )
GO
