-- $Header$
-- _getdb2objects.sql
--
-- Module Description
--   create stored procedure _GetDB2Objects

Use Object
GO

------------------------------------------------------------
-- _GetDB2Objects
PRINT 'create procedure _GetDB2Objects'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = '_GetDB2Objects' AND type = 'P' )
  DROP PROCEDURE _GetDB2Objects
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/_GetDB2Objects.sqv  $
-- $Revision:   1.0  $ 
-- $Date:   Sep 07 2001 16:50:40  $
-- $Author:   Forthman  $
CREATE PROCEDURE _GetDB2Objects
  @cid               ClassID,
  @id                ObjID,
  @OrLocationPattern1  NVARCHAR(255) = NULL,
  @OrLocationPattern2  NVARCHAR(255) = NULL,
  @OrLocationPattern3  NVARCHAR(255) = NULL,
  @NamePattern       NVARCHAR(255) = NULL,
  @MinAlert          INT = 0,
  @MinPriority       INT = 0,
  @OpenNotes         INT = 2,
  @ObjectType        INT = 0, 
  @debug              INT = 0
AS
  RAISERROR( '_GetDB2Objects: Called AS _GetDB2Objects @cid=%s, @id=%d, @OrLocationPattern1=%s, @OrLocationPattern2=%s, @OrLocationPattern3=%s, @NamePattern=%s, @MinAlert=%d, @MinPriority=%d, @OpenNotes=%d, @ObjectType=%d, @debug=%d', 0, 1, @cid, @id, @OrLocationPattern1, @OrLocationPattern2, @OrLocationPattern3, @NamePattern, @MinAlert, @MinPriority, @OpenNotes, @ObjectType, @debug)  

  SET NOCOUNT ON
  SET ROWCOUNT 0

  -- Initialization FOR the timings of steps
  DECLARE @startdate DATETIME
  DECLARE @nowdate   DATETIME
  DECLARE @stepdate  DATETIME
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  NVARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             ( s_step/ s_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Initialization of UPPER threshold FOR the number of records returned.
  DECLARE @maxRowCount AS INT
  SELECT @maxRowCount = 2000

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @maxRowCount'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Get list of DB2 objects under the calling object using object linking 
  CREATE TABLE #descendent
  (
    cid   ClassID PRIMARY KEY (cid, id),
    id    ObjID,
    gen   INT
  )
  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr #descendent'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  DECLARE @where NVARCHAR(500)
  SET @where = 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''D2BA'')'

  DECLARE @ObjectTypeCid NVARCHAR(4)
  SELECT @ObjectTypeCid = NULL

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @where, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Modify the WHERE clause based ON object type requested to get the bare minumim
  -- Either all, just itself, OR just the classes need FOR the physical containments
  -- IF no object type was specified leave the WHERE clause alone
  IF @ObjectType <> 0 
  BEGIN
    -- @ObjectType has been selected get the cid ( @ObjectTypeCid )
    SELECT  @ObjectTypeCid = ev.val_cid FROM enum_value AS ev 
    JOIN enumeration AS e ON ev.eid = e.eid
    WHERE e.name = 'DB2ObjectTypes' AND ev.value = @ObjectType

    SELECT @where = 'IN (''' + ev.val_cid + ''')'
    FROM enum_value AS ev JOIN enumeration AS e ON ev.eid = e.eid
    WHERE e.name = 'DB2ObjectTypes' AND ev.value = @ObjectType
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @where, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  IF @debug >=2
  BEGIN
    PRINT 'Before EXEC asisp_descendents @cid, @id, @where call'
    SELECT @cid, @id, @where
  END

  -- populate #descendents TABLE with those objects with link dependencies
  EXEC asisp_descendents @cid, @id, @where

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Ex asisp_descendents'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  IF @debug >=2
  BEGIN
    PRINT '# of rows IN #descendent: ' 
    SELECT COUNT(*) FROM #descendent
    PRINT 'SELECT * FROM #descendent: '
    SELECT * FROM #descendent
  END

  -- Populate #obj TABLE with all attributes of objects who should be returned
  CREATE TABLE #obj
  (
    src_cid            ClassID NULL,
    src_id             ObjID NULL,
    cid                ClassID PRIMARY KEY (cid, id ),
    id                 ObjID,
    name               ObjName NULL,
    Location           NVARCHAR(1000) NULL,
    Type               NVARCHAR(40) NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        NVARCHAR(255) NULL
  )

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr #obj'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Populate #obj TABLE with all objects known by descendents links
  DECLARE c CURSOR FOR
    SELECT DISTINCT cid FROM #descendent
  OPEN c
  DECLARE @cidTarget ClassID
  FETCH NEXT FROM c INTO @cidTarget
  WHILE @@FETCH_STATUS <> -1 
  BEGIN
    INSERT #obj
      SELECT c.src_cid, c.src_id, dst_cid, dst_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
      FROM #descendent AS d 
        JOIN link AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
        JOIN link_type AS lt ON lt.link_type = c.link_type
      WHERE
            (lt.flag_containment=1 OR lt.flag_parent_child=1)
        AND lt.link_type <> 'USEC'
        AND d.cid = @cidTarget
    FETCH NEXT FROM c INTO @cidTarget
  END
  CLOSE c
  DEALLOCATE c

-- Original
/*
  INSERT #obj
    SELECT DISTINCT c.src_cid, c.src_id, dst_cid, dst_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM #descendent AS d 
      JOIN link AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
      JOIN link_type AS lt ON lt.link_type = c.link_type
    WHERE
          (lt.flag_containment=1 OR lt.flag_parent_child=1)
      AND lt.link_type <> 'USEC'
*/
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #obj'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @debug >=2
  BEGIN
    PRINT '# of rows IN #obj: ' 
    SELECT COUNT(*) FROM #obj
  END

  IF @debug >=2
  BEGIN
    SELECT @cid, @id 
    PRINT '# of rows IN #obj: After object type filtering.' 
    SELECT COUNT(*) FROM #obj
  END

  -- HACK
--  GOTO Output
  DECLARE @NamePatternText      NVARCHAR(255)
  SELECT @NamePatternText  = '' + CASE WHEN @NamePattern        IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @NamePattern        ) + '''' END     

  -- Create the #output table 
  CREATE TABLE #output
  (
     cid               ClassID PRIMARY KEY ( cid, id ),
     id                ObjID,
     name              ObjName        NULL,
     Type              NVARCHAR(80)   NULL,
     Location          NVARCHAR(2000) NULL,
     _AlertStateID     INT,
     _PriorityID       INT,
     _CurrentStateID   INT,
     _DesiredStateID   INT,
     _HasOwnershipNote TINYINT,
     Description       NVARCHAR(510)  NULL
  )

  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr #output'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Create the records to output ( INSERT INTO #ouptut )
  DECLARE c CURSOR FOR
    SELECT DISTINCT cid FROM #obj
  OPEN c
  DECLARE @cidTable NVARCHAR(100), @sql NVARCHAR(2000)
  FETCH NEXT FROM c INTO @cidTarget
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @cidTable = cname + '_C' FROM obj_class WHERE cid = @cidTarget

--    SELECT @OrLocationPattern1  = ' + SELECT CASE WHEN @OrLocationPattern1 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern1 ) + '''' END + '
--    SELECT @OrLocationPattern2  = ' + SELECT CASE WHEN @OrLocationPattern2 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern2 ) + '''' END + '
--    SELECT @OrLocationPattern3  = ' + SELECT CASE WHEN @OrLocationPattern3 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern3 ) + '''' END + '
--    SELECT @NamePattern         = ' + SELECT CASE WHEN @NamePattern        IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @NamePattern        ) + '''' END + '


    SELECT @sql = '
    DECLARE  @OrLocationPattern1  NVARCHAR(255) 
    DECLARE  @OrLocationPattern2  NVARCHAR(255) 
    DECLARE  @OrLocationPattern3  NVARCHAR(255) 
    DECLARE  @NamePattern         NVARCHAR(255) 
    DECLARE  @MinAlert            INT 
    DECLARE  @MinPriority         INT 
    DECLARE  @OpenNotes           INT 
    DECLARE  @ObjectType          INT 
    DECLARE  @ObjectTypeCid       NVARCHAR(255)

    SELECT @OrLocationPattern1  = ' + CASE WHEN @OrLocationPattern1 IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern1 ) + '''' END + '
    SELECT @OrLocationPattern2  = ' + CASE WHEN @OrLocationPattern2 IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern2 ) + '''' END + '
    SELECT @OrLocationPattern3  = ' + CASE WHEN @OrLocationPattern3 IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern3 ) + '''' END + '
    SELECT @NamePattern         = ' + CASE WHEN @NamePattern        IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @NamePattern        ) + '''' END + '
    SELECT @ObjectTypeCid       = ' + CASE WHEN @ObjectTypeCid      IS NULL THEN 'NULL' ELSE '''' + CONVERT(NVARCHAR(255), @ObjectTypeCid      ) + '''' END + '

    SELECT @MinAlert            = ' + CONVERT( NVARCHAR(12), @MinAlert    ) + '
    SELECT @MinPriority         = ' + CONVERT( NVARCHAR(12), @MinPriority ) + '
    SELECT @OpenNotes           = ' + CONVERT( NVARCHAR(12), @OpenNotes   ) + '
    SELECT @ObjectType          = ' + CONVERT( NVARCHAR(12), @ObjectType  ) + '


    INSERT INTO #output 
    SELECT i.cid, i.id, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL
      FROM ' + @cidTable + ' AS i
        INNER JOIN #obj AS o ON o.cid = i.cid AND o.id = i.id
        WHERE (@ObjectType = 0         OR i.cid = @ObjectTypeCid)
          AND (@MinAlert = 0           OR i._AlertStateID >= @MinAlert)
          AND (@NamePattern IS NULL    OR i.name LIKE @NamePattern)
          AND (@MinPriority = 0        OR i._PriorityID <= @MinPriority) -- yes, priorities work IN reverse!
          AND (@OpenNotes = 2          OR i._HasOwnershipNote = @OpenNotes)
          AND ( (@OrLocationPattern1 IS NULL AND @OrLocationPattern2 IS NULL AND @OrLocationPattern3 IS NULL )
            OR  (    i._Path LIKE @OrLocationPattern1
                  OR i._Path LIKE @OrLocationPattern2
                  OR i._Path LIKE @OrLocationPattern3
                )
              )
-- HACK
--     SELECT COUNT(*) FROM #obj
--     SELECT COUNT(*) FROM #output'

    -- HACK
--    PRINT @sql
  
    EXEC ( @sql )
    
    FETCH NEXT FROM c INTO @cidTarget
  END
  CLOSE c
  DEALLOCATE c
  

  -- Output columns based ON final result SET
  DECLARE @countRow INT
  SELECT @countRow = COUNT(*) FROM #output


  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se INTO #output'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- HACK
--  GOTO Output
	   
  -- Output error message
  IF @countRow > @maxRowCount 
  BEGIN 
    CREATE TABLE #errortext
    (
      _AlertStateID      INT,
      Error              NVARCHAR(100)
    )
    INSERT #errortext
      SELECT _AlertStateID = 3,
      Error = 'The query returned ' + CONVERT( NVARCHAR(8), @countRow) + ' rows, please submit query again with more restrictive criteria.'
    SELECT 
      e._AlertStateID, e.Error
    FROM 
      #errortext AS e
  END
  ELSE
  BEGIN

  -- Update Type column, has been left to the end as an efficiency
  -- Create the records to output ( INSERT INTO #ouptut )
  DECLARE c CURSOR FOR
    SELECT DISTINCT cid FROM #obj
  OPEN c
--  DECLARE @cidTable NVARCHAR(100), @sql NVARCHAR(2000)
  FETCH NEXT FROM c INTO @cidTarget
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @cidTable = cname + '_C' FROM obj_class WHERE cid = @cidTarget

--    SELECT @OrLocationPattern1  = ' + SELECT CASE WHEN @OrLocationPattern1 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern1 ) + '''' END + '
--    SELECT @OrLocationPattern2  = ' + SELECT CASE WHEN @OrLocationPattern2 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern2 ) + '''' END + '
--    SELECT @OrLocationPattern3  = ' + SELECT CASE WHEN @OrLocationPattern3 IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @OrLocationPattern3 ) + '''' END + '
--    SELECT @NamePattern         = ' + SELECT CASE WHEN @NamePattern        IS NULL THEN NULL ELSE '''' + CONVERT(NVARCHAR(255), @NamePattern        ) + '''' END + '


    SELECT @sql = '

    UPDATE #output 
    SET 
      name              = i.name,
      Location          = i._Path, 
      _AlertStateID     = i._AlertStateID, 
      _PriorityID       = i._PriorityID, 
      _CurrentStateID   = i._CurrentStateID, 
      _DesiredStateID   = i._DesiredStateID, 
      _HasOwnershipNote = i._HasOwnershipNote, 
      Description       = i.description
      FROM ' + @cidTable + ' AS i
        INNER JOIN #output AS o ON o.cid = i.cid AND o.id = i.id
-- HACK
--     SELECT COUNT(*) FROM #obj
--     SELECT COUNT(*) FROM #output'

    -- HACK
--    PRINT @sql
  
    EXEC ( @sql )
    
    FETCH NEXT FROM c INTO @cidTarget
  END
  CLOSE c
  DEALLOCATE c

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up #output, Step 1'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  UPDATE #output
  SET Type = oc.label
  FROM #output AS i
  INNER JOIN obj_class AS oc ON oc.cid = i.cid       

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Up #output, Step 2'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- RETURN Good Results
  SELECT
    obj._AlertStateID,
    obj._HasOwnershipNote,
    obj.Location,
    obj.Type,
    obj.name,
    obj._PriorityID,
    obj._CurrentStateID,
    obj._DesiredStateID,
    obj.Description,
    obj.cid,
    obj.id
  FROM
    #output AS obj
  ORDER BY
    obj.Location
  END



-- HACK 
Output:

  -- HACK
--  PRINT 'SELECT COUNT(*) FROM #output'
--  SELECT COUNT(*) FROM #output

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Output Results'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ss, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ss, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END
  DROP TABLE #obj
  DROP TABLE #descendent
GO
