-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/_GetManagedObjects.sqv   1.0   Sep 07 2001 16:50:48   Forthman  $
--
-- _GetManagedObjects.sql
--
-- Module Description
--   Defines the _GetManagedObjects stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- _GetManagedObjects
PRINT 'Creating procedure _GetManagedObjects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetManagedObjects' AND type = 'P')
  DROP PROCEDURE _GetManagedObjects
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/_GetManagedObjects.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:50:48  $
-- $Author:   Forthman  $
-- Stored procedure that is used to select items displayed in the Managed Object View

CREATE PROCEDURE _GetManagedObjects
  @cid              ClassID,
  @id               ObjID,
  @LocationPattern  varchar(255) = null,
  @NamePattern      varchar(255) = null,
  @MinAlert         int = 0,
  @MinPriority      int = 0,
  @OpenNotes        int = 2,
  @ObjectType       int = 0
AS
  SET NOCOUNT ON

  --08/07/2001, still keep the limitation, but the customer can configure this value

  -- set searching limitation
  DECLARE @DESCENDENTS_LIMIT INT
  SELECT @DESCENDENTS_LIMIT = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration
  WHERE groupname = '_GetManagedObjects' AND propertyname = 'DESCENDENTS_LIMIT'
  IF @DESCENDENTS_LIMIT IS NULL OR @DESCENDENTS_LIMIT < 0
    SET @DESCENDENTS_LIMIT = 10000

  -- set displaying limitation
  DECLARE @RETURN_LIMIT INT
  SELECT @RETURN_LIMIT = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration
  WHERE groupname = '_GetManagedObjects' AND propertyname = 'RETURN_LIMIT'
  IF @RETURN_LIMIT IS NULL OR @RETURN_LIMIT < 0
    SET @RETURN_LIMIT = 2000

  IF @DESCENDENTS_LIMIT < @RETURN_LIMIT
     SET @DESCENDENTS_LIMIT = @RETURN_LIMIT

  SET ROWCOUNT @DESCENDENTS_LIMIT

  -- Get the list of managed objects under the calling object
  CREATE TABLE #descendent
  (
    cid   ClassID PRIMARY KEY (cid, id),
    id    ObjID,
    gen   INT,
  )
  DECLARE @where VARCHAR(255)
  SET @where = 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''MNO2'')'

  -- If object type was specified, delete anything that doesn't match
  IF @ObjectType <> 0
  BEGIN
    SELECT @where = 'IN (''' + val_cid + ''')'
      FROM enum_value AS ev JOIN enumeration AS e ON ev.eid = e.eid
      WHERE e.name = 'ManagedObjectTypes' AND ev.value = @ObjectType
  END
  EXEC asisp_descendents @cid, @id, @where

  -- Populate table with all managed objects under the calling object
  CREATE TABLE #obj
  (
    src_cid            ClassID NULL,
    src_id             ObjID NULL,
    cid                ClassID,
    id                 ObjID,
    Location           VARCHAR(255) NULL,
    name               ObjName NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        VARCHAR(255) NULL
  )
  INSERT #obj
-- HACK
    SELECT c.src_cid, c.src_id, dst_cid, dst_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM #descendent AS d 
      JOIN link AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
      JOIN link_type AS lt ON lt.link_type = c.link_type
    WHERE
      (lt.flag_containment=1 OR lt.flag_parent_child=1)

  -- Fill in the attribute details for each object
  DECLARE @obj_cid ClassID, @cname VARCHAR(24)
  DECLARE cc INSENSITIVE CURSOR FOR SELECT DISTINCT obj.cid, cname FROM #obj AS obj JOIN obj_class AS c ON c.cid = obj.cid
  OPEN cc
  FETCH NEXT FROM cc INTO @obj_cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC("
    UPDATE obj SET 
      name = o.name,
      Location = o.name,
      _AlertStateID = o._AlertStateID,
      _PriorityID = o._PriorityID,
      _CurrentStateID = o._CurrentStateID,
      _DesiredStateID = o._DesiredStateID,
      _HasOwnershipNote = o._HasOwnershipNote,
      Description = o._Desc
    FROM #obj AS obj, " + @cname + "_V AS o
    WHERE o.id = obj.id AND obj.cid = '" + @obj_cid + "'
    ")
    FETCH NEXT FROM cc INTO @obj_cid, @cname
  END
  DEALLOCATE cc

  -- Remove objects that do not match the filter criteria
  IF @NamePattern IS NOT NULL
    DELETE #obj WHERE name NOT LIKE @NamePattern
  IF @MinAlert <> 0
    DELETE #obj WHERE _AlertStateID < @MinAlert
  IF @MinPriority <> 0
    DELETE #obj WHERE _PriorityID > @MinPriority -- yes, priorities work in reverse!
  IF @OpenNotes <> 2
    DELETE #obj WHERE _HasOwnershipNote <> @OpenNotes
  DECLARE @rc INT, @src_cid ClassID, @loop INT
  SELECT @rc = COUNT(*) FROM #obj
  SET @cname = ''
  SET @loop = 0
  WHILE @rc > 0 AND @loop < 10
  BEGIN
    DECLARE srcc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, cname FROM #obj JOIN obj_class AS c ON c.cid = src_cid
    OPEN srcc
    FETCH NEXT FROM srcc INTO @src_cid, @cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC("
      UPDATE obj SET src_cid = c.src_cid, src_id = c.src_id, Location = parent.name + '/' + Location
      FROM #obj AS obj 
      JOIN " + @cname + "_C AS parent ON parent.id = obj.src_id
      LEFT OUTER JOIN link AS c ON c.dst_cid = obj.src_cid AND c.dst_id = obj.src_id
        JOIN link_type AS lt ON lt.link_type = c.link_type
      WHERE obj.src_cid = '" + @src_cid + "' AND obj.src_cid <> 'BUSC'
        AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1)
      ")
      FETCH NEXT FROM srcc INTO @src_cid, @cname
    END
    DEALLOCATE srcc
    SELECT @rc = COUNT(*) FROM #obj WHERE src_cid <> 'ENT' AND src_cid IS NOT NULL
    SET @loop = @loop + 1
  END

  -- Filter out paths that don't match parameter, if specified
  IF @LocationPattern IS NOT NULL
    DELETE #obj WHERE Location NOT LIKE @LocationPattern

  SET NOCOUNT OFF
  --08/07/2001, still keep the limitation here, but customer can configure this value

  SET ROWCOUNT @RETURN_LIMIT

  -- Return results
  SELECT
    obj._AlertStateID,
    obj._HasOwnershipNote,
    obj.Location,
    c.label AS Type,
    name,
    obj._PriorityID,
    obj._CurrentStateID,
    obj._DesiredStateID,
    Description,
    obj.cid,
    obj.id
  FROM
    #obj AS obj
      JOIN obj_class AS c ON c.cid = obj.cid
  ORDER BY
    obj.Location
  OPTION (MAXDOP 1) -- BUG #: 57757 (SQLBUG_70) Microsoft SQL Server 7.0 bug workaround
                    -- Error appears as DB Error #10007 in Application Server Log when called from Workstation
                    -- Error did not occur when EXEC the procedure directly.
                    -- Error was discovered on a multiple CPU system.
                    -- This Microsoft bug has been fixed in the U.S. Service Pack 3 for SQL Server 7.0 
GO
