-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/_LoadOPCPlan_CreateBCJLOBs.sqv   1.3   Aug 09 2001 10:42:02   BRENT  $
--
-- _LoadOPCPlan_CreateBCJLOBs.sql
--
-- Module Description
--   Defines the _LoadOPCPlan_CreateBCJLOBs stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _LoadOPCPlan_CreateBCJLOBs
PRINT 'Creating procedure _LoadOPCPlan_CreateBCJLOBs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadOPCPlan_CreateBCJLOBs' AND type = 'P')
  DROP PROCEDURE _LoadOPCPlan_CreateBCJLOBs
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/_LoadOPCPlan_CreateBCJLOBs.sqv  $
-- $Revision:   1.3  $
-- $Date:   Aug 09 2001 10:42:02  $
-- $Author:   BRENT  $
-- Supporting procedure for _LoadOPCPlan that creates the LOBs that are required as specified by
-- the contents of a temporary table called #bcj_lob, which is created by _LoadOPCPlan_CreateBCJs
CREATE PROCEDURE _LoadOPCPlan_CreateBCJLOBs
  @plan_id    INT,
  @chkpt      INT OUTPUT,
  @verbose    TINYINT = 0
AS
  -- Transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _LoadOPCPlan_CreateBCJLOBs
  ELSE
    SAVE TRAN _LoadOPCPlan_CreateBCJLOBs

  IF @verbose = 1
  BEGIN
    EXEC asisp_checkpointmessage 'Checking for LOBs for batch jobs', 1, @chkpt
    SELECT '#bcj_lob',
      o.btcy_id AS btcy_id,
      o.job_id AS job_id,
      l.src_id AS btcy_lob_id, 
      o.op_name + ' ' + o.job_name AS bcj_lob_name,
      o.op_desc AS bcj_lob_desc,
      NULL AS bcj_lob_id
    FROM
      opc_operation AS o
        JOIN opc_app_link AS l ON l.btcy_id = o.btcy_id
    WHERE
      plan_id = @plan_id
      AND l.src_cid = 'LOB'
      AND o.status IS NULL -- we don't want jobs that are marked as completed
  END
  SET @chkpt = @chkpt + 1

  CREATE TABLE #bcj_lob
  (
    btcy_id       ObjID,
    bcj_id        ObjID,
    btcy_lob_id   ObjID,
    bcj_lob_name  ObjName,
    bcj_lob_desc  NVARCHAR(255) NULL,
    bcj_lob_id    ObjID NULL
  )
  INSERT #bcj_lob SELECT DISTINCT
    o.btcy_id,
    o.job_id,
    l.src_id, 
    o.op_name + ' ' + o.job_name,
    o.op_desc,
    NULL
  FROM
    opc_operation AS o
      JOIN opc_app_link AS l ON l.btcy_id = o.btcy_id
  WHERE
    plan_id = @plan_id
    AND l.src_cid = 'LOB'
    AND o.status IS NULL -- we don't want jobs that are marked as completed
    AND o.job_id IS NOT NULL
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- Map to pre-existing LOBs for jobs
  IF @verbose = 1
    EXEC asisp_checkpointmessage 'Looking for pre-existing LOBs', 1, @chkpt
  SET @chkpt = @chkpt + 1
  UPDATE b SET bcj_lob_id = lob.id
  FROM #bcj_lob AS b
    JOIN link AS l ON l.src_cid = 'LOB' AND l.src_id = b.btcy_lob_id AND l.dst_cid = 'LOB'
    JOIN LineOfBusiness_V AS lob ON lob.id = l.dst_id AND lob.name = b.bcj_lob_name
  WHERE
    l.deleted = 0
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- Check for nothing to do
  DECLARE @nlob INT
  SELECT @nlob = COUNT(*) FROM #bcj_lob
  IF @nlob = 0
  BEGIN
    IF @verbose = 1
      PRINT '_LoadOPCPlan_CreateBCJLOBs: No new LOBs to create'
    IF @tcount = 0
      COMMIT TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN
  END

  DECLARE @msg NVARCHAR(255)

  CREATE TABLE #new_bcj_lob
  (
    src_id      ObjID,    -- containing LOB
    bcj_id      ObjID,
    name        ObjName,
    description NVARCHAR(255) NULL,
    row         INT IDENTITY(0,1)
  )
  INSERT #new_bcj_lob
    SELECT btcy_lob_id, bcj_id, bcj_lob_name, ISNULL(bcj_lob_desc ,'')
    FROM #bcj_lob 
    WHERE bcj_lob_id IS NULL

  SELECT @nlob = COUNT(*) FROM #bcj_lob
  IF @nlob = 0
  BEGIN
    IF @verbose = 1
      PRINT '_LoadOPCPlan_CreateBCJLOBs: No new LOBs to create'
    IF @tcount = 0
      COMMIT TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN
  END

  IF @verbose = 1
  BEGIN
    SELECT @msg = 'Creating ' + convert(NVARCHAR(9), @nlob) + ' LOBs'
    EXEC asisp_checkpointmessage @msg, 1, @chkpt
    SELECT '#new_bcj_lob', * FROM #new_bcj_lob
  END
  SELECT @chkpt = @chkpt + 1

  -- Allocate the appropriate number of new LOB records
  DECLARE @base_id INT
  EXEC @base_id = alloc_LOB @nlob

  -- Insert the class records
  SELECT @msg = 'Inserting LineOfBusiness_C records'
  IF @verbose = 1
    EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SELECT @chkpt = @chkpt + 1
  INSERT LineOfBusiness_C(id,name) SELECT @base_id + row, name FROM #new_bcj_lob
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- Insert the attribute records
  IF @verbose = 1
  BEGIN
    SELECT @msg = 'Inserting LineOfBusiness_A records'
    EXEC asisp_checkpointmessage @msg, 1, @chkpt
  END
  SELECT @chkpt = @chkpt + 1
  INSERT LineOfBusiness_A(id,_Desc,_ChildEventMatrixID,_ExceptionMatrixID,_PriorityID)
    SELECT DISTINCT @base_id+row, ISNULL(description,''), 0, 0, 6
    FROM #new_bcj_lob
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- Insert LOB containment links
  SELECT @msg = 'Inserting LOB_LOB_LOBC records'
  IF @verbose = 1
    EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SELECT @chkpt = @chkpt + 1
  INSERT LOB_LOB_LOBC(src_id, dst_id) SELECT src_id, @base_id+row FROM #new_bcj_lob
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- If the number of newly created LOBs is greater than 100, then disable the LOB_BATC_LOBL insert trigger
  IF @nlob > 100
  BEGIN
    IF @verbose = 1
    BEGIN
      SELECT @msg = 'Disabling tI_LOB_BATC_LOBL trigger'
      EXEC asisp_checkpointmessage @msg, 1, @chkpt
    END
    SELECT @chkpt = @chkpt + 1
    ALTER TABLE LOB_BATC_LOBL DISABLE TRIGGER tI_LOB_BATC_LOBL
  END

  -- Insert LOB->BATC reference links
  IF @verbose = 1
  BEGIN
    SELECT @msg = 'Inserting LOB_BATC_LOBL records'
    EXEC asisp_checkpointmessage @msg, 1, @chkpt
  END
  SELECT @chkpt = @chkpt + 1
  INSERT LOB_BATC_LOBL(src_id, dst_id) SELECT @base_id+row, bcj_id FROM #new_bcj_lob
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END

  -- If the number of newly created LOBs is greater than 100, then call asisp_rebuild_lob_link and then
  -- reenable the LOB_BATC_LOBL insert trigger
  IF @nlob > 100
  BEGIN
    INSERT link(link_type, src_cid, src_id, dst_cid, dst_id)
      SELECT 'LOBL', 'LOB', @base_id+row, 'BATC', bcj_id
      FROM #new_bcj_lob
    IF @verbose = 1
    BEGIN
      SELECT @msg = 'Calling asisp_rebuild_lob_link'
      EXEC asisp_checkpointmessage @msg, 1, @chkpt
    END
    SELECT @chkpt = @chkpt + 1
    EXEC asisp_rebuild_lob_link
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
      RETURN -30000
    END
    IF @verbose = 1
    BEGIN
      SELECT @msg = 'Enabling tI_LOB_BATC_LOBL trigger'
      EXEC asisp_checkpointmessage @msg, 1, @chkpt
    END
    SELECT @chkpt = @chkpt + 1
    ALTER TABLE LOB_BATC_LOBL ENABLE TRIGGER tI_LOB_BATC_LOBL
  END

  -- Insert the create notifications
  /*
  SELECT @msg = 'Inserting PendingNotification records'
  EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SELECT @chkpt = @chkpt + 1
  INSERT PendingNotification(type, fields, detail)
    SELECT
      EventType.type,
      CONVERT(NVARCHAR(5), LOB.cno) + '|' + CONVERT(NVARCHAR(9), @base_id+l.row) + '|LOB|LOBC|' +
      CONVERT(NVARCHAR(5), LOB.cno) + '|' + CONVERT(NVARCHAR(9), l.src_id) AS fields,
      l.name AS detail
    FROM
      #new_bcj_lob l,
      opc_app_instance i,
      EventType,
      obj_class AS LOB
    WHERE
      EventType.name = 'ObjectCreated'
      AND LOB.cid = 'LOB'
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJLOBs
    RETURN -30000
  END
  */

  IF @tcount = 0
    COMMIT TRANSACTION _LoadOPCPlan_CreateBCJLOBs
GO
