-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/_PollForCompletedSchedules.sqv   1.2   Aug 01 2001 12:22:16   BRENT  $
--
-- _PollForCompletedSchedules.sql
--
-- Module Description
--   Defines the _PollForCompletedSchedules stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _PollForCompletedSchedules
PRINT 'Creating procedure _PollForCompletedSchedules'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_PollForCompletedSchedules' AND type = 'P')
  DROP PROCEDURE _PollForCompletedSchedules
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/_PollForCompletedSchedules.sqv  $
-- $Revision:   1.2  $
-- $Date:   Aug 01 2001 12:22:16  $
-- $Author:   BRENT  $
-- Checks for batch schedules that contain only completed jobs that are not themselves
-- in a completed state. Generates a 'OPCENDED' message on such schedules and directly
-- sets their state to 'Completed'.
-- This procedure is installed as a scheduled task that runs periodically throughout
-- the day.
CREATE PROCEDURE _PollForCompletedSchedules
  @cid ClassID,
  @id  ObjID
AS
  /* BatchCycle states
  id          _State           
  ----------- ---------------- 
  86          <All>
  87          Starting
  88          Stopping
  89          Halted
  90          Pending
  91          Inactive
  92          Running
  93          Completed
  */

  /* BATCH states
  id          _State           
  ----------- ---------------- 
  77          <All>
  78          Starting
  79          Stopping
  80          Active/Inactive
  81          Unknown
  82          Running
  83          Completed
  84          Abended
  376         Pending
  */

  SELECT DISTINCT s.id AS id, s._CurrentStateID AS sched_state, b._CurrentStateID AS job_state, b._StopTime AS stop_time
  INTO #btcy_job_state
  FROM BATCH_V AS b JOIN BatchCycle_V AS s ON s.id = b._ParentID
  WHERE b._ParentCID = 'BTCY' AND b.deleted = 0

  -- delete btcys where its children have job_state other then 83 and 0 
  DELETE b1 FROM 
    #btcy_job_state AS b1
    JOIN #btcy_job_state AS b2 ON b2.id = b1.id
    WHERE b1.sched_state <> 93 
    AND (b1.job_state = 83 AND b2.job_state NOT IN (83,0))

  -- delete btcys of which children have job_state other then 83 
  DELETE #btcy_job_state 
    WHERE sched_state = 93 OR job_state <> 83

  IF EXISTS (SELECT * FROM #btcy_job_state)
  BEGIN
    INSERT INTO StagedMESG(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, DISCOVERY_IND, STATE, MESSAGE, REASON_CD)
      SELECT DISTINCT 'BTCY', js.id, GETDATE(), '', s.name, '', 'MON', 'COMPLETED', 'OPCENDED', 
        FORMATMESSAGE(58003, s.name, CONVERT(VARCHAR(40), js.stop_time, 120))
      FROM #btcy_job_state AS js JOIN BatchCycle_V AS s ON s.id = js.id
      WHERE js.stop_time = (SELECT MAX(stop_time) FROM #btcy_job_state AS latest WHERE latest.id = js.id)
    UPDATE btcy SET _CurrentStateID = 93, _StopTime = js.stop_time, _GoodRuns = _GoodRuns + 1
    FROM BatchCycle_V AS btcy JOIN #btcy_job_state AS js ON js.id = btcy.id
    WHERE js.stop_time = (SELECT MAX(stop_time) FROM #btcy_job_state AS latest WHERE latest.id = js.id)
  END
GO
