-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/_UpdateBatchCounters.sqv   1.1   Aug 14 2001 11:04:24   BRENT  $
--
-- _UpdateBatchCounters
--
-- Module Description
--   Defines the _UpdateBatchCounters stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _UpdateBatchCounters
PRINT 'Creating procedure _UpdateBatchCounters'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_UpdateBatchCounters' AND type = 'P')
  DROP PROCEDURE _UpdateBatchCounters
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/_UpdateBatchCounters.sqv  $
-- $Revision:   1.1  $
-- $Date:   Aug 14 2001 11:04:24  $
-- $Author:   BRENT  $
-- Can be run at any time to synchronize the BATCH_V columns _TotalRuns and _GoodRuns with the
-- current number of Running and Completed messages, respectively.
CREATE PROCEDURE _UpdateBatchCounters
  @cid ClassID = NULL,
  @id  ObjID = NULL
AS

  SET NOCOUNT ON

  PRINT 'row|id|updated|start|end|elapsed_ms|totalruns|goodruns'

  DECLARE bc INSENSITIVE CURSOR FOR
    SELECT DISTINCT b.id
    FROM BATCH_V AS b
    WHERE deleted = 0
  OPEN bc
  DECLARE @TotalRuns INT, @GoodRuns INT, @start DATETIME, @end DATETIME, @n INT
  DECLARE @start_str NVARCHAR(40), @end_str NVARCHAR(40), @ms INT, @updated TINYINT
  SET @n = 0
  FETCH NEXT FROM bc INTO @id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @n = @n + 1
    IF @@TRANCOUNT = 0 BEGIN TRAN UpdateCounts
    ELSE SAVE TRAN UpdateCounts

    SET @start = CURRENT_TIMESTAMP

    SELECT @TotalRuns = COUNT(*) FROM Message_V AS m
      JOIN StateType_V AS sttp ON sttp.id = m._StateID
    WHERE m._ParentCID = 'BATC' AND m._ParentID = @id AND sttp._State = 'Running'

    SELECT @GoodRuns = COUNT(*) FROM Message_V AS m 
      JOIN StateType_V AS sttp ON sttp.id = m._StateID
    WHERE m._ParentCID = 'BATC' AND m._ParentID = @id AND sttp._State = 'Completed'

    UPDATE BATCH_V SET _TotalRuns = @TotalRuns, _GoodRuns = @GoodRuns
    WHERE id = @id AND (_TotalRuns <> @TotalRuns OR _GoodRuns <> @GoodRuns)
    IF @@ROWCOUNT = 1
      SET @updated = 1
    ELSE
      SET @updated = 0

    SET @end = CURRENT_TIMESTAMP

    SET @start_str = CONVERT(NVARCHAR(40), @start, 121)
    SET @end_str = CONVERT(NVARCHAR(40), @end, 121)
    SET @ms = DATEDIFF(MS, @start, @end)

    RAISERROR('%6d|%8d|%d|%s|%s|%5d|%5d|%5d', 0, 1, 
              @n, @id, @updated, @start_str, @end_str, @ms, @TotalRuns, @GoodRuns) WITH NOWAIT

    IF @@TRANCOUNT = 1 COMMIT TRAN UpdateCounts

    FETCH NEXT FROM bc INTO @id
  END
  DEALLOCATE bc

GO
