-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/_UpdateBatchStates.SQv   1.0   Aug 22 2001 19:44:26   BRENT  $
--
-- _UpdateBatchStates.sql
--
-- Module Description
--   Defines the _UpdateBatchStates stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _UpdateBatchStates
PRINT 'Creating procedure _UpdateBatchStates'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_UpdateBatchStates' AND type = 'P')
  DROP PROCEDURE _UpdateBatchStates
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_UpdateBatchStates.SQv  $
-- $Revision:   1.0  $
-- $Date:   Aug 22 2001 19:44:26  $
-- $Author:   BRENT  $
-- Updates the state of all batch jobs to that indicated by the most recent message. 
-- Will also set the AlertState of the job, and generate or clear a child event, if appropriate.
CREATE PROCEDURE _UpdateBatchStates
  @cid ClassID = NULL,
  @id  ObjID = NULL
AS
  SET NOCOUNT ON
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  DECLARE bc INSENSITIVE CURSOR FOR
    SELECT DISTINCT b.id, b._ParentCID, b._ParentID
    FROM BATCH_V AS b
    WHERE deleted = 0
    AND (@id IS NULL OR b.id = @id)
  OPEN bc
  DECLARE @parent_cid ClassID, @parent_id ObjID, @bAlertStateID INT, @bCurrentStateID INT
  DECLARE @mesg_id ObjID, @mAlertStateID INT, @mStateID INT, @mPriorityID INT
  DECLARE @start DATETIME, @end DATETIME, @n INT
  DECLARE @start_str NVARCHAR(40), @end_str NVARCHAR(40), @ms INT, @updated TINYINT
  SET @n = 0
  FETCH NEXT FROM bc INTO @id, @parent_cid, @parent_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @n = @n + 1
    IF @tcount = 0 BEGIN TRAN UpdateStates
    ELSE SAVE TRAN UpdateStates
  
    SET @start = CURRENT_TIMESTAMP
  
    SELECT @bAlertStateID = _AlertStateID, @bCurrentStateID = _CurrentStateID
    FROM BATCH_V WHERE id = @id

    SET @mesg_id = NULL
    SET @updated = 0
    SELECT
      @mesg_id = m.id,
      @mAlertStateID = m._AlertState,
      @mStateID = m._StateID,
      @mPriorityID = m._Priority
    FROM Message_V AS m
    WHERE m._ParentCID = 'BATC' AND m._ParentID = @id
      AND @bCurrentStateID <> m._StateID AND m._Priority <> 5
      AND NOT EXISTS
      (
        SELECT * FROM Message_V AS mlater WHERE mlater._ParentCID=m._ParentCID AND mlater._ParentID=m._ParentID
          AND mlater._TIME>m._TIME
          AND mlater._Priority<>5
      )
    IF @mesg_id IS NULL
      GOTO NextJob
  
    SET @updated = 1

    -- Update the CurrentMessageAlertStatePri table entry for the job
    UPDATE CurrentMessageAlertStatePri SET mesg_id = @mesg_id, AlertState = @mAlertStateID, Priority = @mPriorityID
    WHERE cid = 'BATC' AND id = @id
    IF @@ROWCOUNT = 0
      INSERT INTO CurrentMessageAlertStatePri(cid, id, AlertState, Priority, mesg_id)
      VALUES ( 'BATC', @id, @mAlertStateID, @mPriorityID, @mesg_id )
  
    -- If the message alert state is more severe than the current propagation alert state for the
    -- job then update the alertstate and generate a child event
    DECLARE @pAlertStateID INT
    SET @pAlertStateID = 1
    SELECT @pAlertStateID = 
      MAX(
      CASE WHEN c.Critical>0 THEN c.ArrayIndex
           WHEN c.High>m.High THEN c.ArrayIndex
           WHEN c.Medium>m.Medium THEN c.ArrayIndex
           WHEN c.Low>m.Low THEN c.ArrayIndex
           ELSE 1
      END
      )
    FROM
      PropagationSlot AS c
      JOIN PropagationSlot AS m ON m.OwnerCID = c.OwnerCID AND m.OwnerID=0
        AND c.MatrixIndex = m.MatrixIndex AND c.ArrayIndex = m.ArrayIndex
    WHERE
      c.OwnerCID = 'BATC'
      AND c.OwnerID>0
      AND c.OwnerID=@id
    GROUP BY c.OwnerCID, c.OwnerID
  
    DECLARE @newAlertStateID INT, @evt_cid ClassID, @evt_pri INT
    IF @mAlertStateID > @pAlertStateID
    BEGIN
      SET @newAlertStateID = @mAlertStateID
      SET @evt_cid = 'MESG'
      SET @evt_pri = @mPriorityID
    END
    ELSE
    BEGIN
      SET @newAlertStateID = @pAlertStateID
      SET @evt_cid = 'EXCH'
      SELECT @evt_pri = MAX(_Priority) FROM Exception_V 
      WHERE _ParentCID = 'BATC' AND _ParentID = @id AND deleted = 0
    END

    IF @newAlertStateID > @bAlertStateID
      EXEC asisp_generatechildevent @parent_cid, @parent_id, 'BATC', @id, @evt_cid, 'Generated by _UpdateBatchStates', 1, 1, @evt_pri, @newAlertStateID
    ELSE IF @newAlertStateID < @bAlertStateID
      EXEC asisp_generatechildevent @parent_cid, @parent_id, 'BATC', @id, @evt_cid, 'Generated by _UpdateBatchStates', 1, -1, @evt_pri, @newAlertStateID

    UPDATE BATCH_V SET _CurrentStateID = @mStateID, _AlertStateID = @newAlertStateID
    WHERE id = @id
  
NextJob:
    SET @end = CURRENT_TIMESTAMP

    SET @start_str = CONVERT(NVARCHAR(40), @start, 121)
    SET @end_str = CONVERT(NVARCHAR(40), @end, 121)
    SET @ms = DATEDIFF(MS, @start, @end)

    RAISERROR('%6d|%8d|%d|%s|%s|%5d|%5d|%5d', 0, 1, 
              @n, @id, @updated, @start_str, @end_str, @ms, @bCurrentStateID, @mStateID)
      WITH NOWAIT

    IF @tcount = 0 COMMIT TRAN UpdateStates

    FETCH NEXT FROM bc INTO @id, @parent_cid, @parent_id
  END
  DEALLOCATE bc
GO
