-- archive\db\sql\_ca7createbatc.sql
-- SID: 1.2.1.2
-- Date Modified: 02/01/16 16:04:05
--
-- Define Module
--   _CA7CreateBATC

PRINT 'Create procedure _CA7CreateBATC.'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = '_CA7CreateBATC' AND type = 'P')
  DROP PROCEDURE _CA7CreateBATC
GO
 
-- SID: 1.2.1.2
-- Date Modified: 02/01/16 16:04:05
-- Creates the BATC objects FOR the CA7 plan AS loaded IN the ca7_bcp TABLE
CREATE PROCEDURE _CA7CreateBATC
  @useBCPTable 		INT = 1,
  @autoDiscoveryFlag 	TINYINT = 1,
  @data_source		ObjName
AS

  SET NOCOUNT ON

  DECLARE @tcount INT, @rows INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _CA7CreateBATC
  ELSE
    SAVE TRAN _CA7CreateBATC

  -- fix times of 24:00 IN ca7_jobs
  UPDATE ca7_jobs
  SET time_start = '00:00', day_start = day_start + 1
  WHERE time_start = '24:00'
  UPDATE ca7_jobs
  SET time_stop = '00:00', day_stop = day_stop + 1
  WHERE time_stop = '24:00'

  -- map the obj_id column to actual ids FOR existing objects
  -- unique jobs are based ON parent, name, AND _OPCOpNum(SCHEDID)
  IF @useBCPTable = 1
  BEGIN 
    UPDATE j SET j.obj_id = b.id
      FROM ca7_jobs AS j JOIN BATCH_C AS b ON b._ParentCID = 'BTCY'
        AND b._ParentID = j.btcy_id AND j.bulk_load = @useBCPTable 
        AND b.name = j.name
        AND b._SCHED_PACK = j.sched_id 
        AND b.userid = @data_source 
        AND b.deleted = 0
  END

  -- translate the CA7 time formats INTO SQL time formats
  -- the start_date_time AND end_date_time variables should be defaulted to 19000101,
  -- which should work IN any locale
  UPDATE ca7_jobs SET start_date_time = DATEADD(yyyy, year_start - DATEPART(yyyy, start_date_time), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(dy, day_start - 1, start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(yyyy, year_stop - DATEPART(yyyy, end_date_time), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(dy, day_stop - 1, end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET duration = CONVERT(INT, SUBSTRING(time_duration, 1, 2))*60 + CONVERT(INT, SUBSTRING(time_duration, 4, 5))
  -- bulk CREATE batch objects if not exists
  IF EXISTS (SELECT * FROM ca7_jobs WHERE obj_id = 0 AND bulk_load = @useBCPTable)
  BEGIN
    -- insert into work table for creation
    DELETE Disc_jobs WHERE bulk_load = @useBCPTable AND data_source = @data_source
    INSERT Disc_jobs 
      SELECT name, 'BTCY', btcy_id, sched_id, @data_source, @useBCPTable
      FROM ca7_jobs WHERE obj_id = 0 AND bulk_load = @useBCPTable 

    -- create non-exist objects
    EXEC @rows = asisp_bulkcreateBATC @useBCPTable, @data_source 
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk CREATE batch jobs failed', 0, 1) 
      ROLLBACK TRAN _CA7CreateBATC
      RETURN -30000
    END
 
    -- update job id in ca7_jobs table for cleanup
    UPDATE j SET j.obj_id = b.id
        FROM ca7_jobs AS j JOIN BATCH_C AS b ON b._ParentCID = 'BTCY'
        AND b._ParentID = j.btcy_id AND j.bulk_load = @useBCPTable 
        AND b.name = j.name
        AND CONVERT(SMALLINT, b._OPCOpNum) = j.sched_id 
        AND b.deleted = 0
        AND b.userid = @data_source 
  END

  -- UPDATE BATCH_C attributes except PriorityID which will be updated by TRIGGER
  UPDATE b
    SET b._KeyStatus = j.flag_keyjob,
      b._SchedStartTime = j.start_date_time,
      b._SchedStopTime = j.end_date_time,
      b._SchedDuration = j.duration,
      b._SchedStatusFlags = 1
    FROM BATCH_C AS b JOIN ca7_jobs AS j ON b.name = j.name 
    AND b._SCHED_PACK = j.sched_id
    AND j.bulk_load = @useBCPTable
    JOIN link AS l ON j.btcy_id = l.src_id 
    AND l.src_cid = 'BTCY'
    AND b.id = l.dst_id 
    AND l.dst_cid = 'BATC' 
    AND l.deleted = 0
  -- for clean up batch jobs
  IF @autoDiscoveryFlag = 0
  BEGIN
    DELETE Disc_job_lookups WHERE data_source = @data_source
    INSERT Disc_job_lookups SELECT btcy_id, obj_id, @data_source FROM ca7_jobs WHERE bulk_load = @useBCPTable
  END

  IF @tcount = 0
    COMMIT TRAN _CA7CreateBATC
  
  IF @rows IS NULL
    RETURN 0
  ELSE
    RETURN @rows
  SET NOCOUNT OFF

GO
