-- archive\db\sql\_discoverca7objects.sql
-- SID 1.1
-- Date Modified: 01/10/19 11:35:36
--
--  _discoverca7objects.sql
--
-- Module Discription
--  Define _DiscoverCA7Objects stored procedure
--

USE Object
GO

-----------------------------------------------------------------------------------------------
-- _DiscoverCA7Objects
PRINT 'Create procedure _DiscoverCA7Objects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCA7Objects' AND type = 'P')
  DROP PROCEDURE _DiscoverCA7Objects
GO

-- SID 1.1
-- Date Modified: 01/10/19 11:35:36
CREATE PROCEDURE _DiscoverCA7Objects
AS
  
  RAISERROR( '_DiscoverCA7Objects', 0, 1)
  SET NOCOUNT ON

  -- Standard TRANSACTION logic  
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _DiscoverCA7Objects
  ELSE
    SAVE TRAN _DiscoverCA7Objects

  -- get varibles
  DECLARE @FormatType TINYINT, @autoDiscoveryFlag TINYINT, @data_source ObjName
  SELECT @data_source = 'CA7'
  SELECT @FormatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = @data_source
  SELECT @autoDiscoveryFlag = CONVERT(TINYINT, propertyvalue) FROM SystemConfiguration 
    WHERE groupname = @data_source AND propertyname = 'autoDiscoveryFlag'
  IF @autoDiscoveryFlag IS NULL
    SELECT @autoDiscoveryFlag = 1

  -- if @autoDiscoveryFlag = 0, check lookup table for more filters  
  IF @autoDiscoveryFlag = 0
  BEGIN
    IF EXISTS (SELECT * FROM BJDiscoveryFilter_Lookup WHERE data_source = @data_source)
      DELETE ca7_bcp  
        FROM ca7_bcp AS ca7
        JOIN BJDiscoveryFilter_Lookup AS lookup ON 
        ca7.name_schedule_set like CASE lookup.name_schedule_set WHEN 'ALL' THEN '%' ELSE lookup.name_schedule_set END 
        AND ca7.name_system like CASE lookup.name_schedule WHEN 'ALL' THEN '%' ELSE lookup.name_schedule END 
        AND ca7.name_job like CASE lookup.name_job WHEN 'ALL' THEN '%' ELSE lookup.name_job END
  END

  -- Check IF other process with pstep > 1 running
  IF EXISTS ( SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstep >1 AND pstat = 1)
  BEGIN
    RAISERROR('Other loading OR process IS running.', 0, 1)
    RETURN -30000
  END

  -- SET pstep to 3, pstat = 1 FOR all data ready to process 
  UPDATE DiscoveryBatch SET pstep = 3, pstat = 1 WHERE FormatType = @FormatType AND pstep =2 AND pstat = 0

  -- clean up before process
  DECLARE @name_schedule_set ObjName, @id_schedule_set ObjID, @name_schedule ObjName, @id_schedule ObjID
  DECLARE @name_job ObjName, @id_job ObjID, @sched_id INT, @job_count INT, @like_clause VARCHAR(256)

  DELETE Disc_schedules WHERE bulk_load = 1 AND data_source = @data_source
  DELETE ca7_jobs WHERE bulk_load = 1
  -- handle objects with empty SYSTEM field AND non-executable jobs based ON global variable.
  -- These jobs will NOT be created IF @keep_unknown IS NOT SET to 1. 
  -- UPDATE SYSTEM = '*UNKNOWN*' FOR empty field IF @keep_unknown = 1 
  -- OR ELSE leave it AS NULL AND the objects under it will NOT be created
  DECLARE @keep_unknown TINYINT
  SELECT @keep_unknown = CONVERT(TINYINT, propertyvalue) 
    FROM SystemConfiguration 
    WHERE propertyname = 'keep_unknown' AND groupname = 'CA7'
  IF @keep_unknown = 1
    UPDATE ca7_bcp SET name_system = '*UNKNOWN*' WHERE name_system IS NULL OR name_system = ''
  ELSE 
     DELETE ca7_bcp WHERE name_system = '' OR name_system IS NULL
  -- Currently not handle this type object by desgin
  DELETE ca7_bcp WHERE flag_exec = 0

  -- DELETE multiple entries FOR the same job AND run ON different days
  DELETE c1 FROM ca7_bcp AS c1 
    JOIN ca7_bcp AS c2 ON c1.name_schedule_set = c2.name_schedule_set
    AND c1.name_system = c2.name_system
    AND c1.name_job = c2.name_job
    AND c1.sched_id = c2.sched_id
    AND (c1.day_start > c2.day_start OR c1.time_start > c2.time_start) 

  -- UPDATE schedule SET short name with full name 
  UPDATE b SET b.name_schedule_set = l.full_name FROM ca7_bcp AS b
    JOIN ca7_bcys_name_lookup AS l ON b.name_schedule_set = l.short_name

  -- loop ca7_bcp TABLE by schedule SET name AND created all schedules. INSERT all info FROM ca7_bcp 
  -- with _ParentCID AND _ParentID FOR all jobs.  
  DECLARE bcp CURSOR FOR SELECT DISTINCT name_schedule_set FROM ca7_bcp
  OPEN bcp 
  FETCH NEXT FROM bcp INTO @name_schedule_set
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @like_clause = '%' + short_name + '%' FROM ca7_bcys_name_lookup
      WHERE full_name = '@name_schedule_set'
    -- find schedule SET id, IF NOT EXISTS, RETURN error
    SELECT @id_schedule_set = id FROM BatchCycles_V WHERE deleted = 0 AND name = @name_schedule_set
    IF @id_schedule_set IS NULL
    BEGIN
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Schedule SET was NOT created.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      RAISERROR("Schedule SET was NOT created FOR %s.", 0, 1, @name_schedule_set)
      FETCH NEXT FROM bcp INTO @name_schedule_set
      CONTINUE
    END

    RAISERROR("Processing %s schedule SET.", 0, 1, @name_schedule_set)
    -- insert all schedules in Disc_schedule working table
    INSERT INTO Disc_schedules (name, name_schedule_set, bcys_id, data_source, bulk_load)
      SELECT DISTINCT name_system, name_schedule_set, @id_schedule_set, @data_source, 1
      FROM ca7_bcp WHERE name_schedule_set = @name_schedule_set 

    -- working on Disc_schedules table, update id if exists, else create objects
    DECLARE @rows INT 
    EXEC @rows = asisp_bulkcreateBTCY 
		@name_schedule_set, 
		@id_schedule_set, 
		1, 
		@autoDiscoveryFlag,
		@data_source 
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk CREATE batch schedules failed', 0, 1)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	  pstat = 0, 
	  error = -30000,
	  errtext = 'Bulk CREATE batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      FETCH NEXT FROM bcp INTO @name_schedule_set
      CONTINUE
    END
    -- make sure all monitored batch schedules were created under this schedule SET 
    IF EXISTS (SELECT * FROM ca7_bcp 
                 WHERE name_schedule_set = @name_schedule_set 
                 AND name_system IS NOT NULL AND name_system <> ''
                 AND name_system NOT IN 
                   (SELECT name FROM BatchCycle_V AS b 
                   JOIN BCYS_BTCY_AGGC AS l ON b.id = l.dst_id 
                   AND b.deleted = 0 
                   AND l.src_id = @id_schedule_set))
    BEGIN
      RAISERROR("Batch schedule %s was NOT created.", 0, 1, @name_schedule)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Bulk CREATE batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
    END
    -- copy monitored jobs out of the ca7_bcp TABLE INTO ca7_jobs AND UPDATE btcy_id AND id based ON 
    -- bcys_id, system_name, job name AND sched_id
    INSERT INTO ca7_jobs (
		name, 
	        sched_id,
		cycle_name, 
		flag_keyjob, 
		year_start, 
		day_start, 
		time_start, 
		time_duration, 
		year_stop, 
		day_stop, 
		time_stop,
		btcy_id,
        	bulk_load)
            SELECT c.name_job,
	        c.sched_id,
		c.name_system,
		c.flag_keyjob,
		c.year_start,
		c.day_start,
		c.time_start, 
		c.time_duration, 
		c.year_stop, 
		c.day_stop, 
		c.time_stop,
		btcy.obj_id,
        	1
      FROM ca7_bcp AS c JOIN Disc_schedules AS btcy ON c.name_system = btcy.name 
	AND btcy.name_schedule_set = c.name_schedule_set 
        AND c.name_schedule_set = @name_schedule_set
	AND btcy.bulk_load = 1
	AND btcy.data_source = 'CA7'

    FETCH NEXT FROM bcp INTO @name_schedule_set
  END 
  DEALLOCATE bcp 
  -- CREATE batch job IF NOT EXISTS, THEN UPDATE the plan info FOR jobs using ca7_jobs TABLE
  EXEC @rows = _CA7CreateBATC 1, @autoDiscoveryFlag, @data_source
  IF @rows < 0
  BEGIN
    RAISERROR('Bulk CREATE batch job failed', 0, 1)
    ROLLBACK TRAN _DiscoverCA7Objects
    RETURN -30000
  END

  -- clean up non-monitored batch schedules and batch jobs
  IF @autoDiscoveryFlag = 0
    EXEC asisp_cleanupAutoDiscJob @data_source

  -- Finish TRANSACTION
  IF @tcount = 0
    COMMIT TRAN _DiscoverCA7Objects

  SET NOCOUNT OFF  

  IF NOT EXISTS (SELECT * FROM DiscoveryBatch 
    WHERE FormatType = @FormatType AND pstep = 3 AND error <> 0 AND error IS NOT NULL)
  BEGIN
    -- SET pstep to 3, pstat = 1 FOR all data ready to process 
    DELETE DiscoveryBatch WHERE FormatType = @FormatType AND pstep = 3 AND pstat = 1 
    RETURN 0
  END
  ELSE
  BEGIN
    RAISERROR('Please check DiscoveryBatch TABLE FOR error messages', 0, 1)
    RETURN -30000
  END


