-- archive\db\sql\_getbatchcycles.sql
-- SID 1.1
-- DateModified: 01/10/19 11:28:33
--
-- _GetBatchCycles
--
-- Module Description
--   Defines _GetBatchCycles stored procedure

USE Object
GO

------------------------------------------------------------------------------------------------------------
-- _GetBatchCycles
PRINT 'Creating _GetBatchCycles procedure.'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = '_GetBatchCycles' AND type = 'P')
  DROP PROCEDURE _GetBatchCycles
GO

-- SID 1.1
-- DateModified: 01/10/19 11:28:33
CREATE PROCEDURE _GetBatchCycles
  @cid              ClassID      = NULL,
  @id               ObjID        = NULL,
  @currentStateExpr VARCHAR(255) = NULL,
  @desiredStateExpr VARCHAR(255) = NULL
AS
  RAISERROR('_GetBatchCycles @cid=''%s'', @id=%d, @currentStateExpr=''%s'', @desiredStateExpr=''%s''', 0, 1,
            @cid, @id, @currentStateExpr, @desiredStateExpr)

  -- Assemble the portion of the WHERE-clause HAVING to do with current/desired state
  DECLARE @stateWhereClause VARCHAR(255)
  IF @currentStateExpr IS NOT NULL
    SELECT @stateWhereClause = 'AND _CurrentStateID' + @currentStateExpr
  IF @desiredStateExpr IS NOT NULL
    IF @stateWhereClause IS NOT NULL
      SELECT @stateWhereClause = @stateWhereClause + ' AND _DesiredStateID ' + @desiredStateExpr
    ELSE
      SELECT @stateWhereClause = ' AND _DesiredStateID ' + @desiredStateExpr
  -- Assemble the variable declarations FOR dynamic SQL
  DECLARE @cidDecl VARCHAR(255)
  SELECT @cidDecl = 'DECLARE @cid ClassID SELECT @cid='
  IF @cid IS NULL
    SELECT @cidDecl = @cidDecl + 'NULL '
  ELSE
    SELECT @cidDecl = @cidDecl + '''' + RTRIM(@cid) + ''' '

  CREATE TABLE #descendent
  (
    cid   ClassID,
    id    ObjID,
    gen   INT
  )
  EXEC asisp_descendents @cid, @id, 'IN (''BTCY'')'

  IF NOT EXISTS (SELECT * FROM #descendent)
    RETURN
  
  EXEC("
  SELECT 
    cid='BTCY',
    id=btcy.id,
    'name'=btcy.name, 
    CASE WHEN bcys.name IS NULL THEN btcy.name ELSE bcys.name + '/' + btcy.name END AS 'schedule',
    CASE _Desc WHEN '' THEN NULL ELSE _Desc END AS _Desc,
    _AlertStateID,
    _PriorityID,
    _CurrentStateID,
    _DesiredStateID,
    _HasOwnershipNote, 
    _StartTime,
    _StopTime,
    _Duration,
    _InitEstComplete,
    _CurrEstComplete,
    _EstStartTime,
    _EstDuration, 
    _MedianStartTime,
    _MedianStopTime,
    _MedianDuration,
    _TotalRuns,
    _GoodRuns
  FROM
    #descendent AS d 
      JOIN BatchCycle_V AS btcy ON btcy.id = d.id
      LEFT OUTER JOIN containment AS c ON c.dst_cid = 'BTCY' AND c.dst_id = d.id AND c.deleted = 0
      LEFT OUTER JOIN BatchCycles_C AS bcys ON bcys.id = c.src_id
  WHERE
    1=1 " + @stateWhereClause
  )
