-- archive\db\sql\asisp_bulkcreatebtcy.sql
-- SID 1.1
-- Date Modified: 01/10/18 13:07:51
--
-- asisp_bulkcreateBTCY
--
-- Module Description
--   Defines asisp_bulkcreateBTCY stored procedure

USE Object
GO

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- 
-- asisp_bulkcreateBTCY
PRINT 'Creating procedure asisp_bulkcreateBTCY.'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = 'asisp_bulkcreateBTCY' AND type = 'P')
  DROP PROCEDURE asisp_bulkcreateBTCY
GO                                                                                                                                                                                                                                                             

-- SID 1.1
-- Date Modified: 01/10/18 13:07:51
CREATE PROCEDURE asisp_bulkcreateBTCY
  @name_schedule_set 	ObjName = NULL,
  @id_schedule_set 	ObjID = NULL,
  @useBCPTable 		TINYINT = 1,
  @autoDiscoveryFlag    TINYINT = 1,
  @data_source 		ObjName = NULL
AS

  SET NOCOUNT ON

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_bulkcreateBTCY
  ELSE
    SAVE TRAN asisp_bulkcreateBTCY

  IF @useBCPTable = 1
    -- map the obj_id column to actual ids
    UPDATE cyc SET cyc.obj_id = btcy.id
      FROM Disc_schedules AS cyc, BatchCycle_V AS btcy 
      JOIN containment AS c ON c.dst_cid = 'BTCY' AND c.dst_id = btcy.id
      WHERE c.deleted = 0 
      AND cyc.name = btcy.name 
      AND cyc.bcys_id = c.src_id 
      AND cyc.bulk_load = @useBCPTable
      AND cyc.bcys_id = @id_schedule_set
      AND btcy.userid = @data_source

  -- CREATE new instances
  CREATE TABLE #schedules_new
  (
    id ObjID IDENTITY(0,1),
    name ObjName NULL,
    parent_cid ClassID NULL,
    parent_id ObjID
  )

  INSERT INTO #schedules_new (name, parent_cid, parent_id)
  SELECT name, 'BCYS', bcys_id
    FROM Disc_schedules
    WHERE (obj_id = 0 OR obj_id IS NULL)
    AND bulk_load = @useBCPTable AND data_source = @data_source

  IF EXISTS (SELECT * FROM #schedules_new)
  BEGIN
    DECLARE @base_id ObjID, @rows INT
    SELECT @rows = COUNT(id) FROM #schedules_new
    IF @rows > 0
      EXEC @base_id = alloc_BTCY @rows
    ELSE
      SET @base_id = 0

    -- INSERT INTO _C
    INSERT BatchCycle_C (id, name, userid)
      SELECT id+@base_id, name, @data_source
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    -- INSERT INTO _A
    INSERT BatchCycle_A (id,_Desc,_ChildEventMatrixID,_ExceptionMatrixID,_PriorityID)
      SELECT id+@base_id, '', 0, 0, 6
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    -- INSERT links
    INSERT INTO BCYS_BTCY_AGGC (src_id, dst_id)
      SELECT parent_id, id + @base_id
      FROM #schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN asisp_bulkcreateBTCY
      RETURN -30000
    END

    UPDATE d set d.obj_id = n.id + @base_id 
      FROM Disc_schedules AS d 
      JOIN #schedules_new AS n ON d.bcys_id = n.parent_id
      AND d.name =  n.name  

  END

  IF @tcount = 0
    COMMIT TRAN asisp_bulkcreateBTCY
  
  SET NOCOUNT OFF


