-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/asisp_checkobjmatrices.sqv   1.0   Sep 07 2001 16:35:46   Forthman  $
--
-- asisp_checkobjmatrices.sql
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_checkobjmatrices
PRINT 'Creating procedure asisp_checkobjmatrices'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_checkobjmatrices' AND type = 'P')
  DROP PROCEDURE asisp_checkobjmatrices
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/asisp_checkobjmatrices.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:35:46  $
-- $Author:   Forthman  $
-- Makes sure the count matrices for the specified object exist and have been fully instantiated.
CREATE PROCEDURE asisp_checkobjmatrices
  @cid    ClassID,
  @id     ObjID
AS
--  PRINT 'asisp_checkobjmatrices @cid=''' + @cid + ''', @id=' + CAST(@id AS VARCHAR)

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_checkobjmatrices
  ELSE
    SAVE TRANSACTION asisp_checkobjmatrices

  DECLARE @cname VARCHAR(24), @id_str VARCHAR(9)

  -- Lookup object class name  
  SELECT @cname = cname, @id_str = CAST(@id AS VARCHAR(9))
  FROM obj_class
  WHERE cid = @cid
  IF @cname IS NULL
  BEGIN
    RAISERROR('asisp_checkobjmatrices @cid=''%s'', @id=%d: ''%s'' is not a valid class id', 3, 1, @cid, @id, @cid)
    ROLLBACK TRANSACTION asisp_checkobjmatrices
    RETURN -30000
  END

  -- Lookup matrix indices on the object
  CREATE TABLE #matrixid ( ChildEventMatrixID INT NULL, ExceptionMatrixID INT NULL )
  EXEC("
  INSERT #matrixid SELECT _ChildEventMatrixID, _ExceptionMatrixID
  FROM " + @cname + "_V
  WHERE id = " + @id_str
  )
  IF NOT EXISTS (SELECT * FROM #matrixid)
  BEGIN
    RAISERROR('asisp_checkobjmatrices @cid=''%s'', @id=%d: %d is not a valid object id', 3, 1, @cid, @id, @id)
    ROLLBACK TRANSACTION asisp_checkobjmatrices
    RETURN -30000
  END

  DECLARE @ChildEventMatrixID INT, @ExceptionMatrixID INT
  SELECT @ChildEventMatrixID = ISNULL(ChildEventMatrixID, 0), @ExceptionMatrixID = ISNULL(ExceptionMatrixID, 0)
  FROM #matrixid

  IF @ChildEventMatrixID = 0 OR @ExceptionMatrixID = 0
  BEGIN
    DECLARE @link_table sysname, @acpx_id_str VARCHAR(9), @aepx_id_str VARCHAR(9)

    IF @ChildEventMatrixID = 0
    BEGIN
      PRINT 'Creating ChildEvenMatrix'
      EXEC @ChildEventMatrixID = alloc_PRMX
      IF @ChildEventMatrixID < 0
      BEGIN
        ROLLBACK TRANSACTION asisp_checkobjmatrices
        RETURN @ChildEventMatrixID
      END
      SET @link_table = RTRIM(@cid) + '_PRMX_ACPX'
      SET @acpx_id_str = CAST(@ChildEventMatrixID AS VARCHAR(9))
      EXEC("
      IF EXISTS (SELECT * FROM " + @link_table + " WHERE src_id = " + @id_str + ")
        DELETE " + @link_table + " WHERE src_id = " + @id_str + "
      INSERT " + @link_table + "(src_id, dst_id) SELECT " + @id_str + ", " + @acpx_id_str
      )
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION asisp_checkobjmatrices
        RETURN @ChildEventMatrixID
      END
    END
    ELSE
      SET @acpx_id_str = CAST(@ChildEventMatrixID AS VARCHAR(9))

    IF @ExceptionMatrixID = 0
    BEGIN
      PRINT 'Creating ExceptionMatrix'
      EXEC @ExceptionMatrixID = alloc_PRMX
      IF @ExceptionMatrixID < 0
      BEGIN
        ROLLBACK TRANSACTION asisp_checkobjmatrices
        RETURN @ExceptionMatrixID
      END
      SET @link_table = RTRIM(@cid) + '_PRMX_AEPX'
      SET @aepx_id_str = CAST(@ExceptionMatrixID AS VARCHAR(9))
      EXEC("
      IF EXISTS (SELECT * FROM " + @link_table + " WHERE src_id = " + @id_str + ")
        DELETE " + @link_table + " WHERE src_id = " + @id_str + "
      INSERT " + @link_table + "(src_id, dst_id) SELECT " + @id_str + ", " + @aepx_id_str
      )
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION asisp_checkobjmatrices
        RETURN @ExceptionMatrixID
      END
    END
    ELSE
      SET @aepx_id_str = CAST(@ExceptionMatrixID AS VARCHAR(9))

    EXEC("
    UPDATE " + @cname + "_V SET
      _ChildEventMatrixID = " + @acpx_id_str +  ",
      _ExceptionMatrixID = " + @aepx_id_str + "
    WHERE
      id = " + @id_str
    )
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_checkobjmatrices
      RETURN @ExceptionMatrixID
    END
  END

  DECLARE @r INT

  EXEC @r = asisp_checkobjmatrix @cid, @id, @ChildEventMatrixID, 'ACPX'
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_checkobjmatrices
    RETURN @r
  END

  EXEC asisp_checkobjmatrix @cid, @id, @ExceptionMatrixID, 'AEPX'
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_checkobjmatrices
    RETURN @r
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_checkobjmatrices
GO
