-- archive\db\sql\asisp_cleanupcicstasks.sql
-- SID: 1.2
-- DateModified: 01/11/20 15:54:48
-- 
-- Define module asisp_cleanupcicstasks
-- Fix defect OW51612

USE Object
GO
---------------------------------------------------------------------------------------------------------------------------
-- asisp_cleanupcicstasks
PRINT 'Create procedure asisp_cleanupcicstasks.'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = 'asisp_cleanupcicstasks' AND type = 'P')
  DROP PROCEDURE asisp_cleanupcicstasks
GO

-- Removes all successfully completed CICS tasks older than the specified number of days.
-- This involves removing FROM the msdb..sysjobs, msdb..sysjobsteps, msdb..sysjobserver TABLES, which defines the task, AND FROM the
-- Object..cics_task TABLE which lists the individual regions affected by each task.
CREATE PROCEDURE asisp_cleanupcicstasks
  @daystokeep INT = 7
AS
  RAISERROR('asisp_cleanupcicstasks: @daystokeep=%d', 0, 1, @daystokeep)

  DECLARE @lastrundate INT, @d DATETIME, @onemonthago INT
  SELECT @d = DATEADD(day, -@daystokeep, GETDATE())
  SELECT @lastrundate = DATEPART(year, @d) * 10000 + DATEPART(month, @d) * 100 + DATEPART(day, @d)
  SELECT @d = DATEADD(month, -1, GETDATE())
  SELECT @onemonthago = DATEPART(year, @d) * 10000 + DATEPART(month, @d) * 100 + DATEPART(day, @d)
  CREATE TABLE #task (id UNIQUEIDENTIFIER, name NVARCHAR(255))
  INSERT #task 
    SELECT DISTINCT job.job_id, job.name
    FROM msdb..sysjobs AS job
    RIGHT OUTER JOIN msdb..sysjobsteps AS step ON job.job_id = step.job_id 
    WHERE  
      (job.name like 'CICS Group%' OR job.name like 'CICS-[0-9]%Activate%' OR job.name like 'CICS:[0-9]% to Active%')
      AND
      ( (step.last_run_date < @lastrundate AND job.job_id NOT IN (SELECT job_id FROM msdb..sysjobsteps WHERE last_run_outcome <> 1))
        OR
        step.last_run_date < @onemonthago
      )
  DECLARE @stat INT, @name NVARCHAR(255)
  DECLARE cc CURSOR FOR SELECT DISTINCT name FROM #task
  OPEN cc
  FETCH NEXT FROM cc INTO @name
  WHILE @@fetch_status <> -1
  BEGIN
    RAISERROR('asisp_cleanupcicstasks: Dropping Task: [%s]', 0, 1, @name)
    EXEC msdb..sp_delete_job @job_name=@name
    FETCH NEXT FROM cc INTO @name
  END
  DEALLOCATE cc

GO 
