-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_deleteoldmessages.sqv   1.0   Aug 22 2001 19:27:32   BRENT  $
--
-- asisp_deleteoldmessages.sql
--
-- Module Description
--   Defines the asisp_deleteoldmessages stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_deleteoldmessages
PRINT 'Creating procedure asisp_deleteoldmessages'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_deleteoldmessages' AND type = 'P')
  DROP PROCEDURE asisp_deleteoldmessages
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_deleteoldmessages.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 22 2001 19:27:32  $
-- $Author:   BRENT  $
-- Deletes all messages that can be deleted that are older than a given number of days. A message
-- is considered deletable if it is not currently being used to determine the state of an object.
--
-- Modified by EBH on 8/13/01 to disable deletes from BatchCycleJobHist table
CREATE PROCEDURE asisp_deleteoldmessages
  @daystokeep INT=7
AS
  DECLARE @oldest DATETIME
  SELECT @oldest = DATEADD(day, -@daystokeep, GETDATE())
  DECLARE @num_mesg INT, @num_bch INT, @num_bcjh INT
  -- Make a list of all messages that cannot be deleted. These are the messages older than the 
  -- specified timestamp that are non-ignore and for which there is no later message on the 
  -- object that is also non-ignore.
  SELECT id, ctime, _ParentCID, _ParentID
  INTO #dont_delete
  FROM Message_V m1 
  WHERE 
    ctime < @oldest
    AND _Priority < 5
    AND NOT EXISTS
    (
      SELECT * FROM Message_V m2 
      WHERE
        m2._ParentCID = m1._ParentCID
        AND m2._ParentID = m1._ParentID
        AND m2.id > m1.id
        AND m2._Priority < 5
    )
  -- If any messages that are candidates for deletion have uncleared OwnedEvent records these need
  -- to be cleared so the notes may be properly closed.
  UPDATE OwnedEvent SET status = 1
  FROM Message_V m
  WHERE
    eventcid = 'MESG'
    AND eventid = m.id
    AND m.ctime < @oldest
    AND OwnedEvent.status = 0
    AND NOT EXISTS (SELECT * FROM #dont_delete WHERE id = eventid)
  -- Delete all messages that can be deleted
  DELETE Message_C 
  WHERE 
    ctime < @oldest 
    AND NOT EXISTS (SELECT * FROM #dont_delete WHERE id = Message_C.id)
  SELECT @num_mesg = @@ROWCOUNT
  -- Also delete BatchCycle history records that no longer have corresponding messages
  DELETE BatchCycleHist WHERE eventcid = 'MESG' AND eventid NOT IN (SELECT id FROM Message_C)
  SELECT @num_bch = @@ROWCOUNT
--  DELETE BatchCycleJobHist WHERE eventcid = 'MESG' AND eventid NOT IN (SELECT id FROM Message_C)
--  SELECT @num_bcjh = @@ROWCOUNT
  SET @num_bcjh = 0
  DECLARE @d NVARCHAR(40)
  SELECT @d = CONVERT(NVARCHAR(40), GETDATE(), 113)
  RAISERROR('%s: Total deleted: #Message=%d, #BatchCycleHist=%d, #BatchCycleJobHist=%d', 0, 1,
            @d, @num_mesg, @num_bch, @num_bcjh)
GO
