-------------------------------------------------------------------------------
-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/asisp_discoverDB2Resource.sqv   1.0   Sep 07 2001 16:38:24   Forthman  $
-------------------------------------------------------------------------------

Use Object
GO

-------------------------------------------------------------------------------
-- asisp_discoverDB2Resource
PRINT 'create procedure asisp_discoverDB2Resource'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_discoverDB2Resource' AND type = 'P' )
  DROP PROCEDURE asisp_discoverDB2Resource
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/asisp_discoverDB2Resource.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:38:24  $
-- $Author:   Forthman  $
-- Creates AND processes a discovery batch FOR DB2 using the specified parameters FOR a single object discovery
CREATE PROCEDURE asisp_discoverDB2Resource
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @TimeStamp         NVARCHAR(40),
  @SeqNo             NVARCHAR(16),
  @cid               ClassID ,
  @SystemId          NVARCHAR(40),
  @SubsysName        NVARCHAR(8) ,
  @MasterName        NVARCHAR(40)=NULL,
  @LockMgrName       NVARCHAR(40)=NULL,
  @DBSvcsName        NVARCHAR(40)=NULL,
  @DDFName           NVARCHAR(40)=NULL,
  @SPMgrName         NVARCHAR(40)=NULL,
  @BUFFER_POOLS      NVARCHAR(40)=NULL,
  @EDM_POOL          NVARCHAR(40)=NULL,
  @LOGS              NVARCHAR(40)=NULL,
  @DB2PM             NVARCHAR(40)=NULL,
  @DATABASE_NAME     NVARCHAR(40)=NULL,
  @TABLESPACE_NAME   NVARCHAR(40)=NULL,
  @TABLE_NAME        NVARCHAR(40)=NULL,
  @INDEX_NAME        NVARCHAR(40)=NULL,
  @TABLESPACE_PART   NVARCHAR(40)=NULL,
  @INDEX_PART        NVARCHAR(40)=NULL,
  @VIEW_NAME         NVARCHAR(40)=NULL,
  @STATE             NVARCHAR(40)=NULL,
  @auto_create       TINYINT = 1,
  @obj_cid           ClassID OUTPUT,
  @obj_id            ObjID OUTPUT,
  @obj_name          ObjName OUTPUT,
  @verbose           TINYINT = 0
AS
  -- Initialization FOR the timings of steps
  DECLARE @startdate DATETIME
  DECLARE @nowdate   DATETIME
  DECLARE @stepdate  DATETIME
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  NVARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT
  DECLARE @debug INT

  SELECT @debug = 0

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             ( s_step/ s_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_discoverDB2Resource
  ELSE
    SAVE TRANSACTION asisp_discoverDB2Resource

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'BEGIN TRANSACTION'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- CREATE a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, numrows) 
    SELECT @now, @FormatType, @ActionType, 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr DiscoveryBatch'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @batchid INT
  SELECT @batchid = batchid FROM DiscoveryBatch WHERE ctime = @now
  IF @batchid IS NULL
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Ch batchid'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- SET object name based ON @cid
  SELECT @obj_name =
    CASE @cid
      WHEN 'DBSS' THEN @SubsysName
      WHEN 'DBMS' THEN @MasterName
      WHEN 'DBLK' THEN @LockMgrName
      WHEN 'DBDS' THEN @DBSvcsName
      WHEN 'DBDD' THEN @DDFName
      WHEN 'DBSP' THEN @SPMgrName
      WHEN 'DBBP' THEN @BUFFER_POOLS
      WHEN 'DBEP' THEN @EDM_POOL
      WHEN 'DBLG' THEN @LOGS
      WHEN 'DBPM' THEN @DB2PM
      WHEN 'DBDB' THEN @DATABASE_NAME
      WHEN 'DBTS' THEN @TABLESPACE_NAME
      WHEN 'DBTB' THEN @TABLE_NAME
      WHEN 'DBIX' THEN @INDEX_NAME
      WHEN 'DBTP' THEN @TABLESPACE_PART
      WHEN 'DBIP' THEN @INDEX_PART
      WHEN 'DBVW' THEN @VIEW_NAME
    END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @obj_name'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- SET discovered object type to the type of object this will be after discovery
  -- This IS to handle that the give cid (like DBSS) gets converted during processing (to D2SS),
  -- Certain queries IN this PROCEDURE need to be aware of this conversion 
  DECLARE @cid_discovered NVARCHAR(40) 
  SELECT @cid_discovered  =
    CASE @cid
      WHEN 'DBSS' THEN 'D2SS'
      WHEN 'DBMS' THEN 'D2MS' 
      WHEN 'DBLK' THEN 'D2LK' 
      WHEN 'DBDS' THEN 'D2DS' 
      WHEN 'DBDD' THEN 'D2DD' 
      WHEN 'DBSP' THEN 'D2SP' 
      WHEN 'DBBP' THEN 'D2BP' 
      WHEN 'DBEP' THEN 'D2EP' 
      WHEN 'DBLG' THEN 'D2LG' 
      WHEN 'DBPM' THEN 'D2PM' 
      WHEN 'DBDB' THEN 'D2DB' 
      WHEN 'DBTS' THEN 'D2TS' 
      WHEN 'DBTB' THEN 'D2TB' 
      WHEN 'DBIX' THEN 'D2IX' 
      WHEN 'DBTP' THEN 'D2TP' 
      WHEN 'DBIP' THEN 'D2IP' 
      WHEN 'DBVW' THEN 'D2VW' 
    END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @cid_discovered'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- CREATE temp TABLE that gets populated with the class AND name of
  -- each object that must be created
  CREATE TABLE #object
  (
    obj_cid   ClassID,
    obj_name  ObjName,
    is_object TINYINT
  )

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr #object'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- INSERT record to CREATE the specified object
  INSERT INTO #object VALUES ( @cid, @obj_name, 1 )

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object @obj_name'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- INSERT parent records IF necessary

  -- DB2 TABLE Indexes need parent TABLE record 
  IF @cid = 'DBIX'
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTB', @TABLE_NAME, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object DBIX'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- DB2 TABLE Partition Indexes need parent TABLE Partition record 
  IF @cid = 'DBIP'
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTP', @TABLESPACE_PART, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object DBIP'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- DB2 TABLE AND TABLE Partition AND children need parent Tablespace record 
  IF @cid IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' )
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTS', @TABLESPACE_NAME, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object DBTS'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- DB2 Tablespace AND children need parent Database record 
  IF @cid IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' )
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBDB', @DATABASE_NAME, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object DBDB'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- DB2 Subcomponent AND children need parent Subsystem record 
  IF @cid <> 'DBSS' 
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBSS', @SubsysName, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In #object DBSC'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- INSERT DB2Discovery records based ON the rows that have been entered INTO #object temp TABLE
  INSERT INTO DB2Discovery
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_SUBCOMPONENT,
    DB2_TABLESPACE,
    DB2_TABLE,
    DB2_INDEX,
    dbsu_cid,
    dbsc_cid,
    dbts_cid,
    dbtb_cid,
    dbin_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    STATE
  )
  SELECT DISTINCT
    @batchid,
    @SystemId,
    o.obj_cid,
    CASE
      WHEN @SubsysName    IS NOT NULL THEN @SubsysName
    END AS DB2_SUBSYSTEM,
    CASE
      WHEN o.obj_cid = 'DBSS'         THEN NULL
      WHEN @MasterName    IS NOT NULL THEN @MasterName
      WHEN @LockMgrName   IS NOT NULL THEN @LockMgrName
      WHEN @DBSvcsName    IS NOT NULL THEN @DBSvcsName
      WHEN @DDFName       IS NOT NULL THEN @DDFName
      WHEN @SPMgrName     IS NOT NULL THEN @SPMgrName
      WHEN @BUFFER_POOLS  IS NOT NULL THEN @BUFFER_POOLS
      WHEN @EDM_POOL      IS NOT NULL THEN @EDM_POOL
      WHEN @LOGS          IS NOT NULL THEN @LOGS
      WHEN @DB2PM         IS NOT NULL THEN @DB2PM
      WHEN @DATABASE_NAME IS NOT NULL THEN @DATABASE_NAME
      WHEN @VIEW_NAME     IS NOT NULL THEN @VIEW_NAME
    END AS DB2_SUBCOMPONENT,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLESPACE_NAME IS NOT NULL THEN @TABLESPACE_NAME
    END AS DB2_TABLESPACE,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLE_NAME      IS NOT NULL THEN @TABLE_NAME
      WHEN @TABLESPACE_PART IS NOT NULL THEN @TABLESPACE_PART
    END AS DB2_TABLE,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBIX', 'DBIP' ) THEN NULL
      WHEN @INDEX_NAME IS NOT NULL THEN @INDEX_NAME
      WHEN @INDEX_PART IS NOT NULL THEN @INDEX_PART
    END AS DB2_INDEX,
    CASE
      WHEN @SubsysName    IS NOT NULL THEN 'D2SS'
    END AS dbsu_cid,
    CASE
      WHEN o.obj_cid = 'DBSS'         THEN NULL
      WHEN @MasterName    IS NOT NULL THEN 'D2MS'
      WHEN @LockMgrName   IS NOT NULL THEN 'D2LK'
      WHEN @DBSvcsName    IS NOT NULL THEN 'D2DS'
      WHEN @DDFName       IS NOT NULL THEN 'D2DD'
      WHEN @SPMgrName     IS NOT NULL THEN 'D2SP'
      WHEN @BUFFER_POOLS  IS NOT NULL THEN 'D2BP'
      WHEN @EDM_POOL      IS NOT NULL THEN 'D2EP'
      WHEN @LOGS          IS NOT NULL THEN 'D2LG'
      WHEN @DB2PM         IS NOT NULL THEN 'D2PM'
      WHEN @DATABASE_NAME IS NOT NULL THEN 'D2DB'
      WHEN @VIEW_NAME     IS NOT NULL THEN 'D2VW'
    END AS dbsc_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLESPACE_NAME IS NOT NULL THEN 'D2TS'
    END AS dbts_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLE_NAME      IS NOT NULL THEN 'D2TB'
      WHEN @TABLESPACE_PART IS NOT NULL THEN 'D2TP'
    END AS dbtb_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBIX', 'DBIP' ) THEN NULL
      WHEN @INDEX_NAME IS NOT NULL THEN 'D2IX'
      WHEN @INDEX_PART IS NOT NULL THEN 'D2IP'
    END AS dbin_cid,
    o.obj_name,

    SUBSTRING(@TimeStamp,1,4)+'-'+SUBSTRING(@TimeStamp,5,2)+'-'+SUBSTRING(@TimeStamp,7,2)+' '+SUBSTRING(@TimeStamp,9,12),
    @SeqNo,
    CASE WHEN o.is_object = 1 THEN @STATE ELSE NULL END
  FROM
    #object AS o
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In DB2Discovery'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- Special handling FOR 1708 records:
  -- 1708 records FOR the D2TB actually contain two objects the TABLE AND the VIEW that points to that TABLE.
  -- Need to have the VIEW be discovered with the TABLE so that an INSERT to the D2VW_D2TB_USEC TABLE can be made later.
  IF ( @FormatType = 17 AND @ActionType = 8 AND @cid = 'DBTB' AND @VIEW_NAME IS NOT NULL )
  BEGIN
    INSERT INTO DB2Discovery
    (
      batchid,
      SystemId,
      obj_cid,
      DB2_SUBSYSTEM,
      DB2_SUBCOMPONENT,
      dbsu_cid,
      dbsc_cid,
      obj_name,
      DATE_TIME,
      SEQ_NO
    )
    SELECT DISTINCT
      @batchid,
      @SystemId,
      'DBVW',
      CASE
        WHEN @SubsysName    IS NOT NULL THEN @SubsysName
      END AS DB2_SUBSYSTEM,
      @VIEW_NAME,
      CASE
        WHEN @SubsysName    IS NOT NULL THEN 'D2SS'
      END AS dbsu_cid,
      CASE
        WHEN @VIEW_NAME     IS NOT NULL THEN 'D2VW'
      END AS dbsc_cid,
      CASE
        WHEN @VIEW_NAME     IS NOT NULL THEN @VIEW_NAME
      END AS obj_name,
    SUBSTRING(@TimeStamp,1,4)+'-'+SUBSTRING(@TimeStamp,5,2)+'-'+SUBSTRING(@TimeStamp,7,2)+' '+SUBSTRING(@TimeStamp,9,12),
    @SeqNo

    -- Timing of Step
    IF @debug >= 1
    BEGIN
      SELECT @stepname = 'In DB2Discovery 1708'
      SELECT @nowdate    = GETDATE()
      SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
      SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
      RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
      SELECT @stepdate   = @nowdate
    END
  END
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END


  DECLARE @r INT
  EXEC @r = _DiscoverDB2Resources ROOT, 1, @batchid, @incremental=1, @verbose=@verbose

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = '_DiscoverDB2Resources'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Special handling FOR 1708 records:
  -- Establish USEC link FROM D2VF to D2TB 

  IF ( @FormatType = 17 AND @ActionType = 8 AND @cid = 'DBTB' AND @VIEW_NAME IS NOT NULL )
  BEGIN
    PRINT 'IN THE SPECIAL HANDLING SECTION'
    INSERT INTO D2VW_D2TB_USEC                             -- Establish the USEC relationship
    ( src_id, dst_id )
    SELECT d1.obj_id, d2.obj_id FROM ProcessedDB2Discovery AS d1
    INNER JOIN ProcessedDB2Discovery AS d2 ON d1.batchid = d2.batchid  -- one TABLE FOR each of side of the relationship
    WHERE 
          d1.batchid = @batchid         -- match ON current batch
      AND d1.obj_cid = 'D2VW'           -- src record IS the VIEW 
      AND d2.obj_cid = 'D2TB'           -- dst record IS the TABLE
      AND NOT EXISTS (                  -- but only IF the record has NOT been created before
        SELECT * FROM D2VW_D2TB_USEC AS l 
        WHERE 
              l.src_id = d1.obj_id 
          AND l.dst_id = d2.obj_id
      )
    -- Timing of Step
    IF @debug >= 1
    BEGIN
      SELECT @stepname = 'In D2VW_D2TB_USEC 1708'  
      SELECT @nowdate    = GETDATE()
      SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
      SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
      RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
      SELECT @stepdate   = @nowdate
    END
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = '1708 post discovery processing'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  SELECT @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name
  FROM ProcessedDB2Discovery
  WHERE batchid = @batchid AND obj_cid = @cid_discovered AND obj_name = @obj_name

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se Exit assignments'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  IF @tcount = 0
    COMMIT TRANSACTION asisp_discoverDB2Resource

  RETURN @r
GO