-------------------------------------------------------------------------------
-- $Header:   //Archive/PVCSROOT/Archive/DB/sql/asisp_discoverIMSResource.sqv   1.0   Sep 07 2001 16:38:30   Forthman  $
-------------------------------------------------------------------------------

Use Object
GO

-------------------------------------------------------------------------------
-- asisp_discoverIMSResource
PRINT 'create procedure asisp_discoverIMSResource'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_discoverIMSResource' AND type = 'P' )
  DROP PROCEDURE asisp_discoverIMSResource
GO



-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/asisp_discoverIMSResource.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:38:30  $
-- $Author:   Forthman  $
-- Creates AND processes a discovery batch FOR IMS using the specified parameters
CREATE PROCEDURE asisp_discoverIMSResource
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @DATE_TIME         NVARCHAR(40) = NULL, --  1
  @SEQ_NO            NVARCHAR(40) = NULL, --  2
  @OBJ_TYPE_ID       NVARCHAR(40) = NULL, --  3
  @SystemId          NVARCHAR(40) = NULL, --  4
  @MVS               NVARCHAR(40) = NULL, --  5
  @IMS_TMDB          NVARCHAR(40) = NULL, --  6
  @IMS_DCCTL         NVARCHAR(40) = NULL, --  7
  @IMS_DBCTL         NVARCHAR(40) = NULL, --  8
  @IMS_IRLM          NVARCHAR(40) = NULL, --  9
  @IMS_CQS           NVARCHAR(40) = NULL, -- 10
  @IMS_FDR           NVARCHAR(40) = NULL, -- 11
  @IMS_ITOC          NVARCHAR(40) = NULL, -- 12
  @IMS_MPR           NVARCHAR(40) = NULL, -- 13
  @IMS_FPM           NVARCHAR(40) = NULL, -- 14
  @IMS_BMP           NVARCHAR(40) = NULL, -- 15
  @IMS_LOGS          NVARCHAR(40) = NULL, -- 16
  @IMS_TRANSACTION   NVARCHAR(40) = NULL, -- 17
  @IMS_PROGRAM       NVARCHAR(40) = NULL, -- 18
  @IMS_DATABASE      NVARCHAR(40) = NULL, -- 19
  @IMS_AREA          NVARCHAR(40) = NULL, -- 20
  @IMS_ESAF_DB2      NVARCHAR(40) = NULL, -- 21
  @IMS_ESAF_MQS      NVARCHAR(40) = NULL, -- 22
  @DD                NVARCHAR(40) = NULL, -- 23
  @DSN               NVARCHAR(40) = NULL, -- 24
  @ASSOCMVS          NVARCHAR(40) = NULL, -- 25
  @ASSOCIATE         NVARCHAR(40) = NULL, -- 26
  @STATE             NVARCHAR(40) = NULL, -- 27
  @Event             NVARCHAR(40) = NULL, -- 28
  @EventText         NVARCHAR(255) = NULL, -- 29
  @auto_create       TINYINT = 1,
  @obj_cid           ClassID OUTPUT,
  @obj_id            ObjID OUTPUT,
  @obj_name          ObjName OUTPUT,
  @verbose           TINYINT = 0
AS
  -- Initialization FOR the timings of steps
  DECLARE @startdate DATETIME
  DECLARE @nowdate   DATETIME
  DECLARE @stepdate  DATETIME
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  NVARCHAR(20)
  SELECT @stepname = NULL
  DECLARE @steptime INT
  DECLARE @totaltime INT
  DECLARE @debug INT

  SELECT @debug = 0

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             ( s_step/ s_total)', 0, 1)
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'INIT'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_discoverIMSResource
  ELSE
    SAVE TRANSACTION asisp_discoverIMSResource

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'BEGIN TRANSACTION'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  -- CREATE a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, numrows) 
    SELECT @now, @FormatType, @ActionType, 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Cr DiscoveryBatch'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @batchid INT
  SELECT @batchid = batchid FROM DiscoveryBatch WHERE ctime = @now
  IF @batchid IS NULL
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Ch batchid'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- SET object name based ON OBJ_TYPE_ID
  SELECT @obj_name =
    CASE @OBJ_TYPE_ID
      WHEN 'IMSS' THEN @IMS_TMDB
      WHEN 'IMTM' THEN @IMS_DCCTL
      WHEN 'IMDB' THEN @IMS_DBCTL
      WHEN 'IMIR' THEN @IMS_IRLM
      WHEN 'IMCQ' THEN @IMS_CQS
      WHEN 'IMFD' THEN @IMS_FDR
      WHEN 'IMIT' THEN @IMS_ITOC
      WHEN 'IMMP' THEN @IMS_MPR
      WHEN 'IMFP' THEN @IMS_FPM
      WHEN 'IMBM' THEN @IMS_BMP
      WHEN 'IMLG' THEN @IMS_LOGS
      WHEN 'IMD2' THEN @IMS_ESAF_DB2
      WHEN 'IMMQ' THEN @IMS_ESAF_MQS
      WHEN 'IMPG' THEN @IMS_PROGRAM
      WHEN 'IMTX' THEN @IMS_TRANSACTION
      WHEN 'IMDA' THEN @IMS_DATABASE
      WHEN 'IMAR' THEN @IMS_AREA
    END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @obj_name'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- SET discovered object type to the type of object this will be after discovery
  -- This IS to handle that IMCQ gets converted to IMQA during processing,
  -- Certain queries IN this PROCEDURE need to be aware of this conversion 
  DECLARE @OBJ_TYPE_ID_DISCOVERED NVARCHAR(40) 
  SELECT @OBJ_TYPE_ID_DISCOVERED =
    CASE @OBJ_TYPE_ID
      WHEN 'IMCQ' THEN 'IMQA'
      ELSE @OBJ_TYPE_ID
    END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se @cid_discovered'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Initialize temp table that gets populated with the class and name of 
  -- each object that must be created
  DECLARE @spid INT
  SELECT  @spid = @@SPID
  DELETE FROM DiscoverResourceObject WHERE pid = @spid

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'De DiscoverResourceObject'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- INSERT record to CREATE the specified object
  INSERT INTO DiscoverResourceObject VALUES ( @spid, @OBJ_TYPE_ID, @obj_name, 1 )

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In DRO @obj_name'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- INSERT parent records IF necessary

  -- IMSAreas need a parent database record
  IF @OBJ_TYPE_ID = 'IMAR'
  BEGIN
    INSERT INTO DiscoverResourceObject 
      VALUES ( @spid, 'IMDA', @IMS_DATABASE, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In DRO IMDA'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- IMSSubprograms need IMSProgram record
  IF @OBJ_TYPE_ID IN
  (
    SELECT DISTINCT cid FROM isa_chain WHERE base_cid IN ('IMSP', 'IMAR')
  )
  BEGIN
    INSERT INTO DiscoverResourceObject 
      VALUES ( @spid, 'IMPG', @IMS_PROGRAM, 0 )
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In DRO IMPG'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- IMSSubcomponent AND IMSProgram objects need IMSSubsystem parent record
  IF @OBJ_TYPE_ID_DISCOVERED IN
  (
    SELECT DISTINCT cid FROM isa_chain WHERE base_cid IN ('IMSC', 'IMBM', 'IMPG', 'IMSP', 'IMAR')
  )
  BEGIN
    INSERT INTO DiscoverResourceObject 
    SELECT
    @spid,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN 'IMSS'
      WHEN @IMS_DCCTL IS NOT NULL THEN 'IMTM'
      WHEN @IMS_DBCTL IS NOT NULL THEN 'IMDB'
    END AS obj_cid,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN @IMS_TMDB
      WHEN @IMS_DCCTL IS NOT NULL THEN @IMS_DCCTL
      WHEN @IMS_DBCTL IS NOT NULL THEN @IMS_DBCTL
    END AS obj_name,
    0
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In DRO IMSS/TM/DB'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END



  -- INSERT IMSDiscovery records based ON the rows that have been entered INTO #object temp TABLE
  INSERT INTO IMSDiscovery
  (
    batchid,
    SystemId,
    obj_cid,
    IMS_SUBSYSTEM,
    IMS_SUBCOMPONENT,
    IMS_SUBPROGRAM,
    IMS_AREA,
    imsu_cid,
    imsc_cid,
    imsp_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    MVS,
    DD,
    DSN,
    ASSOCMVS,
    ASSOCIATE,
    STATE,
    Event,
    EventText
  )
  SELECT DISTINCT
    @batchid,
    @SystemId,
    o.obj_cid,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN @IMS_TMDB
      WHEN @IMS_DCCTL IS NOT NULL THEN @IMS_DCCTL
      WHEN @IMS_DBCTL IS NOT NULL THEN @IMS_DBCTL
    END AS IMS_SUBSYSTEM,
    CASE
      WHEN o.obj_cid IN ('IMSS', 'IMTM', 'IMDB') THEN NULL
      WHEN @IMS_IRLM IS NOT NULL THEN @IMS_IRLM
      WHEN @IMS_CQS IS NOT NULL THEN @IMS_CQS
      WHEN @IMS_FDR IS NOT NULL THEN @IMS_FDR
      WHEN @IMS_ITOC IS NOT NULL THEN @IMS_ITOC
      WHEN @IMS_MPR IS NOT NULL THEN @IMS_MPR
      WHEN @IMS_FPM IS NOT NULL THEN @IMS_FPM
      WHEN @IMS_BMP IS NOT NULL THEN @IMS_BMP
      WHEN @IMS_LOGS IS NOT NULL THEN @IMS_LOGS
      WHEN @IMS_ESAF_DB2 IS NOT NULL THEN @IMS_ESAF_DB2
      WHEN @IMS_ESAF_MQS IS NOT NULL THEN @IMS_ESAF_MQS
      WHEN @IMS_PROGRAM IS NOT NULL THEN @IMS_PROGRAM
    END AS IMS_SUBCOMPONENT,
    CASE 
      WHEN o.obj_cid NOT IN ('IMDA', 'IMTX', 'IMAR') THEN NULL
      WHEN @IMS_TRANSACTION IS NOT NULL THEN @IMS_TRANSACTION
      WHEN @IMS_DATABASE IS NOT NULL THEN @IMS_DATABASE
    END AS IMS_SUBPROGRAM,
    CASE
      WHEN o.obj_cid NOT IN ('IMAR')THEN NULL
      ELSE @IMS_AREA
    END AS IMS_AREA,
    CASE
      WHEN @IMS_TMDB IS NOT NULL THEN 'IMSS'
      WHEN @IMS_DCCTL IS NOT NULL THEN 'IMTM'
      WHEN @IMS_DBCTL IS NOT NULL THEN 'IMDB'
    END AS imsu_cid,
    CASE
      WHEN o.obj_cid IN ('IMSS', 'IMTM', 'IMDB') THEN NULL
      WHEN @IMS_IRLM IS NOT NULL THEN 'IMIR'
      WHEN @IMS_CQS IS NOT NULL THEN 'IMQA'
      WHEN @IMS_FDR IS NOT NULL THEN 'IMFD'
      WHEN @IMS_ITOC IS NOT NULL THEN 'IMIT'
      WHEN @IMS_MPR IS NOT NULL THEN 'IMMP'
      WHEN @IMS_FPM IS NOT NULL THEN 'IMFP'
      WHEN @IMS_BMP IS NOT NULL THEN 'IMBM'
      WHEN @IMS_LOGS IS NOT NULL THEN 'IMLG'
      WHEN @IMS_ESAF_DB2 IS NOT NULL THEN 'IMD2'
      WHEN @IMS_ESAF_MQS IS NOT NULL THEN 'IMMQ'
      WHEN @IMS_PROGRAM IS NOT NULL THEN 'IMPG'
    END AS imsc_cid,
    CASE
      WHEN o.obj_cid NOT IN ('IMDA', 'IMTX', 'IMAR') THEN NULL
      WHEN @IMS_TRANSACTION IS NOT NULL THEN 'IMTX'
      WHEN @IMS_DATABASE IS NOT NULL THEN 'IMDA'
    END AS imsp_cid,
    o.obj_name,
    SUBSTRING(@DATE_TIME,1,4)+'-'+SUBSTRING(@DATE_TIME,5,2)+'-'+SUBSTRING(@DATE_TIME,7,2)+' '+SUBSTRING(@DATE_TIME,9,12),
    @SEQ_NO,
  @MVS,
    CASE WHEN o.is_object = 1 THEN @DD ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @DSN ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @ASSOCMVS ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @ASSOCIATE ELSE NULL END,
    CASE WHEN o.is_object = 1 THEN @STATE ELSE NULL END,
    @Event,
    @EventText
  FROM
    DiscoverResourceObject AS o
  WHERE 
        o.pid = @spid
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverIMSResource
    RETURN -30000
  END

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'In IMSDiscovery'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END


  DECLARE @r INT
  EXEC @r = _DiscoverIMSResources ROOT, 1, @batchid, @incremental=1, @verbose=@verbose

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = '_DiscoverIMSResources'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  SELECT @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name
  FROM ProcessedIMSDiscovery
  WHERE batchid = @batchid AND obj_cid = @OBJ_TYPE_ID_DISCOVERED AND obj_name = @obj_name

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Se Exit assignments'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Cleanup of DiscoverResourceObject 
  DELETE FROM DiscoverResourceObject WHERE pid = @spid

  -- Timing of Step
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'De DRO'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( s, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( s, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_discoverIMSResource

  RETURN @r
GO