-- archive\db\sql\asisp_processpendingstatetransitions.sql
-- SID: 1.1
-- DateModified: 01/11/20 12:34:37
-- 
-- Define module asisp_processPendingStateTransitions
-- 

USE Object
GO
-----------------------------------------------------------------------------------------------------------
-- asisp_processPendingStateTransitions
PRINT 'Create procedure asisp_processPendingStateTransitions.'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = 'asisp_processPendingStateTransitions' AND type = 'P')
  DROP PROCEDURE asisp_processPendingStateTransitions
GO

-- Processes the records IN PendingStateTransition that have NULL ptime columns by creating
-- scheduled tasks to run at the specified time to change state fields of the objects.
--
-- Deletes any processed records IN PendingStateTransition that are older than one day.
--
-- RETURN value: the number of rows actually processed.
-- SID: 1.1
-- DateModified: 01/11/20 12:34:37
CREATE PROCEDURE asisp_processPendingStateTransitions
  @maxrows INT = 0 -- maximum number of rows to process
AS
  -- IF we're to process all rows THEN SET @maxrows to a ridiculously high value
  IF @maxrows = 0
    SET @maxrows = 2147483647

  DECLARE @nProcessed INT
  SET @nProcessed = 0

  DECLARE pstc CURSOR FOR 
    SELECT cid, id, aname, state, taskdatetime FROM PendingStateTransition
    WHERE ptime IS NULL
  OPEN pstc
  DECLARE @cid ClassID, @id ObjID, @aname NVARCHAR(24), @state INT, @taskdatetime DATETIME
  FETCH NEXT FROM pstc INTO @cid, @id, @aname, @state, @taskdatetime
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    DECLARE @time DATETIME
    IF @taskdatetime < GETDATE()
      SELECT @time = DATEADD(minute, 1, @taskdatetime)
    ELSE
      SELECT @time = @taskdatetime

    DECLARE @id_str NVARCHAR(9)  
    SELECT @id_str = CONVERT(NVARCHAR(9), @id)

    DECLARE @description NVARCHAR(50)
    SELECT @description = @aname + ' to ' + _State  + ' at '+ CONVERT(NVARCHAR(32), @time, 120)
      FROM StateType_V WHERE id = @state

    DECLARE @task_name NVARCHAR(100)
    SELECT @task_name = RTRIM(@cid) + ':' + CAST(@id AS NVARCHAR(9)) + ':' + @description

    DECLARE @command NVARCHAR(255)
    SELECT @command = '_' + RTRIM(@cid) + 'Set' + @aname + ' ''' + @cid + ''',' + CAST(@id AS NVARCHAR(9)) + ', @' + @aname + 'ID=' + CAST(@state AS NVARCHAR(9))
    FROM obj_class WHERE cid = @cid

    DECLARE @startdate INT
    SELECT @startdate = DATEPART(yy, @time) * 10000 + DATEPART(mm, @time) * 100 + DATEPART(dd, @time)

    DECLARE @starttime INT
    SELECT @starttime = DATEPART(hh, @time) * 10000 + DATEPART(mi, @time) * 100 +  DATEPART(ss, @time)

    DECLARE @cno INT
    SELECT @cno = CONVERT(INT, cno) FROM obj_class WHERE cid = @cid

    DECLARE @taskid INT
    SELECT @taskid = id FROM msdb..systasks WHERE
      name = @task_name
    IF @taskid IS NULL
    BEGIN
      EXEC msdb..sp_addtask
          @name=@task_name,
          @command=@command,
          @description=@description,
          @databasename='Object',
          @freqtype=1,
          @freqsubtype=1,
          @enabled=1,
          @freqrelativeinterval=1,
          @freqrecurrencefactor=1,
          @activestartdate=@startdate,
          @activestarttimeofday=@starttime,
          @retryattempts=5,
          @retrydelay=2,
          @tagobjectid=@id,
          @tagobjecttype=@cno,
          @newid=@taskid OUTPUT
    END
    ELSE
      EXEC msdb..sp_updatetask
          @id=@taskid,
          @command=@command,
          @description=@description,
          @databasename='Object',
          @freqtype=1,
          @freqsubtype=1,
          @enabled=1,
          @freqrelativeinterval=1,
          @freqrecurrencefactor=1,
          @activestartdate=@startdate,
          @activestarttimeofday=@starttime,
          @retryattempts=5,
          @retrydelay=2,
          @tagobjectid=@id,
          @tagobjecttype=@cno
    IF @@ERROR <> 0
    BEGIN
      DEALLOCATE pstc
      RETURN -30000
    END

    -- ADD record to TaskMap so this IS seen AS a task ON the object
    INSERT TaskMap(cid, id, taskid, taskuid)
      SELECT @cid, @id, @taskid, job_id
      FROM msdb..systaskids
      WHERE task_id = @taskid

    -- Mark the current row AS HAVING been processed by setting
    -- its ptime column to the current time
    UPDATE PendingStateTransition SET ptime = GETDATE() WHERE CURRENT OF pstc

    -- Check FOR max rows processed
    SET @nProcessed = @nProcessed + 1
    IF @nProcessed = @maxrows
    BEGIN
      BREAK
    END

    FETCH NEXT FROM pstc INTO @cid, @id, @aname, @state, @taskdatetime
  END
  DEALLOCATE pstc

  RETURN @nProcessed


