-- archive\db\sql\asisp_setdsforlobllinkedcics.sql
-- SID: 1.1
-- DateModified: 01/11/20 12:36:11
-- 
-- Define module asisp_setdsforlobllinkedcics
-- 

USE Object
GO
-----------------------------------------------------------------------------------------------------------
-- asisp_setdsforlobllinkedcics
PRINT 'Create procedure asisp_setdsforlobllinkedcics.'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setdsforlobllinkedcics' AND type = 'P')
  DROP PROCEDURE asisp_setdsforlobllinkedcics
GO

-- Sets the desired state FOR all CICS regions that are linked to the descendent LOBs of the
-- specified LOB
CREATE PROCEDURE asisp_setdsforlobllinkedcics
  @lob_id  ObjID,
  @state   NVARCHAR(40),
  @alert   NVARCHAR(16) = NULL,
  @pri     NVARCHAR(16) = NULL
AS
  -- Look up the mapping of the state name to the StateType id
  DECLARE @stateID INT
  SELECT @stateID=id
  FROM StateType_V, containment c
  WHERE name = 'CICS_' + @state AND dst_cid='STTP' AND dst_id=id AND c.deleted = 0
  IF @@ROWCOUNT = 0
  BEGIN
    RAISERROR('asisp_setdsforlobllinkedcics: no mapping FOR state name ''%s''', 11, 1, @state)
    RETURN -30000
  END
  CREATE TABLE #descendents
  (
    id  ObjID,
    gen INT
  )
  DECLARE @gen INT
  SELECT @gen = 0
 
  INSERT #descendents SELECT @lob_id, @gen
  WHILE @@ROWCOUNT > 0
  BEGIN
    SELECT @gen = @gen + 1
    INSERT #descendents
      SELECT dst_id, @gen
      FROM containment, #descendents
      WHERE gen = @gen - 1 AND src_cid = 'LOB' AND src_id = id AND dst_cid = 'LOB' AND deleted = 0 
  END
  CREATE TABLE #cics
  (
    id ObjID
  )
  INSERT INTO #cics
  SELECT c.id
  FROM 
    #descendents d, 
    link l,
    CICS_V c
  WHERE
    link_type = 'LOBL'
    AND src_cid = 'LOB'
    AND l.src_id = d.id
    AND dst_cid = 'CICS'
    AND c.id = l.dst_id
    AND l.deleted = 0
  ORDER BY c.name
  -- IF the alert state AND priority specified are non-NULL THEN SET the NextSCHVSettings FOR the CICS regions
  IF @alert IS NOT NULL AND @pri IS NOT NULL
  BEGIN
    DECLARE @alert_id INT, @pri_id INT
    SELECT @alert_id = id FROM AlertState_C WHERE name = @alert
    SELECT @pri_id = id FROM SixLevelPri_C WHERE name = @pri
    UPDATE NextSCHVSettings SET alert = @alert_id, pri = @pri_id, ctime = GETDATE() FROM #cics 
    WHERE cid = 'CICS' AND NextSCHVSettings.id = #cics.id
    IF @@ROWCOUNT < (SELECT COUNT(*) FROM #cics)
      INSERT NextSCHVSettings(cid, id, alert, pri) 
        SELECT DISTINCT 'CICS', id, @alert_id, @pri_id 
        FROM #cics
        WHERE NOT EXISTS (SELECT id FROM NextSCHVSettings WHERE cid = 'CICS' AND id = #cics.id)
  END
--  SELECT DISTINCT * FROM #cics
  UPDATE 
    CICS_A 
  SET 
    _DesiredStateID = @stateID
  FROM 
    #cics
  WHERE
    CICS_A.id = #cics.id


