PRINT 'Creating procedure asisp_uploadGetOSInfo'
GO

DROP procedure asisp_uploadGetOSInfo
GO

-- $Archive:   //Archive/PVCSROOT/Archive/DB/sql/asisp_uploadGetOSInfo.sqv  $/Archive/DB/SQL/asisp_uploadGetOSInfo.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 07 2001 16:39:00  $
-- $Author:   Forthman  $
create procedure asisp_uploadGetOSInfo
  @cid ClassID, 
  @id ObjID,
  @formatted_output tinyint=0
as
  raiserror('asisp_uploadGetOSInfo @cid=''%s'', @id=%d, @formatted_output=%d', 0, 1, @cid, @id, @formatted_output)

  declare @gen int

  -- Standard transaction logic
  declare @tcount int
  select @tcount = @@trancount
  if @tcount = 0
    begin tran asisp_uploadGetOSInfo
  else
    save tran asisp_uploadGetOSInfo

  create table #uploadGetOSInfo
  (
    cid             ClassID,
    id              ObjID,
    _UploadUserID   varchar(8) NULL DEFAULT NULL,
    _UploadPassword varchar(8) NULL DEFAULT NULL
  )

  -- Special handling for OS
  if @cid='OS'
    insert #uploadGetOSInfo(cid, id) select @cid, @id

  -- Check if 'OS' is an immediate parent class of @cid
  declare @os_id ObjID
  select @os_id=src_id from link where dst_cid=@cid and dst_id=@id and src_cid='OS' and link_type='PHYC' and deleted=0
  if @os_id is not null
    insert #uploadGetOSInfo(cid, id) select 'OS', @os_id
  else
  begin
    -- If the calling object derives from MJOB then we see if it has an OS anywhere in its parent chain
    declare @parent_cid ClassID, @parent_id ObjID
    select @parent_cid = src_cid, @parent_id = src_id 
      from containment 
      where dst_cid = @cid and dst_id = @id and deleted = 0
    while @@rowcount > 0 and @parent_cid <> 'BUSC'
    begin
      if @parent_cid = 'OS'
        break
      select @parent_cid = src_cid, @parent_id = src_id
        from containment
        where dst_cid = @parent_cid and dst_id = @parent_id and deleted = 0
    end
    if @parent_cid = 'OS'
    begin
      select @os_id = @parent_id
      insert #uploadGetOSInfo(cid, id) select @parent_cid, @parent_id
    end
  end

  -- Check if @cid is a ascendent cid of 'OS'
  select @gen=generation from link_chain
    where cid='OS'
      and src_cid=@cid
  if @gen is not null
  begin
    insert #uploadGetOSInfo(cid, id)
      select @cid, @id

    declare @stat int
    declare @sql varchar(255)
    declare @src_cid ClassID, @dst_cid ClassID

    select @sql='select src_cid, dst_cid from link_chain
      where cid=''OS''
        and generation<='+convert(varchar(255),@gen)+'
      order by generation desc'

    -- Populate #uploadGetOSInfo with all OS's below @cid
    exec @stat = asisp_declarecursor asisp_uploadGetOSInfo_c, @sql, 1
    if @stat < 0
    begin
       rollback tran asisp_uploadGetOSInfo
       return -3000
    end
    open asisp_uploadGetOSInfo_c
    fetch next from asisp_uploadGetOSInfo_c into @src_cid, @dst_cid
    while @@fetch_status <> -1
    begin

      insert #uploadGetOSInfo(cid, id)
        select dst_cid, dst_id
          from link, #uploadGetOSInfo
          where src_cid=@src_cid 
            and src_cid=cid
            and src_id=id
            and dst_cid=@dst_cid
            and deleted=0

      select @gen=@gen-1
      fetch next from asisp_uploadGetOSInfo_c into @src_cid, @dst_cid
    end
    deallocate asisp_uploadGetOSInfo_c

  end
  delete from #uploadGetOSInfo where cid<>'OS'

  -- obtain the values for the overridable attributes UploadUserID, UploadPassword
  exec asisp_updateOverrideAttrs #uploadGetOSInfo, cid, id, '_UploadUserID', OS, UploadUserID
  exec asisp_updateOverrideAttrs #uploadGetOSInfo, cid, id, '_UploadPassword', OS, UploadPassword

--select '#uploadGetOSInfo', * from #uploadGetOSInfo

  -- Output the OS's found in #uploadGetOSInfo
  if @formatted_output=1
    select 'DATA_ROW|'+rtrim(t.cid)+'|'+convert(varchar(255),t.id)+'|'+name+'|'+t._UploadUserID+'|'+t._UploadPassword+'|'+m.svc_host+'|'+m.svc_name
      from OS_V as v
        join #uploadGetOSInfo as t on t.id = v.id
        join link as l on l.link_type = 'PHYC' AND l.dst_cid = 'OS' and l.dst_id = v.id
        left outer join MVSSenderSvcMap as m on m.os_id = v.id
      where 
        l.deleted=0
        and v._UploadEnabled=1
  else
    select t.cid, t.id, name, t._UploadUserID, t._UploadPassword, m.svc_host, m.svc_name
      from OS_V as v
        join #uploadGetOSInfo as t on t.id = v.id
        join link as l on l.link_type = 'PHYC' AND l.dst_cid = 'OS' and l.dst_id = v.id
        left outer join MVSSenderSvcMap as m on m.os_id = v.id
      where 
        l.deleted=0
        and v._UploadEnabled=1

  drop table #uploadGetOSInfo

  -- Finish up transaction
  if @tcount = 0
    commit tran asisp_uploadGetOSInfo
GO
