-- archive/db/sql/attachdatabases.sql
-- SID: 1.10
-- Date Modified: 01/09/28 15:16:19
--
-- attachdatabases.sql
--
-- Module Description
--   Procedure for attaching TBSM databases that have been previously detached
--   and saved in .mdf files.

USE master
GO
SET NOCOUNT ON
GO

PRINT 'Removing obsolete database(s)...'
GO
IF EXISTS (SELECT * FROM sysdatabases WHERE name='ASIWebServer')
  EXEC sp_detach_db ASIWebServer
GO
IF EXISTS (SELECT * FROM sysdevices WHERE name='ASIWebServer')
  EXEC sp_dropdevice 'ASIWebServer', 'DELFILE'
GO

-- Special-CASE removal of msdb database log file
EXEC xp_cmdshell 'rm -f D:\MSSQL7\Data\msdblog.ldf'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'xp_rule_stoppulsing' AND type = 'X')
  EXEC xp_rule_stoppulsing
GO

PRINT 'Attaching database(s)...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_install_database' AND type = 'P')
  DROP PROCEDURE asisp_install_database
GO

CREATE PROCEDURE asisp_install_database
  @dbname     sysname,
  @data_file  sysname = NULL,
  @data_dir   NVARCHAR(255) = 'D:\MSSQL7\Data',
  @backup_dir NVARCHAR(255) = 'D:\MSSQL7\Backup',
  @conditional TINYINT = 0
AS
  DECLARE @cmd NVARCHAR(256), @rc INT
  IF @data_file IS NULL
    SET @data_file = @dbname + '_Data'

  IF (@conditional = 1) BEGIN
    SET @cmd = 'test -f "' + @data_dir + '\' + @data_file + '.mdf"'
    EXEC @rc = xp_cmdshell @cmd
    IF @rc <> 0 BEGIN
      RAISERROR('%s does not exist so it will not be attached', 0, 1, @dbname)
      RETURN 0
    END
  END

  SET @cmd = 'rm -f "' + @data_dir + '\' + @dbname + '_Log.ldf"'
  EXEC @rc = xp_cmdshell @cmd
  IF @rc <> 0
  BEGIN
    RAISERROR('ERROR: cannot DELETE file [%s\%s_Log.ldf]', 1, 11, @data_dir, @dbname)
    RETURN -30000
  END

  DECLARE @file_path NVARCHAR(255)
  SET @file_path = @data_dir + '\' + @data_file + '.mdf'
  EXEC sp_attach_single_file_db @dbname, @file_path
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  RAISERROR('Setting database options for database %s', 0, 1, @dbname)

  EXEC sp_dboption @dbname, 'trunc. log ON chkpt.', true
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  EXEC sp_dboption @dbname, 'SELECT INTO/bulkcopy', true
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  EXEC sp_dboption @dbname, 'single user', false
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  EXEC sp_dboption @dbname, 'autoshrink', true
  IF @@ERROR <> 0
  BEGIN
    RETURN -30000
  END

  IF NOT EXISTS (SELECT * FROM sysdevices WHERE name = @dbname)
  BEGIN
    RAISERROR('Creating backup device for database %s', 0, 1, @dbname)
    SET @file_path = @backup_dir + '\' + @dbname + '.BAK'
    EXEC sp_addumpdevice 'disk', @dbname, @file_path
    IF @@ERROR <> 0
    BEGIN
      RETURN -30000
    END
  END
GO

EXEC asisp_install_database Meta
EXEC asisp_install_database Object
EXEC asisp_install_database ObjectEvents
EXEC asisp_install_database ObjectQueues
EXEC asisp_install_database ASIRuleSvc
EXEC asisp_install_database WebServer
EXEC asisp_install_database msdb, msdbdata
EXEC asisp_install_database RODM, RODM
EXEC asisp_install_database RODMLoad, RODMLoad
EXEC asisp_install_database OPCLoad, OPCLoad
EXEC asisp_install_database EventHistory, EventHistory, @conditional=1
GO

IF EXISTS (SELECT * FROM sysdatabases WHERE name = 'Object')
BEGIN
  PRINT 'Changing DEFAULT database to Object...' 
  EXEC sp_defaultdb 'BUILTIN\Administrators', 'Object'
  EXEC sp_defaultdb 'sa', 'Object'
END
GO

PRINT 'Changing database options FOR database model...' 
EXEC sp_dboption 'model', 'trunc. log ON chkpt.', true
EXEC sp_dboption 'model', 'SELECT INTO/bulkcopy', true
GO

EXEC asisp_rename_sqlserver @db_host='XXXX', @pa_host='YYYY', @websvr_host='ZZZZ'
GO

-- Table to hold replacement host records for Installation Management / Version Utility 
CREATE TABLE #replacementHost
(
  host       NVARCHAR(64),
  pseudoHost NVARCHAR(64)
)

-- Insert as many records as necessary to translate real host names to replacement host names
-- All the host names entered here should all be upper case
INSERT #replacementHost ( host, pseudoHost ) VALUES ( 'TBSM01', 'S1' )
INSERT #replacementHost ( host, pseudoHost ) VALUES ( 'TBSM02', 'S2' )
INSERT #replacementHost ( host, pseudoHost ) VALUES ( 'TBSM03', 'S3' )
INSERT #replacementHost ( host, pseudoHost ) VALUES ( 'TBSM04', 'S4' )
INSERT #replacementHost ( host, pseudoHost ) VALUES ( 'TBSM05', 'S5' )

-- Prepare Installation Management / Version Utility data to be attached
UPDATE Object..imObjectList
  SET host = r.host
  FROM Object..imObjectList AS o
  INNER JOIN #replacementHost AS r ON o.host = r.pseudoHost

PRINT 'UPDATE Object..imObjectList...'

UPDATE Object..imExclusionPattern
  SET host = r.host
  FROM Object..imExclusionPattern AS o
  INNER JOIN #replacementHost AS r ON o.host = r.pseudoHost

PRINT 'UPDATE Object..imExclusionPattern...'

UPDATE Object..imObjectSeed
  SET host = r.host
  FROM Object..imObjectSeed AS o
  INNER JOIN #replacementHost AS r ON o.host = r.pseudoHost

PRINT 'UPDATE Object..imObjectSeed...'
-- Cleanup

DROP TABLE #replacementHost


PRINT 'Done.'
GO
