-- archive/db/sql/backupeventhistory.sql
-- SID: 1.1
-- Date Modified: 01/09/28 15:18:40
--
-- backupeventhistory.sql
--
-- Module Description
--   Backup the EventHistory database, if it exists.
--   If the database exists but the dump device doesn't, the dump device is created.

USE master
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'backupEventHistory' AND type = 'P')
  DROP PROCEDURE backupEventHistory
PRINT 'Creating backupEventHistory'
GO

CREATE PROCEDURE backupEventHistory
AS

SET NOCOUNT ON

IF EXISTS (SELECT * FROM master..sysdatabases WHERE name = 'EventHistory') BEGIN
  IF NOT EXISTS (SELECT * FROM master..sysdevices WHERE name = 'EventHistory') BEGIN
    DECLARE @path NVARCHAR(260)
    SELECT @path = REPLACE(filename,".MDF",".BAK")
      FROM master..sysdatabases WHERE name = 'EventHistory'
    SET @path = REPLACE(@path,".Mdf",".BAK")
    SET @path = REPLACE(@path,".mdf",".BAK")
    SET @path = REPLACE(@path,"\DATA\","\Backup\")
    SET @path = REPLACE(@path,"\Data\","\Backup\")
    SET @path = REPLACE(@path,"\data\","\Backup\")
    PRINT 'Adding dump device ' + @path + ' for EventHistory database'
    EXEC sp_addumpdevice 'disk', 'EventHistory', @path
  END

  SELECT 'message'='Backing up ' + db.filename + ' to ' + dev.phyname
    FROM master..sysdatabases db
    JOIN master..sysdevices dev ON dev.name = db.name
    WHERE db.name = 'EventHistory'

  BACKUP DATABASE EventHistory TO EventHistory WITH INIT, NOSKIP, NOFORMAT, STATS
END

GO
