# $Header:   //ARCHIVE/Archive/DB/sql/clsql.ksv   1.8   Sep 07 2000 21:53:18   BRENT  $

# clsql.ksh

# Uses m4 to produce a .sql file from a .sqi input file

usage()
{
  print 'SYNPOSIS'
  print '  Produce a SQL output file from an input file with m4-style preprocessing'
  print '  directives.'
  print ''
  print 'USAGE'
  print "  $0 [<options>] <input-file>"
  print ''
  print 'Options are:'
  print '  -h                     prints this message'
  print '  -D <name>[=<value>]    defines a symbol with the given name and value'
  print '  -I <sql-directory>     specifies directory containing required macro file'
  print '                         BusinessObject.sqi. If not specified then it will'
  print '                         be looked for in the SQL subdirectory of the Tivoli'
  print '                         Business Systems Manager installation.'
  print '  -o <output-directory>  specifies directory that output file will be'
  print '                         written to. Defaults to current directory.'
  print 'If the input file name is <name>.sqi then the output file name is <name>.sql.'
}

compile()
{
  (
  cd $sqldir
  input=$1
  output=$2
  m4 $symlist $input > $output
  )
}

# check for regfind utility
rc=$(which regfind.exe); rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find REGFIND.EXE.  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure REGFIND.EXE is in the path." >&2
  exit 1
fi

#
# main
#

outdir=$PWD

while getopts D:I:ho: c; do
  case $c in
  h)  usage; exit 0;;
  o)  outdir=$OPTARG;;
  D)  symlist="$symlist -D$OPTARG";;
  I)  sqldir="$OPTARG";;
  *)  usage>&2; exit 1;;
  esac
done

shift $OPTIND-1

if [ -z "$sqldir" ]; then
  # Get install directory from registry
  eval $(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | awk '/InstallDirectory/ {gsub("[\\]", "/", $0); printf("sqldir=%s\n",$3);}')
fi

if [ -z "$sqldir" ]; then
  sqldir=.
else
  sqldir="$sqldir/sql"
fi

if [ ! -f "$sqldir/BusinessObject.sqi" ]; then
  print >&2 "Could not find BusinessObject.sqi, aborting"
  exit 2
fi

while [ ! -z "$1" ]; do
  input=$PWD/$1

  root=$(echo $input | sed 's/[.][Ss][Qq][Ii]//')
  if [ "$input" = "$root" ]; then
    input=${root}.sqi
    if [ ! -f "$input" ]; then
      input=$root
    fi
  fi

  if [ ! -f "$input" ]; then
    print >&2 "Could not open input file $input"
    exit 1
  fi

  root=$(echo $root | sed 's?.*/??')
  outdir=$(cd $outdir; echo $PWD;)
  output=$outdir/${root}.sql

  compile $input $output

  shift
done
