-- archive\db\sql\createmissinglinkdeletetriggers.sql
-- SID 1.1
-- Date Modified: 01/10/19 12:27:27
--
-- Module Description
--   Creates the delete trigger for any link table that is missing it

USE Object
GO

---------------------------------------------------------------------------------------------------
-- CreateMissingLinkDeleteTriggers
PRINT 'Creating procedure CreateMissingLinkDeleteTriggers'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateMissingLinkDeleteTriggers' AND type = 'P')
  DROP PROCEDURE CreateMissingLinkDeleteTriggers
GO

-- SID 1.1
-- Date Modified: 01/10/19 12:27:27
-- Creates the delete triggers for the individual link tables that are missing them in the
-- current database. Any that are already in place are left alone.
CREATE PROCEDURE CreateMissingLinkDeleteTriggers
AS
  SET NOCOUNT ON
  DECLARE linkc INSENSITIVE CURSOR FOR
    SELECT ol.src_cid, ol.dst_cid, ol.link_type, so.name, lt.flag_containment FROM sysobjects AS so
      JOIN derived_obj_link AS ol ON so.name = RTRIM(ol.src_cid) + '_' + RTRIM(ol.dst_cid) + '_' + RTRIM(ol.link_type)
      JOIN obj_class AS soc ON soc.cid = ol.src_cid
      JOIN obj_class AS doc ON doc.cid = ol.dst_cid
      JOIN link_type AS lt ON lt.link_type = ol.link_type
    WHERE
      soc.flag_abstract = 0
      AND doc.flag_abstract = 0
      AND NOT EXISTS (SELECT * FROM sysobjects AS t WHERE t.name = 'tD_' + so.name AND type = 'TR')
    ORDER BY
      so.name
  OPEN linkc
  DECLARE @src_cid ClassID, @dst_cid ClassID, @link_type VARCHAR(4), @table sysname, @flag_containment TINYINT
  DECLARE @containment_delete VARCHAR(1000), @sql VARCHAR(8000)
  FETCH NEXT FROM linkc INTO @src_cid, @dst_cid, @link_type, @table, @flag_containment
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @flag_containment = 1
      SET @containment_delete = "  -- Delete containment records
    DELETE c 
      FROM containment AS c
      JOIN deleted AS d ON c.src_cid = '" + @src_cid + "' AND c.src_id = d.src_id AND c.dst_cid = '" + @dst_cid + "' AND c.dst_id = d.dst_id
  "
    ELSE
      SET @containment_delete = "  -- Not a containment link, so no delete from containment
  "
    SET @sql = "-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/CreateMissingLinkDeleteTriggers.sqv  $
-- SID 1.1
-- Date Modified: 01/10/19 12:27:27
CREATE TRIGGER tD_" + @table + " ON " + @table + " FOR DELETE
AS
" + @containment_delete + "
  -- Delete link records
  DELETE c
    FROM link AS c
    JOIN deleted AS d ON c.link_type = '" + @link_type + "' AND c.src_cid = '" + @src_cid + "' AND c.src_id = d.src_id AND c.dst_cid = '" + @dst_cid + "' AND c.dst_id = d.dst_id
"
    RAISERROR('Creating trigger tD_%s', 0, 1, @table) WITH NOWAIT
    EXEC(@sql)
    FETCH NEXT FROM linkc INTO @src_cid, @dst_cid, @link_type, @table, @flag_containment
  END
  DEALLOCATE linkc
GO

EXEC CreateMissingLinkDeleteTriggers
GO
