IF EXISTS(SELECT * FROM sysobjects WHERE type='P' AND name='delete_obj')
  DROP PROCEDURE delete_obj
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/ISSUE_01389_20000919.sqv  $
-- $Revision:   1.0  $
-- $Date:   Sep 19 2000 20:35:32  $
-- $Author:   DAVID  $
CREATE procedure delete_obj
  @cid  ClassID,
  @id   ObjID
as
  raiserror('delete_obj: "%s", %d', 0, 1, @cid, @id)

  -- Transaction logic
  declare @tcount int
  select @tcount = @@trancount
  if @tcount > 0
    save tran delete_obj
  else
    begin tran delete_obj

  -- Notify all logically linked objects that this object has been deleted
  if exists (select * from derived_obj_link where link_type = 'LOBL' and dst_cid = @cid)
  begin
    raiserror('delete_obj: notifying all LOBL-linked objects that this object has been deleted', 0, 1)
    exec asisp_notifylobsofdelete @cid, @id
  end
 
  -- Perform the delete
  declare @id_decl varchar(80), @err int
  select @id_decl = 'declare @cid ClassID, @id ObjID select @cid='''+rtrim(@cid)+''', @id='+convert(varchar(9),@id)
  exec
  (@id_decl + "
    -- Build table of objects to be deleted
    raiserror('delete_obj: building a list of objects to be deleted', 0, 1)
    create table #deleted_objs
    (
      cid         ClassID,    -- cid of descendent object to be deleted
      id          ObjID,      -- id of descendent object to be deleted
      generation  int,        -- generation of this object from original object
      cno         smallint,   -- obj_class.cno corresponding to cid
      propagation tinyint     -- obj_class.flag_propagation setting
    )
    declare @generation int
    select @generation = 0
    insert #deleted_objs select @cid, @id, @generation, cno, flag_propagation
    from obj_class
    where cid = @cid
    while @@rowcount > 0
    begin
      select @generation = @generation + 1
      insert #deleted_objs select dst_cid, dst_id, @generation, o.cno, o.flag_propagation
      from containment, #deleted_objs d, obj_class o
      where
        generation = @generation-1 and src_cid=d.cid and src_id=id and deleted=0
        and o.cid = dst_cid 
    end
--    select * from #deleted_objs

    -- Mark all containment records for the deleted objects
    raiserror('delete_obj: marking link and containment records as deleted', 0, 1)
    update containment
      set deleted=1 from #deleted_objs where dst_cid=cid and dst_id=id and deleted=0
    update containment
      set deleted=1 from #deleted_objs where src_cid=cid and src_id=id and deleted=0

    -- Mark all link records for the deleted objects
    update link
      set deleted=1 from #deleted_objs where dst_cid=cid and dst_id=id and deleted=0
    update link
      set deleted=1 from #deleted_objs where src_cid=cid and src_id=id and deleted=0

    -- begin gem processing
    raiserror('delete_obj: begin GEM-specific processing...', 0, 1)
    if exists (select * from sysobjects where name = 'GEM_IDLookup' and type = 'U')
      delete g from GEM_IDLookup as g join #deleted_objs as d on d.cid = g.CID and d.id = g.IID
    -- gem lob processing
    if exists (select * from sysobjects where name = 'GEM_LOBLookup' and type = 'U')
    begin
      update GEM_LOBLookup set LOBID = 0 from #deleted_objs d where d.cid = 'LOB' and d.id = LOBID 
      UPDATE GEM_LOBLookup SET parentLOBID = 0 FROM #deleted_objs d WHERE d.cid = 'LOB' AND d.id = parentLOBID AND parentLOBKey <> 'LOBC'
    end
    raiserror('delete_obj: end GEM-specific processing.', 0, 1)
    -- end gem processing

    -- Send notification to the dispatcher of all propagation objects that have just been deleted
    select 
      type = et.type,
      fields=
        convert(varchar(9), isnull(d.cno, 0)) + '|' +
        convert(varchar(9), isnull(d.id, 0)) + '|' +
        case generation when 0 then '1' else '0' end
    into #ObjectDeletedEvent
    from 
      #deleted_objs d,
      EventType et
    where
      d.propagation = 1
      and et.name = 'ObjectDeleted'
    insert PendingEvent(type, fields, detail) select type, fields, '' from #ObjectDeletedEvent
    insert PendingNotification(type, fields, detail) select type, fields, '' from #ObjectDeletedEvent
    raiserror('delete_obj: sent %d Event and Notification row(s) denoting objects have been deleted', 0, 1, @@rowcount)
  ")
  select @err = @@error
  if @err <> 0
  begin
    raiserror('delete_obj ''%s'', %d: Could not delete due to database error %d, aborting', 11, 1, @cid, @id, @err)
    rollback tran delete_obj
    return -30000
  end

  -- cleanup child events generated by this object
  raiserror('delete_obj: generating negating child events', 0, 1)
  exec asisp_negatechildevents @cid, @id

  -- Decrement the child count in our parent
  raiserror('delete_obj: decrementing the child count of parent', 0, 1)
  declare @sql varchar(255)
  select
    @sql = 'update ' + cname + '_C set nchild = nchild - 1 where id = ' + convert(varchar(9), src_id)
  from 
    obj_class c, 
    link l,
    link_type lt
  where
    l.dst_cid = @cid
    and l.dst_id = @id
    and l.link_type = lt.link_type
    and lt.flags & 1 = 1
    and c.cid = l.src_cid
  if @@rowcount = 1
  begin
    exec(@sql)
    if @@error != 0
    begin
      raiserror('delete_obj ''%s'', %d: could not decrement child count in parent', 11, 1, @cid, @id)
      rollback tran delete_obj
      return -30000
    end
  end
  
  if @tcount = 0
    commit tran delete_obj

