-- $Header
-- 
-- 
-- 

use History
GO

/* create the eventhist table which is a mirror of the event table */
if not exists (select * from sysobjects where id = object_id('dbo.eventhist') and sysstat & 0xf = 3)
begin
  exec ('create table eventhist
	(
	/* Agent generating event */
  	agent_cid	ClassID,
  	agent_id	ObjID,
	/* Event identifiers from agent */
  	agent_ctime   datetime,
  	agent_seqno	varchar(16),
	/* Object receiving event */
  	obj_cid	ClassID,
  	obj_id	ObjID,
	/* Event attributes */
  	cid		ClassID,
  	id    	numeric(16),
  	name		ObjName,
  	ctime		datetime,
  	detail	varchar(255) null,
  	alertstate	int,
  	priority	int,
  	state		int,
  	tstamp	binary(8)
  	constraint event_PK primary key clustered (obj_cid, obj_id, cid, id)
	)')
end
GO

use master
GO

------
--Procedure for separating directory and file name from a full path
------

if exists (select * from sysobjects where id = object_id('dbo.asisp_split_file_path') and sysstat & 0xf = 4)
  drop procedure dbo.asisp_split_file_path
GO

create procedure dbo.asisp_split_file_path
  @path varchar(520),
  @dir varchar(520) output,
  @file varchar(520) output
AS

  declare @pos int

  select @dir=''
  select @pos=patindex("%[/\]%",@path)
  if @pos=0
  begin
    select @dir='', @file=@path
    return
  end
  while @pos>1
  begin
    if @dir=''
      select @dir=left(@path,@pos)
    else
      select @dir=@dir+left(@path,@pos)
    select @path=substring(@path,@pos+1,520)
    select @pos=patindex("%[/\]%",@path)
  end

  select @file=@path
GO

------
--This procedure builds the 'move' clause for the database restore command.
--First it derives the logical file names from the backup along with the
--original paths to the data devices.  Then it sets up move commands to move
--the device files to either the existing device location for the database, or
--the location of master if the database doesn't exist.
------

if exists (select * from sysobjects where id = object_id('dbo.asisp_get_restore_move') and sysstat & 0xf = 4)
  drop procedure dbo.asisp_get_restore_move
GO

create procedure dbo.asisp_get_restore_move
  @bupath varchar(255),
  @dbname sysname,
  @moveclause varchar(2000) output
AS
  declare @checkdb sysname, @datapath varchar(520), @datadir varchar(520), @filename varchar(255)
  declare @logfile varchar(128), @physfile varchar(260)
--Get directory for database devices, either current location of database or location of master
  if exists(select * from master..sysdatabases where name=@dbname)
    select @checkdb=@dbname
  else
    select @checkdb='master'
  create table #pathtemp (path nchar(520))
  exec ('insert #pathtemp select filename from '+@checkdb+'..sysfiles where fileid=1')
  select @datapath=path from #pathtemp
  drop table #pathtemp
  exec asisp_split_file_path @path=@datapath, @dir=@datadir output, @file=''

--Get logical file info from backup
  create table #fileinfo (
    LogicalName nvarchar(128),
    PhysicalName nvarchar(260),
    Type char(1),
    FileGroupName nvarchar(128),
    Size numeric(20,0),
    MaxSize numeric(20,0)
  )
  insert #fileinfo exec('RESTORE FILELISTONLY FROM  DISK = '''+@bupath+''' WITH  FILE = 1 ,  NOUNLOAD')
  declare filecur insensitive cursor for select LogicalName, PhysicalName from #fileinfo
  open filecur
  select @moveclause=''
  fetch next from filecur into @logfile, @physfile
  while @@fetch_status<>-1
  begin
    exec asisp_split_file_path @path=@physfile, @file=@filename output, @dir=''
    if @moveclause<>''
      select @moveclause=@moveclause+','
    select @moveclause=@moveclause+'move '''+@logfile+''' to '''+@datadir+@filename+''''
    fetch next from filecur into @logfile, @physfile
  end
  deallocate filecur
  drop table #fileinfo
  if @moveclause<>''
    select @moveclause=@moveclause+','
GO

-------------------------------------------------------------------------------------------
--Procedure to restore and initialize Object and Meta databases
-------------------------------------------------------------------------------------------
PRINT 'Creating procedure asisp_historyinit'
GO

if exists (select * from sysobjects where id = object_id('dbo.asisp_historyinit') and sysstat & 0xf = 4)
	drop procedure dbo.asisp_historyinit
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/event_hist.sqv  $
-- $Revision:   1.12  $
-- $Date:   Aug 10 2001 10:40:44  $
-- $Author:   lin  $
-- Restore Object and Meta databases from live db backups precopied at specified location
-- rebuild views for historical report

create procedure asisp_historyinit
  @path ObjName = "d:\mssql7\backup",
  @file_extension varchar(4) = "bak"
AS
  declare @bupath varchar(255), @moveclause varchar(2000)
  if not (right(@path,1) = "\" or right(@path,1) = "/")
    select @path=@path + "\"
  --stop web server and kill all processes
  exec xp_cmdshell 'sc stop w3svc'
  exec xp_cmdshell 'sleep 90'
  declare @spid smallint, @spid_str char(2)
  while exists (select * from sysprocesses where dbid = OBJECT_ID('Meta'))
  begin
    declare p_id cursor for select spid from sysprocesses where dbid = OBJECT_ID('Meta')
    open p_id 
    fetch next from p_id into @spid
    while @@fetch_status <> -1
    begin
      select @spid_str = convert(char(2), @spid)
      exec("kill " + @spid_str)
      fetch next from id into @spid
    end
    deallocate p_id
  end

  select @bupath=@path+'Meta.'+@file_extension,@moveclause=''
  exec asisp_get_restore_move @bupath, 'Meta', @moveclause output
  exec('RESTORE DATABASE Meta FROM  DISK = "' + @bupath + '" WITH '+@moveclause+' NOUNLOAD, FILE = 1, REPLACE')

  -- start to load Object db
  while exists (select * from sysprocesses where dbid = OBJECT_ID('Object'))
  begin
    declare p_id cursor for select spid from sysprocesses where dbid = OBJECT_ID('Object')
    open p_id 
    fetch next from p_id into @spid
    while @@fetch_status <> -1
    begin
      select @spid_str = convert(char(2), @spid)
      exec("kill " + @spid_str)
      fetch next from id into @spid
    end
    deallocate p_id
  end

  select @bupath=@path+'Object.'+@file_extension,@moveclause=''
  exec asisp_get_restore_move @bupath, 'Object', @moveclause output
  exec('RESTORE DATABASE Object FROM  DISK = "' + @bupath + '" WITH '+@moveclause+' NOUNLOAD, FILE = 1, REPLACE')
  if exists (select * from Object..sysobjects where id = object_id('Object.dbo.event') and sysstat & 0xf = 3)
    drop table Object.dbo.event
  exec ('use Object
         exec (''create view event as select * from History..eventhist'')
        ')
  exec xp_cmdshell 'sc start w3svc'
GO

