/* $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/event_live.sqv   1.7   Aug 10 2001 10:42:02   lin  $ */

/* Normalized event table structure */

use Object
GO

-- Drop event table if it exists

if exists (select * from sysobjects where id = object_id('dbo.event') and sysstat & 0xf = 3)
	drop table dbo.event
GO

-- Create new event table

create table event
(
  /* Agent generating event */
  agent_cid	ClassID,
  agent_id	ObjID,

  /* Event identifiers from agent */
  agent_ctime   datetime,
  agent_seqno	varchar(16),

  /* Object receiving event */
  obj_cid	ClassID,
  obj_id	ObjID,

  /* Event attributes */
  cid		ClassID,
  id    	numeric(16) identity,
  name		ObjName,
  ctime		datetime,
  detail	varchar(255) null,
  alertstate	int,
  priority	int,
  state		int,
  tstamp	timestamp
  constraint event_PK primary key clustered (tstamp)
)

GO

-- Create view excluding identity column

if exists (select * from sysobjects where id = object_id('dbo.event_V') and sysstat & 0xf = 2)
	drop view dbo.event_V
GO

create view event_V as
  select
    agent_cid, agent_id, agent_ctime, agent_seqno, obj_cid, obj_id, cid, name, ctime, detail, alertstate, priority, state
  from event

GO

-- Create all insert triggers to create event table

if exists (select * from sysobjects where id = object_id('dbo.tI_ExceptionHistory_A') and sysstat & 0xf = 8)
	drop trigger dbo.tI_ExceptionHistory_A
GO

create trigger tI_ExceptionHistory_A on ExceptionHistory_A for insert as

--First insert from exceptions with valid OS in _SEQ_NO using os as agent

  insert event_V
  select
    'OS', os.id, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCH',
    o.name, a._TIME, a._EXCP_CD, isnull(e._AlertState,0), isnull(e._Priority,0), 0
  from inserted a, ExceptionHistory_C o, Exception_V e, OS_C os
  where
    a.id=o.id and e.id=a._ParentID and os.name=substring(a._SEQ_NO,1,4)

--Next insert from exceptions without valid os in _SEQ_NO using parent of history as agent

  insert event_V
  select
    e._ParentCID, e._ParentID, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCH',
    o.name, a._TIME, a._EXCP_CD, isnull(e._AlertState,0), isnull(e._Priority,0), 0
  from inserted a, ExceptionHistory_C o, Exception_V e
  where
    a.id=o.id and e.id=a._ParentID and a.id not in (select a.id from inserted a, OS_C os where substring(a._SEQ_NO,1,4)=os.name)

--Uncommenting these will enable delete of history
--delete from ExceptionHistory_A where id in (select id from inserted)
--delete from ExceptionHistory_C where id in (select id from inserted)

GO

if exists (select * from sysobjects where id = object_id('dbo.tI_Message_A') and sysstat & 0xf = 8)
	drop trigger dbo.tI_Message_A
GO

create trigger tI_Message_A on Message_A for insert as

--First insert from messages with valid OS in _SEQ_NO using os as agent

  insert event_V
  select
    'OS', os.id, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG',
    a._MESSAGE, a._TIME, a._REASON_CD, isnull(a._AlertState,0), isnull(a._Priority,0), isnull(a._StateID,0)
  from inserted a, OS_C os
  where
    os.name=substring(a._SEQ_NO,1,4)

--Next insert from messages without valid os in _SEQ_NO using parent of message as agent

  insert event_V
  select
    a._ParentCID, a._ParentID, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG',
    a._MESSAGE, a._TIME, a._REASON_CD, isnull(a._AlertState,0), isnull(a._Priority,0), isnull(a._StateID,0)
  from inserted a
  where
    a.id not in (select a.id from inserted a, OS_C os where substring(a._SEQ_NO,1,4)=os.name)

--Uncommenting this will enable delete of message
--delete from Message_C where id in (select id from inserted)

GO

if exists (select * from sysobjects where id = object_id('dbo.tI_ChildEventHistory_A') and sysstat & 0xf = 8)
	drop trigger dbo.tI_ChildEventHistory_A
GO

create trigger tI_ChildEventHistory_A on ChildEventHistory_A for insert as

--Insert from child events using parent as agent

  insert event_V
  select
    c._ParentCID, c._ParentID, o.ctime, '', c._ParentCID, c._ParentID, 'CHVH',
    o.name, o.ctime, a._EventDetail, isnull(c._AlertState,0), isnull(c._Priority,0), 0
  from inserted a, ChildEventHistory_C o, ChildEvent_V c
  where
    a.id=o.id and c.id=a._ParentID

--Uncommenting these will enable delete of history
--delete from ChildEventHistory_A where id in (select id from inserted)
--delete from ChildEventHistory_C where id in (select id from inserted)

GO

--Create eventbcp table

if exists (select * from sysobjects where id = object_id('dbo.eventbcp') and sysstat & 0xf = 3)
	drop table dbo.eventbcp
GO

create table eventbcp
(
  /* Agent generating event */
  agent_cid	ClassID,
  agent_id	ObjID,

  /* Event identifiers from agent */
  agent_ctime   datetime,
  agent_seqno	varchar(16),

  /* Object receiving event */
  obj_cid	ClassID,
  obj_id	ObjID,

  /* Event attributes */
  cid		ClassID,
  id    	numeric(16),
  name		ObjName,
  ctime		datetime,
  detail	varchar(255) null,
  alertstate	int,
  priority	int,
  state		int,
  tstamp	binary(8)
  constraint eventbcp_PK primary key clustered (tstamp)
)

GO

--Create eventbcp_V view for bcp out preserving datetime seconds and milliseconds

if exists (select * from sysobjects where id = object_id('dbo.eventbcp_V') and sysstat & 0xf = 2)
	drop view dbo.eventbcp_V
GO

create view eventbcp_V as
  select
    agent_cid,
    agent_id,
    agent_ctime=convert(varchar(30),agent_ctime,109),
    agent_seqno,
    obj_cid,
    obj_id,
    cid,
    id,
    name,
    ctime=convert(varchar(30),ctime,109),
    detail,
    alertstate,
    priority,
    state,
    tstamp
  from eventbcp

GO

--Create event_lastbcp table

if exists (select * from sysobjects where id = object_id('dbo.event_lastbcp') and sysstat & 0xf = 3)
	drop table dbo.event_lastbcp
GO

create table event_lastbcp
(
  lastbcp_tstamp binary(8)
)

GO

insert event_lastbcp select 0
GO

--Create asisp_moveevent procedure

if exists (select * from sysobjects where id = object_id('dbo.asisp_moveevent') and sysstat & 0xf = 4)
	drop procedure dbo.asisp_moveevent
GO

create procedure dbo.asisp_moveevent
  @targetserv sysname,
  @bcpdir varchar(255),
  @targetdb sysname='History',
  @targettable sysname='eventhist',
  @frompassword varchar(30)='',
  @topassword varchar(30)=''
AS

  declare @rtn int, @server sysname, @cmd varchar(255), @targetservlc sysname
  declare @tstamp binary(8)

  set nocount on

  select @server=lower(@@servername)
  select @targetservlc = lower(@targetserv)
  if @frompassword=''
    select @frompassword='sa_' + @server
  if @topassword=''
    select @topassword='sa_' + @targetservlc

--Clear eventbcp
  truncate table eventbcp
--Load all events into eventbcp exit if none to process
  select @tstamp=lastbcp_tstamp from event_lastbcp
  insert eventbcp select * from event where tstamp>@tstamp
  if @@rowcount=0
    return 0

--bcp out

  select @cmd='bcp Object.dbo.eventbcp_V out ' + @bcpdir + '\temp_event.txt -S' + @server + ' -Usa -P' + @frompassword + ' -c'
  exec @rtn=master..xp_cmdshell @cmd, no_output
  if @rtn<>0
  begin
    raiserror ('Unable to bcp eventbcp table out.', 0, 1)
    return -1
  end

--bcp into history

  select @cmd='bcp ' + @targetdb + '.dbo.' + @targettable + ' in ' + @bcpdir + '\temp_event.txt -S' + @targetservlc + ' -Usa -P' + @topassword + ' -c -b 1000 -m 10000 -e ' + @bcpdir + '\temp_event.err'
  exec @rtn=master..xp_cmdshell @cmd, no_output
  if @rtn<>0
  begin
    raiserror ('Unable to bcp into eventhist table in server: %s, database: %s', 0, 1, @targetservlc, @targetdb)
    return -1
  end

--Update last timestamp bcped
  update event_lastbcp set lastbcp_tstamp=(select max(tstamp) from eventbcp)

  return 0
GO

--Create asisp_deleteoldevent

if exists (select * from sysobjects where id = object_id('dbo.asisp_deleteoldevent') and sysstat & 0xf = 4)
	drop procedure dbo.asisp_deleteoldevent
GO

create procedure dbo.asisp_deleteoldevent
AS
  declare @tstamp binary(8)
  select @tstamp=lastbcp_tstamp from event_lastbcp
  delete event where tstamp<=@tstamp
GO




