-- archive/db/sql/hist_phys_objtree.sql
-- SID: 1.2
-- Date Modified: 01/11/01 18:19:33
--
-- Module Description
--   Defines the hist_phys_objtree stored procedure

USE Object
GO


--------------------------------------------------------------------------------------------------
-- hist_phys_objtree
PRINT 'Creating procedure hist_phys_objtree'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'hist_phys_objtree' AND type = 'P')
  DROP PROCEDURE hist_phys_objtree
GO

-- archive/db/sql/hist_phys_objtree.sql
-- SID: 1.2
-- Date Modified: 01/11/01 18:19:33

CREATE PROCEDURE hist_phys_objtree
  @pcid ClassID='ENT',
  @pid  ObjID=1
AS

  SET NOCOUNT ON

  CREATE TABLE #phystree
  (
    generation INT,
    cid ClassID,
    id  ObjID,
    parent_cid ClassID,
    parent_id ObjID,
    name ObjName,
    nodeid NVARCHAR(255),
    shortnodeid NVARCHAR(255)
  )


-- This PROCEDURE returns a result SET containing the entire indented
-- structure of pysical object.  It IS used to load the puysical structure
-- INTO a hyperbolic tree IN the same order AS the Access1 product.

  DECLARE @generation SMALLINT, @rc INT, @cid ClassID, @ctable sysname, @classdesc NVARCHAR(255)
  DECLARE @lvl SMALLINT, @holdpcid ClassID, @holdpid ObjID, @bDone TINYINT
  DECLARE @dot NCHAR(1), @pad SMALLINT, @parnodeid NVARCHAR(255), @shortparnodeid NVARCHAR(255)
  SELECT @generation = 1
  INSERT #phystree
    SELECT @generation, @pcid, @pid, '', 0, '', '', ''

  SELECT @rc = @@ROWCOUNT
  WHILE @rc > 0
  BEGIN
    SELECT @generation = @generation + 1
    INSERT #phystree
    SELECT DISTINCT @generation, dst_cid, dst_id, src_cid, src_id, '', '', ''
    FROM #phystree d, link l, link_type lt
    WHERE
        l.src_cid=d.cid
        AND src_id=id
        AND deleted=0

        AND l.link_type=lt.link_type
        AND lt.flags&2=2
        AND d.generation=@generation-1
    SELECT @rc = @@ROWCOUNT
  END

-- Load the object names FOR one class at a time

  DECLARE cc INSENSITIVE CURSOR FOR SELECT DISTINCT cid FROM #phystree
  OPEN cc
  FETCH NEXT FROM cc INTO @cid
  WHILE @@FETCH_STATUS<>-1
  BEGIN
    SELECT @ctable=cname + "_V" FROM obj_class WHERE cid=@cid
    EXEC ('UPDATE #phystree SET name=c.name FROM ' + @ctable + ' c, #phystree d WHERE d.cid=''' + @cid + ''' AND c.id=d.id')
    FETCH NEXT FROM cc INTO @cid
  END
  DEALLOCATE cc

-- The following builds the nodeid NVARCHAR column IN the form 00001.00001....etc
-- This represents the indented structure of the tree sorted IN the same order
-- AS IS used IN the Access1 product.  The #lvlcounter TABLE, by USE of an
-- identity column, IS used to CREATE the numbered fields of this nodeid.

  SELECT @lvl=1, @bDone=0, @pad=5, @parnodeid='', @shortparnodeid=''

  CREATE TABLE #lvlcounter (cid ClassID, id ObjID, ident INT identity)
  SELECT @dot=''

  WHILE @bDone=0
  BEGIN
    DECLARE cc INSENSITIVE CURSOR FOR
    SELECT DISTINCT parent_cid, parent_id FROM #phystree WHERE generation=@lvl
    OPEN cc
    FETCH NEXT FROM cc INTO @holdpcid, @holdpid
    IF @@FETCH_STATUS=-1
      SELECT @bDone=1
    WHILE @@FETCH_STATUS<>-1 AND @bDone=0
    BEGIN
      TRUNCATE TABLE #lvlcounter
      SELECT @parnodeid = LTRIM(nodeid), @shortparnodeid=LTRIM(shortnodeid) FROM #phystree WHERE cid=@holdpcid AND id=@holdpid
      INSERT #lvlcounter (cid, id) SELECT d.cid, d.id FROM #phystree d, obj_class o WHERE d.parent_cid=@holdpcid AND d.parent_id=@holdpid AND o.cid=d.cid ORDER BY o.label, d.name
      UPDATE #phystree SET nodeid = LTRIM(@parnodeid + @dot + right(replicate("0",@pad) + LTRIM(str(c.ident)),@pad)), shortnodeid=LTRIM(@shortparnodeid + @dot + LTRIM(str(c.ident)))
        FROM #lvlcounter c, #phystree d
        WHERE c.cid=d.cid AND c.id=d.id AND generation=@lvl
      FETCH NEXT FROM cc INTO @holdpcid, @holdpid
    END
    DEALLOCATE cc
    SELECT @lvl = @lvl + 1
    SELECT @dot='.'

  END

-- RETURN result SET IN the proper order

  SELECT p.generation-1 generation, p.shortnodeid, p.cid, o.label, p.id, p.name FROM #phystree p, obj_class o WHERE o.cid=p.cid ORDER BY nodeid

  SET NOCOUNT OFF

