-- archive\db\sql\tu_cics_a.sql
-- SID: 1.1
-- DateModified: 01/11/20 12:11:55
-- 
-- Define module tU_CICS_A
-- Fix defect OW51612

USE Object
GO
-----------------------------------------------------------------------------------------------------------
-- tU_CICS_A
PRINT 'Create trigger tU_CICS_A.'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_CICS_A' AND type = 'TR')
  DROP TRIGGER tU_CICS_A
GO 

-- SID: 1.1
-- DateModified: 01/11/20 12:11:55
CREATE TRIGGER tU_CICS_A ON CICS_A FOR UPDATE AS
  DECLARE @rc INT
  SELECT @rc = @@ROWCOUNT

  RAISERROR('tU_CICS_A: @@ROWCOUNT=%d, @@TRANCOUNT=%d, @@nestlevel=%d', 
            0, 1, @rc, @@TRANCOUNT, @@nestlevel)

  DECLARE @id ObjID

  -- CREATE the link attribute specification strings FOR CICS states Active, Inactive, Starting, AND Stopping

  -- This makes the comparisons FOR CurrentState AND DesiredState more readable.
  DECLARE @active INT, @inactive INT, @starting INT, @stopping INT, @abended INT, @dontcare INT
  SELECT @active = 27
  SELECT @inactive = 28

  SELECT @starting = 29
  SELECT @stopping = 30
  SELECT @abended = 32
  SELECT @dontcare = 72

  -------------------------------------------------------------------------------------------------
  -- Beginning of custom code FOR handling changing _DesiredState.
  -------------------------------------------------------------------------------------------------
  DECLARE @StartupDelay INT, @ShutdownDelay INT, @time DATETIME, @now DATETIME, @ids INT,
          @ics INT, @dds INT, @unvf INT
  IF UPDATE(_DesiredStateID)
  BEGIN
    RAISERROR('tU_CICS_A: _DesiredStateID updated', 0, 1)
    INSERT cics_state_observation
      SELECT i.id, GETDATE(), 'desired', d._DesiredStateID, i._DesiredStateID, NULL
      FROM deleted d, inserted i
      WHERE d.id = i.id AND i._DesiredStateID <> d._DesiredStateID
    -----------------------------------------------------------------------------------------------
    -- FOR all rows WHERE _DesiredStateID IS being changed to 'Starting' OR 'Stopping' we schedule a
    -- task to run at a number of minutes IN the future, AS determined by the CICS _StartupDelay OR
    -- _ShutdownDelay attribute, which will change the desired state of the CICS to 'Active' OR
    -- 'Inactive'
    -----------------------------------------------------------------------------------------------
--      SELECT i.id,d._DesiredStateID,i._DesiredStateID FROM inserted i, deleted d 
--      WHERE 
--        i.id = d.id
    IF EXISTS
    (
      SELECT * FROM inserted i, deleted d 
      WHERE 
        i.id = d.id
        AND i._DesiredStateID IN (@starting, @stopping)
        AND d._DesiredStateID <> i._DesiredStateID
    )
    BEGIN
      RAISERROR('tU_CICS_A: rows with _DesiredStateID SET to %d OR %d OR %d', 0, 1,
                @starting, @stopping, @dontcare)
      IF @rc = 1
      BEGIN
        SELECT 
          @id = id,
          @ids = _DesiredStateID,
          @StartupDelay = _StartupDelay,
          @ShutdownDelay = _ShutdownDelay
        FROM
          inserted
        IF @ids = @starting AND @StartupDelay <> 0
          EXEC asisp_schedulecicstransition @id, @active, @StartupDelay
        ELSE IF @ids = @stopping AND @ShutdownDelay <> 0
          EXEC asisp_schedulecicstransition @id, @inactive, @ShutdownDelay
      END
      ELSE
      BEGIN
        DECLARE cics_startup_c CURSOR FOR 
          SELECT DISTINCT i.id, i._DesiredStateID, i._StartupDelay 
          FROM inserted i, deleted d
          WHERE 
            i.id = d.id 
            AND i._DesiredStateID = @starting 
            AND d._DesiredStateID <> i._DesiredStateID
            AND i._StartupDelay <> 0
        OPEN cics_startup_c
        FETCH NEXT FROM cics_startup_c INTO @id, @ids, @StartupDelay
        WHILE @@FETCH_STATUS <> -1
        BEGIN
          EXEC asisp_schedulecicstransition @id, @active, @StartupDelay
          FETCH NEXT FROM cics_startup_c INTO @id, @ids, @StartupDelay
        END
        DEALLOCATE cics_startup_c
        DECLARE cics_shutdown_c CURSOR FOR 
          SELECT DISTINCT i.id, i._DesiredStateID, i._ShutdownDelay 
          FROM inserted i, deleted d
          WHERE 
            i.id = d.id
            AND i._DesiredStateID = @stopping 
            AND d._DesiredStateID <> i._DesiredStateID
            AND i._ShutdownDelay <> 0
        OPEN cics_shutdown_c
        FETCH NEXT FROM cics_shutdown_c INTO @id, @ids, @ShutdownDelay
        WHILE @@FETCH_STATUS <> -1
        BEGIN
          EXEC asisp_schedulecicstransition @id, @inactive, @ShutdownDelay
          FETCH NEXT FROM cics_shutdown_c INTO @id, @ids, @ShutdownDelay
        END
        DEALLOCATE cics_shutdown_c
      END
    END

    -----------------------------------------------------------------------------------------------
    -- Iterate through all rows with changing DesiredState to check FOR SCHV (schedule violation)
    -- AND Unavailable Files (UNVF) conditions that need to be generated OR cleared.
    -----------------------------------------------------------------------------------------------
    DECLARE @excp_cd NVARCHAR(255)
    DECLARE cics_a_c CURSOR FOR 
    SELECT 
      i.id,
      d._DesiredStateID,
      i._DesiredStateID,
      i._CurrentStateID,
      i._UnavailableFiles
    FROM
      inserted i,
      deleted d
    WHERE
      i.id = d.id
      AND i._DesiredStateID <> d._DesiredStateID
    OPEN cics_a_c
    FETCH NEXT FROM cics_a_c INTO @id, @dds, @ids, @ics, @unvf
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      RAISERROR('DesiredState UPDATE: (DDS=%d) (IDS=%d) (ICS=%d) (UF=%d)',
            0, 1, @dds, @ids, @ics, @unvf)

      DECLARE @cno SMALLINT, @r INT
      SELECT @cno = cno FROM obj_class WHERE cid = 'CICS'
      EXEC @r = asisp_notifyDesiredStateChange
        17,    -- ASIXpEvent::DesiredStateChanged
        @cno,
        @id,
        @dds,
        @ids

      IF @r < 0
      BEGIN
        RAISERROR ('tU_CICS_A: asisp_notifyDesiredStateChange failed', 11, 1, @id)
        ROLLBACK TRANSACTION
        DEALLOCATE cics_a_c
        RETURN
      END
      -- RAISERROR ('tU_CICS_A: asisp_notifyDesiredStateChange succeeded', 0, 1, @id)

      -- Check to see whether SCHV (schedule violation) needs to be generated OR cleared
      SELECT @excp_cd = NULL
      IF @ids <> @ics
      BEGIN
        IF @ids = @active
        BEGIN
          -- Confirm that @ics IS correct -- it appears NOT to be!!!
          DECLARE @verify_ics INT
          SELECT @verify_ics = _CurrentStateID FROM CICS_A WHERE id = @id
          IF @ics <> @verify_ics
            INSERT cics_state_observation SELECT @id, GETDATE(), 'anomoly', @ics, @verify_ics, 'state1 FROM TRIGGER. state2 FROM database'
          IF @verify_ics <> @active
          BEGIN   
            DECLARE @ids_name NVARCHAR(16), @ics_name NVARCHAR(16)
            SELECT @ids_name = _State FROM StateType_A WHERE id = @ids
            SELECT @ics_name = _State FROM StateType_A WHERE id = @ics
            SELECT @excp_cd = 'REGION SHOULD NOW BE ''' + @ids_name + ''' (' + CONVERT(NVARCHAR(9),@ids) + '). ' +
                              'ACTUAL STATE IS '''  + @ics_name + ''' (' + CONVERT(NVARCHAR(9),@ics) + ').'
          END
        END
        ELSE IF @ids = @inactive
          SELECT @excp_cd = 'REGION SHOULD BE INACTIVE'
        ELSE IF @ics IN (@active, @inactive) 
             AND @ids IN (@starting, @stopping, @dontcare) 
             AND @dds IN (@inactive, @active) 
             AND @dds <> @ics
          -- DesiredState IS going FROM non-transitional to transitional, AND the old
          -- DesiredState differs FROM the CurrentState.  There should therefore be
          -- AND outstanding SCHV that needs to be cleared.
          SELECT @excp_cd = 'OK'
      END
      ELSE       -- DesiredState=CurrentState
      BEGIN
        -- IF the old DesiredState was NOT one of the transitional states, THEN there should be 
        -- an outstanding SCHV that needs to be cleared
        IF @dds IN (@active, @inactive)
          SELECT @excp_cd = 'OK' 
      END
      IF @excp_cd IS NOT NULL
      BEGIN
        RAISERROR('asisp_SCHVException ''%s'', %d, ''%s''', 0, 1, 'CICS', @id, @excp_cd)
        INSERT cics_state_observation SELECT @id, GETDATE(), 'current-desired', @ics, @ids, @excp_cd
        EXEC asisp_SCHVException 'CICS', @id, @excp_cd
      END
        
      -- IF there are unavailable files THEN we may SET OR clear a UNVF based ON DesiredState
      IF @unvf > 0
        IF @ids = @active
          -- DesiredState becoming active, so UnavailableFiles>0 causes UNVF
          EXEC asisp_UNVFException 'CICS', @id, 'ONE OR MORE FILES ARE UNAVAILABLE'
        ELSE IF @dds = @active
          -- DesiredState becoming !active, so UnavailableFiles>0 implies existing UNVF that needs
          -- to be cleared
          EXEC asisp_UNVFException 'CICS', @id, 'OK'
                    
      -- Get the NEXT row
      FETCH NEXT FROM cics_a_c INTO @id, @dds, @ids, @ics, @unvf
    END
    DEALLOCATE cics_a_c
  END
  -------------------------------------------------------------------------------------------------
  -- END of custom code FOR handling changing _DesiredStateID
  -------------------------------------------------------------------------------------------------

  -------------------------------------------------------------------------------------------------
  -- Beginning of custom code FOR handling changes to _UnavailableFiles.  Under normal 
  -- circumstances this attribute only gets updated ON active regions.  IF the region's
  -- DesiredState IS also 'Active' THEN what we do IS generate a UNVF exception ON any region WHERE
  -- _UnavailableFiles IS going FROM 0 to >0, AND clear the UNVF WHERE _UnavailableFiles IS going
  -- FROM >0 to 0.
  -------------------------------------------------------------------------------------------------
  DECLARE @duf INT, @iuf INT
  IF UPDATE(_UnavailableFiles)
  BEGIN
    IF @rc = 1  -- single row UPDATE
    BEGIN
      SELECT @id = i.id FROM inserted i, deleted d
      WHERE i._UnavailableFiles > 0 AND d._UnavailableFiles = 0 AND i._DesiredStateID = @active
      IF @@ROWCOUNT = 1
        EXEC asisp_UNVFException 'CICS', @id, 'ONE OR MORE FILES ARE UNAVAILABLE'
      ELSE
      BEGIN
        -- Check FOR change to no unavailable files ON active region
        SELECT @id = i.id FROM inserted i, deleted d
        WHERE i._UnavailableFiles = 0 AND d._UnavailableFiles > 0 AND i._DesiredStateID = @active
        IF @@ROWCOUNT = 1  -- clear the exception
          EXEC asisp_UNVFException 'CICS', @id, 'OK'
      END
    END
    ELSE        -- multirow UPDATE
    BEGIN
      INSERT StagedEXCP
      (
        parent_cid,
        parent_id,
        TIME,
        SEQ_NO,
        OBJECT_NAME,
        NATIVE_KEY,
        EXCP_OBJ_TYPE,
        EXCP_OBJ_NAME,
        EXCP_NAME,
        EXCP_CD,
        nEXCH,
        AlertState,
        Priority
      )
        SELECT 'CICS', i.id, GETDATE(), NULL, NULL, NULL, NULL, NULL, 'UNVF', 
          CASE WHEN (i._UnavailableFiles > 0 AND d._UnavailableFiles = 0) THEN 'ONE OR MORE FILES ARE UNAVAILABLE' ELSE 'OK' END,
          1, NULL, NULL
        FROM inserted i, deleted d
        WHERE 
          i.id = d.id 
          AND (
               (i._UnavailableFiles > 0 AND d._UnavailableFiles = 0)
               OR
               (i._UnavailableFiles = 0 AND d._UnavailableFiles > 0)
              )
          AND d._CurrentStateID = @active
          AND i._CurrentStateID = @active
          AND i._DesiredStateID = @active
    END
  END

  -------------------------------------------------------------------------------------------------
  -- Beginning of custom code FOR handling changes to _CurrentStateID.  Most changes to CurrentState
  -- are single-row updates based ON state-change events we receive.
  -------------------------------------------------------------------------------------------------
  DECLARE @dcs INT, @dsactive TINYINT
  IF UPDATE(_CurrentStateID)
  BEGIN
    RAISERROR('tU_CICS_A: _CurrentStateID updated', 0, 1)
    INSERT cics_state_observation

      SELECT i.id, GETDATE(), 'current', d._CurrentStateID, i._CurrentStateID, NULL
      FROM deleted d, inserted i
      WHERE d.id = i.id AND i._CurrentStateID <> d._CurrentStateID
    IF @rc = 1      -- single row UPDATE
    BEGIN
      SELECT 
        @id = i.id,
        @dcs = d._CurrentStateID,
        @ics = i._CurrentStateID,
        @dsactive = CASE i._DesiredStateID WHEN @active THEN 1 ELSE 0 END,
        @unvf = i._UnavailableFiles
      FROM inserted i, deleted d
      IF @ics = @active                     -- region becoming active
      BEGIN
        IF @dcs = @abended
          EXEC asisp_processactivecics @id, @unvf, 1, @dsactive
        ELSE
          EXEC asisp_processactivecics @id, @unvf, 0, @dsactive
        -- Check FOR schedule violation
        SELECT @ids = _DesiredStateID FROM inserted
        IF @ids  = @inactive
        BEGIN
          INSERT cics_state_observation SELECT @id, GETDATE(), 'current-desired', @ics, @ids, 'REGION SHOULD BE INACTIVE'
          EXEC asisp_SCHVException 'CICS', @id, 'REGION SHOULD BE INACTIVE'
        END
        ELSE IF @ids = @active
        BEGIN
          INSERT cics_state_observation SELECT @id, GETDATE(), 'current-desired', @ics, @ids, 'OK'
          EXEC asisp_SCHVException 'CICS', @id, 'OK'
        END
      END
      ELSE IF @dcs = @active                -- region becoming !active
      BEGIN
        IF @ics = @inactive
        BEGIN
          IF @unvf > 0  -- Mark FILEs AS stale, reset UnavailableFiles
            EXEC asisp_processinactivecics @id, 1, 1
          ELSE          -- Mark FILEs AS stale, don't reset UnavailableFiles
            EXEC asisp_processinactivecics @id, 1, 0
          -- Check FOR schedule violation
          SELECT @ids = _DesiredStateID FROM inserted
          IF @ids  = @active
          BEGIN
            SELECT @ids_name = _State FROM StateType_A WHERE id = @ids
            SELECT @ics_name = _State FROM StateType_A WHERE id = @ics
            SELECT @excp_cd = 'REGION SHOULD BE ''' + @ids_name + ''' (' + CONVERT(NVARCHAR(9),@ids) + '). ' +
                              'ACTUAL STATE IS NOW '''  + @ics_name + ''' (' + CONVERT(NVARCHAR(9),@ics) + ').'
            EXEC asisp_SCHVException 'CICS', @id, @excp_cd
          END
          ELSE IF @ids = @inactive
            EXEC asisp_SCHVException 'CICS', @id, 'OK'
        END
        ELSE
          -- Mark FILEs AS stale, don't reset UnavailableFiles (abnormal END)
          EXEC asisp_processinactivecics @id, 1, 0
      END
      ELSE IF @dcs = @abended AND @ics = @inactive
        -- Don't mark FILEs, reset UnavailableFiles
        IF @unvf > 0
          EXEC asisp_processinactivecics @id, 0, 1
    END
  END

  RAISERROR('tU_CICS_A: finished', 0, 1)

